/* ============================================================================================================================================= /
 * PROJECT:         Modulo remoto per stazione meteo
 * FILE:            Main.c
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F26K22
 * CREATED:         08/12/2016
 * UPDATED:         28/02/2016
 *
 * DESCRIP:         Gestione modulo remoto sensori per stazione meteo. Il modulo legge ogni N secondi i sensori estraendo solo i dati grezzi e li
 *                  invia alla stazione tramite modulo HMTRP 433MHz.
 * ============================================================================================================================================= /
 * Change History:
 * Rev.             Date                Changes
 * 1.1              09/08/2016          Correzione tempi di invio
 * 1.0              28/02/2016          Prima versione completa
 *                  26/02/2016          Inizio sviluppo firmware
 *                  08/12/2016          Test Convertitore Step-up 3.3V e sensore tensione batteria.
 *                                      Test SLEEP
 /* ============================================================================================================================================ */

 /* INCLUDES =================================================================================================================================== */
#include <xc.h>
#include "Main.h"
#include "Prototipi.h"
#include "Delay.c"
#include "USART_init.c"
#include "I2C_init.c"

#include "HMTRP.c"

#include "HDC1080.c"
#include "LPS25HB.c"
#include "VEML6075.c"

#include "EEPROM.c"

/* GLOBAL VARIABLES =========================================================================================================================== */
unsigned char vbat = 0;
unsigned int temp_hdc1080_raw = 0;
unsigned int humidity_hdc1080_raw = 0;
unsigned long press_lps25_raw = 0;
signed int temp_lps25_raw = 0;
veml6075_uv_raw uv_readed_raw;
volatile unsigned int timer_preload = 0;
volatile char timer_sec = 0;


/* INTERRUPT ================================================================================================================================== */
interrupt high_priority void inter(void) {
    
/* INTERRUPT TIMER 0 PER USCIRE DA IDLE */
    if (INTCONbits.TMR0IE && INTCONbits.TMR0IF) {  
        T0CONbits.TMR0ON = 0;                                                           // spengo il timer e 
        OSCCONbits.IRCF0 = 1;                                                           // imposto oscillatore interno 1 MHz
        OSCCONbits.IRCF1 = 1;    
        OSCCONbits.IRCF2 = 0; 

        INTCONbits.TMR0IF = 0;
    }
     
/* INTERRUPT PER HMTRP */    
    else if (HMTRP_INTERRUPT_IE && HMTRP_INTERRUPT_IF) {                              // se l'interrupt  stato causato da una ricezione dati USART
        
        hmtrp_buffer[hmtrp_buffer_index] = HMTRP_receive();                      // salvo i dati ricevuti nell'array usart_received
        
    if (hmtrp_buffer_index>0){                                                          // Se trovo caratteri di fine stringa
        if ((hmtrp_buffer[hmtrp_buffer_index]== 0x0A) && (hmtrp_buffer[hmtrp_buffer_index-1])== 0x0D){
            hmtrp_string_received = 1;                                                  // imposto flag di ricezione avvenuta
            hmtrp_buffer[hmtrp_buffer_index] = 0x00;                                    // e tolgo caratteri di fine stringa
            hmtrp_buffer[hmtrp_buffer_index-1] = 0x00;
        }
    }
    
        hmtrp_buffer_index++;
        
        if (hmtrp_buffer_index > HMTRP_BUFFER_LENGTH)                                   // se supero la dimensione massima resetto l'indice
            hmtrp_buffer_index = 0;    
    }
}

interrupt low_priority void interlow(void) {
    if (PIE1bits.TMR1IE && PIR1bits.TMR1IF){
        TMR1 = 34286;
        
        timer_sec++;
        PIR1bits.TMR1IF = 0;
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:        Main
 *
 * Overview:        - All' avvio il modulo invia  per 3 secondi segnale di presenza (0xF0 via HMTRP) e attende risposta di presenza dalla stazione (0xF0)
 *                    a connessione avvenuta i led lampeggia 2 volte e controlla se sono stati ricevuti comandi di hardware check o settaggio tempo di invio
 *                      - Se riceve 0xF1 (hardware check) legge ID e invia risposte poi inizia il ciclo continuo
 *                      - Se riceve 0xF4 impostazione tempo di invio
 *                  - se non sono stati ricevuti comandi o se sono stati ricevuti ed elaborati entra in ciclo continuo 
 *                  - ciclo continuo:
 *                      - legge sensori
 *                      - legge batteria
 *                      - abilita  modulo HMTRP
 *                      - invia valori
 *                      - disabilita modulo HMTRP
 *                      - avvia timer 0 per uscire da sleep
 *                      - sleep
/* ============================================================================================================================================ */


void main(void) {
    Define_IO();
    Module_settings();
    
    HMTRP_init();                                                                       // Inizializzo modulo HMTRP
    USART_enable(USART_BAUDR_9600);                                                     // Inizializzo USART  per HMTRP
    
    I2C_init_master (400);                                                              // Inizializzo I2C modulo 1 per HDC1080, LPS25HB, VEML6075
    
    Read_time_eeprom();                                                                 // leggo da EEPROM il valore del tempo di invio e imposto il preload del timer
    
/* IMPOSTAZIONE DEI SENSORI */    
    HDC1080_set(HDC1080_ONE_AQ,HDC1080_TEMP_RES_14bit,HDC1080_HUMI_RES_14bit);
    LPS25HB_set_resolution(LPS25HB_DEV_ADDR,LPS25HB_TEMP_RES_8,LPS25HB_PRES_RES_8);  
    LPS25HB_set_mode(LPS25HB_DEV_ADDR,LSP25HB_ODR_ONE_SHOT);                            
    LPS25HB_set_onepoint_cal(LPS25HB_DEV_ADDR, LPS25HB_ONE_POINT_CAL);
    VEML6075_set(VEML6075_ONE_AQ,VEML6075_NORMAL_DR,VEML6075_IT_50MS);
    
/* CONNESSIONE ALLA STAZIONE */
    
    HMTRP_mode(HMTRP_NORMAL_MODE);                                                      // attivo modulo HMTRP
    T1CONbits.TMR1ON = 1;
    do{
        HMTRP_send_char(0xF0);                                                          // invio continuo di 0xF0 fino a ricezione stringa o per 3 secondi
        HMTRP_send_char(0x0D);
        HMTRP_send_char(0x0A);
        Delay_ms(50);
    } while (hmtrp_string_received == 0 && timer_sec != TIMEOUT_SETTINGS);

    T1CONbits.TMR1ON  = 0;
    if (hmtrp_buffer[0] == 0xF0){                                                       // se ricevo 0xF0 sono in attesa di ricevere un comando
        HMTRP_reset_buffer();                                                           // reset buffer e lampeggio led

        LED_1 = 1;
        Delay_ms(200);
        LED_1 = 0;
        Delay_ms(600);
        LED_1 = 1;
        Delay_ms(200);
        LED_1 = 0;
    }

    if (hmtrp_string_received){                                                         // Se ricevo un'altra stringa

/* CHECK HARDWARE 0xF1 */  
        if (hmtrp_buffer[0] == 0xF1){                                               
            Delay_ms(100);      

            if (HDC1080_read_device_id() == 0x1050)                                     // leggo id sensori e invio 
                HMTRP_send_char (0xF2);                                                 // 0xF2 se id corretto
            else
                HMTRP_send_char (0xF3);                                                 // 0xF3 se id errato o sensore non trovato

            if (LPS25HB_read_device_id(LPS25HB_DEV_ADDR) == 0xBD)
                HMTRP_send_char (0xF2);
            else
                HMTRP_send_char (0xF3);

            if (VEML6075_read_device_id() == 0x0026)
                HMTRP_send_char (0xF2);
            else
                HMTRP_send_char (0xF3);

            HMTRP_send_char(0x0D);                                                      // fine stringa
            HMTRP_send_char(0x0A);
        } 

/* IMPOSTAZIONE TEMPO ACQUISIZIONE E INVIO */         
        else if (hmtrp_buffer[0] == 0xF4){                                              // se ricevo 0xF4
            EEPROM_write(hmtrp_buffer[1], 0x00);                                        // salvo il valore ricevuto in EEPROM
            Read_time_eeprom();                                                         // rileggo il valore
        }
        HMTRP_reset_buffer();
    }

/* CICLO CONTINUO LETTURA E INVIO DATI */    
    while (1){
        temp_hdc1080_raw = HDC1080_read_temp_raw();                                     // leggo temperatura e umidit
        humidity_hdc1080_raw = HDC1080_read_humidity_raw();  

        LPS25HB_power_mode(LPS25HB_DEV_ADDR,LSP25HB_ACTIVE);                            // attivo il sensore LPS25HB
        LPS25HB_shot(LPS25HB_DEV_ADDR);                                                 // shot
        press_lps25_raw = LPS25HB_read_press_raw(LPS25HB_DEV_ADDR);                     // leggo pressione e temperatura
        temp_lps25_raw = LPS25HB_read_temp_raw(LPS25HB_DEV_ADDR);                      
        LPS25HB_power_mode(LPS25HB_DEV_ADDR,LSP25HB_POWER_DOWN);                        // disattivo il sensore

        VEML6075_power_mode(VEML6075_ACTIVE);                                           // attivo il sensore VEML6075
        VEML6075_shot();
        Delay_ms(200);                                                                  // attesa lettura e conversione
        uv_readed_raw = VEML6075_read_uv_raw();                                         // leggo valori UV
        VEML6075_power_mode(VEML6075_POWER_DOWN);

        vbat =  Read_vbat();                                                            // leggo livello batteria

        HMTRP_mode(HMTRP_NORMAL_MODE);                                                  // attivo modulo HMTRP

        HMTRP_send_char(0xFA);                                                          // invio carattere inizio stringa con dati

        HMTRP_send_char(temp_hdc1080_raw >> 8);                                         // invio tutti i valori (19 byte + 2 di chiusura)
        HMTRP_send_char(temp_hdc1080_raw);

        HMTRP_send_char(humidity_hdc1080_raw >> 8);
        HMTRP_send_char(humidity_hdc1080_raw);

        HMTRP_send_char(press_lps25_raw >> 24);
        HMTRP_send_char(press_lps25_raw >> 16);
        HMTRP_send_char(press_lps25_raw >> 8);
        HMTRP_send_char(press_lps25_raw);

        HMTRP_send_char(temp_lps25_raw >> 8);
        HMTRP_send_char(temp_lps25_raw);

        HMTRP_send_char(uv_readed_raw.uva >> 8);
        HMTRP_send_char(uv_readed_raw.uva);
        HMTRP_send_char(uv_readed_raw.uvb >> 8);
        HMTRP_send_char(uv_readed_raw.uvb);
        HMTRP_send_char(uv_readed_raw.uv_comp_1 >> 8);
        HMTRP_send_char(uv_readed_raw.uv_comp_1);
        HMTRP_send_char(uv_readed_raw.uv_comp_2 >> 8);
        HMTRP_send_char(uv_readed_raw.uv_comp_2);

        HMTRP_send_char(vbat);

        HMTRP_send_char(0x0D);                                                          // byte di chiusura stringa
        HMTRP_send_char(0x0A);

        HMTRP_reset_buffer();

        HMTRP_mode(HMTRP_SLEEP_MODE);                                                   // disattivo il modulo HMTRP

        OSCCONbits.IRCF0 = 0;                                                           //cambio clock da 1 MHz a 31.25Khz
        OSCCONbits.IRCF1 = 0;    
        OSCCONbits.IRCF2 = 0;  

        WRITETIMER0(timer_preload);
        T0CONbits.TMR0ON = 1;                                                           // avvio timer per uscire da idle

        SLEEP();
        Nop();
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:        Read_vbat
 * Output:          0: VBAT < 1.2V
 *                  1: 1.2V >= VBAT > 1.8V
 *                  2: 1.8V >= VBAT > 2.4V
 *                  3: 2.4V >= VBAT > 3.2V    
 *                  4: VBAT >= 3.2V (Alimentatore)
 * 
 * Overview:        Controlla la tensione di alimentazione pre convertitore (VBAT) e torna 1 se  maggiore della soglia impostata, 0 se  minore
/* ============================================================================================================================================ */
unsigned char Read_vbat(void){
    unsigned int _vbat = 0;
    unsigned int _read = 0;
    unsigned int _return = 0;
    
    TRISAbits.RA5 = 1;                                                                  // abilito AN4
    ANSELAbits.ANSA5 = 1;
    
    for (unsigned char _i = 0; _i<20; _i++){                                            // ciclo 20 letture
        ADCON0bits.ADON = 1;
        ADCON0bits.GO_DONE = 1;
        while (ADCON0bits.GO_DONE);
        ADCON0bits.ADON = 0;

        _read = (unsigned int) ADRESH << 8;
        _read |= ADRESL;

    _vbat = _read + _vbat;
    }    
    
    _vbat = _vbat/20;                                                                   // media del risultato
    
    if (_vbat < VBAT_L1)
        _return = 0;
    else if (_vbat >= VBAT_L1 && _vbat < VBAT_L2)
        _return =  1;
    else if (_vbat >= VBAT_L2 && _vbat < VBAT_L3)
        _return =  2;
    else if (_vbat >= VBAT_L3 && _vbat < VBAT_L4)
        _return =  3;
    else if (_vbat >= VBAT_L4) {                                                        // Vbat > 3.2 V significa che l'alimentazione  pi alta. in casi in cui VCC   tra o 4 e i 6V  
        TRISAbits.RA5 = 0;                                                              // la tensione in eccesso si somma a VCC. Per evitare questo problema
        ANSELAbits.ANSA5 = 0;                                                           // disabilito AN4 quando non serve impostandolo come uscita digitale
        _return =  4;
    }
    
    return _return;    
}

/* ============================================================================================================================================= /
 * FUNCTION:        Read_time_eeprom
 * 
 * Overview:        Legge il valore salvato in eeprom (1~8) e imposta il preload del Timer 0 di conseguenza, se legge 0 o FF imposta di default il
 *                  preload minore
/* ============================================================================================================================================ */
void Read_time_eeprom(void){
    unsigned char _readed = 0;
    
    _readed = EEPROM_read(0x00);
    
    switch (_readed){
        
        case 1: timer_preload = TMR0_PRELOAD_1;
            break;
        case 2: timer_preload = TMR0_PRELOAD_2;
            break;
        case 3: timer_preload = TMR0_PRELOAD_3;
            break;
        case 4: timer_preload = TMR0_PRELOAD_4;
            break;
        case 5: timer_preload = TMR0_PRELOAD_5;
            break;
        case 6: timer_preload = TMR0_PRELOAD_6;
            break;
        case 7: timer_preload = TMR0_PRELOAD_7;
            break;
        case 8: timer_preload = TMR0_PRELOAD_8;
            break;
            
        default: timer_preload = TMR0_PRELOAD_1;
            break;            
    }
    
}