/* ============================================================================================================================================ */
/* LIBRARY:         Lettura e scrittura da EEPROM interna
/* FILE:            EEPROM.c
/* LAST REV:        1.2
/* AUTHOR:          Ivo Colleoni
/* DEVICE:          PIC18F
/* CREATED:         24/09/2013
/* UPDATED:         22/04/20176
/*
/* DESCRIP:         Gestione lettura e scrittura EEPROM interna
/*
/* FUNCTIONS:       - EEPROM_read
/*                  - EEPROM_write
/*                  - EEPROM_read_int
/*                  - EEPROM_write_int
/* ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.2               22/04/2017          - Cicli di abilitazione alla scrittura in EEPROM in ASM per problemi di compilazione con PIC18F67K22
  1.1               21/07/2016          - Implementata compatibilit  con PIC con pi di 256Mb di EEPROM
                                        - Scrittura dato finch non viene scritto correttamente
  1.0               24/09/2013          - Prima release
//! [History]
/* ============================================================================================================================================ */

#ifndef EEPROM_INT
#define EEPROM_INT

/* INCLUDES =================================================================================================================================== */
#include "EEPROM.h"

/* ============================================================================================================================================ */
/* FUNCTION:        EEPROM_read
/* Input:           _addr: indirizzo da leggere (unsigned int)
/* Output           valore letto (unsigned char)
/*
/* Overview:        Restituisce il valore contenuto nell'indirizzo dichiarato
/* ============================================================================================================================================ */

unsigned char EEPROM_read(unsigned int _addr) {
    unsigned char _readed = 0;
    
    if (_addr < _EEPROMSIZE) {                                                          // leggo solo se l'indirizzo non supera la memoria massima  
        EEADR = _addr & 0x00FF;                                                         // imposto indirizzo byte basso
        #if _EEPROMSIZE > 256                                                           // MCU con eeprom > 256
            EEADRH = _addr >> 8;                                                        // imposto indirizzo byte alto
        #endif                                                                  
        EECON1bits.EEPGD = 0;
        EECON1bits.CFGS = 0;
        EECON1bits.RD = 1;                                                              // leggo il dato
        _readed = EEDATA;
    }
    return _readed;
}

/* ============================================================================================================================================ */
/* FUNCTION:        EEPROM_write
/* Input:           _data: valore da scrivere (unsigned char)
/*                  _addr: indirizzo in cui scrivere (unsigned int)
/*
/* Overview:        Salva il valore nell'indirizzo dichiarato
/* ============================================================================================================================================ */

void EEPROM_write(unsigned char _data, unsigned int _addr) {     
    unsigned char _flag_interrupt = 0;
    
    if (_addr < _EEPROMSIZE) {                                                          // scrivo il dato solo se indirizzo non supera la memoria massima
        do {
            _flag_interrupt = INTCONbits.GIE;
            
            EEIF = 0;                                                                   // resetto flag di fine scrittura
            EEADR = _addr & 0x00FF;                                                     // imposto indirizzo byte basso
            #if _EEPROMSIZE > 256                                                       // MCU con eeprom > 256
                EEADRH = _addr >> 8;                                                    // imposto indirizzo byte alto
            #endif
            EEDATA = _data;                                                             // imposto dato da scrivere
            EECON1bits.EEPGD = 0;
            EECON1bits.CFGS = 0;
            EECON1bits.WREN = 1;                                                        // abilito la scrittura

            INTCONbits.GIE = 0;
            
            asm("MOVLW 0x55");
            asm("MOVWF EECON2");
            asm("MOVLW 0xAA");
            asm("MOVWF EECON2");
            
            EECON1bits.WR = 1;                                                          // scrivo il dato
            
            while (EECON1bits.WR)                                                       // attendo che il dato sia scritto (WR=0)
                continue;

            EECON1bits.WREN = 0;                                                        // disabilito scrittura
            EEIF = 0;  

            INTCONbits.GIE = _flag_interrupt;

        }while (EEPROM_read(_addr) != _data);
    }
}

/* ============================================================================================================================================ */
/* FUNCTION:        EEPROM_read_int
/* Input:           _addr: indirizzo in cui scrivere (unsigned int)
/* Output           valore letto in _addr e _addr+1 (unsigned int)
/*
/* Overview:        restituisce un valore intero letto da 2 indirizzi consecutivi
/* ============================================================================================================================================ */

unsigned int EEPROM_read_int(unsigned int _addr) {
    unsigned int _ret_value;
    unsigned char _high;
    unsigned char _low;

    _high = EEPROM_read(_addr);                                                         // leggo il byte alto
    _low = EEPROM_read(_addr+1);                                                        // leggo il byte basso

    _ret_value = _high;                                                                 // metto _high nel byte alto di _ret_value
    _ret_value = _ret_value <<8;
    _ret_value = _ret_value + _low;                                                     // aggiungo _low nel byte basso

    return _ret_value;
}

/* ============================================================================================================================================ */
/* FUNCTION:        EEPROM_write_int
/* Input:           _data: valore da scrivere (unsigned int)
/*                  _addr: indirizzo in cui scrivere (unsigned int)
/*
/* Overview:        Salva il valore intero nell'indirizzo dichiarato e in quello successivo (2 byte)
/* ============================================================================================================================================ */

void EEPROM_write_int(unsigned int _data, unsigned int _addr) {
    unsigned char _high;
    unsigned char _low;

    _low = _data & 0x00FF;                                                              // maschero il byte basso
    _data = _data >>8;                                                                  // sposto il byte alto nel byte basso
    _high = _data;

    EEPROM_write(_high ,_addr);                                                         // scrivo il byte alto
    Delay_ms(10);
    EEPROM_write(_low, _addr+1);                                                        // scrivo il byte basso
}

#endif