/* ============================================================================================================================================= /
 * LIBRARY:         Gestione HDC1080
 * FILE:            HDC1080.c
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         30/08/2016
 * UPDATED:         08/09/2016
 *
 * DESCRIP:         Gestione sensore temperatura e umidit Texas Instruments HDC1080
 *
 * FUNCTIONS:       - HDC1080_reset()    
 *                  - HDC1080_set()
 *                  - HDC1080_set_heater()
 *                  - HDC108_dry_sensor()
 *                  - HDC1080_read_battery_stat()
 *                  - HDC1080_read_temp_float()
 *                  - HDC1080_read_temp_raw()
 *                  - HDC1080_read_humidity_float()
 *                  - HDC1080_read_humidity_raw()
 *                  - HDC1080_temp_raw_2_float()
 *                  - HDC1080_humidity_raw_2_float()
/* ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.0               08/09/2016          - Prima release
  1.0               30/08/2016          - Inizio sviluppo
//! [History]
/* ============================================================================================================================================ */

#ifndef HDC1080
#define HDC1080

/* INCLUDES =================================================================================================================================== */
#include    "HDC1080_settings.h"
#include    "HDC1080.h"
#include    <xc.h>

/* VARIABILI GLOBALI LIBRERIA================================================================================================================== */

/* ============================================================================================================================================= /
 * FUNCTION:    HDC1080_reset
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision 
 *
 * Overview:    Resetta il sensore e cancella il config register
/* ============================================================================================================================================ */
signed char HDC1080_reset(void) {
    signed char _return = 0;
    _return = _HDC1080_I2C_1ByteAddr_2ByteData_write(HDC1080_ADDR, HDC1080_CONFIG_REG, HDC1080_RESET << 8); // resetto il registro

    Delay_ms(20);                                                                       // tempo necessario per l'avvio

    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    HDC1080_set
 * Input:       _mode: settaggio per la modalit di acquisizione (HDC1080_SEQUENTIAL_AQ/HDC1080_ONE_AQ)
 *              _temp_res: settaggio per la risoluzione di acquisizione della temperatura (HDC1080_TEMP_RES_14bit/HDC1080_TEMP_RES_11bit)
 *              _humidity_res: settaggio per la risoluzione di acquisizione dell'umidit(HDC1080_HUMI_RES_14bit/HDC1080_HUMI_RES_11bit/HDC1080_HUMI_RES_8bit)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision 
 *
 * Overview:    Imposta la modalit di acquisizione: sequenziale, vengono acquisiti ed inviati in sequenza sia temperatura che umidit, one: 
 *              viene acquisito ed inviato solo il dato richiesto; imposta la risoluzione della temperatura e dell'umidit
/* ============================================================================================================================================ */
signed char HDC1080_set(unsigned char _mode, unsigned char _temp_res, unsigned char _humidity_res) {
    unsigned char _setting = 0;
    _setting = _mode | _temp_res | _humidity_res;
    
    return _HDC1080_I2C_1ByteAddr_2ByteData_write(HDC1080_ADDR, HDC1080_CONFIG_REG, _setting <<8);      // scrivo il nuovo valore negli 8 bit pi alti
}

/* ============================================================================================================================================= /
 * FUNCTION:    HDC1080_set_heater
 * Input:       _settings: settaggio per attivare o disattivare l'elemento riscaldante (HDC1080_HEATER_ON/HDC1080_HEATER_OFF)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision 
 * 
 * Overview:    Accende o spegne l'elemento riscaldante del sensore
 * Note:        Se abilitato si accende solo durante le misurazioni.
/* ============================================================================================================================================ */
signed char HDC1080_set_heater(unsigned char _setting) {
    unsigned char _readed = 0;

    _readed = _HDC1080_I2C_1ByteAddr_2ByteData_read(HDC1080_ADDR, HDC1080_CONFIG_REG) >>8;// leggo le impostazioni attuali e tengo solo gli 8 bit pi alti
    _readed &= ~HDC1080_HEATER_ON;                                                        // cancello eventuale impostazione precedente
    _readed |= _setting;                                                                  // aggiungo impostazione attuale

    return _HDC1080_I2C_1ByteAddr_2ByteData_write(HDC1080_ADDR, HDC1080_CONFIG_REG, _readed <<8);      // scrivo il nuovo valore negli 8 bit pi alti
}

/* ============================================================================================================================================= /
 * FUNCTION:    HDC108_dry_sensor
 *
 * Overview:    Abilita il riscaldatore del sensore ed avvia un ciclo di lettura per asciugare il sensore.
 * Note:        Bloccante per circa 1 secondo
/* ============================================================================================================================================ */
void HDC108_dry_sensor (void){
    HDC1080_set_heater (HDC1080_HEATER_ON);
  
    for (unsigned char _i = 0; _i<130; _i++){   // 130*7ms = 900ms bloccato  minimo
        HDC1080_read_humidity_raw();
    }
    HDC1080_set_heater (HDC1080_HEATER_OFF);
}

/* ============================================================================================================================================= /
 * FUNCTION:    HDC1080_read_battery_stat
 * Output:       1: Batteria < 2.8V
 *               0: Batteria > 2.8V
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision 
 *
 * Overview:    Controlla il voltaggio di alimentazione
/* ============================================================================================================================================ */
signed char HDC1080_read_battery_stat(void) {
    signed char _readed = 0;
    
    _readed = _HDC1080_I2C_1ByteAddr_2ByteData_read(HDC1080_ADDR, HDC1080_CONFIG_REG) >> 8;           // leggo le impostazioni attuali e tengo solo gli 8 bit pi alti

    if (_readed == -3 || _readed == -2 || _readed == -1)                                // se ci sono errori di comunicazione _HDC1080_I2C
        return _readed;                                                                 // torno l'errore
    else if (_readed & 0b00001000)                                                      // verifico lo stato della batteria
        return 0;
    else
        return 1;
}

/* ============================================================================================================================================= /
 * FUNCTION:    HDC1080_read_temp_float
 * Output:      temperatura letta (float)
 *
 * Overview:    Legge la temperatura e restituisce il valore convertito in gradi centigradi
 * Note:        Non torna eventuali errori di comunicazione _HDC1080_I2C
/* ============================================================================================================================================ */
float HDC1080_read_temp_float(void) {                                          
    unsigned int _readed = 0;
    
    _readed = _HDC1080_I2C_1ByteAddr_2ByteData_read_delay(HDC1080_ADDR, HDC1080_TEMP_REG);    // leggo il registro_HDC1080_I2C_1ByteAddr_2ByteData_read(HDC1080_ADDR, HDC1080_TEMP_REG);       // leggo il registro
    return HDC1080_temp_raw_2_float(_readed);
}

/* ============================================================================================================================================= /
 * FUNCTION:    HDC1080_read_temp_raw
 * Output:      temperatura letta (unsigned int)
 *
 * Overview:    Legge la temperatura e restituisce il valore letto dal registro
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
/* ============================================================================================================================================ */
unsigned int HDC1080_read_temp_raw(void) {
    return _HDC1080_I2C_1ByteAddr_2ByteData_read_delay(HDC1080_ADDR, HDC1080_TEMP_REG);       // leggo il registro
}

/* ============================================================================================================================================= /
 * FUNCTION:    HDC1080_read_humidity_float
 * Output:      umidit letta (float)
 *
 * Overview:    Legge l'umidit e restituisce il valore convertito percentuale
 * Note:        Non torna eventuali errori di comunicazione _HDC1080_I2C
/* ============================================================================================================================================ */
float HDC1080_read_humidity_float(void) {
    unsigned int _readed = 0;
    
    _readed = _HDC1080_I2C_1ByteAddr_2ByteData_read_delay(HDC1080_ADDR, HDC1080_HUMID_REG);   // leggo il registro
    return HDC1080_humidity_raw_2_float(_readed);
}

/* ============================================================================================================================================= /
 * FUNCTION:    HDC1080_read_humidity_raw
 * Output:      umidit letta (unsigned int)
 *
 * Overview:    Legge l'umidit e restituisce il valore letto dal registro
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
/* ============================================================================================================================================ */
unsigned int HDC1080_read_humidity_raw(void) {
    return _HDC1080_I2C_1ByteAddr_2ByteData_read_delay(HDC1080_ADDR, HDC1080_HUMID_REG);      // leggo il registro
}

/* ============================================================================================================================================= /
 * FUNCTION:    HDC1080_temp_raw_2_float
 * Input:       _readed: dato letto dal registro da convertire (unsigned int)
 * Output:      temperatura convertita in gradi centigradi (float)
 *
 * Overview:    Converte il dato letto della temperatura in gradi centigradi
/* ============================================================================================================================================ */
float HDC1080_temp_raw_2_float (unsigned int _readed){
    float _temp = 0;

    _temp = ((float)_readed / 65536);
    _temp = _temp * 165;
    _temp = _temp - 40;

    return _temp;    
}

/* ============================================================================================================================================= /
 * FUNCTION:    HDC1080_humidity_raw_2_float
 * Input:       _readed: dato letto dal registro da convertire (unsigned int)
 * Output:      umidit convertita in percentuale (float)
 *
 * Overview:    Converte il dato letto dell'umidit in percentuale
/* ============================================================================================================================================ */
float HDC1080_humidity_raw_2_float(unsigned int _readed){
    float _humid = 0;

    _humid = ((float)_readed / 65536);
    _humid = _humid * 100;
    
    return _humid;    
}

/* ============================================================================================================================================= /
 * FUNCTION:    HDC1080_read_manuf_id
 * Output:      Manufacturer ID (unsigned int)
 *
 * Overview:    Legge l'id del costruttore (0x5449)
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
/* ============================================================================================================================================ */
unsigned int HDC1080_read_manuf_id (void){
    return _HDC1080_I2C_1ByteAddr_2ByteData_read(HDC1080_ADDR, HDC1080_MANUFID_REG);    
}

/* ============================================================================================================================================= /
 * FUNCTION:    HDC1080_read_device_id
 * Output:      Device ID (unsigned int)
 *
 * Overview:    Legge l'id del dispositivo (0x1050) 
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
/* ============================================================================================================================================ */
unsigned int  HDC1080_read_device_id (void){
    return _HDC1080_I2C_1ByteAddr_2ByteData_read(HDC1080_ADDR, HDC1080_DEVICEID_REG);    
}

#endif
