/* ============================================================================================================================================= /
 * LIBRARY:         I2C_init.c
 * FILE:            I2C_init.c
 * LAST REV:        2.6
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         17/03/2014
 * UPDATED:         07/07/2017
 *
 * DESCRIP:         Gestione protocollo i2c, implementato solo MASTER mode
 *
 * PRECONDITIONS:   - definire la frequenza del clock in MHz XTAL_FREQ
 *                  - definire le linee e il modulo I2C in I2C_init_settings.h
 *                  
 * FUNCTIONS:       - I2C_init_master()
 *                  - I2C_open()   
 *                  - I2C_idle()
 *                  - I2C_start()
 *                  - I2C_restart()
 *                  - I2C_stop()
 *                  - I2C_ack()
 *                  - I2C_not_ack()
 *                  - I2C_write()
 *                  - I2C_read()
 *                  - I2C_1ByteAddr_1ByteData_read()
 *                  - I2C_1ByteAddr_1ByteData_write()
 *                  - I2C_2ByteAddr_1ByteData_read()
 *                  - I2C_2ByteAddr_1ByteData_write()
 *                  - I2C_1ByteAddr_2ByteData_read()
 *                  - I2C_1ByteAddr_2ByteData_read_delay()
 *                  - I2C_1ByteAddr_2ByteData_write()
 *                  - I2C2_init_master()
 *                  - I2C2_open()   
 *                  - I2C2_idle()
 *                  - I2C2_start()
 *                  - I2C2_restart()
 *                  - I2C2_stop()
 *                  - I2C2_ack()
 *                  - I2C2_not_ack()
 *                  - I2C2_write()
 *                  - I2C2_read()
 *                  - I2C2_1ByteAddr_1ByteData_read()
 *                  - I2C2_1ByteAddr_1ByteData_write()
 *                  - I2C2_2ByteAddr_1ByteData_read()
 *                  - I2C2_2ByteAddr_1ByteData_write()
 *                  - I2C2_1ByteAddr_2ByteData_read()
 *                  - I2C_1ByteAddr_2ByteData_read_delay()
 *                  - I2C2_1ByteAddr_2ByteData_write()
 *
/* ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  2.6               07/07/2017          - Corretto bug in funzioni con indirizzi a 2 byte
  2.5               06/03/2017          - Inserite funzioni I2C_1ByteAddr_2ByteData_read_delay e I2C2_1ByteAddr_2ByteData_read_delay per lettura da sensore HDC1080
  2.4               17/02/2017          - Modifica da  PIR3bits.BCL2IF a BCL2IF per compatibilit con PIC18F67K22 (si trova su PIR2 invece che su PIR3)
  2.3               16/12/2016          - Compatibilit con PIC18F26K22 e PIC18F67K22   
  2.2               27/08/2016          - Adattamento per poter utilizzare 2 moduli contemporaneamente, compatibilit con PIC18F46K22
                                        - Linee SCL e SDA impostate automaticamente a seconda del PIC in uso e del modulo selezionato in setting
                                          dispositivo (gestione in I2C_init.h)
                                        - Abilitare il modulo I2C usato in I2C_init_settings
  2.1               15/08/2016          - Implementate funzioni per scrittura e lettura da dispositivi con indirizzi a 2 byte
                                        - Implementate funzioni per scrittura e lettura da dispositivi con indirizzi a 1 byte e data a 2 byte
  2.0               07/08/2016          - Integrate funzioni di base
                                        - Integrati cicli lettura e scrittura di byte da indirizzo I2C, ex EERandomRead e EEByteWrite
                                        - Gestione secondo modulo I2C    
  1.0               17/03/2014          - Prima release
//! [History]
/* ============================================================================================================================================ */

#ifndef I2C
#define I2C

/* INCLUDES =================================================================================================================================== */
#include "I2C_init_settings.h"
#include "I2C_init.h"
#include <xc.h>

/* ============================================================================================================================================= /
 * FUNCTION:    I2C_init
 * Input:       _baud_rate: indicare il baudrate espresso in KHz (unsigned int)
 *
 * Overview:    Imposta il protocollo I2C e calcola SSPADD. Per ora gestito solo come master
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_1  
void I2C_init_master(unsigned int _baud_rate) {
    I2C_SCL_TRIS = 1;
    I2C_SDA_TRIS = 1;
    
    SSPADD = (((XTAL_FREQ * 1000)/4)/_baud_rate)-1;                                     // calcolo SSPADD
    I2C_open(MASTER, SLEW_ON);                                                          // i2c master
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_init
 * Input:       _baud_rate: indicare il baudrate espresso in KHz (unsigned int)
 *
 * Overview:    Imposta il protocollo I2C del secondo modulo e calcola SSPADD. Per ora gestito solo come master
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_2 
void I2C2_init_master(unsigned int _baud_rate) {
 
    I2C2_SCL_TRIS = 1;
    I2C2_SDA_TRIS = 1;
    
    SSP2ADD = (((XTAL_FREQ * 1000)/4)/_baud_rate)-1;                                    // calcolo SSPADD
    I2C2_open(MASTER, SLEW_ON);                                                         // i2c master
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C_open
 * Input:       _sync_mode: indicare la modalit , per ora solo MASTER (unsigned char)
 *              _slew: SLEW_ON, SLEW_OFF (unsigned char)
 *
 * Overview:    Abilita il modulo I2C primario
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_1  
void I2C_open(unsigned char _sync_mode, unsigned char _slew) {
  SSPSTAT &= 0x3F;                                                                      // power on state 
  SSPCON1 = 0x00;                                                                       // power on state
  SSPCON2 = 0x00;                                                                       // power on state
  SSPCON1 |= _sync_mode;                                                                // select serial mode 
  SSPSTAT |= _slew;                                                                     // slew rate on/off 
  
  SSPCON1 |= 0b00100000;                                                                // enable synchronous serial port 
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_open
 * Input:       _sync_mode: indicare la modalit , per ora solo MASTER (unsigned char)
 *              _slew: SLEW_ON, SLEW_OFF (unsigned char)
 *
 * Overview:    Abilita il modulo I2C secondario
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_2
void I2C2_open(unsigned char _sync_mode, unsigned char _slew) {
  SSP2STAT &= 0x3F;                                                                     // power on state 
  SSP2CON1 = 0x00;                                                                      // power on state
  SSP2CON2 = 0x00;                                                                      // power on state
  SSP2CON1 |= _sync_mode;                                                               // select serial mode 
  SSP2STAT |= _slew;                                                                    // slew rate on/off 
                                                                                          
  SSP2CON1 |= 0b00100000;                                                               // enable synchronous serial port 
}
#endif                                                                                    

/* ============================================================================================================================================= /
 * FUNCTION:    I2C_idle
 *
 * Overview:    Attende che il bus del modulo I2C primario sia libero
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_1  
void I2C_idle(void){
  while ( ( SSPCON2 & 0x1F ) || ( SSPSTATbits.R_W ) )
     continue;
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_idle
 *
 * Overview:    Attende che il bus del modulo I2C secondario sia libero
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_2
void I2C2_idle(void){
   
  while ( ( SSP2CON2 & 0x1F ) || ( SSP2STATbits.R_W ) )
     continue;
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C_start
 *
 * Overview:    Invia il bit di start sul modulo I2C primario ed attende che venga ricevuto 
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_1
void I2C_start(void) {
 
  SSPCON2bits.SEN = 1;                                                                  // initiate bus start condition
  while(SSPCON2bits.SEN);                                                                 
}
#endif                                                                                    
 
/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_start
 *
 * Overview:    Invia il bit di start sul modulo I2C secondario ed attende che venga ricevuto 
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_2 
void I2C2_start(void) {                                                                   
                                                                    
  SSP2CON2bits.SEN = 1;                                                                 // initiate bus start condition
  while(SSP2CON2bits.SEN);                                                                
}
#endif                                                                                    

/* ============================================================================================================================================= /
 * FUNCTION:    I2C_restart
 *
 * Overview:    Riavvia il modulo I2C primario ed attende che il bit venga ricevuto
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_1   
void I2C_restart(void) {                                                                  
                                                                 
  SSPCON2bits.RSEN = 1;                                                                 // initiate bus restart condition
  while(SSPCON2bits.RSEN);                                                                
}
#endif                                                                                    

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_restart
 *
 * Overview:    Riavvia il modulo I2C secondario ed attende che il bit venga ricevuto
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_2                                                                    
void I2C2_restart(void) {                                                                 
  SSP2CON2bits.RSEN = 1;                                                                // initiate bus restart condition
  while(SSP2CON2bits.RSEN);                                                               
}
#endif                                                                                    
   
/* ============================================================================================================================================= /
 * FUNCTION:    I2C_stop
 *
 * Overview:    Invia il bit di stop sul modulo I2C primario ed attende che venga ricevuto 
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_1    
void I2C_stop(void) {                                                                     
                                                                
  SSPCON2bits.PEN = 1;                                                                  // initiate bus stop condition
  while(SSPCON2bits.PEN);                                                                 
}
#endif                                                                                    

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_stop
 *
 * Overview:    Invia il bit di stop sul modulo I2C secondario ed attende che venga ricevuto 
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_2
void I2C2_stop(void) {                                                                    
                                                                   
  SSP2CON2bits.PEN = 1;                                                                 // initiate bus stop condition
  while(SSP2CON2bits.PEN);                                                                
}
#endif                                                                                    
 
/* ============================================================================================================================================= /
 * FUNCTION:    I2C_ack
 *
 * Overview:    Invia ACK sul modulo I2C primario ed attende che venga ricevuto
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_1
void I2C_ack(void) {                                                                      
                                                                     
  SSPCON2bits.ACKDT = 0;                                                                // set acknowledge bit state for ACK
  SSPCON2bits.ACKEN = 1;                                                                // initiate bus acknowledge sequence
  while(SSPCON2bits.ACKEN);                                                               
}
#endif                                                                                    

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_ack
 *
 * Overview:    Invia ACK sul modulo I2C secondario ed attende che venga ricevuto
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_2 
void I2C2_ack(void) {                                                                     
                                                                  
  SSP2CON2bits.ACKDT = 0;                                                               // set acknowledge bit state for ACK
  SSP2CON2bits.ACKEN = 1;                                                               // initiate bus acknowledge sequence
  while(SSP2CON2bits.ACKEN);                                                              
}
#endif                                                                                    
 
/* ============================================================================================================================================= /
 * FUNCTION:    I2C_not_ack
 *
 * Overview:    Invia NOT ACK sul modulo I2C primario ed attende che venga ricevuto
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_1
void I2C_not_ack(void) {                                                                  
                                                                      
  SSPCON2bits.ACKDT = 1;                                                                // set acknowledge bit for not ACK
  SSPCON2bits.ACKEN = 1;                                                                // initiate bus acknowledge sequence
  while(SSPCON2bits.ACKEN);                                                             
}
#endif                                                                                    

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_not_ack
 *
 * Overview:    Invia NOT ACK sul modulo I2C secondario ed attende che venga ricevuto
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_2  
void I2C2_not_ack(void) {                                                                 
                                                                 
  SSP2CON2bits.ACKDT = 1;                                                               // set acknowledge bit for not ACK
  SSP2CON2bits.ACKEN = 1;                                                               // initiate bus acknowledge sequence
  while(SSP2CON2bits.ACKEN);                                                              
}
#endif                                                                                    

/* ============================================================================================================================================= /
 * FUNCTION:    I2C_write
 * Input:       _data_out: byte da inviare (unsigned char)
 * Output:      0: ACK ricevuto
 *              -1: Bus Collision
 *              -2: ACK non ricevuto
 *              -3: errore non specificato
 *
 * Overview:    Invia un byte sul BUS I2C primario
/* ============================================================================================================================================ */                                  
#ifdef I2C_MODULE_1
signed char I2C_write(unsigned char _data_out) {
            
  SSPBUF = _data_out;                                                                   // write single byte to SSPBUF
  if ( SSPCON1bits.WCOL )                                                               // test if write collision occurred
   return ( -1 );                                                                       // if WCOL bit is set return negative #
  else                            
  {                               
	if( ((SSPCON1&0x0F)!=0x08) && ((SSPCON1&0x0F)!=0x0B) )	                            // Slave mode only
	{                              
	      SSPCON1bits.CKP = 1;                                                          // release clock line 
	      while ( !PIR1bits.SSPIF );                                                    // wait until ninth clock pulse received
                                 
	      if ( ( !SSPSTATbits.R_W ) && ( !SSPSTATbits.BF ) )                            // if R/W=0 and BF=0, NOT ACK was received
	      {                      
	        return ( -2 );                                                              // return NACK
	      }                      
		  else                     
		  {                       
			return ( 0 );                                                               // return ACK
		  }	                      
	}                           
	else if( ((SSPCON1&0x0F)==0x08) || ((SSPCON1&0x0F)==0x0B) )	                        // master mode only
	{ 
	    while( SSPSTATbits.BF );                                                        // wait until write cycle is complete   
	    I2C_idle();                                                                     // ensure module is idle
	    if ( SSPCON2bits.ACKSTAT )                                                      // test for ACK condition received
	    	 return ( -2 );			                                                    // return NACK
		else return ( 0 );                                                              // return ACK
	}	
  }   
  return ( -3);
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_write (unsigned char)
 * Input:       _data_out: byte da inviare
 * Output:      0: ACK ricevuto
 *              -1: Bus Collision
 *              -2: ACK non ricevuto
 *              -3: errore non specificato
 *
 * Overview:    Invia un byte sul BUS I2C secondario
/* ============================================================================================================================================ */ 
#ifdef I2C_MODULE_2
signed char I2C2_write(unsigned char _data_out) {
 
  SSP2BUF = _data_out;                                                                  // write single byte to SSPBUF
  if ( SSP2CON1bits.WCOL )                                                              // test if write collision occurred
   return ( -1 );                                                                       // if WCOL bit is set return negative #
  else
  {
	if( ((SSP2CON1&0x0F)!=0x08) && ((SSP2CON1&0x0F)!=0x0B) )	                        // Slave mode only
	{
	      SSP2CON1bits.CKP = 1;                                                         // release clock line 
//	      while ( !PIR3bits.SSP2IF );                                                   // wait until ninth clock pulse received
          while ( !SSP2IF );                                                            // wait until ninth clock pulse received

	      if ( ( !SSP2STATbits.R_W ) && ( !SSP2STATbits.BF ) )                          // if R/W=0 and BF=0, NOT ACK was received
	      {
	        return ( -2 );                                                              // return NACK
	      }
		  else
		  {
			return ( 0 );				                                                // return ACK
		  }	
	}
	else if( ((SSP2CON1&0x0F)==0x08) || ((SSP2CON1&0x0F)==0x0B) )	                    // master mode only
	{ 
	    while( SSP2STATbits.BF );                                                       // wait until write cycle is complete   
	    I2C2_idle();                                                                    // ensure module is idle
	    if ( SSP2CON2bits.ACKSTAT )                                                     // test for ACK condition received
	    	 return ( -2 );			                                                    // return NACK
		else return ( 0 );                                                              // return ACK
	}	
  }
  return ( -3);
}
#endif
/* ============================================================================================================================================= /
 * FUNCTION:    I2C_read
 * Output:      ritorna il byte ricevuto (unsigned char)
 *
 * Overview:    riceve un byte sul BUS I2C primario
/* ============================================================================================================================================ */ 
#ifdef I2C_MODULE_1
unsigned char I2C_read(void){
 
if( ((SSPCON1&0x0F)==0x08) || ((SSPCON1&0x0F)==0x0B) )	                                // master mode only
  SSPCON2bits.RCEN = 1;                                                                 // enable master for 1 byte reception
  while ( !SSPSTATbits.BF );                                                            // wait until byte received  
  return ( SSPBUF );                                                                    // return with read byte 
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_read
 * Output:      ritorna il byte ricevuto (unsigned char)
 *
 * Overview:    riceve un byte sul BUS I2C secondario
/* ============================================================================================================================================ */ 
#ifdef I2C_MODULE_2  
unsigned char I2C2_read(void){
if( ((SSP2CON1&0x0F)==0x08) || ((SSP2CON1&0x0F)==0x0B) )	                            // master mode only
  SSP2CON2bits.RCEN = 1;                                                                // enable master for 1 byte reception
  while ( !SSP2STATbits.BF );                                                           // wait until byte received  
  return ( SSP2BUF );                                                                   // return with read byte 
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C_1ByteAddr_1ByteData_read
 * Input:       _control: indirizzo del dispositivo I2C (unsigned char)
 *              _address: registro da leggere (unsigned char)
 * Output:      byte ricevuto (unsigned char) 
 *
 * Overview:    Legge registri a 1 byte da dispositivi I2C con indirizzi dei registri a 1 byte (modulo I2C 1)
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
/* ============================================================================================================================================ */ 
#ifdef I2C_MODULE_1
unsigned char I2C_1ByteAddr_1ByteData_read (unsigned char _control, unsigned char _address) {
  I2C_idle();                                                                           // ensure module is idle
  I2C_start();                                                                          // initiate START condition
  if (PIR2bits.BCLIF) {                                                                 // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
  } else {
    if (I2C_write(_control)) {                                                          // write 1 byte
        I2C_stop(); 
        return ( -3 );                                                                  // return with write collision error
    }
    
    if (!SSPCON2bits.ACKSTAT) {                                                         // test for ACK condition, if received
      if (I2C_write(_address)) {                                                        // WRITE word address
        I2C_stop(); 
        return ( -3 );                                                                  // return with write collision error
      }
      if (!SSPCON2bits.ACKSTAT) {                                                       // test for ACK condition, if received
        I2C_restart();                                                                  // generate I2C bus restart condition
        if (PIR2bits.BCLIF) {                                                           // test for bus collision
          return ( -1 );                                                                // return with Bus Collision error 
        }
                          
        if (I2C_write(_control+1)) {                                                    // write 1 byte - R/W bit should be 1
          I2C_stop(); 
          return ( -3 );                                                                // return with write collision error
        }

        if ( !SSPCON2bits.ACKSTAT ) {                                                   // test for ACK condition, if received
          SSPCON2bits.RCEN = 1;                                                         // enable master for 1 byte reception
          while ( SSPCON2bits.RCEN );                                                   // check that receive sequence is over
          I2C_not_ack();                                                                // send ACK condition
          I2C_stop();                                                                   // send STOP condition
          if (PIR2bits.BCLIF) {                                                         // test for bus collision
            return ( -1 );                                                              // return with Bus Collision error 
          }
        } else {
		  I2C_stop(); 
          return ( -2 );                                                                // return with Not Ack error
        }

      } else {
	    I2C_stop(); 
        return ( -2 );                                                                  // return with Not Ack error
      }
    } else {
	  I2C_stop(); 
      return ( -2 );                                                                    // return with Not Ack error
    }
  }
  return (SSPBUF);                                                                      // return with data
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_1ByteAddr_1ByteData_read
 * Input:       _control: indirizzo del dispositivo I2C (unsigned char)
 *              _address: registro da leggere (unsigned char)
 * Output:      byte ricevuto (unsigned char)
 *
 * Overview:    Legge registri a 1 byte da dispositivi I2C con indirizzi dei registri a 1 byte (modulo I2C 2)
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
/* ============================================================================================================================================ */ 
#ifdef I2C_MODULE_2
unsigned char I2C2_1ByteAddr_1ByteData_read (unsigned char _control, unsigned char _address) {  
  I2C2_idle();                                                                          // ensure module is idle
  I2C2_start();                                                                         // initiate START condition
//  if (PIR3bits.BCL2IF) {                                                                // test for bus collision
  if (BCL2IF) {                                                                         // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
  } else {
    if (I2C2_write(_control)) {                                                         // write 1 byte
        I2C2_stop(); 
        return ( -3 );                                                                  // return with write collision error
    }
    
    if (!SSP2CON2bits.ACKSTAT) {                                                        // test for ACK condition, if received
      if (I2C2_write(_address)) {                                                       // WRITE word address
        I2C2_stop(); 
        return ( -3 );                                                                  // return with write collision error
      }
      if (!SSP2CON2bits.ACKSTAT) {                                                      // test for ACK condition, if received
        I2C2_restart();                                                                 // generate I2C bus restart condition
//        if (PIR3bits.BCL2IF) {                                                          // test for bus collision
          if (BCL2IF) {                                                                         // test for bus collision
          return ( -1 );                                                                // return with Bus Collision error 
        }
                          
        if (I2C2_write(_control+1)) {                                                   // write 1 byte - R/W bit should be 1
          I2C2_stop(); 
          return ( -3 );                                                                // return with write collision error
        }

        if ( !SSP2CON2bits.ACKSTAT ) {                                                  // test for ACK condition, if received
          SSP2CON2bits.RCEN = 1;                                                        // enable master for 1 byte reception
          while ( SSP2CON2bits.RCEN );                                                  // check that receive sequence is over
          I2C2_not_ack();                                                               // send ACK condition
          I2C2_stop();                                                                  // send STOP condition
//          if (PIR3bits.BCL2IF) {                                                        // test for bus collision
            if (BCL2IF) {                                                                         // test for bus collision
            return ( -1 );                                                              // return with Bus Collision error 
          }
        } else {
		  I2C2_stop(); 
          return ( -2 );                                                                // return with Not Ack error
        }

      } else {
	    I2C2_stop(); 
        return ( -2 );                                                                  // return with Not Ack error
      }
    } else {
	  I2C2_stop(); 
      return ( -2 );                                                                    // return with Not Ack error
    }
  }
  return (SSP2BUF);                                                                     // return with data
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C_1ByteAddr_1ByteData_write
 * Input:       _control_ indirizzo di scrittura del dispositivo I2C (unsigned char)
 *              _address: registro da scrivere (unsigned char)
 *              _data: byte da scrivere (unsigned char)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision   
 *
 * Overview:    Scrive registri a 1 byte su dispositivi I2C con indirizzi dei registri a 1 byte (modulo I2C 1)
/* ============================================================================================================================================ */ 
#ifdef I2C_MODULE_1
signed char I2C_1ByteAddr_1ByteData_write(unsigned char _control, unsigned char _address, unsigned char _data) {
  I2C_idle();                                                                           // ensure module is idle
  I2C_start();                                                                          // initiate START condition
  if (PIR2bits.BCLIF) {                                                                 // test for bus collision
    return ( -1 );                                                                      // return with Bus Collision error 
  } else {                                                                              // start condition successful
    if (I2C_write(_control)) {                                                          // write byte - R/W bit should be 0
      I2C_stop(); 
      return ( -3 );                                                                    // set error for write collision
    }

    if (!SSPCON2bits.ACKSTAT) {                                                         // test for ACK condition received
      if (I2C_write(_address)) {                                                        // write word address
        I2C_stop(); 
        return ( -3 );                                                                  // set error for write collision
      }

      if (!SSPCON2bits.ACKSTAT) {                                                       // test for ACK condition received
        if (I2C_write(_data)){                                                          // data byte
          I2C_stop(); 
          return ( -3 );                                                                // set error for write collision
        }
      } else {
	    I2C_stop(); 
        return ( -2 );                                                                  // return with Not Ack error condition   
      }
    } else {
	  I2C_stop(); 
      return ( -2 );                                                                    // return with Not Ack error condition   
    }
  }

  I2C_stop();                                                                           // send STOP condition
  if (PIR2bits.BCLIF) {                                                                 // test for bus collision
    return ( -1 );                                                                      // return with Bus Collision error 
  }
  return ( 0 );                                                                         // return with no error
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_1ByteAddr_1ByteData_write 
 * Input:       _control_ indirizzo di scrittura del dispositivo I2C (unsigned char)
 *              _address: registro da scrivere (unsigned char)
 *              _data: byte da scrivere (unsigned char)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision   
 *              
 * Overview:    Scrive registri a 1 byte su dispositivi I2C con indirizzi dei registri a 1 byte (modulo I2C 2)
/* ============================================================================================================================================ */ 
#ifdef I2C_MODULE_2
signed char I2C2_1ByteAddr_1ByteData_write(unsigned char _control, unsigned char _address, unsigned char _data) {
  I2C2_idle();                                                                          // ensure module is idle
  I2C2_start();                                                                         // initiate START condition
//  if (PIR3bits.BCL2IF) {                                                                // test for bus collision
    if (BCL2IF) {                                                                         // test for bus collision
    return ( -1 );                                                                      // return with Bus Collision error 
  } else {                                                                              // start condition successful
    if (I2C2_write(_control)) {                                                         // write byte - R/W bit should be 0
      I2C2_stop(); 
      return ( -3 );                                                                    // set error for write collision
    }

    if (!SSP2CON2bits.ACKSTAT) {                                                        // test for ACK condition received
      if (I2C2_write(_address)) {                                                       // write word address
        I2C2_stop(); 
        return ( -3 );                                                                  // set error for write collision
      }

      if (!SSP2CON2bits.ACKSTAT) {                                                      // test for ACK condition received
        if (I2C2_write(_data)){                                                         // data byte for
          I2C2_stop(); 
          return ( -3 );                                                                // set error for write collision
        }
      } else {
	    I2C2_stop(); 
        return ( -2 );                                                                  // return with Not Ack error condition   
      }
    } else {
	  I2C2_stop(); 
      return ( -2 );                                                                    // return with Not Ack error condition   
    }
  }

  I2C2_stop();                                                                          // send STOP condition
//  if (PIR3bits.BCL2IF) {                                                                // test for bus collision
    if (BCL2IF) {                                                                         // test for bus collision
    return ( -1 );                                                                      // return with Bus Collision error 
  }
  return ( 0 );                                                                         // return with no error
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C_2ByteAddr_1ByteData_read
 * Input:       _control: indirizzo del dispositivo I2C (unsigned char)
 *              _address: registro da leggere (unsigned int)
 * Output:      byte ricevuto (unsigned char) 
 *
 * Overview:    Legge registri a 1 byte da dispositivi I2C con indirizzi dei registri a 2 byte (modulo I2C 1)
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
/* ============================================================================================================================================ */ 
#ifdef I2C_MODULE_1
unsigned char I2C_2ByteAddr_1ByteData_read (unsigned char _control, unsigned int _address) {
    unsigned char _address_h = (_address & 0xFF00) >> 8;
    unsigned char _address_l = (_address & 0x00FF);
    
    I2C_idle();
    I2C_start();
    
    if (PIR2bits.BCLIF) {                                                               // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
    }
    
    if (I2C_write(_control)) {                                                          // send control
        return ( -3 );
    }
    
    if (!SSPCON2bits.ACKSTAT){                                                          // if ack received
        if (I2C_write(_address_h)){                                                     // send address_h
            return (-3);                                                                
        }
    } else {
        return ( -2 );                                                                  // no ack
    }
    
    if (!SSPCON2bits.ACKSTAT){                                                          
        if (I2C_write(_address_l)){
            return (-3);
        }
    } else {
        return ( -2 );
    }
    
    if (!SSPCON2bits.ACKSTAT){
        I2C_restart();                                                                  // restart
    
        if (PIR2bits.BCLIF) {                                                           // test for bus collision
            return ( -1 );                                                              // return with Bus Collision error 
        }
        
        if (I2C_write(_control+1)) {
            return ( -3 );
        }
        
        if (!SSPCON2bits.ACKSTAT){
            I2C_read();
            I2C_not_ack();                                                              // send ACK condition
            I2C_stop();                                                                 // send STOP condition
            if (PIR2bits.BCLIF) {                                                       // test for bus collision
                return ( -1 );                                                          // return with Bus Collision error 
            }
        } else {
            return ( -2 );
        }
    } else {
        return ( -2 );
    }
    
    return SSPBUF;
}
    
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_2ByteAddr_1ByteData_read
 * Input:       _control: indirizzo del dispositivo I2C (unsigned char)
 *              _address: registro da leggere (unsigned int)
 * Output:      byte ricevuto (unsigned char) 
 *
 * Overview:    Legge registri a 1 byte da dispositivi I2C con indirizzi dei registri a 2 byte (modulo I2C 2)
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
/* ============================================================================================================================================ */ 
#ifdef I2C_MODULE_2
unsigned char I2C2_2ByteAddr_1ByteData_read (unsigned char _control, unsigned int _address) {
    unsigned char _address_h = (_address & 0xFF00) >> 8;
    unsigned char _address_l = (_address & 0x00FF);
    
    I2C2_idle();
    I2C2_start();
    
//    if (PIR3bits.BCL2IF) {                                                              // test for bus collision
      if (BCL2IF) {                                                                         // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
    }
    
    if (I2C2_write(_control)) {
        return ( -3 );
    }
    
    if (!SSP2CON2bits.ACKSTAT){
        if (I2C2_write(_address_h)){
            return (-3);
        }
    } else {
        return ( -2 );
    }
    
    if (!SSP2CON2bits.ACKSTAT){
        if (I2C2_write(_address_l)){
            return (-3);
        }
    } else {
        return ( -2 );
    }
    
    if (!SSP2CON2bits.ACKSTAT){
        I2C2_restart();
    
//        if (PIR3bits.BCL2IF) {                                                          // test for bus collision
          if (BCL2IF) {                                                                         // test for bus collision
            return ( -1 );                                                              // return with Bus Collision error 
        }
        
        if (I2C2_write(_control+1)) {
            return ( -3 );
        }
        
        if (!SSP2CON2bits.ACKSTAT){
            I2C2_read();
            I2C2_not_ack();                                                             // send ACK condition
            I2C2_stop();                                                                // send STOP condition
//            if (PIR3bits.BCL2IF) {                                                      // test for bus collision
              if (BCL2IF) {                                                                         // test for bus collision
                return ( -1 );                                                          // return with Bus Collision error 
            }
        } else {
            return ( -2 );
        }
    } else {
        return ( -2 );
    }
    
    return SSP2BUF;
}
    
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C_2ByteAddr_1ByteData_write
 * Input:       _control: indirizzo del dispositivo I2C (unsigned char)
 *              _address: registro da scrivere (unsigned int)
 *              _data: dato da scrivere (unsigned char)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision    
 *
 * Overview:    Scrive registri a 1 byte su dispositivi I2C con indirizzi dei registri a 2 byte (modulo I2C 1)
/* ============================================================================================================================================ */ 
#ifdef I2C_MODULE_1
signed char I2C_2ByteAddr_1ByteData_write (unsigned char _control, unsigned int _address, unsigned char _data) {
    unsigned char _address_h = (_address & 0xFF00) >> 8;
    unsigned char _address_l = (_address & 0x00FF);    
    
    I2C_idle();
    I2C_start();
    
    if (PIR2bits.BCLIF) {                                                               // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
    }
    
    if (I2C_write(_control)){
        return ( -3 );
    }
    
    if (!SSPCON2bits.ACKSTAT){
        if (I2C_write(_address_h)){
            return (-3);
        }
    } else {
        return ( -2 );
    }
    
    if (!SSPCON2bits.ACKSTAT){
        if (I2C_write(_address_l)){
            return (-3);
        }
    } else {
        return ( -2 );
    }
    
    if (!SSPCON2bits.ACKSTAT){
        if (I2C_write(_data)){
            return ( -3 );
        } 
    } else {
        return ( -2 );
    }
       
    if (!SSPCON2bits.ACKSTAT){
        I2C_stop();
    } else {
        return ( -2 );
    }
    
    if (PIR2bits.BCLIF) {                                                               // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
    }
    
    return (0);
} 
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_2ByteAddr_1ByteData_write
 * Input:       _control: indirizzo del dispositivo I2C (unsigned char)
 *              _address: registro da scrivere (unsigned int)
 *              _data: dato da scrivere (unsigned char)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision   
 *
 * Overview:    Scrive registri a registri a 1 byte su dispositivi I2C con indirizzi dei registri a 2 byte (modulo I2C 2)
/* ============================================================================================================================================ */ 
#ifdef I2C_MODULE_2
signed char I2C2_2ByteAddr_1ByteData_write (unsigned char _control, unsigned int _address, unsigned char _data) {
    unsigned char _address_h = (_address & 0xFF00) >> 8;
    unsigned char _address_l = (_address & 0x00FF);    
    
    I2C2_idle();
    I2C2_start();
    
//    if (PIR3bits.BCL2IF) {                                                              // test for bus collision
      if (BCL2IF) {                                                                         // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
    }
    
    if (I2C2_write(_control)){
        return ( -3 );
    }
    
    if (!SSP2CON2bits.ACKSTAT){
        if (I2C2_write(_address_h)){
            return (-3);
        }
    } else {
        return ( -2 );
    }
    
    if (!SSP2CON2bits.ACKSTAT){
        if (I2C2_write(_address_l)){
            return (-3);
        }
    } else {
        return ( -2 );
    }
    
    if (!SSP2CON2bits.ACKSTAT){
        if (I2C2_write(_data)){
            return ( -3 );
        } 
    } else {
        return ( -2 );
    }
       
    if (!SSP2CON2bits.ACKSTAT){
        I2C2_stop();
    } else {
        return ( -2 );
    }
    
//    if (PIR3bits.BCL2IF) {                                                              // test for bus collision
      if (BCL2IF) {                                                                         // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
    }
    
    return (0);
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C_1ByteAddr_2ByteData_read
 * Input:       _control: indirizzo del dispositivo I2C (unsigned char)
 *              _address: registro da leggere (unsigned char)
 * Output:      2 byte ricevuti (unsigned int) 
 *
 * Overview:    Legge registri a 2 byte da dispositivi I2C con indirizzi dei registri a 1 byte (modulo I2C 1)
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 *              MSB primo dato ricevuto
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_1
unsigned int  I2C_1ByteAddr_2ByteData_read (unsigned char _control, unsigned char _address) {
    unsigned char _b1 = 0;
    unsigned char _b2 = 0;
    signed int _return = 0;
    
    I2C_idle();
    I2C_start();                                                                        // start I2C
    
    if (PIR2bits.BCLIF) {                                                               // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
    }
    
    if (I2C_write(_control)){                                        
        return ( -3 );
    }
    
    if (!SSPCON2bits.ACKSTAT){
        if (I2C_write(_address)){                                                       // scrivo indirizzo del registro
            return (-3);
        }
    } else {
        return ( -2 );
    }   
       
    if (!SSPCON2bits.ACKSTAT){
        I2C_restart();                                                      
        if (PIR2bits.BCLIF) {                                                           // test for bus collision
            return ( -1 );                                                              // return with Bus Collision error 
        }
    
        if (I2C_write(_control+1)){                                                     // scrivo indirizzo del registro
            return (-3);
        }
    
        if (!SSPCON2bits.ACKSTAT){ 
            _b1 = I2C_read();                                                           // leggo il byte 
            I2C_ack();                                                                  // invio ACK
            _b2 = I2C_read();                                                           // e leggo il secondo byte
            I2C_not_ack();                                                              // invio NotACK
            I2C_stop();                                                                 // stop I2C
            if (PIR2bits.BCLIF) {                                                       // test for bus collision
                return ( -1 );                                                          // return with Bus Collision error 
            }
        } else {
            return ( -2 );
        }
    } else {
        return ( -2 );
    }
    
    _return = _b1 << 8;
    _return = _return | _b2;

    return _return;
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_1ByteAddr_2ByteData_read
 * Input:       _control: indirizzo del dispositivo I2C (unsigned char)
 *              _address: registro da leggere (unsigned char)
 * Output:      2 byte ricevuti (unsigned int) 
 *
 * Overview:    Legge registri a 2 byte da dispositivi I2C con indirizzi dei registri a 1 byte (modulo I2C 2)
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 *              MSB primo dato ricevuto
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_2
unsigned int  I2C2_1ByteAddr_2ByteData_read (unsigned char _control, unsigned char _address) {
    unsigned char _b1 = 0;
    unsigned char _b2 = 0;
    signed int _return = 0;
    
    I2C2_idle();
    I2C2_start();                                                                       // start I2C
    
//    if (PIR3bits.BCL2IF) {                                                              // test for bus collision
      if (BCL2IF) {                                                                     // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
    }
    
    if (I2C2_write(_control)){                                        
        return ( -3 );
    }
    
    if (!SSP2CON2bits.ACKSTAT){
        if (I2C2_write(_address)){                                                      // scrivo indirizzo del registro
            return (-3);
        }
    } else {
        return ( -2 );
    }   

    if (!SSP2CON2bits.ACKSTAT){
        I2C2_restart();                                                      
//        if (PIR3bits.BCL2IF) {                                                          // test for bus collision
          if (BCL2IF) {                                                                 // test for bus collision
            return ( -1 );                                                              // return with Bus Collision error 
        }
        
        if (I2C2_write(_control+1)){                                                    // scrivo indirizzo del registro
            return (-3);
        }        
        
        if (!SSP2CON2bits.ACKSTAT){ 
            _b1 = I2C2_read();                                                          // leggo il byte 
            I2C2_ack();                                                                 // invio ACK
            _b2 = I2C2_read();                                                          // e leggo il secondo byte
            I2C2_not_ack();                                                             // invio NotACK
            I2C2_stop();                                                                // stop I2C
//            if (PIR3bits.BCL2IF) {                                                      // test for bus collision
              if (BCL2IF) {                                                                         // test for bus collision
                return ( -1 );                                                          // return with Bus Collision error 
            }
        } else {
            return ( -2 );
        }
    } else {
        return ( -2 );
    }
    
    _return = _b1 << 8;
    _return = _return | _b2;

    return _return;
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C_1ByteAddr_2ByteData_read_delay
 * Input:       _control: indirizzo del dispositivo I2C (unsigned char)
 *              _address: registro da leggere (unsigned char)
 * Output:      2 byte ricevuti (unsigned int) 
 *
 * Overview:    Legge registri a 2 byte da dispositivi I2C con indirizzi dei registri a 1 byte con delay tra indirizzo e lettura del dato (HDC1080) (modulo I2C 1)
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 *              MSB primo dato ricevuto
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_1
unsigned int  I2C_1ByteAddr_2ByteData_read_delay (unsigned char _control, unsigned char _address) {
    unsigned char _b1 = 0;
    unsigned char _b2 = 0;
    signed int _return = 0;
    
    I2C_idle();
    I2C_start();                                                                        // start I2C
    
    if (PIR2bits.BCLIF) {                                                               // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
    }
    
    if (I2C_write(_control)){                                        
        return ( -3 );
    }
    
    if (!SSPCON2bits.ACKSTAT){
        if (I2C_write(_address)){                                                       // scrivo indirizzo del registro
            return (-3);
        }
    } else {
        return ( -2 );
    }   
       
    Delay_ms(10);                                                                        // delay per attesa conversione HDC1080
    
    if (!SSPCON2bits.ACKSTAT){
        I2C_restart();                                                      
        if (PIR2bits.BCLIF) {                                                           // test for bus collision
            return ( -1 );                                                              // return with Bus Collision error 
        }
    
        if (I2C_write(_control+1)){                                                     // scrivo indirizzo del registro
            return (-3);
        }
    
        if (!SSPCON2bits.ACKSTAT){ 
            _b1 = I2C_read();                                                           // leggo il byte 
            I2C_ack();                                                                  // invio ACK
            _b2 = I2C_read();                                                           // e leggo il secondo byte
            I2C_not_ack();                                                              // invio NotACK
            I2C_stop();                                                                 // stop I2C
            if (PIR2bits.BCLIF) {                                                       // test for bus collision
                return ( -1 );                                                          // return with Bus Collision error 
            }
        } else {
            return ( -2 );
        }
    } else {
        return ( -2 );
    }
    
    _return = _b1 << 8;
    _return = _return | _b2;

    return _return;
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_1ByteAddr_2ByteData_read_delay
 * Input:       _control: indirizzo del dispositivo I2C (unsigned char)
 *              _address: registro da leggere (unsigned char)
 * Output:      2 byte ricevuti (unsigned int) 
 *
 * Overview:    Legge registri a 2 byte da dispositivi I2C con indirizzi dei registri a 1 byte con delay tra indirizzo e lettura del dato (HDC1080) (modulo I2C 2)
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 *              MSB primo dato ricevuto
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_2
unsigned int  I2C2_1ByteAddr_2ByteData_read_delay (unsigned char _control, unsigned char _address) {
    unsigned char _b1 = 0;
    unsigned char _b2 = 0;
    signed int _return = 0;
    
    I2C2_idle();
    I2C2_start();                                                                       // start I2C
    
//    if (PIR3bits.BCL2IF) {                                                              // test for bus collision
      if (BCL2IF) {                                                                         // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
    }
    
    if (I2C2_write(_control)){                                        
        return ( -3 );
    }
    
    if (!SSP2CON2bits.ACKSTAT){
        if (I2C2_write(_address)){                                                      // scrivo indirizzo del registro
            return (-3);
        }
    } else {
        return ( -2 );
    }   

    Delay_ms(7);                                                                        // delay per attesa conversione HDC1080

    if (!SSP2CON2bits.ACKSTAT){
        I2C2_restart();                                                      
//        if (PIR3bits.BCL2IF) {                                                          // test for bus collision
          if (BCL2IF) {                                                                         // test for bus collision
            return ( -1 );                                                              // return with Bus Collision error 
        }
        
        if (I2C2_write(_control+1)){                                                    // scrivo indirizzo del registro
            return (-3);
        }        
        
        if (!SSP2CON2bits.ACKSTAT){ 
            _b1 = I2C2_read();                                                          // leggo il byte 
            I2C2_ack();                                                                 // invio ACK
            _b2 = I2C2_read();                                                          // e leggo il secondo byte
            I2C2_not_ack();                                                             // invio NotACK
            I2C2_stop();                                                                // stop I2C
//            if (PIR3bits.BCL2IF) {                                                      // test for bus collision
              if (BCL2IF) {                                                                         // test for bus collision
                return ( -1 );                                                          // return with Bus Collision error 
            }
        } else {
            return ( -2 );
        }
    } else {
        return ( -2 );
    }
    
    _return = _b1 << 8;
    _return = _return | _b2;

    return _return;
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C_1ByteAddr_2ByteData_write
 * Input:       _control: indirizzo del dispositivo I2C (unsigned char)
 *              _address: registro da scrivere (unsigned char)
 *              _data: dato da scrivere (unsigned int)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision   
 *
 * Overview:    Scrive registri a 2 byte su dispositivi I2C con indirizzi dei registri a 1 byte (modulo I2C 1)
 * Note:        MSB primo dato scritto
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_1
signed char I2C_1ByteAddr_2ByteData_write (unsigned char _control, unsigned char _address, unsigned int _data) {

    unsigned char _b2 = _data >> 8;
    unsigned char _b1 = _data & 0b0000000011111111;
    
    I2C_idle();
    I2C_start();                                                                        // start I2C
    
    if (PIR2bits.BCLIF) {                                                              // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
    }
    
    
    if (I2C_write(_control)){                                             
         return ( -3 );
    }
    
    if (!SSPCON2bits.ACKSTAT){
        if (I2C_write(_address)){
            return (-3);
        }
    } else {
        return ( -2 );
    }

    if (!SSPCON2bits.ACKSTAT){
        if (I2C_write(_b2)){
            return (-3);
        }
    } else {
        return ( -2 );
    }
    
    if (!SSPCON2bits.ACKSTAT){
        if (I2C_write(_b1)){
            return (-3);
        }
    } else {
        return ( -2 );
    }
    
    if (!SSPCON2bits.ACKSTAT){
        I2C_stop();
    } else {
        return ( -2 );
    }
    
    if (PIR2bits.BCLIF) {                                                               // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
    }
    
    return (0);
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    I2C2_1ByteAddr_2ByteData_write
 * Input:       _control: indirizzo del dispositivo I2C (unsigned char)
 *              _address: registro da scrivere (unsigned char)
 *              _data: dato da scrivere (unsigned int)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision   
 *
 * Overview:    Scrive registri 2 byte su dispositivi I2C con indirizzi dei registri a 1 byte (modulo I2C 2)
 * Note:        MSB primo dato scritto
/* ============================================================================================================================================ */
#ifdef I2C_MODULE_2
signed char I2C2_1ByteAddr_2ByteData_write (unsigned char _control, unsigned char _address, unsigned int _data) {

    unsigned char _b2 = _data >> 8;
    unsigned char _b1 = _data & 0b0000000011111111;
    
    I2C2_idle();
    I2C2_start();                                                                       // start I2C
    
//    if (PIR3bits.BCL2IF) {                                                              // test for bus collision
      if (BCL2IF) {                                                                         // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
    }
    
    
    if (I2C2_write(_control)){                                             
         return ( -3 );
    }
    
    if (!SSP2CON2bits.ACKSTAT){
        if (I2C2_write(_address)){
            return (-3);
        }
    } else {
        return ( -2 );
    }

    if (!SSP2CON2bits.ACKSTAT){
        if (I2C2_write(_b2)){
            return (-3);
        }
    } else {
        return ( -2 );
    }
    
    if (!SSP2CON2bits.ACKSTAT){
        if (I2C2_write(_b1)){
            return (-3);
        }
    } else {
        return ( -2 );
    }
    
    if (!SSPCON2bits.ACKSTAT){
        I2C2_stop();
    } else {
        return ( -2 );
    }
    
//    if (PIR3bits.BCL2IF) {                                                              // test for bus collision
      if (BCL2IF) {                                                                         // test for bus collision
        return ( -1 );                                                                  // return with Bus Collision error 
    }
    
    return (0);
}
#endif

#endif