/* ============================================================================================================================================= /
 * LIBRARY:         Gestione LPS25HB
 * FILE:            LPS25HB.c
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         22/09/2016
 * UPDATED:         09/09/2016
 *
 * DESCRIP:         Gestione sensore pressione ST LPS25HB
 *
 * FUNCTIONS:       - LPS25HB_set_resolution()
 *                  - LPS25HB_set_mode()
 *                  - LPS25HB_power_mode()
 *                  - LPS25HB_sw_reset()
 *                  - LPS25HB_memory_reset()
 *                  - LPS25HB_shot()
 *                  - LPS25HB_read_press_raw() 
 *                  - LPS25HB_press_raw_2_float()         
 *                  - LPS25HB_read_press_float() 
 *                  - LPS25HB_read_temp_raw()
 *                  - LPS25HB_temp_raw_2_float()
 *                  - LPS25HB_read_temp_float()
 *                  - LPS25HB_auto_zero()
 *                  - LPS25HB_set_onepoint_cal()
 *                  - LPS25HB_check_press_data_ready()
 *                  - LPS25HB_check_temp_data_ready()
 *                  - LPS25HB_set_fifo()
 *                  - LPS25HB_fifo_get_status()
 *                  - LPS25HB_set_interrupt()
 *                  - LPS25HB_set_interrupt_ref()
 *                  - LPS25HB_check_interrupt_source()
 *                  - LPS25HB_read_device_id()       
/* ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.1               26/09/2017          - corretto (_dev_addr << 1)
  1.0               22/09/2016          - Prima release 
  1.0               09/09/2016          - Inizio sviluppo
//! [History]
/* ============================================================================================================================================ */

#ifndef LPS25HB
#define LPS25HB

/* INCLUDES =================================================================================================================================== */
#include    "LPS25HB_settings.h"
#include    "LPS25HB.h"
#include    <xc.h>

/* VARIABILI GLOBALI LIBRERIA================================================================================================================== */

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_set_resolution
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 *              _readed_res: settaggio per la risoluzione di acquisizione della temperatura (LPS25HB_TEMP_RES_8/LPS25HB_TEMP_RES_16/LPS25HB_TEMP_RES_32/LPS25HB_TEMP_RES_64)
 *              _press_res: settaggio per la risoluzione di acquisizione della pressione(LPS25HB_PRES_RES_8/LPS25HB_PRES_RES_32/LPS25HB_PRES_RES_128/LPS25HB_PRES_RES_512)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision 
 *
 * Overview:    Imposta la risoluzione della temperatura e della pressione
/* ============================================================================================================================================ */
signed char LPS25HB_set_resolution(unsigned char _dev_addr, unsigned char _temp_res, unsigned char _press_res){
    signed char _value = 0;
    
    _value = _temp_res | _press_res;                                                    // valore risoluzioni da impostare
    return _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_RES_CONF_REG, _value);    
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_set_mode
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 *              _mode: seleziona la modalit di funzionamento (LSP25HB_ODR_ONE_SHOT/LSP25HB_ODR_1HZ/LSP25HB_ODR_7HZ/LSP25HB_ODR_12HZ/LSP25HB_ODR_25HZ)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision 
 *
 * Overview:    Imposta la modalit di funzionamento: one shot oppure con aggiornamento a varie frequenze
 * Note:        Sia in modalit one shot che in acquisizione continua abilitare il dispositivo tramite funzione LPS25HB_power_mode(); 
 *              per funzionamento one shot abilitare l'acquisizione del dato tramite funzione LPS25HB_power_shot();
/* ============================================================================================================================================ */
signed char LPS25HB_set_mode(unsigned char _dev_addr, unsigned char _mode){        
    unsigned char _readed = 0;

    _readed = _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG1);  
    
    _readed &= 0b10001111;                                                              // cancello modalit attuale e mantengo gli altri bit
    _readed |= _mode;                                                                   // imposto nuova modalit
    
    return _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG1, _readed);    
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_power_mode
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 *              _mode: seleziona lo stato (LSP25HB_POWER_DOWN/LSP25HB_ACTIVE)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision 
 *
 * Overview:    Abilita o disabilita il sensore
/* ============================================================================================================================================ */
signed char LPS25HB_power_mode(unsigned char _dev_addr, unsigned char _mode){
    unsigned char _readed = 0;
    
    _readed = _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG1);  
    
    _readed &= ~LSP25HB_ACTIVE;                                                         // cancello modalit attuale e mantengo gli altri bit
    _readed |= _mode;                                                                   // imposto nuovo stato
    
    return _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG1, _readed);    
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_sw_reset
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision
 *
 * Overview:    Resetta i registri allo stato di avvio ed attende che il reset sia completo
 * Note:        Bloccante se il sensore non funziona correttamente
/* ============================================================================================================================================ */
signed char LPS25HB_sw_reset(unsigned char _dev_addr){
    signed char _return = 0;

    _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG2, LPS25HB_SWRESET);
    
    if (_return == 0)                                                                   // se non ci sono errori I2C proseguo
        while (_LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG2 & LPS25HB_SWRESET));       // attendo che LPS25HB_SWRESET torni a 0
        
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_memory_reset
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision
 *
 * Overview:    Resetta la memoria interna ai valori di fabbrica ed attende che il reset sia completo
 * Note:        Bloccante se il sensore non funziona correttamente
/* ============================================================================================================================================ */
signed char LPS25HB_memory_reset(unsigned char _dev_addr){
    signed char _return = 0;

    // reset memoria
    _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG2, LPS25HB_BOOT);
    if (_return == 0)                                                                   // se non ci sono errori I2C proseguo
        while (_LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG2 & LPS25HB_BOOT));      // attendo che LPS25HB_BOOT torni a 0     
        
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_shot
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision 
 *
 * Overview:    Avvia l'acquisizione dei dati ed attende che i dati siano pronti per essere letti
 * Note:        Non  necessario verificare che il dato sia pronto tramite funzione LPS25HB_check_press_data_ready() o LPS25HB_check_temp_data_ready.        
 *              Bloccante se il sensore non funziona correttamente
/* ============================================================================================================================================ */
signed char LPS25HB_shot(unsigned char _dev_addr){
    unsigned char _readed = 0; 
    signed char _return = 0;
    
    _readed = _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG2);  

    _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG2, _readed | LPS25HB_ONE_SHOT);      // shot
    
    if (_return == 0)                                                                   // se non ci sono errori di scrittura I2C
        while (_LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG2) & LPS25HB_ONE_SHOT);      // attendo che il dato venga acquisito (a dato acquisito il bit one shot torna a 0)
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_read_press_raw
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * Output:      valore grezzo del registro della pressione (signed long)
 *
 * Overview:    Legge il valore dai 3 byte di registro della pressione e torna il valore grezzo
 * Note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
/* ============================================================================================================================================ */
unsigned long LPS25HB_read_press_raw(unsigned char _dev_addr){
    unsigned long _press = 0;
    
    _press = _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_PRESS_OUT_H_REG);
    _press = _press << 8;
    _press |= _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_PRESS_OUT_L_REG);
    _press = _press << 8;
    _press |= _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_PRESS_OUT_XL_REG);

    return _press;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_press_raw_2_float
 * Input:       _press: valore grezzo del registro della pressione (unsigned long)
 * Output:      valore della pressione convertito (float)
 *
 * Overview:    Converte il valore grezzo del registro della pressione in valore float
 * Note:        Risoluzione massima 0,0002hPa
/* ============================================================================================================================================ */
float LPS25HB_press_raw_2_float(unsigned long _press){
    float _return = 0;
    unsigned char _negative_flag = 0;
                                                                                        // Il dato della pressione  in complemento a 2 24 bit. non 
                                                                                        // esiste una variabile XC8 per questo formato. devo usare quindi il formato unsigned long
    
    if (_press & 0b00000000100000000000000000000000) {                                  // se il bit 23  1 significa che  un valore negativo
        _negative_flag = 1;                                                             
        _press = ~_press +1;                                                            // faccio il complemento a 2
        _press &= 0b00000000111111111111111111111111;                                   // ed elimino i bit superflui
    
    }
    _return = (float) _press /4096;                                                     // calcolo il float
    
    if (_negative_flag)                                                                 // se il dato era negativo moltiplico il float * -1
        _return *= -1;
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_read_press_float
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * Output:      valore della pressione convertito (float)
 *
 * Overview:    Legge il valore dai 3 byte di registro della pressione e torna il valore il valore convertito in float
 * Note:        Risoluzione massima 0,0002hPa
 *              Non torna eventuali errori di comunicazione I2C
/* ============================================================================================================================================ */
float LPS25HB_read_press_float(unsigned char _dev_addr){
    unsigned long _readed = 0;
    
    _readed = LPS25HB_read_press_raw (_dev_addr);
    return LPS25HB_press_raw_2_float (_readed);
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_read_temp_raw
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * Output:      valore grezzo del registro della temperatura (signed int)
 *
 * Overview:    Legge il valore dai 2 byte di registro della temperatura e torna il valore grezzo
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
/* ============================================================================================================================================ */
signed int LPS25HB_read_temp_raw(unsigned char _dev_addr){
    signed int _temp = 0;
    
    _temp = _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_TEMP_OUT_H_REG);
    _temp = _temp << 8;
    _temp |= _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_TEMP_OUT_L_REG);

    return _temp;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_temp_raw_2_float
 * Input:       _temp: valore grezzo del registro della temperatura (signed int)
 * Output:      valore della temperatura convertito (float)
 *
 * Overview:    Converte il valore grezzo del registro della temperatura in valore float
 * Note:        Risoluzione massima 0,002C
/* ============================================================================================================================================ */
float LPS25HB_temp_raw_2_float(signed int _temp){
    float _return = 0;                                                                  // il dato raw della temperatura  a 16 bit in complemento a 2
                                                                                        // mi basta usare una variabile signed int                                                                                                                                                                        
    _return = 42.5 + ((float) _temp/480);
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_read_temp_float
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * Output:      valore della temperatura convertito (float)
 *
 * Overview:    Legge il valore dai 2 byte di registro della temperatura e torna il valore il valore convertito in float
 * Note:        Risoluzione massima 0,002C
 *              Non torna eventuali errori di comunicazione I2C
/* ============================================================================================================================================ */
float LPS25HB_read_temp_float(unsigned char _dev_addr){
    signed int _readed = 0;
    
    _readed = LPS25HB_read_temp_raw (_dev_addr);
    return LPS25HB_temp_raw_2_float (_readed);
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_auto_zero
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 *              _on_off: auto zero On/Off (unsigned char)
  * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision
 *
 * Overview:    Abilita o disabilita la funzione Auto Zero. (la pressione attuale viene presa come valore di riferimento, i valori letti successivamente
 *              vengono letti come variazione rispetto allo zero impostato).
 * ============================================================================================================================================ */
signed char LPS25HB_auto_zero(unsigned char _dev_addr, unsigned char _on_off){
    unsigned char _readed1 = 0;
    unsigned char _readed2 = 0;
    signed char _return = 0;
    
    _readed1 = _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG1);      // leggo CTRL_REG1 e cancello lo stato attuale di RESET_AZ
    _readed2 = _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG2);      // leggo CTRL_REG2 e cancello lo stato attuale di AUTO_ZERO
    _readed2 &= ~LPS25HB_AUTO_ZERO;
    
    if(_on_off==ON) {
        _readed2 |= LPS25HB_AUTO_ZERO;                                                    // Setto CTRL_REG2 AUTO_ZERO
    } else {
        _readed1 |= LPS25HB_AUTO_ZERO;                                                    // Setto CTRL_REG1 RESET_AZ
    }
    
    _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG1, _readed1);
    if (_return == 0)
        _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG2, _readed2);
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_set_onepoint_cal
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 *              _offset_float: valore da aggiungere o togliere dal dato letto (float)
  * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision
 *
 * Overview:    Aggiunge o toglie un offset al dato di pressione letto prima di essere salvato nei registri PRESS_OUT
 * ============================================================================================================================================ */
signed char LPS25HB_set_onepoint_cal(unsigned char _dev_addr, float _offset_float){
    signed int _offset_raw = 0;
    signed char _return = 0;
    
    /* NB RPDS_L e RPDS_H corrispondono ai 2 byte alti della pressione, tratto il dato come signed int.
       L'LPS25HB  fa una sottrazione tra il valore letto e il valore impostato nei registi RPDS. Per mantenere una logica positiva inverto
       il dato facendo un complemento a 2
    */
    
    if (_offset_float != 0){                                                            // se il valore dell'offset  diverso da 0

        _offset_float = _offset_float * 16;                                             // calcolo i valori da impostare nei registri RPDS                                                
        _offset_raw = (signed int) _offset_float;

        _offset_raw = ~_offset_raw +1;                                                  // complemento a 2

        _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_RPDS_L, _offset_raw);        // scrivo i valori nei registri RPDS_L e RPDS_H  
        if (_return == 0)
            _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_RPDS_H, _offset_raw >> 8);
            
    } else {                                                                            // se il valore  0 resetto i registri RPDS_L e RPDS_H
        _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_RPDS_L, 0);
        if (_return == 0)
            _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_RPDS_H, 0);
    }
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_check_temp_press_ready
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 *              
 * Output:      0: dato pressione non pronto
 *              1: dato pressione pronto
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision
 *
 * Overview:    Controlla se il dato della pressione  pronto
 * Note:        Si setta quando un nuovo dato viene convertito e si resetta in automatico quando viene letto il dato 
 * ============================================================================================================================================ */
signed char LPS25HB_check_press_data_ready(unsigned char _dev_addr){
    signed char _readed = 0;
    signed char _return = 0;
    
    _readed = _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_STATUS_REG);      // leggo STATUS_REG
     
    if (_readed >= 0){                                                                  // dato valido
        if ((_readed & LPS25HB_P_DA) == LPS25HB_P_DA)                                   // LPS25HB_P_DA = 1
            _return = 1;                                                                // torno 1
        else if ((_readed & LPS25HB_P_DA) == 0)
            _return = 0;
    }else                                                                               // dato non valido torno l'errore
        _return =  _readed;
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_check_temp_data_ready
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 *              
 * Output:      0: dato temperatura non pronto
 *              1: dato temperatura pronto
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision
 *
 * Overview:    Controlla se il dato della temperatura  pronto
 * Note:        Si setta quando un nuovo dato viene convertito e si resetta in automatico quando viene letto il dato 
 * ============================================================================================================================================ */
signed char LPS25HB_check_temp_data_ready(unsigned char _dev_addr){
    signed char _readed = 0;
    signed char _return = 0;
    
    _readed = _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_STATUS_REG);
     
    if (_readed >= 0){
        if ((_readed & LPS25HB_T_DA) == LPS25HB_T_DA)
            _return = 1;
        else if ((_readed & LPS25HB_T_DA) == 0)
            _return = 0;
    }else
        _return =  _readed;
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_set_fifo
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 *              _fifo_mode: modalit FIFO (LPS25HB_FIFO_OFF/LPS25HB_F_MODE_FIFO/LPS25HB_F_MODE_STREAM/LPS25HB_F_MODE_FIFO_MEAN)
 *              _wtm: Watermark level (LPS25HB_FIFO_WTM_0/LPS25HB_FIFO_WTM_2/LPS25HB_FIFO_WTM_4/LPS25HB_FIFO_WTM_8/LPS25HB_FIFO_WTM_16/LPS25HB_FIFO_WTM_32)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision
 *
 * Overview:    Imposta la modalit FIFO e relativi parametri
 * Note:        Implementati solo FIFO main mode
 * ============================================================================================================================================ */
signed char LPS25HB_set_fifo(unsigned char _dev_addr, unsigned char _fifo_mode, unsigned char _wtm){
    signed char _return = 0;
    unsigned char _readed = 0;
    
    _readed = _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG2);
    _readed &= ~LPS25HB_FIFO_EN;                                                        // cancello lo stato attuale di FIFO_EN
    
    if (_fifo_mode != LPS25HB_FIFO_OFF)                                                   
        _readed |= LPS25HB_FIFO_EN;                                                     // abilito FIFO se richiesta una modalit valida
    
    _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_FIFO_CTRL, LPS25HB_F_MODE_BYPASS);   
    if (_return == 0)
        _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG2, _readed); 
    if (_return == 0)
        _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_FIFO_CTRL, _fifo_mode | _wtm);       // imposto FIFO e WTM     
        
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_fifo_get_status
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 *              _to_check: parametro del registro FIFO_STATUS da controllare (LPS25HB_FIFO_STAT_FTH/LPS25HB_FIFO_STAT_OVR/LPS25HB_FIFO_STAT_EMPTY/LPS25HB_FIFO_STAT_FSS/LPS25HB_FIFO_STAT_ALL
 * Output:      Per LPS25HB_FIFO_STAT_FTH, LPS25HB_FIFO_STAT_OVR, LPS25HB_FIFO_STAT_EMPTY
 *                  - 1: stato verificato
 *                  - 2: stato non verificato
 *              Per LPS25HB_FIFO_STAT_FSS torna il valore dell'ultimo livello FIFO usato
 *              Per LPS25HB_FIFO_STAT_ALL torna l'intero registro FIFO_STATUS
 *
 * Overview:    Legge il parametro desiderato dello stato del FIFO
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 * ============================================================================================================================================ */
signed char LPS25HB_fifo_get_status(unsigned char _dev_addr, unsigned char _to_check){
    unsigned char _readed = 0;
    signed char _return = 0;
    
    _readed = _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_FIFO_STATUS);
    
    if (_to_check == LPS25HB_FIFO_STAT_ALL)
        _return = _readed;                                                              // torno tutto il valore del registro
    else if (_to_check == LPS25HB_FIFO_STAT_FSS)
        _return = _readed & LPS25HB_FIFO_STAT_FSS;                                      // torno il valore del livello WTM usato
    else {
        if (_readed & _to_check)                                                        // altrimenti verifico lo stato dei singoli bit richiesto e 
            return 1;                                                                   // torno 1 se lo stato  verificato
        else
            return 0;        
    }    
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_set_interrupt
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 *              _source: sorgente interrupt (LPS25HB_INT_DRDY/LPS25HB_INT_F_OVR/LPS25HB_INT_F_FTH/LPS25HB_INT_F_EMPTY/LPS25HB_INT_P_HIGH/LPS25HB_INT_P_LOW/LPS25HB_INT_P_HIGH_LOW)
 *              _logic: logica interrupt (LPS25HB_INT_ACTIVE_HIGH/LPS25HB_INT_ACTIVE_LOW)
 *              _connection: collegamento hardware del pin INT_DRDY (LPS25HB_INT_PUSH_PULL/LPS25HB_INT_OPEN_DRAIN)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision
 *
 * Overview:    Imposta la modalit e attiva interrupt sul pin INT_DRDY.
 *              - In modalit DATA viene generato un interrupt quando  disponibile un nuovo dato di pressione oppure per lo stato FIFO selezionato 
 *              - Nelle modalit di interrupt differenziali viene generato un interrupt a seconda delle impostazioni selezionate (Pressione alta/pressione bassa rispetto ai valori di riferimento)
 * Note:        Per utilizzare le funzioni di interrupt differenziali impostare i valori di riferimento e di threshold tramite la funzione LPS25HB_set_interrupt_ref()
 *              prima di impostare gli interrupt con la funzione LPS25HB_set_interrupt()
 * ============================================================================================================================================ */
signed char LPS25HB_set_interrupt(unsigned char _dev_addr, unsigned char _source, unsigned char _logic, unsigned char _connection){
    unsigned char _ctrl_reg1 = 0;
    signed char _return = 0;

    _ctrl_reg1 = _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG1);
    _ctrl_reg1 &= ~LPS25HB_DIFF_EN;                                                     // cancello il valore attuale di DIFF_EN
    
    // interrupt data
    if ((_source == LPS25HB_INT_DRDY) || (_source == LPS25HB_INT_F_OVR) || (_source == LPS25HB_INT_F_FTH) || (_source == LPS25HB_INT_F_EMPTY)){
        _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG1, _ctrl_reg1);
        if (_return == 0)
            _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG3,_logic | _connection);
        if (_return == 0)
            _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG4, _source >> 4);
        if (_return == 0)
            _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_INTERRUPT_CFG, 0x00);                    // resetto registro    
    
        
    // interrupt differenziali    
    } else {
        _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG1, _ctrl_reg1 | LPS25HB_DIFF_EN);
        if (_return == 0)
            _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG3, _source |_logic | _connection);
        if (_return == 0)
            _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_INTERRUPT_CFG, _source | LPS25HB_LIR);   // configurazione di cosa causa l'interrupt. uso _source perch i bit corrispondono a PL_E e PJ_E 
                                                                                                                                        // e forzo LIR per salvare lo stato dell'interrupt nel registro INT_SOURCE
        if (_return == 0)
            _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_CTRL_REG4, 0x00);                        // resetto registro
    }
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_set_interrupt_ref
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 *              _press: pressione di riferimento per l'interrupt in hPa (float)
 *              _threshold: threshold di riferimento per l'interrupt in hPa (float)
  * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision
 *
 * Overview:    Imposta la pressione di riferimento e la threshold per far scattare l'interrupt
 * Note:        Richiamare questa funzione prima della funzione LPS25HB_set_interrupt_ref()
 * ============================================================================================================================================ */
signed char LPS25HB_set_interrupt_ref(unsigned char _dev_addr, float _press, float _threshold){
    unsigned long _press_raw = 0;
    signed int  _threshold_raw = 0; 
    signed char _return = 0;
    
    _press_raw =  (unsigned int) _press * 4096;
    _threshold_raw = (unsigned int) _threshold * 16;
    
    _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_REF_P_XL, _press_raw);
    if (_return == 0)
        _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_REF_P_L, _press_raw >> 8);
    if (_return == 0)
        _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_REF_P_H, _press_raw >> 16);
    if (_return == 0)
        _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_THS_P_L, _threshold_raw);
    if (_return == 0)
        _return = _LPS25HB_I2C_1ByteAddr_1ByteData_write(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_THS_P_H, _threshold_raw >> 8);
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_check_interrupt_source
 * Input:       _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * Output:      0: Nessun interrupt
 *              1: Interrupt pressione bassa 
 *              2: Interrupt pressione alta 
 *              3: Interrupt pressione alta e bassa
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision
 *
 * Overview:    Verifica l'evento differenziale che ha causato l'interrupt
 * ============================================================================================================================================ */
signed char LPS25HB_check_interrupt_source(unsigned char _dev_addr) {
    return _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_INT_SOURCE);
}

/* ============================================================================================================================================= /
 * FUNCTION:    LPS25HB_read_device_id
 * Input        _dev_addr: indirizzo impostato sul pin SA0 (unsigned char)
 * Output:      Device ID (unsigned char)
 *
 * Overview:    Legge l'id del dispositivo (0xBD) 
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
/* ============================================================================================================================================ */
unsigned char LPS25HB_read_device_id(unsigned char _dev_addr){
    return _LPS25HB_I2C_1ByteAddr_1ByteData_read(LPS25HB_ADDR | (_dev_addr << 1), LPS25HB_WHO_AM_I_REG);    
}

#endif          