/* ============================================================================================================================================= /
 * LIBRARY:         USART_init.c
 * FILE:            USART.c
 * LAST REV:        2.5
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         29/09/2013
 * UPDATED:         31/07/2017
 *
 * DESCRIP:         gestione del bus USART primario e secondario (se disponibile)
 *
 * FUNCTIONS:       - USART_enable
 *                  - USART_send_char
 *                  - USART_send_string
 *                  - USART_send_n_chars
 *                  - USART_receive o interrupt
 *                  - USART2_enable
 *                  - USART2_send_char
 *                  - USART2_send_string
 *                  - USART2_send_n_chars
 *                  - USART2_receive o interrupt
 * ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  2.5               31/07/2017          - Aggiunte funzioni USART_send_n_chars() e USART2_send_n_chars() per inviare stringhe di lughezza fissa
                                          inviando anche i caratteri di fine stringa '\0'
  2.4               10/01/2017          - baudrate impostabile in funzioni USART_enable(_baudr) e USART2_enable(_baudr)
  2.3               16/12/2016          - Compatibilit con PIC18F26K22 e PIC18F67K22   
  2.2               27/08/2016          - Adattamento per poter utilizzare 2 moduli contemporaneamente, compatibilit con PIC18F46K22
                                        - Linee TX e RX impostate automaticamente a seconda del PIC in uso e del modulo selezionato in setting
                                          dispositivo (gestione in USART_init.h)
  2.1               25/08/2016          - Aggiunti settaggi per gestione clock 64MHz
  2.0               08/08/2016          - Gestione secondo modulo I2C
                                        - Rivisto ciclo di ricezione (bloccante)
  1.0               24/09/2013          - Prima release
//! [History]
/* ============================================================================================================================================ */

#ifndef USART
#define USART

/* INCLUDES =================================================================================================================================== */
#include "USART_init_settings.h"
#include "USART_init.h"

/* ============================================================================================================================================= /
 * FUNCTION:        USART_enable
 * Input:           _baudr: settaggio baud rate (unsigned char)
 *
 * Overview:        Abilita l'invio e la ricezione dei dati via USART, Baud rate impostabile, 8 bit, senza parit . MODULO 1
/* ============================================================================================================================================ */
#ifdef USART_MODULE_1
void USART_enable(unsigned char _baudr) {
#ifdef _12F1840
    if (USART_TX_RA == 0)                                                               // TX su RA0
        APFCONbits.TXCKSEL = 0;    
    else if (USART_TX_RA == 4)                                                          // TX su RA4
        APFCONbits.TXCKSEL = 1;   
    
    if (USART_RX_RA == 1)                                                               // RX su RA1
        APFCONbits.RXDTSEL = 0;
    else if (USART_RX_RA == 5)                                                          // RX su RA5    
        APFCONbits.RXDTSEL = 1;
#endif
    
    USART_TX_TRIS = 1;
    USART_RX_TRIS = 1;
        
    TXSTA = 0x00;
    RCSTA =0x00;

    TXSTAbits.TX9 = 0;                                                                  // trasmissione 8 bit
    RCSTAbits.RX9 = 0;                                                                  // trasmissione 8 bit
    
    TXSTAbits.SYNC = 0;                                                                 // trasmissione asincrona
    TXSTAbits.CSRC = 0;                                                                 // 
    
    RCSTAbits.CREN = 1;                                                                 // Continuos Reception
    RCSTAbits.SREN = 0;

    TXSTAbits.BRGH = 1;                                                                 // baud 9600
    
    switch(_baudr) {
        case USART_BAUDR_9600:
                                BAUDCONbits.BRG16 = USART_9600_BRG16;
                                SPBRGH = USART_9600_SPBRGH;
                                SPBRG = USART_9600_SPBRG;
                                break;
        case USART_BAUDR_19200:
                                BAUDCONbits.BRG16 = USART_19200_BRG16;
                                SPBRGH = USART_19200_SPBRGH;
                                SPBRG = USART_19200_SPBRG;
                                break;                            
#if (XTAL_FREQ >= 8)                                                                    // con frequenze di clock inferiori a 8 MHz il baud rate non pu  andare oltre a 19200
        case USART_BAUDR_57600:
                                BAUDCONbits.BRG16 = USART_57600_BRG16;
                                SPBRGH = USART_57600_SPBRGH;
                                SPBRG = USART_57600_SPBRG;
                                break;
        case USART_BAUDR_115200:
                                BAUDCONbits.BRG16 = USART_115200_BRG16;
                                SPBRGH = USART_115200_SPBRGH;
                                SPBRG = USART_115200_SPBRG;
                                break;
#endif
                                
        default:                
                                BAUDCONbits.BRG16 = USART_9600_BRG16;
                                SPBRGH = USART_9600_SPBRGH;
                                SPBRG = USART_9600_SPBRG;
                                break;
    }
    
    
    TXSTAbits.TXEN = 1;                                                                 // abilito trasmissione
    RCSTAbits.SPEN = 1;
    
//        RXDTP=1;
//        TXCKP=1;
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:        USART_enable
 * Input:           _baudr: settaggio baud rate (unsigned char)
 *
 * Overview:        Abilita l'invio e la ricezione dei dati via USART, Baud rate impostabile, 8 bit, senza parit . MODULO 2
/* ============================================================================================================================================ */
#ifdef USART_MODULE_2
void USART2_enable(unsigned char _baudr) {
    USART2_TX_TRIS = 1;
    USART2_RX_TRIS = 1;
        
    TXSTA2 = 0x00;
    RCSTA2 =0x00;

    TXSTA2bits.TX9 = 0;                                                                 // trasmissione 8 bit
    RCSTA2bits.RX9 = 0;                                                                 // trasmissione 8 bit
    
    TXSTA2bits.SYNC = 0;                                                                // trasmissione asincrona
    TXSTA2bits.CSRC = 0;                                                                // 
    
    RCSTA2bits.CREN = 1;                                                                // Continuos Reception
    RCSTA2bits.SREN = 0;

    TXSTA2bits.BRGH = 1;                                                                // baud 9600
    
    switch(_baudr) {
        case USART_BAUDR_9600:
                                BAUDCON2bits.BRG16 = USART_9600_BRG16;
                                SPBRGH2 = USART_9600_SPBRGH;
                                SPBRG2 = USART_9600_SPBRG;
                                break;
        case USART_BAUDR_19200:
                                BAUDCON2bits.BRG16 = USART_19200_BRG16;
                                SPBRGH2 = USART_19200_SPBRGH;
                                SPBRG2 = USART_19200_SPBRG;
                                break;                            
#if (XTAL_FREQ >= 8)
                                
        case USART_BAUDR_57600:
                                BAUDCON2bits.BRG16 = USART_57600_BRG16;
                                SPBRGH2 = USART_57600_SPBRGH;
                                SPBRG2 = USART_57600_SPBRG;
                                break;
        case USART_BAUDR_115200:
                                BAUDCON2bits.BRG16 = USART_115200_BRG16;
                                SPBRGH2 = USART_115200_SPBRGH;
                                SPBRG2 = USART_115200_SPBRG;
                                break;
#endif
                                
        default:                
                                BAUDCON2bits.BRG16 = USART_9600_BRG16;
                                SPBRGH2 = USART_9600_SPBRGH;
                                SPBRG2 = USART_9600_SPBRG;
                                break;
    }
    
    
    TXSTA2bits.TXEN = 1;                                                                // abilito trasmissione
    RCSTA2bits.SPEN = 1;
    
//        RXDTP=1;
//        TXCKP=1;
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:        USART_send_char
 * Input:           _data: byte da inviare (unsigned char)
 *
 * Overview:        Invia un byte via USART e attende il flag di invio completato. MODULO 1
/* ============================================================================================================================================ */
#ifdef USART_MODULE_1
void USART_send_char(unsigned char _data) {
    TXREG = _data;
    while (!TXSTAbits.TRMT)
        continue;
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:        USART_send_char
 * Input:           _data: byte da inviare (unsigned char)
 *
 * Overview:        Invia un byte via USART e attende il flag di invio completato. MODULO 2
/* ============================================================================================================================================ */
#ifdef USART_MODULE_2
void USART2_send_char(unsigned char _data) {
    TXREG2 = _data;
    while (!TXSTA2bits.TRMT)
        continue;
}
#endif        

/* ============================================================================================================================================= /
 * FUNCTION:        USART_send_string
 * Input:           _string[]: Stringa da inviare
 *
 * Overview:        Invia una stringa via USART. MODULO 1
/* ============================================================================================================================================ */
#ifdef USART_MODULE_1
void USART_send_string(const unsigned char _string[]) {
    unsigned char _i=0;
    while (_string[_i] != '\0') {                                                       // finch il valore da scrivere non  /0 (ultimo valore stringhe)
        USART_send_char(_string[_i]);                                                   // mando i caratteri
        _i++;
    }
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:        USART_send_string
 * Input:           _string[]: Stringa da inviare
 *
 * Overview:        Invia una stringa via USART. MODULO 2
/* ============================================================================================================================================ */
#ifdef USART_MODULE_2
void USART2_send_string(const unsigned char _string[]) {
    unsigned char _i=0;
    while (_string[_i] != '\0') {                                                       // finch il valore da scrivere non  /0 (ultimo valore stringhe)
        USART2_send_char(_string[_i]);                                                  // mando i caratteri
        _i++;
    }
}
#endif    

/* ============================================================================================================================================= /
 * FUNCTION:        USART_send_n_chars
 * Input:           _string[]: Stringa da inviare
 *                  _n: numero di byte da inviare (unsigned char)
 *
 * Overview:        Invia una stringa di dati di linghezza fissa via USART. MODULO 1 
 * Note:            a differenza della funzione USART_send_string non si ferma se il byte da inviare  0x00 (carattere di fine stringa ('\0')
/* ============================================================================================================================================ */
#ifdef USART_MODULE_1
void USART_send_n_chars(const unsigned char _string[], unsigned char _n) {
    unsigned char _i=0;
    while (_i < _n) {                                                       // finch il valore da scrivere non  /0 (ultimo valore stringhe)
        USART_send_char(_string[_i]);                                                   // mando i caratteri
        _i++;
    }
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:        USART2_send_n_chars
 * Input:           _string[]: Stringa da inviare
 *                  _n: numero di byte da inviare (unsigned char)
 *
 * Overview:        Invia una stringa di dati di linghezza fissa via USART. MODULO 2 
 * Note:            a differenza della funzione USART_send_string non si ferma se il byte da inviare  0x00 (carattere di fine stringa ('\0')
/* ============================================================================================================================================ */
#ifdef USART2_MODULE_2
void USART_send_n_chars(const unsigned char _string[], unsigned char _n) {
    unsigned char _i=0;
    while (_i < _n) {                                                       // finch il valore da scrivere non  /0 (ultimo valore stringhe)
        USART2_send_char(_string[_i]);                                                   // mando i caratteri
        _i++;
    }
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:        USART_receive
 * Output:          torna il byte ricevuto (unsigned char)
 *
 * Overview:        Restituisce il byte ricevuto, bloccante se non si riceve nulla. MODULO 1
 /* ============================================================================================================================================ */
#ifdef USART_MODULE_1  
unsigned char USART_receive(void) {
 
    while (!PIR1bits.RCIF);
        PIR1bits.RCIF = 0;
        return RCREG;                                                                  
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:        USART_receive
 * Output:          torna il byte ricevuto (unsigned char)
 *
 * Overview:        Restituisce il byte ricevuto, bloccante se non si riceve nulla. MODULO 2
 /* ============================================================================================================================================ */
#ifdef USART_MODULE_2
unsigned char USART2_receive(void) {
    while (!PIR3bits.RC2IF);
    PIR3bits.RC2IF = 0;
   
    return RCREG2;                                                  
}
#endif                                                                        

/* ============================================================================================================================================= /
 * FUNCTION:        INTERRUPT
 * Output:          torna il byte ricevuto (unsigned char)
 * Preconditions:   abilitare ricezione interrupt PIE1bits.RC1IE (modulo 1)
 *
 * Overview:        Esempio di interrupt da inserire in Main.c per la ricezione dei dati USART MODULO 1
/* ============================================================================================================================================ */

//void interrupt INTERRUPT(void) {
//    if (PIR1bits.RCIF) {                                                                // se l'interrupt  stato causato da una ricezione dati USART
//        unsigned char _data = USART_receive();                                          // salvo i dati ricevuti nell'array usart_received
//        usart_received[index] = _data;
//        index++;
//    #ifdef __DEBUG
////        if(_data != 0x0D && _data != 0x0A)                                              // Esposizione dato ricevuto su LCD per debug
////        {LCD_send_char(_data);}
//    #endif
//    }
//}

/* ============================================================================================================================================= /
 * FUNCTION:        INTERRUPT
 * Output:          torna il byte ricevuto (unsigned char)
 * Preconditions:   abilitare ricezione interrupt PIE3bits.RC2IE (modulo 2) 
 *
 * Overview:        Esempio di interrupt da inserire in Main.c per la ricezione dei dati USART MODULO 2
/* ============================================================================================================================================ */
//void interrupt INTERRUPT(void) {
//    if (PIR3bits.RC2IF) {                                                                // se l'interrupt  stato causato da una ricezione dati USART
//        unsigned char _data = USART2_receive();                                          // salvo i dati ricevuti nell'array usart_received
//        usart2_received[index2] = _data;
//        index2++;
//    #ifdef __DEBUG
////        if(_data != 0x0D && _data != 0x0A)                                              // Esposizione dato ricevuto su LCD per debug
////        {LCD_send_char(_data);}
//    #endif
//    }
//}


#endif