/* ============================================================================================================================================= /
 * LIBRARY:         Gestione VEML6075
 * FILE:            VEML6075.c
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         09/05/2017
 * UPDATED:         20/12/2016
 *
 * DESCRIP:         gestione del sensore di raggi UVA e UVB Vishay VEML6075
 *
 * FUNCTIONS:       - VEML6075_reset()
 *                  - VEML6075_set()
 *                  - VEML6075_power_mode()
 *                  - VEML6075_shot()
 *                  - VEML6075_read_uv_raw()
 *                  - VEML6075_convert_uv_raw()
 *                  - VEML6075_read_uv()
 *                  - VEML6075_read_device_id();
/* ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.1               09/05/2017          - Corretta formula per calcolo UVA e UVB in uW/cm^2
 *                                      - Calcolo UVA e UVB responsivity in funzione VEML6075_set a seconda di IT selezionato
  1.0               20/12/2016          - Prima release completa
  1.0               14/10/2016          - Inizio sviluppo
//! [History]
/* ============================================================================================================================================ */

#ifndef VEML6075 
#define VEML6075

/* VARIABILI GLOBALI LIBRERIA================================================================================================================== */
typedef struct {                                                                        // struct valori raw
    unsigned int uva;
    unsigned int uvb;
    unsigned int uv_comp_1;
    unsigned int uv_comp_2;
} veml6075_uv_raw; 

typedef struct {                                                                        // struct valori convertiti
    float uva;
    float uvb;
    float uvi;
} veml6075_uv;  

float veml6075_uva_resp = 0.0;
float veml6075_uvb_resp = 0.0;

/* INCLUDES =================================================================================================================================== */
#include    "VEML6075_settings.h"
#include    "VEML6075.h"
#include    "math.h"
#include    <xc.h>

/* ============================================================================================================================================= /
 * FUNCTION:    VEML6075_reset
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision 
 *
 * Overview:    Resetta il sensore ai valori di default e lo imposta in power down mode
/* ============================================================================================================================================ */
signed char VEML6075_reset(void) {
    
    return _VEML6075_I2C_1ByteAddr_2ByteData_write(VEML6075_ADDR, VEML6075_CONFIG_REG, (VEML6075_RESET << 8));       // resetto il registro ai valori di default e imposto shut down
}

/* ============================================================================================================================================= /
 * FUNCTION:    VEML6075_set
 * Input:       _mode: settaggio per la modalit di acquisizione (VEML6075_SEQUENTIAL_AQ/VEML6075_ONE_AQ)
 *              _dr: settaggio per il range dinamico (VEML6075_NORMAL_DR/VEML6075_HIGH_DR)
 *              _uv_it: setaggio per il tempo di integrazione(VEML6075_IT_50MS, VEML6075_IT_100MS, VEML6075_IT_200MS, VEML6075_IT_400MS, VEML6075_IT_800MS)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision 
 *
 * Overview:    Imposta la modalit di acquisizione: continua o one acquisition, il range dinamico: normale o alto e il tempo di integrazione. Imposta il 
 *              sensore in power down         
/* ============================================================================================================================================ */
signed char VEML6075_set(unsigned char _mode, unsigned char _dr, unsigned char _uv_it) {
    unsigned int _setting = 0;
     
    veml6075_uva_resp = (float)VEML6075_UVA_RESPONSIVITY;
    veml6075_uvb_resp = (float)VEML6075_UVB_RESPONSIVITY;
    
//    // calcolo uva e uvb responsivity a seconda di IT: raddoppia ad ogni raddoppiare di IT
//    for (unsigned char _i = 0; _i < (_uv_it >> 4); _i++){
//    veml6075_uva_resp *= 2;    
//    veml6075_uvb_resp *= 2;    
//    }
    
    // calcolo uva e uvb responsivity a seconda di IT: dimezza ad ogni raddoppiare di IT
    
    veml6075_uva_resp = veml6075_uva_resp / (pow(2,_uv_it >> 4));
    veml6075_uvb_resp = veml6075_uvb_resp / (pow(2,_uv_it >> 4));
    
    _setting = _mode | _dr | _uv_it | VEML6075_POWER_DOWN;                              // imposto il valore de scrivere nel registro
    
    return _VEML6075_I2C_1ByteAddr_2ByteData_write(VEML6075_ADDR, VEML6075_CONFIG_REG, (_setting << 8));      // scrivo il valore nel registro di configurazione
}  

/* ============================================================================================================================================= /
 * FUNCTION:    VEML6075_cal_responsivity
 * Input:       _uv_it: setaggio per il tempo di integrazione(VEML6075_IT_50MS, VEML6075_IT_100MS, VEML6075_IT_200MS, VEML6075_IT_400MS, VEML6075_IT_800MS)
 *
 * Overview:    Calcola i coefficienti da utilizzare per i calcoli di UVA e UVB a seconda dell' IT selezionato
 * Nota:        Questa funzione va usata solo se prima non  stata usata la funzione VEML6075_set (il sensore  su un altro circuito)     
/* ============================================================================================================================================ */
void VEML6075_cal_responsivity (unsigned char _uv_it){
    
    veml6075_uva_resp = (float)VEML6075_UVA_RESPONSIVITY;
    veml6075_uvb_resp = (float)VEML6075_UVB_RESPONSIVITY;
    
//    // calcolo uva e uvb responsivity a seconda di IT: raddoppia ad ogni raddoppiare di IT
//    for (unsigned char _i = 0; _i < (_uv_it >> 4); _i++){
//        veml6075_uva_resp /= 2;    
//        veml6075_uvb_resp /= 2;    
//    }
    
    // calcolo uva e uvb responsivity a seconda di IT: dimezza ad ogni raddoppiare di IT
    
    veml6075_uva_resp = veml6075_uva_resp / (pow(2,_uv_it >> 4));
    veml6075_uvb_resp = veml6075_uvb_resp / (pow(2,_uv_it >> 4));
}

/* ============================================================================================================================================= /
 * FUNCTION:    VEML6075_power_mode
 * Input:       _mode: seleziona lo stato (VEML6075_POWER_DOWN/VEML6075_ACTIVE)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision 
 *
 * Overview:    Abilita o disabilita il sensore
/* ============================================================================================================================================ */
signed char VEML6075_power_mode(unsigned char _mode){
    unsigned int _readed = 0;
    
    _readed = _VEML6075_I2C_1ByteAddr_2ByteData_read(VEML6075_ADDR, VEML6075_CONFIG_REG) >>8;   // leggo lo stato attuale del registro
    
    _readed &= ~VEML6075_POWER_DOWN;                                                    // cancello modalit attuale e mantengo gli altri bit
    _readed |= _mode;                                                                   // imposto nuovo stato
    
    return _VEML6075_I2C_1ByteAddr_2ByteData_write(VEML6075_ADDR, VEML6075_CONFIG_REG, (_readed << 8));    
}

/* ============================================================================================================================================= /
 * FUNCTION:    VEML6075_shot
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision
 *              -2: ACK non ricevuto
 *              -3: Write collision 
 *
 * Overview:    Imposta il trigger per l'acquisizione del dato ed attende che il dato sia stato acquisito
 * Nota:        Il sensore deve essere in modalit VEML6075_ONE_AQ
 *              Bloccante se il sensore non funziona correttamente
/* ============================================================================================================================================ */
signed char VEML6075_shot (void){
    unsigned int _readed = 0;
    unsigned char _return = 0;
    
    _readed = _VEML6075_I2C_1ByteAddr_2ByteData_read(VEML6075_ADDR, VEML6075_CONFIG_REG) >> 8;                      // leggo lo stato attuale del registro
    _readed |= VEML6075_UV_TRIG;                                                                                    // aggiungo il bit trigger
    _return = _VEML6075_I2C_1ByteAddr_2ByteData_write(VEML6075_ADDR, VEML6075_CONFIG_REG, (_readed <<8));           // scrivo il nuovo dato nel registro
    
     while ((_VEML6075_I2C_1ByteAddr_2ByteData_read(VEML6075_ADDR, VEML6075_CONFIG_REG) >> 8) & VEML6075_UV_TRIG);  // attendo finch il trigger resta a 1
     
     return _return;
}    

/* ============================================================================================================================================= /
 * FUNCTION:    VEML6075_read_uv_raw
 * Output:      struct contenente i valori letti dal sensore (uva, uvb, uv_comp_1, uv_comp_2)
 *
 * Overview:    Legge i valori acquisiti dal sensore
 * Note:        In caso di errori di comunicazione potrebbero essere salvati i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
/* ============================================================================================================================================ */
veml6075_uv_raw VEML6075_read_uv_raw (void) {
    veml6075_uv_raw _return;
    unsigned int _temp = 0;
    
    /* il sensore manda per primo l'LSB e per secondo l'MSB, la libreria I2C interpreta il dato come MSB + LSB, vanno quindi invertiti i 2 byte ricevuti */
    
    _temp = _VEML6075_I2C_1ByteAddr_2ByteData_read(VEML6075_ADDR, VEML6075_UVA_REG);    // leggo registro UVA
    _return.uva = ((_temp & 0xFF) << 8) | ((_temp & 0xFF00) >> 8);                      // inverto i 2 byte ricevuti
    _temp = _VEML6075_I2C_1ByteAddr_2ByteData_read(VEML6075_ADDR, VEML6075_UVB_REG);
    _return.uvb = ((_temp & 0xFF) << 8) | ((_temp & 0xFF00) >> 8);
    _temp = _VEML6075_I2C_1ByteAddr_2ByteData_read(VEML6075_ADDR, VEML6075_UVCOMP_1_REG);
    _return.uv_comp_1 = ((_temp & 0xFF) << 8) | ((_temp & 0xFF00) >> 8);
    _temp = _VEML6075_I2C_1ByteAddr_2ByteData_read(VEML6075_ADDR, VEML6075_UVCOMP_2_REG);
    _return.uv_comp_2 = ((_temp & 0xFF) << 8) | ((_temp & 0xFF00) >> 8);
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    VEML6075_convert_uv_raw
 * Input:       struct contenente i valori letti dal sensore (uva, uvb, uv_comp_1, uv_comp_2)
 * Output:      struct contenente i valori convertiti UVA in uW/cm^2, UVB in uW/cm^2, UVI (float)
 *
 * Overview:    Converte i dati raw in ingresso in valori UVA UVB e UVI 
 * Note:        Se prima di usare questa funzione non  stata usata la funzione VEML6075_set impostare prima i coefficienti di calcolo tramite
 *              funzione VEML6075_cal_responsivity (caso in cui il sensore  su un altro circuito)
 /* ============================================================================================================================================ */
veml6075_uv VEML6075_convert_uv_raw (veml6075_uv_raw _raw_input){
    veml6075_uv _return;
    float _uva_calc = 0;
    float _uvb_calc = 0;
    float _uvia = 0;
    float _uvib = 0;
    
    // Calcolo 
    // Calcolo suggerito da assistenza vishay
    _return.uva = (float) _raw_input.uva * veml6075_uva_resp;   
    _return.uvb = (float) _raw_input.uvb * veml6075_uvb_resp;
    // correzione 09/05/2017
//    _return.uva = (float) _raw_input.uva / veml6075_uva_resp;   
//    _return.uvb = (float) _raw_input.uvb / veml6075_uvb_resp;
 
/* nuovi calcoli UVI in base all'application note del 25/11/2016 */
    _uva_calc = (float)_raw_input.uva - ((float) VEML6075_A_COEF * (float)_raw_input.uv_comp_1) - ((float) VEML6075_B_COEF * (float)_raw_input.uv_comp_2);
    _uvb_calc = (float)_raw_input.uvb - ((float) VEML6075_C_COEF * (float)_raw_input.uv_comp_1) - ((float) VEML6075_D_COEF * (float)_raw_input.uv_comp_2);
    
    _uvia = (_uva_calc * (float) K1) * (float) VEML6075_UVI_UVA_RESPONSIVITY;                             // K1 e K2 sono fattori di calibrazione non potendoli calcolare sono impostati a 1
    _uvib = (_uvb_calc * (float) K2) * (float) VEML6075_UVI_UVB_RESPONSIVITY;
    
    _return.uvi = (_uvia + _uvib) / 2.0;
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    VEML6075_read_uv
 * Output:      struct contenente i valori convertiti UVA in uW/cm^2, UVB in uW/cm^2, UVI (float)
 *
 * Overview:    Legge i dati dal sensore e li converte in valori UVA UVB e UVI 
 /* ============================================================================================================================================ */
veml6075_uv VEML6075_read_uv (void) {
    veml6075_uv_raw _readed_raw;
    
    _readed_raw = VEML6075_read_uv_raw();
    return VEML6075_convert_uv_raw(_readed_raw);
}

/* ============================================================================================================================================= /
 * FUNCTION:    VEML6075_read_device_id
 * Output:      Device ID (unsigned int)
 *
 * Overview:    Legge l'id del dispositivo (0x0026) 
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
/* ============================================================================================================================================ */
unsigned char VEML6075_read_device_id (void){
    return _VEML6075_I2C_1ByteAddr_2ByteData_read(VEML6075_ADDR, VEML6075_DEVICEID_REG) >> 8;       // Ritrorno LSB (primo dato ricevuto)
}

#endif
