/* ============================================================================================================================================= /
 * LIBRARY:         Gestione HDC1080
 * FILE:            HDC1080.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         30/08/2016
 * UPDATED:         
 *
 * DESCRIP:         parametri e prototipi per libreria HDC1080.c
 *
/* ============================================================================================================================================= /
 *
/* CONTROLLI DI COMPILAZIONE ================================================================================================================== */

#ifndef I2C
    #error  Includere I2C_init.c
#endif

#ifdef HDC1080_I2C_MODULE_1                                                             // Ridefinisco le chiamate alle funzioni a seconda del modulo I2C usato
    #ifndef I2C_MODULE_1
        #error I2C_MODULE_1 non abilitato in I2C_init_settings.h
    #endif
    #define _HDC1080_I2C_1ByteAddr_2ByteData_write      I2C_1ByteAddr_2ByteData_write
    #define _HDC1080_I2C_1ByteAddr_2ByteData_read       I2C_1ByteAddr_2ByteData_read
    #define _HDC1080_I2C_1ByteAddr_2ByteData_read_delay I2C_1ByteAddr_2ByteData_read_delay

#elif defined HDC1080_I2C_MODULE_2
    #ifndef I2C_MODULE_2
        #error I2C_MODULE_2 non abilitato in I2C_init_settings.h
    #endif
    #define _HDC1080_I2C_1ByteAddr_2ByteData_write      I2C2_1ByteAddr_2ByteData_write
    #define _HDC1080_I2C_1ByteAddr_2ByteData_read       I2C2_1ByteAddr_2ByteData_read
    #define _HDC1080_I2C_1ByteAddr_2ByteData_read_delay I2C2_1ByteAddr_2ByteData_read_delay

#else
    #error Definire il modulo I2C da utilizzare per HDC1080
#endif

/* INDIRIZZI REGISTRI ========================================================================================================================= */
#define HDC1080_ADDR                    0x80                                            // indirizzo HDC1080

#define HDC1080_TEMP_REG                0x00
#define HDC1080_HUMID_REG               0x01
#define HDC1080_CONFIG_REG              0x02
#define HDC1080_SERIALID_REG            0xFB                                            // (+ 0xFC e 0xFD)
#define HDC1080_MANUFID_REG             0xFE
#define HDC1080_DEVICEID_REG            0xFF                

/* COSTANTI ================================================================================================================================== */

#define HDC1080_RESET                   0b10000000
#define HDC1080_HUMI_RES_14bit          0b00000000
#define HDC1080_HUMI_RES_11bit          0b00000001
#define HDC1080_HUMI_RES_8bit           0b00000010
#define HDC1080_TEMP_RES_14bit          0b00000000
#define HDC1080_TEMP_RES_11bit          0b00000100
#define HDC1080_SEQUENTIAL_AQ           0b00010000
#define HDC1080_ONE_AQ                  0b00000000
#define HDC1080_HEATER_ON               0b00100000
#define HDC1080_HEATER_OFF              0b00000000

/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file HDC1080.c
 * Libreria per la gestione del sensore di temperatura e umidit Texas Instruments HDC1080 via I2C
 * - Per un corretto funzionamento:
 *  + Includere la libreria I2C_init.c prima delle libreria HDC1080.c
 *  + Impostare il modulo I2C da utilizzare per l'HDC1080 in HDC1080_settings.h
 *  + Abilitare il modulo I2C utilizzato in I2C_init_settings.h
 *  + Avviare il modulo I2C nel programma principale richiamando la funzione I2C_init() o I2C2_init()
 * - Utilizzo:
 *  + Impostare il sensore in modalit acquisizione singola tramite la funzione HDC1080_set();
 *  + Leggere i dati del sensore tramite le apposite funzioni.
 * - Files:
 *  + HDC1080.c: Codice funzioni 
 *  + HDC1080.h: Parametri e prototipi
 *  + HDC1080_settings.h: Impostazioni specifiche per progetto
 */


/**
 * Resetta il sensore e cancella il config register
 * @param   void
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 */
signed char HDC1080_reset (void);

/**
 * Imposta la risoluzione della temperatura e dell'umidit e la modalit di acquisizione: 
 * @param   _mode: settaggio per la modalit di acquisizione (unsigned char):
 *              - HDC1080_SEQUENTIAL_AQ: acquisizione temperatura ed umidit in sequenza (non implementata a causa della libreria I2C, dovrei creare una funzione per leggere 4 byte in sequenza)
 *              - HDC1080_ONE_AQ: acquisizione singola
 * @param   _temp_res: settaggio per la risoluzione di acquisizione della temperatura (unsigned char):
 *              - HDC1080_TEMP_RES_14bit
 *              - HDC1080_TEMP_RES_11bit
 * @param   _humidity_res: settaggio per la risoluzione di acquisizione dell'umidit (unsigned char):
 *              - HDC1080_HUMI_RES_14bit
 *              - HDC1080_HUMI_RES_11bit
 *              - HDC1080_HUMI_RES_8bit
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 * @note    Implementato solo il funzionamento in modalit ONE AQ.
 */
signed char HDC1080_set(unsigned char _mode, unsigned char _temp_res, unsigned char _humidity_res);

/**
 * Accende o spegne l'elemento riscaldante del sensore
 * @param   _settings: settaggio per attivare o disattivare l'elemento riscaldante (unsigned char):
 *              - HDC1080_HEATER_ON: heater abilitato
 *              - HDC1080_HEATER_OFF: heater disabilitato
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 * @note    Se abilitato si accende solo durante le misurazioni.
 */
signed char HDC1080_set_heater (unsigned char _setting);

/**
 * Abilita il riscaldatore del sensore ed avvia un ciclo di lettura per asciugare il sensore.
 * @param   void
 * @return  void
 * @note    Bloccante per circa 1 secondo
 */
void HDC108_dry_sensor (void);

/**
 * Controlla il voltaggio di alimentazione
 * @param   void
 * @return  
 *          - 1: Batteria < 2.8V    
 *          - 0: Batteria > 2.8V    
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision 
 */
signed char HDC1080_read_battery_stat (void);

/**
 * Legge la temperatura e restituisce il valore convertito in gradi centigradi
 * @param   void
 * @return  temperatura letta (float)
 * @note    Non torna eventuali errori di comunicazione I2C
 */
float HDC1080_read_temp_float(void);

/**
 * Legge la temperatura e restituisce il valore letto dal registro
 * @param   void
 * @return  temperatura letta (unsigned int)
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 */
unsigned int HDC1080_read_temp_raw(void);

/**
 * Legge l'umidit e restituisce il valore convertito percentuale
 * @param   void
 * @return  umidit letta (float)
 * @note    Non torna eventuali errori di comunicazione I2C
 */
float HDC1080_read_humidity_float(void);

/**
 * Legge l'umidit e restituisce il valore letto dal registro
 * @param   void
 * @return  umidit letta (unsigned int)
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 */
unsigned int HDC1080_read_humidity_raw(void);

/**
 * Converte il dato letto della temperatura in gradi centigradi
 * @param   _readed: dato letto dal registro da convertire (unsigned int)
 * @return  temperatura convertita in gradi centigradi (float)
 */
float HDC1080_temp_raw_2_float (unsigned int _readed);

/**
 * Converte Converte il dato letto dell'umidit in percentuale
 * @param   _readed: dato letto dal registro da convertire (unsigned int)
 * @return  umidit convertita in percentuale (float)
 */
float HDC1080_humidity_raw_2_float(unsigned int _readed);

/**
 * Legge l'id del costruttore (0x5449)
 * @param   void
 * @return  Manufacturer ID (unsigned int)
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 */
unsigned int HDC1080_read_manuf_id (void);

/**
 * Legge l'id del dispositivo (0x1050)
 * @param   void
 * @return  Device ID (unsigned int)
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 */
unsigned int  HDC1080_read_device_id (void);