/* ============================================================================================================================================= /
 * LIBRARY:         I2C_init.c
 * FILE:            I2C_init.h
 * LAST REV:        2.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         17/03/2013
 * UPDATED:         07/08/2016
 *
 * DESCRIP:         Parametri e prototipi per libreria I2C_init.c
 *
 /* =========================================================================================================================================== */

/* CONTROLLI DI COMPILAZIONE ================================================================================================================== */
#ifndef XTAL_FREQ
    #error Definire XTAL_FREQ
#endif

#ifdef I2C_MODULE_1
    #if defined (_18F4550) || defined (_18F2550)
        #define I2C_SCL_TRIS TRISBbits.TRISB1
        #define I2C_SDA_TRIS TRISBbits.TRISB0
    #elif defined (_18F46K22) || defined (_18F26K22) || defined (_18LF26K22)|| defined (_18F67K22)
        #define I2C_SCL_TRIS TRISCbits.TRISC3
        #define I2C_SDA_TRIS TRISCbits.TRISC4
    #elif defined _12F1840
        #define I2C_SCL_TRIS TRISAbits.TRISA1
        #define I2C_SDA_TRIS TRISAbits.TRISA2     
    #else
        #error PIC non Gestito. Definire costanti PIN di uscita per il nuovo PIC in I2C_init.h
    #endif
#endif

#ifdef I2C_MODULE_2
    #ifdef _18F46K22
        #define I2C2_SCL_TRIS TRISDbits.TRISD0
        #define I2C2_SDA_TRIS TRISDbits.TRISD1
    #elif defined _18F26K22 || defined (_18LF26K22)
        #define I2C2_SCL_TRIS TRISBbits.TRISB1
        #define I2C2_SDA_TRIS TRISBbits.TRISB2
    #elif defined _18F67K22
        #define I2C2_SCL_TRIS TRISDbits.TRISD6
        #define I2C2_SDA_TRIS TRISDbits.TRISD5
    #elif defined (_18F4550) || defined (_18F2550) ||  defined (_12F1840)
        #error il PIC selezionato non ha il secondo modulo I2C
    #else
        #error PIC non Gestito. Definire costanti PIN di uscita per il nuovo PIC in I2C_init.h
    #endif

    #ifdef __18F67K22
        #define SSP2IF  PIR2bits.SSP2IF
    #else
        #define SSP2IF  PIR3bits.SSP2IF
    #endif
#endif



/* COSTANTI =============== ================================================================================================================== */
//#define   SLAVE_7_BIT_ADDRESS   	0b00000110     	
//#define   SLAVE_10_BIT_ADDRESS  	0b00000111    	
#define     MASTER                    0b00001000   

#define     SLEW_ON                 0b00000000 
#define     SLEW_OFF                0b10000000 

/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file I2C_init.c
 * Libreria per la gestione del bus I2C primario e secondario (se disponibile). Al momento sono state sviluppate solo le funzionalit  per dispositivo MASTER.
 * - Per un corretto funzionamento:
 *  + Definire la frequenza del clock in MHz (XTAL_FREQ)
 *  + Impostare i moduli I2C abilitati in I2C_init_settings.h
 *  + Impostare il modulo I2C specifico per il dispositivo nei settaggi dei vari dispositivi
 * - Files:
 *  + I2C_init.c: Codice funzioni
 *  + I2C_init.h: Parametri e prototipi
 *  + I2C_init_settings.h: Impostazioni specifiche per progetto
 */

/**
 * Imposta il protocollo I2C e calcola SSPADD per il modulo I2C primario. Per ora gestito solo come master
 * @param   _baud_rate: indicare il baudrate espresso in KHz (unsigned int)
 * @return  void
 */
void I2C_init_master(unsigned int _baud_rate);

/**
 * Imposta il protocollo I2C e calcola SSPADD per il modulo I2C secondario. Per ora gestito solo come master
 * @param   _baud_rate: indicare il baudrate espresso in KHz (unsigned int)
 * @return  void
 */
void I2C2_init_master(unsigned int _baud_rate);

/**
 * Abilita il modulo I2C primario
 * @param   _sync_mode: indicare il la modalit , per ora solo MASTER (unsigned char)
 * @param   _slew:
 *              - SLEW_ON: slew rate on
 *              - SLEW_OFF: slew rate off
 * @return  void
 */
void I2C_open(unsigned char _sync_mode, unsigned char _slew);

/**
 * Abilita il modulo I2C secondario
 * @param   _sync_mode: indicare il la modalit , per ora solo MASTER (unsigned char)
 * @param   _slew:
 *              - SLEW_ON: slew rate on
 *              - SLEW_OFF: slew rate off
 * @return  void
 */
void I2C2_open(unsigned char _sync_mode, unsigned char _slew);

/**
 * Attende che il bus del modulo I2C primario sia libero
 * @param   void
 * @return  void
 */
void I2C_idle(void);

/**
 * Attende che il bus del modulo I2C secondario sia libero
 * @param   void
 * @return  void
 */
void I2C2_idle(void);

/**
 * Invia il bit di start sul modulo I2C primario ed attende che venga ricevuto 
 * @param   void
 * @return  void
 */
void I2C_start(void);

/**
 * Invia il bit di start sul modulo I2C secondario ed attende che venga ricevuto 
 * @param   void
 * @return  void
 */
void I2C2_start(void);

/**
 * Riavvia il modulo I2C primario ed attende che il bit venga ricevuto
 * @param   void
 * @return  void
 */
void I2C_restart(void);

/**
 * Riavvia il modulo I2C secondario ed attende che il bit venga ricevuto
 * @param   void
 * @return  void
 */
void I2C2_restart(void);

/**
 * Invia il bit di stop sul modulo I2C primario ed attende che venga ricevuto 
 * @param   void
 * @return  void
 */
void I2C_stop(void);

/**
 * Invia il bit di stop sul modulo I2C secondario ed attende che venga ricevuto 
 * @param   void
 * @return  void
 */
void I2C2_stop(void);

/**
 * Invia ACK sul modulo I2C primario ed attende che venga ricevuto
 * @param   void
 * @return  void
 */
void I2C_ack(void);

/**
 * Invia ACK sul modulo I2C secondario ed attende che venga ricevuto
 * @param   void
 * @return  void
 */
void I2C2_ack(void);

/**
 * Invia NOT ACK sul modulo I2C primario ed attende che venga ricevuto
 * @param   void
 * @return  void
 */
void I2C_not_ack(void);

/**
 * Invia NOT ACK sul modulo I2C secondario ed attende che venga ricevuto
 * @param   void
 * @return  void
 */
void I2C2_not_ack(void);

/**
 * Invia un byte sul BUS I2C primario
 * @param   _data_out: byte da inviare (unsigned char)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision      
 */
signed char I2C_write(unsigned char _data_out);

/**
 * Invia un byte sul BUS I2C secondario
 * @param   _data_out: byte da inviare (unsigned char)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision          
 */
signed char I2C2_write(unsigned char _data_out);

/**
 * Riceve un byte sul BUS I2C primario
 * @param   void
 * @return  byte ricevuto (unsigned char)
 */
unsigned char I2C_read(void);

/**
 * Riceve un byte sul BUS I2C secondario
 * @param   void
 * @return  byte ricevuto (unsigned char)
 */
unsigned char I2C2_read(void);

/**
 * Legge registri a 1 byte da dispositivi I2C con indirizzi dei registri a 1 byte (modulo I2C 1)
 * @param   _control: indirizzo del dispositivo I2C (unsigned char)
 * @param   _address: registro da leggere (unsigned char)
 * @return  byte ricevuto (unsigned char) 
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 */
unsigned char I2C_1ByteAddr_1ByteData_read (unsigned char _control, unsigned char _address);

/**
 * Legge registri a 1 byte da dispositivi I2C con indirizzi dei registri a 1 byte (modulo I2C 2)
 * @param   _control: indirizzo del dispositivo I2C (unsigned char)
 * @param   _address: registro da leggere (unsigned char)
 * @return  byte ricevuto (unsigned char)
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1) 
 */
unsigned char I2C2_1ByteAddr_1ByteData_read (unsigned char _control, unsigned char _address);

/**
 * Scrive registri a 1 byte su dispositivi I2C con indirizzi dei registri a 1 byte (modulo I2C 1)
 * @param   _control: indirizzo del dispositivo I2C (unsigned char)
 * @param   _address: registro da scrivere (unsigned char)
 * @param   _data: byte da scrivere (unsigned char)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision            
 */
signed char I2C_1ByteAddr_1ByteData_write(unsigned char _control, unsigned char _address, unsigned char _data);

/**
 * Scrive registri a 1 byte su dispositivi I2C con indirizzi dei registri a 1 byte (modulo I2C 2)
 * @param   _control: indirizzo del dispositivo I2C (unsigned char)
 * @param   _address: registro da scrivere (unsigned char)
 * @param   _data: byte da scrivere (unsigned char)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision             
 */
signed char I2C2_1ByteAddr_1ByteData_write(unsigned char _control, unsigned char _address, unsigned char _data);

/**
 * Legge registri a 1 byte da dispositivi I2C con indirizzi dei registri a 2 byte (modulo I2C 1)
 * @param   _control: indirizzo del dispositivo I2C (unsigned char)
 * @param   _address: registro da leggere (unsigned int)
 * @return  byte ricevuto (unsigned char) 
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 */
unsigned char I2C_2ByteAddr_1ByteData_read (unsigned char _control, unsigned int _address);

/**
 * Legge registri a 1 byte da dispositivi I2C con indirizzi dei registri a 2 byte (modulo I2C 2)
 * @param   _control: indirizzo del dispositivo I2C (unsigned char)
 * @param   _address: registro da leggere (unsigned int)
 * @return  byte ricevuto (unsigned char) 
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 */
unsigned char I2C2_2ByteAddr_1ByteData_read (unsigned char _control, unsigned int _address);

/**
 * Scrive registri a 1 byte su dispositivi I2C con indirizzi dei registri a 2 byte (modulo I2C 1)
 * @param   _control: indirizzo del dispositivo I2C (unsigned char)
 * @param   _address: registro da scrivere (unsigned int)
 * @param   _data: byte da scrivere (unsigned char)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision            
 */
signed char I2C_2ByteAddr_1ByteData_write (unsigned char _control, unsigned int _address, unsigned char _data);

/**
 * Scrive registri a 1 byte su dispositivi I2C con indirizzi dei registri a 2 byte (modulo I2C 2)
 * @param   _control: indirizzo del dispositivo I2C (unsigned char)
 * @param   _address: registro da scrivere (unsigned int)
 * @param   _data: byte da scrivere (unsigned char)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision            
 */
signed char I2C2_2ByteAddr_1ByteData_write (unsigned char _control, unsigned int _address, unsigned char _data);

/**
 * Legge registri a 2 byte da dispositivi I2C con indirizzi dei registri a 1 byte (modulo I2C 1)
 * @param   _control: indirizzo del dispositivo I2C (unsigned char)
 * @param   _address: registro da leggere (unsigned char)
 * @return  2 byte ricevuti (unsigned int) 
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 *          MSB primo dato letto
 */
unsigned int  I2C_1ByteAddr_2ByteData_read (unsigned char _control, unsigned char _address);

/**
 * Legge registri a 2 byte da dispositivi I2C con indirizzi dei registri a 1 byte (modulo I2C 2)
 * @param   _control: indirizzo del dispositivo I2C (unsigned char)
 * @param   _address: registro da leggere (unsigned char)
 * @return  2 byte ricevuti (unsigned int) 
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 *          MSB primo dato letto
 */
unsigned int  I2C2_1ByteAddr_2ByteData_read (unsigned char _control, unsigned char _address);

/**
 * Legge registri a 2 byte da dispositivi I2C con indirizzi dei registri a 1 byte con delay tra indirizzo e lettura del dato (HDC1080) (modulo I2C 1)
 * @param   _control: indirizzo del dispositivo I2C (unsigned char)
 * @param   _address: registro da leggere (unsigned char)
 * @return  2 byte ricevuti (unsigned int) 
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 *          MSB primo dato letto
 */
unsigned int  I2C2_1ByteAddr_2ByteData_read_delay (unsigned char _control, unsigned char _address);

/**
 * Legge registri a 2 byte da dispositivi I2C con indirizzi dei registri a 1 byte con delay tra indirizzo e lettura del dato (HDC1080) (modulo I2C 2)
 * @param   _control: indirizzo del dispositivo I2C (unsigned char)
 * @param   _address: registro da leggere (unsigned char)
 * @return  2 byte ricevuti (unsigned int) 
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 *          MSB primo dato letto
 */
unsigned int  I2C2_1ByteAddr_2ByteData_read_delay (unsigned char _control, unsigned char _address);

/**
 * Scrive registri a 2 byte su dispositivi I2C con indirizzi dei registri a 2 byte (modulo I2C 1)
 * @param   _control: indirizzo del dispositivo I2C (unsigned char)
 * @param   _address: registro da scrivere (unsigned char)
 * @param   _data: byte da scrivere (unsigned int)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision           
 * @note    MSB primo dato scritto
 */
signed char I2C_1ByteAddr_2ByteData_write (unsigned char _control, unsigned char _address, unsigned int _data);

/**
 * Scrive registri a 2 byte su dispositivi I2C con indirizzi dei registri a 2 byte (modulo I2C 2)
 * @param   _control: indirizzo del dispositivo I2C (unsigned char)
 * @param   _address: registro da scrivere (unsigned char)
 * @param   _data: byte da scrivere (unsigned int)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision       
 * @note    MSB primo dato scritto     
 */
signed char I2C2_1ByteAddr_2ByteData_write (unsigned char _control, unsigned char _address, unsigned int _data);