/* ============================================================================================================================================= /
 * LIBRARY:         Decodifica DCF77
 * FILE:            DCF77.c
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         28/09/2016
 * UPDATED:         
 *
 * DESCRIP:         Decodifica del segnale radio orario DCF77
 *
 * FUNCTIONS:       - DCF77_init ()
 *                  - DCF77_search_minute_mark()
 *                  - DCF77_receive_data()
 *                  - DCF77_decode_data()
 *                  - DCF77_start_receive_via_interrupt()
 *                  - DCF77_stop_receive_via_interrupt()
 *                  - DCF77_receive_via_interrupt()=
 *            
/* ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.0               03/10/2016          - Prima release completa
  1.0               27/09/2016          - Inizio sviluppo
//! [History]
/* ============================================================================================================================================ */

#ifndef DCF77
#define DCF77

/* VARIABILI GLOBALI LIBRERIA ================================================================================================================= */
/** \brief buffer di ricezione */
unsigned char _dcf77_buffer[60];
/** \brief dato ricevuto */
bit dcf77_data_ready = 0;
/** \brief stato sincronizzazione */
bit dcf77_sync = 0;
/** \brief a 1 per 200ms ad ogni secondo */
bit dcf77_seconds_pulse = 0;

/** \brief struttura contenente i dati decodificati */
typedef struct {
    unsigned char hours;                // ora
    unsigned char minutes;              // minuto
    unsigned char day;                  // giorno mese
    unsigned char month;                // mese
    unsigned char year;                 // anno
    unsigned char day_w;                // giorno settimana
    unsigned char flag_bits;            // bit 000-A2-Z2-Z1-A1-R     R: abnormal transmitter, A1: summertime annuncement (1 hour before change), Z1: CEST in effect, Z2: CET in effect, A2 Leap sedond annunc.
    unsigned int  meteo;                // pacchetto meteo criptato
}dcf77_decoded_data;

/* INCLUDES =================================================================================================================================== */
#include    "DCF77_settings.h"
#include    "DCF77.h"
#include    <xc.h>

/* ============================================================================================================================================= /
 * FUNCTION:    DCF77_init
 *
 * Overview:    Inizializzazione ed avvio del modulo, da usare solo se si usa il modulo DCF77 tramite funzioni e con pin di abilitazione
/* ============================================================================================================================================ */
void DCF77_init(void){
    #ifdef DCF77_PDN
        DCF77_PDN = DCF77_OFF;
        Delay_ms(20);
        DCF77_PDN = DCF77_ON;
        Delay_ms(800);
    #endif
}

/* ============================================================================================================================================= /
 * FUNCTION:    DCF77_search_minute_mark
 * Output:      1: Inizio trasmissione trovato
 *
 * Overview:    Attende il minute mark nella trasmissione, quando lo trova torna 1
/* ============================================================================================================================================ */
unsigned char DCF77_search_minute_mark(void){
    unsigned char _counter = 0;
    
    while (_counter < 150) {                                                            // 150 *10 = 1.5s
        if (DCF77_SIGNAL == DCF77_SIGNAL_IDLE)
            _counter++;
        else
            _counter = 0;
        
        Delay_ms(10);
    }
    return 1;
}

/* ============================================================================================================================================= /
 * FUNCTION:    DCF77_receive_data
 *
 * Overview:    Riceve il dato (58 secondi) e lo salva nell'array _dcf77_buffer, decodificare il dato tramite la funzione  DCF77_decode_data per 
 *              salvarlo in una struct di variabili
/* ============================================================================================================================================ */
// a 130 ms controlla il segnale per 5 volte ogni 10 ms e conta quanto sta alto
void DCF77_receive_data(void){
    unsigned char _dcf77_signal_check = 0;
    
    for (unsigned char _i = 0; _i<59; _i++) {
        
        while(DCF77_SIGNAL == DCF77_SIGNAL_IDLE);                                       // attende che il segnale vada alto
        
        Delay_ms(120);                                                                  // attesa 120ms
        
        for (unsigned char _i = 0; _i < 5; _i++ ){                                      // controllo il segnale ogni 10 ms per 5 volte 130~170ms
            Delay_ms(10);
            if (DCF77_SIGNAL == DCF77_SIGNAL_ACTIVE)
                _dcf77_signal_check++;
        }

        if (_dcf77_signal_check > 3){                                                   // se il segnale  stato attivo per 4 o pi volte
            _dcf77_buffer[_i] = 1;                                                      // salvo 1
        }
        else
            _dcf77_buffer[_i] = 0;                                                      // altrimenti salvo 0
        
        _dcf77_signal_check = 0;
        
        if (_i < 58) {                                                                  // tutti i bit tranne l'ultimo aggiungo un delay per aspettare il nuovo secondo. l'ultimo si rialiinea con il minute mark
            Delay_ms(800);                                                              // 800+170 = 970ms
        }
    }  
}

/* ============================================================================================================================================= /
 * FUNCTION:    DCF77_decode_data
 * Output:      restituisce una struct di variabili contenente i dati decodificati (dcf77_decoded_data)
 *
 * Overview:    Decodifica il dato contenuto nell'array di ricezione _dcf77_buffer, effettua i controlli di parit e valori massimi su ore,
 *              minuti e data. 
 * Note:        Se la parit non  corretta imposta il valore a F1.
 *              Se il dato  fuori dai valori massimi imposta il valore a F0.
/* ============================================================================================================================================ */
dcf77_decoded_data DCF77_decode_data(void) {
    
    dcf77_decoded_data _data;
    
    if ((_dcf77_buffer[0] == 0) && (_dcf77_buffer[20] == 1)){                           // proseguo con la decodifica solo se questi 2 bit sono corretti altrimenti torno tutto a 0
        if ((_dcf77_buffer[21]^_dcf77_buffer[22]^_dcf77_buffer[23]^_dcf77_buffer[24]^_dcf77_buffer[25]^_dcf77_buffer[26]^_dcf77_buffer[27]^_dcf77_buffer[28]) == 0) {                     // controllo parit pari (deve essere 0) per il dato dei minuti
            _data.minutes = (_dcf77_buffer[21] | _dcf77_buffer[22]<<1 | _dcf77_buffer[23]<<2 | _dcf77_buffer[24]<<3 | _dcf77_buffer[25]<<4 | _dcf77_buffer[26]<<5 | _dcf77_buffer[27]<<6);// se corretto salvo i minuti 
            if (_data.minutes > 0x59)
                _data.minutes == 0x88;
        }
        else                                                                                                                                                                              // altrimenti scrivo 0x99
            _data.minutes = 0x99;

        if ((_dcf77_buffer[29]^_dcf77_buffer[30]^_dcf77_buffer[31]^_dcf77_buffer[32]^_dcf77_buffer[33]^_dcf77_buffer[34]^_dcf77_buffer[35]) == 0) {                                       // controllo parit pari (deve essere 0) per il dato delle ore  
            _data.hours = (_dcf77_buffer[29] | _dcf77_buffer[30]<<1 | _dcf77_buffer[31]<<2 | _dcf77_buffer[32]<<3 | _dcf77_buffer[33]<<4 | _dcf77_buffer[34]<<5);                         // se corretto salvo le ore
            if (_data.hours > 0x23)
                _data.hours = 0x88;
        }
        else
            _data.hours = 0x99;

       if ((_dcf77_buffer[36]^_dcf77_buffer[37]^_dcf77_buffer[38]^_dcf77_buffer[39]^_dcf77_buffer[40]^_dcf77_buffer[41]^_dcf77_buffer[42]^_dcf77_buffer[43]^_dcf77_buffer[44]^_dcf77_buffer[45]^_dcf77_buffer[46]^_dcf77_buffer[47]^
            _dcf77_buffer[48]^_dcf77_buffer[49]^_dcf77_buffer[50]^_dcf77_buffer[51]^_dcf77_buffer[52]^_dcf77_buffer[53]^_dcf77_buffer[54]^_dcf77_buffer[55]^_dcf77_buffer[56]^_dcf77_buffer[57]^_dcf77_buffer[58]) == 0){             // controllo parit pari (deve essere 0) per il dato della data
            _data.day = (_dcf77_buffer[36] | _dcf77_buffer[37]<<1 | _dcf77_buffer[38]<<2 | _dcf77_buffer[39]<<3 | _dcf77_buffer[40]<<4 | _dcf77_buffer[41]<<5);                                                                       // se corretto salvo i dati
            _data.day_w = (_dcf77_buffer[42]|_dcf77_buffer[43]<<1|_dcf77_buffer[44]<<2);
            _data.month = (_dcf77_buffer[45] | _dcf77_buffer[46]<<1 | _dcf77_buffer[47]<<2 | _dcf77_buffer[48]<<3 | _dcf77_buffer[49]<<4);
            _data.year = (_dcf77_buffer[50] | _dcf77_buffer[51]<<1 | _dcf77_buffer[52]<<2 | _dcf77_buffer[53]<<3 | _dcf77_buffer[54]<<4 | _dcf77_buffer[55]<<5 | _dcf77_buffer[56]<<6 | _dcf77_buffer[57]<<7);
            
            if (_data.day > 0x31)
                _data.day = 0xF0;
            if (_data.day_w > 0x07)
                _data.day_w = 0xF0;
            if (_data.month > 0x12)
                _data.month = 0xF0;
            if (_data.year > 0x99)
                _data.year = 0xF0;
       } else {
            _data.day = 0xF1;
            _data.month = 0xF1;
            _data.day_w = 0xF1;
            _data.year = 0xF1;
       }

        _data.flag_bits =  (_dcf77_buffer[15] | _dcf77_buffer[16]<<1 | _dcf77_buffer[17] <<2 | _dcf77_buffer[18] <<3 | _dcf77_buffer[19] << 4) &0b00011111;       // salvo i flag bits

        for (unsigned char _i=1; _i<15; _i++) {
            _data.meteo = _data.meteo <<1;
            _data.meteo |=  _dcf77_buffer[_i];
        }
    
    }
    return _data;                                                                       // torno la struct
}

/* ============================================================================================================================================= /
 * FUNCTION:    DCF77_start_receive_via_interrupt
 *
 * Overview:    Abilita interrupt e relativo timer di ricezione. 
 *              - I dati vengono ricevuti in interrupt e salvati nel buffer di ricezione _dcf77_buffer
 *              - A sincronizzazione avvenuta viene settata la variabile globale dcf77_sync 
 *              - A ricezione completata viene settata la variabile dcf77_data_ready
 *              - Ad ogni bit ricevuto viene settata la variabile globale dcf77_seconds_pulse (pulsa ad ogni secondo) 
/* ============================================================================================================================================ */
void DCF77_start_receive_via_interrupt(void){

    /* INTERRUPT GLOBALE e PRIORITA'*/
    RCONbits.IPEN = 0;
    INTCONbits.GIEH = 0;
    INTCONbits.GIEL = 0;
    
    #ifdef DCF77_PDN
        DCF77_PDN = DCF77_OFF;
        Delay_ms(20);
        DCF77_PDN = DCF77_ON;
        Delay_ms(800);
    #endif
    
    DCF77_set_timer();

/* INTERRUPT on selected edge RBx */
    
#if defined DCF77_ACTIVE_HIGH                                                           // se DCF77 attivo alto
    DCF77_INT_EDGE = 1;                                                                 // interrupt su rising edge
#elif defined DCF77_ACTIVE_LOW                                                          // se DCF77 attivo basso
    DCF77_INT_EDGE = 0;                                                                 // interrupt su falling edge
#endif
    
#if (DCF77_SIGNAL_RB != 0)
    DCF77_INT_PRIOROIRY = 1;
#endif

    DCF77_INT_IF = 0;
    DCF77_INT_IE = 1; 
    
    RCONbits.IPEN = 1;
    INTCONbits.GIEH = 1;
    INTCONbits.GIEL = 1;
}   

/* ============================================================================================================================================= /
 * FUNCTION:    DCF77_stop_receive_via_interrupt
 *
 * Overview:    Disabilita interrupt e relativo timer di ricezione. (attenzione, disabilita tutti gli interrupt on change su PORTB)
/* ============================================================================================================================================ */
void DCF77_stop_receive_via_interrupt(void){
    #ifdef DCF77_PDN
        DCF77_PDN = 1;
    #endif 
    
    dcf77_data_ready = 0;
    dcf77_sync = 0;    
    dcf77_seconds_pulse = 0;
    
    DCF77_INT_IE = 0;
    DCF77_INT_IF = 0;
    DCF77_TIMER_INT_IE = OFF;
    DCF77_TIMER_ON = OFF;
}

/* ============================================================================================================================================= /
 * FUNCTION:    DCF77_receive_via_interrupt
 *
 * Overview:    Routine di interrupt per la ricezione dei dati DCF77, se necessario commentare e spostare nel main del programma.
 *              - I dati vengono salvati nel buffer di ricezione _dcf77_buffer 
 *              - A sincronizzazione avvenuta viene settata la variabile globale dcf77_sync 
 *              - A ricezione completata viene settata la variabile dcf77_data_ready (resettare la variabile una volta letto il dato per evitare doppie letture)
 *              - Ad ogni bit ricevuto viene settata la variabile globale dcf77_seconds_pulse (pulsa ad ogni secondo) 
/* ============================================================================================================================================ */
//    volatile static unsigned int _dcf77_timer_counter = 0;
//    volatile static unsigned char _dcf77_bit_counter = 0;
//    volatile static unsigned char _dcf77_signal_check = 0;
//interrupt high_priority void DCF77_receive_via_interrupt(void) {
//     
//    if (DCF77_INT_IE && DCF77_INT_IF){
//        if (_dcf77_timer_counter > 185) {                                               // minute mark con 185 va benissimo
//                _dcf77_timer_counter = 0;
//                DCF77_TMR = DCF77_TIMER_PRELOAD;
//                _dcf77_bit_counter = 0;   
//                dcf77_sync = 1;
//                dcf77_seconds_pulse = 1;
//                DCF77_INT_IE = 0;                                                       // disabilito interrupt INT
//        } else if ((_dcf77_timer_counter >= 90) && (_dcf77_timer_counter <= 110) ){     // impulso ad inizio secondo valido
//            _dcf77_timer_counter = 0;
//            DCF77_TMR = DCF77_TIMER_PRELOAD;
//            _dcf77_bit_counter++;
//            dcf77_seconds_pulse = 1;
//            DCF77_INT_IE = 0;                                                           // disabilito interrupt INT
//        } 
//        
//        DCF77_TIMER_ON = ON;                                                            // avvio timer (dopo primo interrupt)
//        DCF77_TIMER_INT_IF = 0;
//        DCF77_TIMER_INT_IE= ON;
//        DCF77_INT_IF = 0;
//    }
//      
//    else if (DCF77_TIMER_INT_IE && DCF77_TIMER_INT_IF) {
//        DCF77_TMR = DCF77_TIMER_PRELOAD;                                                // resetto il preload
//        
//        _dcf77_timer_counter++;  
//        
//        if (dcf77_seconds_pulse == 1){                                                  // se ho gi un impulso ad inizio secondo valido, se arriva un impulso prima di 130ms non viene considerato e il timer continua
//            if ((_dcf77_timer_counter >=13) && (_dcf77_timer_counter <=17)) {           // controllo il segnale a 130ms, 140ms, 150ms, 160ms, 170ms,
//                if (DCF77_SIGNAL == DCF77_SIGNAL_ACTIVE)                                // e ne salvo il valore
//                    _dcf77_signal_check++;
//            } else if (_dcf77_timer_counter == 18) {
//                                           
//                if (_dcf77_signal_check > 3)                                            // se ho trovato piu 1 salvo 1 altrimenti 0
//                    _dcf77_buffer[_dcf77_bit_counter] = 1;    
//                else
//                    _dcf77_buffer[_dcf77_bit_counter] = 0;    
//
//                dcf77_seconds_pulse = 0;
//                _dcf77_signal_check = 0;                                                
//                
//                if (_dcf77_bit_counter == 58 && dcf77_sync) {                           // se il contatore  arrivato a 58 ed il minute_mark  stato trovato almeno una volta
//                    dcf77_data_ready = 1;                                               // imposto data ready
//                    dcf77_sync = 0;                                                     // azzero per attesa nuovo minute mark, serve anche in caso di second leap
//                }
//            }
//        } else {                                                                        
//            if (_dcf77_timer_counter == 85){                                            // Abilito interrupt INT in attesa del secondo
//                DCF77_INT_IF = 0;
//                DCF77_INT_IE = 1; 
//            }
//        }
//        DCF77_TIMER_INT_IF = 0;
//    }



#endif          