/* ============================================================================================================================================= /
 * LIBRARY:         EEPROM I2C
 * FILE:            EEPROM_I2C.c
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         13/08/2016
 * UPDATED:         
 *
 * DESCRIP:         Gestione EEPROM I2C, Compatibile con EEPROM con indirizzi a 2 byte (24x32...)
 *
 * PRECONDITIONS:   - inizializzare il protocollo I2C tramite libreria I2C_init.c
 * FUNCTIONS:       - EEPROM_I2C_read()
 *                  - EEPROM_I2C_write()
 *                  - EEPROM_I2C_write_e_check()
 *                  - EEPROM_I2C_read_int()
 *                  - EEPROM_I2C_write_int()
 *                  - EEPROM_I2C_write_int_e_check()        
/* ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.0               13/08/2016          - Prima release
//! [History]
/* ============================================================================================================================================ */

#ifndef EEPROM_I2C
#define EEPROM_I2C

/* INCLUDES =================================================================================================================================== */
#include "EEPROM_I2C_settings.h"
#include "EEPROM_I2C.h"
#include <xc.h>      

/* ============================================================================================================================================= /
 * FUNCTION:    EEPROM_I2C_read
 * Input:       _dev_addr: indirizzo impostato sui pin A0-A2 (unsigned char)
 *              _addr: indirizzo da leggere (unsigned int)
 * Output       dato letto all'indirizzo richiesto (unsigned char)
 *
 * Overview:    Legge il dato all'indirizzo EEprom richiesto e torna il valore
 * Note:        In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
/* ============================================================================================================================================ */
unsigned char EEPROM_I2C_read (signed char _dev_addr, unsigned int _addr){
    
    _dev_addr &= 0b00000111;                                                            // maschero solo i 3 bit utili
    _dev_addr = _dev_addr <<1;                                                          // sposto a sinistra di 1 bit
    
    return _EEPROM_I2C_2ByteAddr_1ByteData_read((EEPROM_I2C_ADDR | _dev_addr), _addr);
}

/* ============================================================================================================================================= /
 * FUNCTION:    EEPROM_I2C_write
 * Input:       _dev_addr: indirizzo impostato sui pin A0-A2 (unsigned char)
 *              _addr: indirizzo da scrivere (unsigned int)
 *              _data: dato da scrivere (unsigned char)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision  
 *              -2: ACK non ricevuto
 *              -3: Write collision 
 *
 * Overview:    Scrive il dato all'indirizzo EEprom richiesto
/* ============================================================================================================================================ */
signed char EEPROM_I2C_write (signed char _dev_addr, unsigned int _addr, unsigned char _data){
    
    _dev_addr &= 0b00000111;                                                            // maschero solo i 3 bit utili
    _dev_addr = _dev_addr <<1;                                                          // sposto a sinistra di 1 bit
    
    return _EEPROM_I2C_2ByteAddr_1ByteData_write((EEPROM_I2C_ADDR | _dev_addr), _addr, _data);
}

/* ============================================================================================================================================= /
 * FUNCTION:    EEPROM_I2C_write_e_check
 * Input:       _dev_addr: indirizzo impostato sui pin A0-A2 (unsigned char)
 *              _addr: indirizzo da scrivere (unsigned int)
 *              _data: dato da scrivere (unsigned char)
 * Output:      0: Dato scritto correttamente
 *              -1: Bus collision  
 *              -2: ACK non ricevuto
 *              -3: Write collision 
 *              -4: Dato da salvare e dato salvato non corrispondono
 *
 * Overview:    Scrive il dato all'indirizzo EEprom richiesto e verifica che sia stato scritto correttamente
/* ============================================================================================================================================ */
signed char EEPROM_I2C_write_e_check (signed char _dev_addr, unsigned int _addr, unsigned char _data){
    signed char _return = 0;
    unsigned char _data_read = 0;

    _return = EEPROM_I2C_write(_dev_addr, _addr, _data);                                // scrivo il dato e salvo il risultato
    
    if (_return == 0) {                                                                 // se non ci sono errori in scrittura
        Delay_ms(10);                                                                   // ritardo per attendere fine scrittura
        _data_read = EEPROM_I2C_read(_dev_addr, _addr);                                 // rileggo il dato
        
        if (_data_read == _data)                                                        // se dato letto e dato inviato sono uguali
            _return = 0;                                                                // torno 1
        else
            _return = -4;                                                               // se sono diversi torno -4
        
    }
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    EEPROM_I2C_read_int
 * Input:       _dev_addr: indirizzo impostato sui pin A0-A2 (unsigned char)
 *              _addr: indirizzo da leggere (unsigned int)
 * Output       dato letto in _addr e _addr+1 (unsigned int)
 *
 * Overview:    Legge il dato all'indirizzo EEprom richiesto e torna il valore valore intero letto da 2 indirizzi consecutivi
 * Note:        Non torna eventuali errori di comunicazione I2C
/* ============================================================================================================================================ */
unsigned int EEPROM_I2C_read_int(unsigned char _dev_addr, unsigned int _addr) {
    unsigned int _ret_value;
    unsigned char _high;
    unsigned char _low;

    _high = EEPROM_I2C_read(_dev_addr, _addr);                                          // leggo il byte alto
    _low = EEPROM_I2C_read(_dev_addr, _addr+1);                                         // leggo il byte basso

    _ret_value = _high;                                                                 // metto _high nel byte alto di _ret_value
    _ret_value = _ret_value <<8;
    _ret_value = _ret_value + _low;                                                     // aggiungo _low nel byte basso

    return _ret_value;
}

/* ============================================================================================================================================= /
 * FUNCTION:    EEPROM_write_int
 * Input:       _dev_addr: indirizzo impostato sui pin A0-A2 (unsigned char)
 *              _addr: indirizzo da leggere (unsigned int)
 *              _data: dato da scrivere (unsigned int)
 * 
 * Overview:    Scrive il dato intero nell'indirizzo dichiarato e in quello successivo (2 byte)
 * Note:        Non torna eventuali errori di comunicazione I2C
/* ============================================================================================================================================ */
void EEPROM_I2C_write_int(unsigned char _dev_addr, unsigned int _addr, unsigned int _data) {
    unsigned char _high;
    unsigned char _low;

    _low = _data & 0b0000000011111111;                                                  // maschero il byte basso
    _data = _data >>8;                                                                  // sposto il byte alto nel byte basso
    _high = _data;

    EEPROM_I2C_write(_dev_addr, _addr, _high);                                          // scrivo il byte alto
    Delay_ms(10);
    EEPROM_I2C_write(_dev_addr, _addr+1, _low);                                         // scrivo il byte basso
}

/* ============================================================================================================================================= /
 * FUNCTION:    EEPROM_I2C_write_int_e_check
 * Input:       _dev_addr: indirizzo impostato sui pin A0-A2 (unsigned char)
 *              _addr: indirizzo da scrivere (unsigned int)
 *              _data: dato da scrivere (unsigned char)
 * Output:      0: Dato scritto correttamente
 *              -4: Dato da salvare e dato salvato non corrispondono
 *
 * Overview:    Scrive il dato intero nell'indirizzo dichiarato e in quello successivo (2 byte) e verifica che sia stato scritto correttamente
 * Note:        Non torna eventuali errori di comunicazione I2C
/* ============================================================================================================================================ */
signed char EEPROM_I2C_write_int_e_check(unsigned char _dev_addr, unsigned int _addr, unsigned int _data) { 
    unsigned int _data_read = 0;
    
    EEPROM_I2C_write_int(_dev_addr, _addr, _data);
    Delay_ms(10);
    _data_read = EEPROM_I2C_read_int(_dev_addr, _addr);
    
    if (_data == _data_read)
        return 0;
    else
        return -4;
}


#endif