/* ============================================================================================================================================= /
 * LIBRARY:         ESP8266
 * FILE:            ESP8266.c
 * LAST REV:        1.1
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         10/01/2017
 * UPDATED:         31/07/2017
 *
 * DESCRIP:         Gestione modulo ESP8266
 *                  - firmware xxxx
 *                  - AT v. 1.3.0.0 14/07/2016
 *                  - SDK v.2.0.0(656ebdf) 19/07/2016
 *
 * PRECONDITIONS:   libreria USART_init.c, Delay.c
 * FUNCTIONS:       ESP8266_init()
 *                  ESP8266_soft_reset()
 *                  ESP8266_hard_reset()
 *                  ESP8266_power()
 *                  ESP8266_at()
 *                  ESP8266_read_firmware()
 *                  ESP8266_echo() 
 *                  ESP8266_restore_factory()             
 *                  ESP8266_uart_conf()
 *                  ESP8266_set_tx_power()
 *                  ESP8266_set_mode()
 *                  ESP8266_read_mode()        
 *                  ESP8266_join_ap()  
 *                  ESP8266_check_ap_con()
 *                  ESP8266_search_ap_opt()
 *                  ESP8266_search_ap()
 *                  ESP8266_quit_ap()       
 *                  ESP8266_set_softap()
 *                  ESP8266_read_softap()
 *                  ESP8266_read_connected_stations()
 *                  ESP8266_set_dhcp()
 *                  ESP8266_read_dhcp()
 *                  ESP8266_set_dhcp_ip_range()
 *                  ESP8266_set_dhcp_ip_range_default()
 *                  ESP8266_read_dhcp_ip_range() 
 *                  ESP8266_auto_con()   
 *                  ESP8266_set_mac_sta()
 *                  ESP8266_read_mac_sta()
 *                  ESP8266_set_mac_ap()
 *                  ESP8266_read_mac_ap()
 *                  ESP8266_set_ip_sta()
 *                  ESP8266_read_ip_sta()
 *                  ESP8266_set_ip_ap()
 *                  ESP8266_read_ip_ap()
 *                  ESP8266_read_ip()  
 *                  ESP8266_multiple_con()
 *                  ESP8266_create_server()
 *                  ESP8266_set_server_timeout()
 *                  ESP8266_read_server_timeout()
 *                  ESP8266_close_server()
 *                  ESP8266_get_dns_ip()
 *                  ESP8266_open_tcp_connection()
 *                  ESP8266_open_udp_connection()
 *                  ESP8266_close_connection()
 *                  ESP8266_send_data_over_con_l()
 *                  ESP8266_send_data_over_con()
 *                  ESP8266_check_connection_status()
 *                  ESP8266_ping()
 *                  ESP8266_check_incoming_data()
 *                  ESP8266_wait_resp()
 *                  ESP8266_decode_resp()
 *                  ESP8266_wait_decode_resp() 
 *                  ESP8266_check_ready()
 *                  ESP8266_reset_buffer()
 *                  ESP8266_clean_buffer()
 *                  ESP8266_clean_buffer_until_char()
 *                  ESP8266_set_timer()
 * ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.1               31/07/2017          - Aggiornata funzione ESP8266_send_data_over_con_l utilizzando UART_send_char_n per inviare anche 0x00 
  1.0               14/02/2017          - Prima build completa
  1.0               10/01/2017          - Inizio studio modulo ESP8266
//! [History]
/* ============================================================================================================================================ */
#ifndef ESP8266
#define ESP8266

/* INCLUDES =================================================================================================================================== */
#include "ESP8266_settings.h"
#include "ESP8266.h"
#include <stdlib.h>

/* VARIABILI GLOBALI =========================================================================================================================== */
// variabili globali perch comunque posso ricevere e verificare solo una risposta alla volta.
//  inutile usare puntatori o passare le variabili alle funzioni
/** \brief indice del buffer di ricezione */
volatile unsigned int esp8266_buffer_index = 0;
/** \brief buffer di ricezione */
unsigned char esp8266_buffer[ESP8266_BUFFER_LENGTH] = 0;
/** \brief stringa ricevuta */
volatile bit esp8266_string_completed = 0;
/** \brief buffer pulito */
volatile bit esp8266_buffer_cleaned = 0;

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_init
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x02: ESP8266_RESP_ATE0_OK (modulo correttamente avviato e ATE0 impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Imposta le linee di ricezione, abilita e resetta il modulo e disabilita l'echo automatico.
/* ============================================================================================================================================ */
unsigned char ESP8266_init(unsigned int _timeout){
    unsigned char _return = ESP8266_RESP_TIMEOUT;
    
    ESP8266_reset_buffer();
    
    // assicurarsi che siano abilitati interrupt globali
    // abilitazione interrupt in ricezione in base al modulo UART selezionato
    ESP8266_INTERRUPT_IP = ESP8266_INT_PRIORITY;
    ESP8266_INTERRUPT_IF = 0;
    ESP8266_INTERRUPT_IE = 1;
    
    // abilitazione timer e relativi interrupt
    ESP8266_set_timer();
    
    // settaggio delle linee
    ESP8266_CHPD = 1;
    ESP8266_RST = 1;
    Delay_ms(100);
    
    // reset hardware
    ESP8266_RST = 0;                                                                    // reset hardware
    Delay_ms(100);
    ESP8266_RST = 1;

    _return = ESP8266_check_ready(_timeout);                                            // attendo timeout o ready
    
    if (_return == ESP8266_READY)                                                       // se ready
        _return = ESP8266_echo(OFF, _timeout);                                          // invio ATE0, torna 0 se timed out torna 1 se ..ok..
                                                                                        // altrimenti torna _return che pu essere ESP8266_NOT_READY, ESP8266_RESP_TIMEOUT o ESP8266_RESP_NOT_HANDLED
    return _return;                                                                       
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_soft_reset
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x02: ESP8266_RESP_ATE0_OK (modulo correttamente avviato e ATE0 impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Invia comando di reset, verifica il corretto riavvio del modulo ESP8266 e disabilita l'echo automatico.
/* ============================================================================================================================================ */
unsigned char ESP8266_soft_reset(unsigned int _timeout){
    unsigned char _return = ESP8266_RESP_TIMEOUT;
    
    ESP8266_reset_buffer();
     
    _ESP8266_USART_send_string("AT+RST\r\n");                                           // invio comando di reset
    
    _return =  ESP8266_wait_decode_resp(_timeout);                                      // attendo risposta
    
    if (_return == ESP8266_RESP_OK) {                                                   // se primo OK verifico ready
        _return = ESP8266_check_ready(_timeout);
        if (_return == ESP8266_READY)                                                   // se ready
            _return = ESP8266_echo(OFF, _timeout);    
    }
    return _return;                                                                     
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_hard_reset
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (impostare 1000ms)(unsigned int)
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x02: ESP8266_RESP_ATE0_OK (modulo correttamente avviato e ATE0 impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Resetta tramite PIN CH_PD il modulo ESP8266, verifica il corretto riavvio e disabilita l'echo automatico.
/* ============================================================================================================================================ */
unsigned char ESP8266_hard_reset(unsigned int _timeout){
    unsigned char _return = ESP8266_RESP_TIMEOUT;
    
    ESP8266_reset_buffer();
     
    // reset hardware
    ESP8266_RST = 0;            
    Delay_ms(100);
    ESP8266_RST = 1;
    
    _return = ESP8266_check_ready(_timeout);
    
    if (_return == ESP8266_READY)                                                       // se ready                                                      // se ready
        _return = ESP8266_echo(OFF, _timeout);                                          // invio ATE0, torna 0 se timed out torna 1 se ..ok..

    return _return;                                                                      
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_power
 * Input:           _on_off: stato da impostare: ON/OFF (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)       
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x02: ESP8266_RESP_ATE0_OK (modulo correttamente avviato e ATE0 impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Spegne o accende tramite PIN CH_PD il modulo ESP8266, verifica il corretto avvio e disabilita l'echo automatico.
/* ============================================================================================================================================ */
unsigned char ESP8266_power(unsigned char _on_off, unsigned int _timeout){
    unsigned char _return = ESP8266_RESP_TIMEOUT;
        
    ESP8266_reset_buffer();
    
    ESP8266_CHPD = _on_off;
    if (_on_off == ON) {                                                                // se il modulo va avviato
        _return = ESP8266_check_ready(_timeout);
        
        if (_return == ESP8266_READY)                                                   // se ready
            _return = ESP8266_echo(OFF, _timeout);                                      // invio ATE0, torna 0 se timed out torna 1 se ..ok..
    } else {
        _return = 2;                                                                    // se il modulo va spento torno subito 1
    }
    return _return;     
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_at
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)       
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (il modulo ESP8266 ha restituito OK)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Invia comando AT e verifica la risposta
/* ============================================================================================================================================ */
unsigned char ESP8266_at(unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return; 
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_read_firmware
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (IMPOSTARE > 300ms) (unsigned int)       
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (firmware letto correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Invia comando di lettura versione firmware del modulo. Torna 1 se i dati sono ricevuti correttamente.
 * Note:            In esp8266_buffer torna i dati letti:
 *                  ATversion:1.3.0.0(Jul14201618:54:01)SDKversion:2.0.0(656edbf)compiletime:Jul19201618:43:55OK
/* ============================================================================================================================================ */                          
unsigned char ESP8266_read_firmware(unsigned int _timeout){
    unsigned char _return = ESP8266_RESP_TIMEOUT;
    
    _ESP8266_USART_send_string("AT+GMR\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);
        
    if (_return == ESP8266_RESP_ATVER){
            // pulizia buffer per formattazione risultato
        _return = 1;
    }
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_echo
 * Input:           _on_off: stato da impostare: ON/OFF (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)       
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (parametro impostato correttamente)
 *                  - 0x02: ESP8266_RESP_ATE0_OK (ATE0 impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Attiva o disattiva l'echo automatico da parte del modulo ESP8266
 * Note:            Per ridurre il buffer di ricezione La libreria funziona con echo disattivato, l'unico comando interpretato con echo attivato 
 *                  ATE0 per disabilitare l'echo automatico.
/* ============================================================================================================================================ */
unsigned char ESP8266_echo(unsigned char _on_off, unsigned int _timeout){
    unsigned char _return = 0;
   
    _ESP8266_USART_send_string("ATE");                                                  // Invio ATE0 o ATE1
    _ESP8266_USART_send_char('0' + _on_off);                                            // ASCII 1 o 0
    _ESP8266_USART_send_string("\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);
    
    return _return;                                                                     
}              

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_restore_factory
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (IMPOSTARE > 1000ms) (unsigned int)       
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x02: ESP8266_RESP_ATE0_OK (impostazioni ripristinate e modulo riavviato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 * 
 * Overview:        Invia comando di ripristino impostazioni di programmazione (che non sono quelle programmate con funzioni DEF) reimposta la modalit
 *                  di trasmissione del modulo UART e reinizializza il modulo ESP8266
/* ============================================================================================================================================ */
unsigned char ESP8266_restore_factory(unsigned int _timeout){
    unsigned char _return = 0;

    _ESP8266_USART_send_string("AT+RESTORE\r\n");                                       
    
    _return =  ESP8266_wait_decode_resp(_timeout);                                      // primo OK
    
    if (_return == ESP8266_RESP_OK){ 
        _ESP8266_USART_enable(ESP8266_BAUDR_115200);                                    // reimposto il modulo UART a 115200 di default
        _return = ESP8266_hard_reset(_timeout);                                         // riavvio il modulo ESP8266
    }
    
    return _return;  
}    

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_uart_conf
 * Input:           _baud_r: velocit di trasmissione: ESP8266_BAUDR_9600/ESP8266_BAUDR_19200/ESP8266_BAUDR_57600/ESP8266_BAUDR_115200
 *                  _data_bits: numero di bit della trasmissione: 5~8 (unsigned char)
 *                  _stop_bits: numero di bit di stop: 1:1, 2:1.5, 3:2 (unsigned char)
 *                  _parity: impostazione check parit: ESP8266_PARITY_NONE/ESP8266_PARITY_ODD, ESP8266_PARITY_EVEN
 *                  _flow_ctrl: impostazione controllo flusso: ESP8266_FLAW_CTRL_NONE/ESP8266_FLAW_CTRL_RTS/ESP8266_FLAW_CTRL_CTS/ESP8266_FLAW_CTRL_BOTH
 *                  _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                  
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (parametri impostati correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 * 
 * Overview:        Imposta la modalit di trasmissione UART (attuale o default) e reimposta la velocit di trasmissione del modulo UART
/* ============================================================================================================================================ */
unsigned char ESP8266_uart_conf(unsigned char _baud_r, unsigned char _data_bits, unsigned char _stop_bits, unsigned char _parity, unsigned char _flow_ctrl, unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)                                              // impostazione corrente
        _ESP8266_USART_send_string("AT+UART_CUR=");
    else                                                                                // o default
        _ESP8266_USART_send_string("AT+UART_DEF=");
    
    switch(_baud_r) {
        case ESP8266_BAUDR_9600:
                                _ESP8266_USART_send_string("9600");
                                break;
        case ESP8266_BAUDR_19200:
                                _ESP8266_USART_send_string("19200");
                                break;                            
        case ESP8266_BAUDR_57600:
                                _ESP8266_USART_send_string("57600");
                                break;
        case ESP8266_BAUDR_115200:
                                _ESP8266_USART_send_string("115200");
                                break;
    }
    
    _ESP8266_USART_send_char(','); 
    _ESP8266_USART_send_char('0' + _data_bits);  
    _ESP8266_USART_send_char(','); 
    _ESP8266_USART_send_char('0' + _stop_bits);  
    _ESP8266_USART_send_char(','); 
    _ESP8266_USART_send_char('0' + _parity);  
    _ESP8266_USART_send_char(','); 
    _ESP8266_USART_send_char('0' + _flow_ctrl);
    _ESP8266_USART_send_string("\r\n");   

    _return =  ESP8266_wait_decode_resp(_timeout);                                      // attendo la risposta
   
    if (_return == ESP8266_RESP_OK)
        _ESP8266_USART_enable(_baud_r);                                                 // se OK imposto il modulo UART a seconda del baud rate selezionato
                                                                                        // (i valori di ESP8266_BAUDR_xx e USART_BAUDR_xx sono equivalenti)   
    return _return;  
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_set_tx_power
 * Input:           _rf_power[]: potenza di trasmissione: stringa "0"~"82" a passi di 0.25dBm per unit (stringa)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                  
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (parametro impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Imposta la potenza di trasmissione
 * Note:            Non so se torna al valore standard al riavvio o dopo restore.
/* ============================================================================================================================================ */
unsigned char ESP8266_set_tx_power(const unsigned char _rf_power[], unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+RFPOWER=");
    _ESP8266_USART_send_string (_rf_power);
    _ESP8266_USART_send_string("\r\n");   

    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return;  
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_set_mode
 * Input:           _mode: modalit di funzionamento: ESP8266_STATION_MODE/ESP8266_SOFTAP_MODE/ESP8266_BOTH_MODE (unsigned char)
 *                  _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (parametro impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Imposta la modalit di funzionamento del modulo (attuale o default)
/* ============================================================================================================================================ */
unsigned char ESP8266_set_mode(unsigned char _mode,  unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CWMODE_CUR=");
    else
        _ESP8266_USART_send_string("AT+CWMODE_DEF=");
        
    _ESP8266_USART_send_char('0' + (_mode & 0x0F));
    _ESP8266_USART_send_string("\r\n"); 
    
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return; 
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_read_mode
 * Input:           _cur_def: impostazione da leggere : ESP8266_CURRENT_VALUE: legge il valore attuale, ESP8266_DEFAULT_VALUE: legge  il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x91: ESP8266_STATION_MODE (modulo ESP8266 in modalit Stazione)
 *                  - 0x92: ESP8266_SOFTAP_MODE (modulo ESP8266 in modalit Soft Access Point)
 *                  - 0x93: ESP8266_SOFTAP_MODE (modulo ESP8266 in modalit Stazione + Soft Access Point)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        legge la modalit di funzionamento del modulo (attuale o default)
/* ============================================================================================================================================ */
unsigned char ESP8266_read_mode(unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CWMODE_CUR?\r\n");
    else
        _ESP8266_USART_send_string("AT+CWMODE_DEF?\r\n");
        
    _return =  ESP8266_wait_decode_resp(_timeout);                                  
        
    if (_return == ESP8266_RESP_CWMODE)
        _return = esp8266_buffer[12] - '0' + ESP8266_CWMODE_OFFSET;                     // Torno codice modalit + offset
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_join_ap
 * Input:           _ssid[]: nome della rete (stringa)
 *                  _password[]: password (stringa)
 *                  _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore fino al riavvio, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (IMPOSTARE > 18000ms) (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: Rete connessa e IP assegnato
 *                  - 0x05: EPS8266_RESP_WIFI_CONNEC (WiFi connesso ma IP non assegnato)
 *                  - 0x07: EPS8266_RESP_WIFI_GOT_IP (WiFi indirizzo IP assegnato)
 *                  - 0x81: ESP8266_TIMEOUTCON (timeout connessione)
 *                  - 0x82: ESP8266_PWD_ERR (password errata)
 *                  - 0x83: ESP8266_NO_AP (access Point non trovato)
 *                  - 0x84: ESP8266_CON_FAIL (Connessione fallita)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Accede alla rete WiFi
 * Preconditions:   Il modulo ESP8266 deve essere impostato in Station Mode o AP + Station Mode (ESP8266_set_mode())
 *                  - FUNZIONAMENTO CON DHCP:
 *                    - Abilitare il DHCP sul modulo (impostazione di default) e nell'access point 
 *                  - FUNZIONAMENTO CON IP STATICO
 *                    - Disabilitare il DHCP (ESP8266_set_dhcp())
 *                    - Assegnare un indirizzo IP valido (ESP8266_set_IP())
/* ============================================================================================================================================ */
unsigned char ESP8266_join_ap(const unsigned char _ssid[], const unsigned char _password[], unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;

    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CWJAP_CUR=");
    else
        _ESP8266_USART_send_string("AT+CWJAP_DEF=");
    _ESP8266_USART_send_char(0x22);     //"    
    _ESP8266_USART_send_string(_ssid);
    _ESP8266_USART_send_char(0x22);     //"    
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_char(0x22);     //"    
    _ESP8266_USART_send_string(_password); 
    _ESP8266_USART_send_char(0x22);     //"    
    _ESP8266_USART_send_string("\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);                                      // leggo la risposta

    if (_return == EPS8266_RESP_WIFI_DISCON)                                            // se torna WIFI DISCONNECT significa che era connesso ad un'altra rete
        _return =  ESP8266_wait_decode_resp(_timeout);                                  // leggo nuovamente la risposta
        
    if (_return == EPS8266_RESP_WIFI_CONNEC){                                           // se  WIFI CONNECTED
        _return =  ESP8266_wait_decode_resp(_timeout);                                  // attendo nuova risposta
        if (_return == EPS8266_RESP_WIFI_GOT_IP){                                       // se  GOT IP attendo nuova risposta (OK)
            _return =  ESP8266_wait_decode_resp(_timeout);                              // 
        } else if (_return == EPS8266_RESP_CWJAP){
            _return = esp8266_buffer[7] - '0' + ESP8266_CWJAP_OFFSET;                   // torno codice errore pi relativo offset
        }
    } else if (_return == EPS8266_RESP_CWJAP){
        _return = esp8266_buffer[7] - '0' + ESP8266_CWJAP_OFFSET;                       // torno codice errore pi relativo offset
    }
    return _return; 
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_check_ap_con
 * Input:           _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore fino al riavvio, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                   
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: Access point connesso
 *                  - 0x09: EPS8266_RESP_NOAP (nessun Access Point connesso)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Legge access point connesso e torna 1 se i dati sono ricevuti correttamente.
 * Preconditions:   Il modulo ESP8266 deve essere impostato in Station Mode o AP + Station Mode (ESP8266_set_mode())
 * Note:            In esp8266_buffer torna i dati letti:
 *                  Se connesso: "Home","e0:91:f5:4d:0c:3c",10,-49OK
 *                  Se non connesso: NoAPOK
/* ============================================================================================================================================ */
unsigned char ESP8266_check_ap_con(unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;

    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CWJAP_CUR?\r\n");
    else
        _ESP8266_USART_send_string("AT+CWJAP_DEF?\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);
     
    if (_return == EPS8266_RESP_CWJAP){                                                 // se torna CWJAP ho l'AP nel buffer
        ESP8266_clean_buffer(11);
        _return = 1;    
    }
     return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_search_ap_opt
 * Input:           _on_off: ordinamento in base alla potenza del segnale: ON/OFF (unsigned char)
 *                  _mask[]: stringa di mascheramento dei risultati ("31" per mostrare solo ecn, ssid, rssi, mac, ch) (stringa)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                   
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (parametro impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Imposta i parametri da restituire durante la scansione degli access point (ESP8266_search_ap())
 * Note:            l'impostazione si resetta al riavvio del modulo
/* ============================================================================================================================================ */
unsigned char ESP8266_search_ap_opt(unsigned char _on_off, const unsigned char _mask[], unsigned int _timeout){
    unsigned char _return = 0;
 
    _ESP8266_USART_send_string("AT+CWLAPOPT=");
    _ESP8266_USART_send_char('0' + _on_off);
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_string(_mask);
    
    _ESP8266_USART_send_string("\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return; 
}  

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_search_ap
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (IMPOSTARE > 18000ms) (unsigned int)                                   
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (AP trovati)
 *                  - 0x09: ESP8266_RESP_NOAP (nessun AP trovato)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Legge gli access point nelle vicinanze. Torna 1 se ha finito e ha trovato access point.
 * Preconditions:   Il modulo ESP8266 deve essere  impostato in Station Mode o AP + Station Mode (ESP8266_set_mode())
 * Note:            In esp8266_buffer torna i dati letti:
 *                  +CWLAP:(1,"Home",-36,"e0:91:f5:4d:0c:3c",10)+CWLAP:(1,"Home2",-75,"4c:9e:ff:04:5a:48",3
 *                  +CWLAP:(4,"Alice-49348481",-80,"d4:d1:84:30:cc:38",13)OK)
 * ToDo:            verifica cosa succede se non trova access point e se chiude con ..ok..
/* ============================================================================================================================================ */
unsigned char ESP8266_search_ap(unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+CWLAP\r\n");
       
    _return =  ESP8266_wait_decode_resp(_timeout);
        
    if (_return == EPS8266_RESP_CWLAP){
      // pulizia buffer
        _return = 1;
    } else if (_return == ESP8266_RESP_OK){
        _return = ESP8266_RESP_NOAP;
    }
     return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_quit_ap
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (IMPOSTARE > 5000ms) (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x06: EPS8266_RESP_WIFI_DISCON (WiFi disconnesso)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Disconnette dalla rete attualmente connessa
/* ============================================================================================================================================ */
unsigned char ESP8266_quit_ap(unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+CWQAP");
    _ESP8266_USART_send_string("\r\n");

    
    _return =  ESP8266_wait_decode_resp(_timeout);
        // torna subito OK.. WIFI DISDCONNECT, troppo veloce per gestire come se fossero 2 risposte separate
    
    if (_return == ESP8266_RESP_OK){
        ESP8266_clean_buffer(2);                                                        // cancello l'ok dal buffer
        _return = ESP8266_decode_resp();                                                // controllo wifi disconnect
                                                                                        // a 9600 prende questo OK + WIFI DISCONNECT
        if (_return != EPS8266_RESP_WIFI_DISCON)                                        // se non si  disconnesso
            _return =  ESP8266_wait_decode_resp(_timeout);                              // A 115200 DEVO ATTENDERE NUOVO INVIO
    } 
    return _return; 
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_set_softap
 * Input:           _ssid[]: nome della rete (stringa)
 *                  _password[]: password (8~64 byte) (stringa)
 *                  _channel[]: canale della trasmissione (stringa)
 *                  _enc: modi di criptazione (ESP8266_ENC_OPEN/ESP8266_ENC_WPA_PSK/ESP8266_ENC_WPA2_PSK/ESP8266_ENC_WPA_WPA2_PSK) (unsigned char)
 *                  _max_con: numero di connessioni massime (1~4) (unsigned char)
 *                  _hidden: ssid nascosto (ON/OFF) (unsigned char)
 *                  _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore fino al riavvio, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (IMPOSTARE > 18000ms) (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (soft Access Point impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Imposta il modulo ESP8266 come Access Point
 * Preconditions:   Il modulo ESP8266 deve essere impostato in Soft AP mode o AP + Station Mode (ESP8266_set_mode())
 *                  - FUNZIONAMENTO CON DHCP:
 *                    - Abilitare il DHCP sul modulo (impostazione di default) e nell'access point 
 *                    - Definire eventualmente l'intervallo di indirizzi che il DHCP pu assegnare (funzione settaggio indirizzi DHCP)
 *                  - FUNZIONAMENTO CON IP STATICO
 *                    - Disabilitare il DHCP (ESP8266_set_dhcp())
 *                    - Assegnare un indirizzo IP valido al client da connettere al modulo ESP8266
 * Note:            WEP e WPA2_ENT non supportati                    
/* ============================================================================================================================================ */
unsigned char ESP8266_set_softap(const unsigned char _ssid[], const unsigned char _password[], const unsigned char _channel[], unsigned char _enc, unsigned char _max_con, unsigned char _hidden, unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CWSAP_CUR=");
    else
        _ESP8266_USART_send_string("AT+CWSAP_DEF=");
    
    _ESP8266_USART_send_char(0x22);     //"    
    _ESP8266_USART_send_string(_ssid);
    _ESP8266_USART_send_char(0x22);     //"    
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_char(0x22);     //"    
    _ESP8266_USART_send_string(_password); 
    _ESP8266_USART_send_char(0x22);     //"
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_string(_channel);
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_char('0' + (_enc & 0x0F));
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_char('0' + _max_con);
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_char('0' + _hidden);
    _ESP8266_USART_send_string("\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);                                      
    
    return _return;
} 

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_read_softap
 * Input:           _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore fino al riavvio, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (IMPOSTARE > 18000ms) (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (dati soft Access Point letti correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        legge i parametri del soft Access Point
 * Preconditions:   Il modulo ESP8266 deve essere impostato in Soft AP mode o AP + Station Mode (ESP8266_set_mode())
 * Note:            In esp8266_buffer torna i dati letti:
 *                  es: "TEST","123456789",5,2,4,0
/* ============================================================================================================================================ */
unsigned char ESP8266_read_softap(unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CWSAP_CUR?\r\n");
    else
        _ESP8266_USART_send_string("AT+CWSAP_DEF?\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);                                      
    
    if (_return == EPS8266_RESP_CWSAP){                                                 // se torna CWJAP ho l'AP nel buffer
        ESP8266_clean_buffer(11);
        _return = 1;    
    }
     return _return;  
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_read_connected_stations
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (IMPOSTARE > 18000ms) (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (stazioni connesse lette correttamente)
 *                  - 0xFA: ESP8266_NO_STATION_CON (nessuna stazione connessa)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        legge gli indirizzi IP e MAC di eventuali stazioni connesse al Soft AP
 * Preconditions:   - Il modulo ESP8266 deve essere impostato in Soft AP mode o AP + Station Mode (ESP8266_set_mode())
 *                  - Il DHCP deve essere attivo per la modalit Soft AP (ESP8266_set_dhcp())
 * Note:            In esp8266_buffer torna i dati letti:
 *                  es: 192.168.4.2,90:18:7c:25:aa:db  OK
/* ============================================================================================================================================ */
unsigned char ESP8266_read_connected_stations(unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+CWLIF\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);                                      // leggo la risposta
    
    if (_return == ESP8266_RESP_OK){                                                    // torna OK se non ci sono stazioni connesse
        _return = ESP8266_NO_STATION_CON;
    } else if (_return == ESP8266_RESP_NOT_HANDLED){                                    // torna IP e mac se ci sono stazioni (192.168.4.2,90:18:7c:25:aa:db) con OK finale
        _return = ESP8266_RESP_OK;     
    }
    
    return _return;
}


/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_set_dhcp
 * Input:           _mode: modalit di funzionamento sulla quale abilitare o disabilitare il DHCP: ESP8266_DHCP_AP/ESP8266_DHCP_STATION/ESP8266_DHCP_BOTH (unsigned char)
 *                  _on_off: ON/OFF (unsigned char)
 *                  _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (parametri DHCP impostati correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Attiva o disattiva il DHCP per la modalit di funzionamento selezionata (attuale o default)
 * Preconditions:   per abilitare o disabilitare il DHCP il modulo ESP8266 deve avere la relativa modalit abilitata (ESP8266_set_mode())
 * Note:            - Di default il DHCP  abilitato per entrambe le modalit di funzionamento
 *                  - Pu essere impostato il DHCP solo per la modalit selezionata (ESP8266_set_mod())
/* ============================================================================================================================================ */
unsigned char ESP8266_set_dhcp(unsigned char _mode, unsigned char _on_off, unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CWDHCP_CUR=");
    else
        _ESP8266_USART_send_string("AT+CWDHCP_DEF=");
        
    _ESP8266_USART_send_char('0' + ((_mode -1) & 0x0F));
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_char('0' + _on_off);
    _ESP8266_USART_send_string("\r\n"); 
    
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return; 
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_read_dhcp
 * Input:           _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0xA0: ESP8266_DHCP_OFF (DHCP disabilitato per Soft Access Point e Stazione)
 *                  - 0xA1: ESP8266_DHCP_AP (DHCP abilitato per Soft Access Point)
 *                  - 0xA2: ESP8266_DHCP_STATION (DHCP abilitato per Stazione) 
 *                  - 0xA3: ESP8266_BOTH_MODE (DHCP abilitato per Soft Access Point e Stazione)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:       Legge lo stato del DHCP per tutti i modi di funzionamento (attuale o default)
/* ============================================================================================================================================ */
unsigned char ESP8266_read_dhcp(unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CWDHCP_CUR?\r\n");
    else
        _ESP8266_USART_send_string("AT+CWDHCP_DEF?\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);   
        
    if (_return == ESP8266_RESP_CWDHCP)
        _return = esp8266_buffer[12] - '0' + ESP8266_DHCP_OFFSET;                   
    
    return _return;
}


/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_set_dhcp_ip_range
 * Input:           _lease_time[]: tempo di mantenimento ip in minuti, max 2880 (stringa)
 *                  _start_ip[]: indirizzo IP di partenza (stringa)
 *                  _end_ip[]: indirizzo IP finale (stringa)
 *                  _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (indirizzi DHCP impostati correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Imposta il range di IP che il DHCP pu assegnare
 * Preconditions:   - Il modulo ESP8266 deve essere impostato in Soft AP mode o AP + Station Mode (ESP8266_set_mode())
 *                  - Il modulo ESP8266 deve avere il DHCP per la modalit Soft AP abilitato (ESP8266_set_dhcp()
 * Note:            Gli indirizzi ip devono essere sullo stesso segmento di rete dell'ip assegnato alla modalita Soft Access Point                 
/* ============================================================================================================================================ */
unsigned char ESP8266_set_dhcp_ip_range(const unsigned char _lease_time[], const unsigned char _start_ip[], const unsigned char _end_ip[],  unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CWDHCPS_CUR=");
    else
        _ESP8266_USART_send_string("AT+CWDHCPS_DEF=");
        
    _ESP8266_USART_send_string("1,");
    _ESP8266_USART_send_string(_lease_time);
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_char(0x22);
    _ESP8266_USART_send_string(_start_ip);
    _ESP8266_USART_send_char(0x22);
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_char(0x22);
    _ESP8266_USART_send_string(_end_ip);
    _ESP8266_USART_send_char(0x22);  
    _ESP8266_USART_send_string("\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_set_dhcp_ip_range_default
 * Input:           _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (indirizzi DHCP impostati correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Imposta il range di IP di default che il DHCP pu assegnare
 * Preconditions:   - Il modulo ESP8266 deve essere impostato in Soft AP mode o AP + Station Mode (ESP8266_set_mode())
 *                  - Il modulo ESP8266 deve avere il DHCP per la modalit Soft AP abilitato (ESP8266_set_dhcp()
/* ============================================================================================================================================ */
unsigned char ESP8266_set_dhcp_ip_range_default(unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CWDHCPS_CUR=0");
    else
        _ESP8266_USART_send_string("AT+CWDHCPS_DEF=0");
    
    _ESP8266_USART_send_string("\r\n");
        
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return;     
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_read_dhcp_ip_range
 * Input:           _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (indirizzi DHCP letti correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Legge gli access point nelle vicinanze. Torna 1 se ha finito e ha trovato access point.
 * Preconditions:   - Il modulo ESP8266 deve essere impostato in Soft AP mode o AP + Station Mode (ESP8266_set_mode())   
 *                  - Il modulo ESP8266 deve avere il DHCP per la modalit Soft AP abilitato (ESP8266_set_dhcp()  
 * Note:            In esp8266_buffer torna i dati letti:
 *                  60,192.168.4.10,192.168.4.50OK
/* ============================================================================================================================================ */
unsigned char ESP8266_read_dhcp_ip_range(unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CWDHCPS_CUR?\r\n");
    else
        _ESP8266_USART_send_string("AT+CWDHCPS_DEF?\r\n");
        
    _return =  ESP8266_wait_decode_resp(_timeout);                                      // leggo la risposta
    
    if (_return == ESP8266_RESP_CWDHCPS){                                               // se torna CWJAP ho l'AP nel buffer
        ESP8266_clean_buffer(13);
        _return = 1;    
    }
     return _return;  
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_auto_con
 * Input:           _on_off: ON/OFF (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (IMPOSTARE > 18000ms) (unsigned int)                                   
 * Output:          0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  0x01: ESP8266_RESP_OK (parametro impostato correttamente)
 *                  0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Imposta l'autoconnessione all'avvio
 * Preconditions:   Il modulo ESP8266 deve essere impostato in Station Mode o AP + Station Mode (ESP8266_set_mode())
 * Note:            Impostazione salvata e mantenuta al riavvio
/* ============================================================================================================================================ */
unsigned char ESP8266_auto_con(unsigned char _on_off, unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+CWAUTOCONN=");
    _ESP8266_USART_send_char('0' + _on_off);
    _ESP8266_USART_send_string("\r\n");    
       
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return; 
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_set_mac_sta
 * Input:           _mac[]: Indirizzo IP da impostare (stringa)
 *                  _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (indirizzo MAC stazione impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Imposta l'indirizzo MAC della modalit stazione(attuale o default)
 * Preconditions:   Il modulo ESP8266 deve essere impostato in Station Mode o AP + Station Mode (ESP8266_set_mode()) 
 * Note:            - l'indirizzo MAC della modalit stazione e della modalit Soft Access Point devono essere diversi
 *                  - il bit 0 del primo byte deve essere 0 (es: 1A va bene, 15 no)
/* ============================================================================================================================================ */
unsigned char ESP8266_set_mac_sta(const unsigned char _mac[], unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CIPSTAMAC_CUR=");
    else
        _ESP8266_USART_send_string("AT+CIPSTAMAC_DEF=");
    
    _ESP8266_USART_send_char(0x22);     //"    
    _ESP8266_USART_send_string(_mac);
    _ESP8266_USART_send_char(0x22);     //"  
    _ESP8266_USART_send_string("\r\n"); 
    
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return; 
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_read_mac_sta
 * Input:           _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (indirizzo MAC stazione letto correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Legge l'indirizzo MAC della modalit stazione(attuale o default). Torna 1 se i dati sono ricevuti correttamente
 * Note:            In esp8266_buffer torna i dati letti:
 *                  es: "5c:cf:7f:8b:5c:d0"OK
/* ============================================================================================================================================ */
unsigned char ESP8266_read_mac_sta(unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CIPSTAMAC_CUR?\r\n");
    else
        _ESP8266_USART_send_string("AT+CIPSTAMAC_DEF?\r\n");
   
    _return =  ESP8266_wait_decode_resp(_timeout);
        
    if (_return == ESP8266_RESP_CIPSTAMAC) {
        ESP8266_clean_buffer(15);
        _return = 1;
    }
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_set_mac_ap
 * Input:           _mac[]: indirizzo MAC da impostare (stringa)
 *                  _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (indirizzo MAC Soft Access Point impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Imposta l'indirizzo MAC della modalit Soft Access Point (attuale o default)
 * Preconditions:   - Il modulo ESP8266 deve essere impostato in Soft AP mode o AP + Station Mode (ESP8266_set_mode())  
 * Note:            - l'indirizzo MAC della modalit stazione e della modalit Soft Access Point devono essere diversi
 *                  - il bit 0 del primo byte deve essere 0 (es: 1A va bene, 15 no)
/* ============================================================================================================================================ */
unsigned char ESP8266_set_mac_ap(const unsigned char _mac[], unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CIPAPMAC_CUR=");
    else
        _ESP8266_USART_send_string("AT+CIPAPMAC_DEF=");
    
    _ESP8266_USART_send_char(0x22);     //"    
    _ESP8266_USART_send_string(_mac);
    _ESP8266_USART_send_char(0x22);     //"  
    _ESP8266_USART_send_string("\r\n"); 
    
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return; 
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_read_mac_ap
 * Input:           _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (indirizzo MAC Soft Access Point letto correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Legge l'indirizzo MAC della modalit Soft Access Point (attuale o default). Torna 1 se i dati sono ricevuti correttamente
 * Note:            In esp8266_buffer torna i dati letti:
 *                  es: "5c:cf:7f:8b:5c:d0"OK
/* ============================================================================================================================================ */
unsigned char ESP8266_read_mac_ap(unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CIPAPMAC_CUR?\r\n");
    else
        _ESP8266_USART_send_string("AT+CIPAPMAC_DEF?\r\n");
   
    _return =  ESP8266_wait_decode_resp(_timeout);
        
    if (_return == ESP8266_RESP_CIPAPMAC) {
        ESP8266_clean_buffer(14);
        _return = 1;
    }
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_set_ip_sta
 * Input:           _ip[]: Indirizzo IP da impostare (stringa)
 *                  _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (indirizzo IP Stazione impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Imposta l'indirizzo IP della modalit stazione(attuale o default)
 * Preconditions:   Il DHCP per la modalit stazione deve essere disattivato (ESP8266_set_dhcp)
/* ============================================================================================================================================ */
unsigned char ESP8266_set_ip_sta(const unsigned char _ip[], unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CIPSTA_CUR=");
    else
        _ESP8266_USART_send_string("AT+CIPSTA_DEF=");
    
    _ESP8266_USART_send_char(0x22);     //"    
    _ESP8266_USART_send_string(_ip);
    _ESP8266_USART_send_char(0x22);     //"  
    _ESP8266_USART_send_string("\r\n"); 
    
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return; 
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_read_ip_sta
 * Input:           _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (indirizzo IP Stazione letto correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Legge l'indirizzo IP, l'indirizzo Gateway e la Subnet mask della modalit stazione(attuale o default). Torna 1 se i dati sono ricevuti correttamente
 * Preconditions:   - Il modulo ESP8266 deve essere impostata in modalit stazione o Stazione + Soft AP (ESP8266_set_mode())
 *                  - Il modulo ESP8266 deve essere connesso ad un Acces Point
 * Note:            In esp8266_buffer torna i dati letti:
 *                  +CIPSTA_CUR:ip:"192.168.0.50"
 *                  +CIPSTA_CUR:gateway:"192.168.0.1"
 *                  +CIPSTA_CUR:netmask:"255.255.255.0"
/* ============================================================================================================================================ */
unsigned char ESP8266_read_ip_sta(unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CIPSTA_CUR?\r\n");
    else
        _ESP8266_USART_send_string("AT+CIPSTA_DEF?\r\n");
   
    _return =  ESP8266_wait_decode_resp(_timeout);
        
    if (_return == ESP8266_RESP_CIPSTA) {
            // pulizia buffer per formattazione risultato
        _return = 1;
    }
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_set_ip_ap
 * Input:           _ip[]: indirizzo IP da impostare (stringa)
 *                  _gateway[]: indirizzo IP gateway (stringa)
 *                  _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (indirizzo IP Soft Access Point impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Imposta l'indirizzo IP e l'indirizzo Gateway della modalit Soft Access Point (attuale o default)
/* ============================================================================================================================================ */
unsigned char ESP8266_set_ip_ap(const unsigned char _ip[], const unsigned char _gateway[], unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CIPAP_CUR=");
    else
        _ESP8266_USART_send_string("AT+CIPAP_DEF=");
    
    _ESP8266_USART_send_char(0x22);     //"    
    _ESP8266_USART_send_string(_ip);
    _ESP8266_USART_send_char(0x22);     //"  
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_char(0x22);
    _ESP8266_USART_send_string(_gateway);
    _ESP8266_USART_send_char(0x22);
    _ESP8266_USART_send_string("\r\n"); 
    
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return; 
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_read_ip_ap
 * Input:           _cur_def: impostazione di salvataggio: ESP8266_CURRENT_VALUE: imposta il valore attuale, ESP8266_DEFAULT_VALUE: imposta il valore di default (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (indirizzo IP Soft Access Point letto correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Legge l'indirizzo IP, l'indirizzo Gateway e la Subnet mask della modalit Soft Access Point (attuale o default). Torna 1 se i dati sono ricevuti correttamente
 * Preconditions:   Il modulo ESP8266 deve essere impostata in modalit Soft AP (ESP8266_set_mode())
 * Note:            In esp8266_buffer torna i dati letti:
 *                  +CIPAP_CUR:ip:"192.168.0.50"
 *                  +CIPAP_CUR:gateway:"192.168.0.1"
 *                  +CIPAP_CUR:netmask:"255.255.255.0"
/* ============================================================================================================================================ */
unsigned char ESP8266_read_ip_ap(unsigned char _cur_def, unsigned int _timeout){
    unsigned char _return = 0;
    
    if (_cur_def == ESP8266_CURRENT_VALUE)
        _ESP8266_USART_send_string("AT+CIPAP_CUR?\r\n");
    else
        _ESP8266_USART_send_string("AT+CIPAP_DEF?\r\n");
   
    _return =  ESP8266_wait_decode_resp(_timeout);
        
    if (_return == ESP8266_RESP_CIPAP) {
            // pulizia buffer per formattazione risultato
        _return = 1;
    }
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_read_ip
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)       
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (indirizzi IP letti correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Legge indirizzo IP e MAC di stazione e soft ap. Torna 1 se i dati sono ricevuti correttamente
 * Note:            In esp8266_buffer torna i dati letti:
 *                  +CIFSR:APIP,"192.168.4.1"
 *                  +CIFSR:APMAC,"5e:cf:7f:8b:5c:d0"
 *                  +CIFSR:STAIP,"192.168.0.50"
 *                  +CIFSR:STAMAC,"5c:cf:7f:8b:5c:d0"
/* ============================================================================================================================================ */
unsigned char ESP8266_read_ip(unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+CIFSR\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);
    
    if (_return == ESP8266_RESP_CIFSR) {
            // pulizia buffer per formattazione risultato
        _return = 1;
    }
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_multiple_con
 * Input:           _on_off: ON/OFF (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                   
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: 0x01: ESP8266_RESP_OK (parametro impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Abilita o disabilita le connessioni multiple (massimo 5)
 * Preconditions:   Il modulo ESP8266 deve essere impostato in modalit normale (CIPMODE = 0, default)
 * Note:            - Tutte le funzioni sono implementate per funzionare in modalit connessioni multiple. impostare ESP8266_multiple_con(ON, 200)
 *                  - Si pu impostare solo se non ci sono connessioni gi attive
 *                  - Prima di impostare disconnettere eventuali connessioni gi attive ed eliminare eventuali server TCP
 *                  - Se esiste gi una connessione attiva torna ERROR
/* ============================================================================================================================================ */
unsigned char ESP8266_multiple_con(unsigned char _on_off,  unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+CIPMUX=");
    _ESP8266_USART_send_char('0' + _on_off);
    _ESP8266_USART_send_string("\r\n");    
       
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return; 
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_create_server
 * Input:           _port[]: porta di connessione (stringa)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                   
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (server creato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Crea un server ed abilita monitor connessioni in ingresso
 * Preconditions:   Il modulo ESP8266 deve essere in modalit connessioni multiple (ESP8266_multiple_con())
 * Note:            - Si pu impostare solo se non ci sono connessioni gi attive
 *                  - Prima di impostare disconnettere eventuali connessioni gi attive ed eliminare eventuali server TCP
/* ============================================================================================================================================ */
unsigned char ESP8266_create_server(const unsigned char _port[],   unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+CIPSERVER=1,");
    _ESP8266_USART_send_string(_port);
    _ESP8266_USART_send_string("\r\n");  
    
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return; 
    
}                                                                                                                                                 

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_set_server_timeout
 * Input:           _time[]: secondi senza comunicazione con il client da attendere prima di disconnettere il client 0~7200 secondi (stringa)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                                   
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (parametro impostato correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Imposta il tempo di timeout per la disconnessione di un client connesso al server ma senza comunicazione 
 * Preconditions:   - Il modulo ESP8266 deve essere in modalit connessioni multiple (ESP8266_multiple_con())
 *                  - il server deve essere attivo (ESP8266_create_server())
 * Note:            - Impostare il timeout a 0 significa che il client non verr mai disconnesso ma questa impostazione  da evitare.
 *                  - Torna al valore di default al riavvio (180)
/* ============================================================================================================================================ */
unsigned char ESP8266_set_server_timeout(const unsigned char _time[],  unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+CIPSTO=");
    _ESP8266_USART_send_string(_time);
    _ESP8266_USART_send_string("\r\n");  
    
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return; 
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_read_server_timeout
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)       
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (parametro letto correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Legge il tempo di timeout per la disconnessione di un client connesso al server ma senza comunicazione 
 * Note:            In esp8266_buffer torna i dati letti:
 *                  es: 3600 (60 minuti)
/* ============================================================================================================================================ */
unsigned char ESP8266_read_server_timeout(unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+CIPSTO?\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);
    
    if (_return == ESP8266_RESP_CIPSTO){
        ESP8266_clean_buffer(8);
        _return = 1;
    }
    return _return;    
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_close_server
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)       
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (server chiuso correttamente)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Chiude il server
/* ============================================================================================================================================ */
unsigned char ESP8266_close_server(unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+CIPSERVER=0\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);

    return _return; 
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_get_dns_IP
 * Input:           _domain[]: indirizzo internet (stringa)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)          
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: ESP8266_RESP_OK (dominio trovato)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Ottiene l'indirizzo IP del sito richiesto
 * Preconditions:   - Il modulo ESP8266 deve essere impostata in modalit stazione o Stazione + Soft AP (ESP8266_set_mode())
 *                  - Il modulo ESP8266 deve essere connesso ad un Acces Point
 * Note:            In esp8266_buffer torna i dati letti:
 *                  es: 80.88.85.16
/* ============================================================================================================================================ */
unsigned char ESP8266_get_dns_ip(const unsigned char _domain[], unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+CIPDOMAIN=");
    _ESP8266_USART_send_char(0x22);     //"   
    _ESP8266_USART_send_string(_domain);   
    _ESP8266_USART_send_char(0x22);     //"   
    _ESP8266_USART_send_string("\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);
    
    if (_return == ESP8266_RESP_CIPDOMAIN){
        ESP8266_clean_buffer(11);
        _return = 1;
    }
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_open_tcp_connection
 * Input:           _connection: numero della connessione da usare 0~4 (unsigned char)
 *                  _domain[]: dominio a cui connettersi, indirizzo o IP (stringa)
 *                  _port[]: porta di connessione (stringa)
 *                  _time[]: tempo per mantenere la connessione attiva (stringa)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (impostare > 10000) (unsigned int)          
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x30: ESP8266_CON_0_FAIL (connessione 0 fallita)
 *                  - 0x31: ESP8266_CON_1_FAIL (connessione 1 fallita)
 *                  - 0x32: ESP8266_CON_2_FAIL (connessione 2 fallita)
 *                  - 0x33: ESP8266_CON_3_FAIL (connessione 3 fallita)
 *                  - 0x34: ESP8266_CON_4_FAIL (connessione 4 fallita)
 *                  - 0x40: ESP8266_CON_0_CONNECT (connessione 0 connessa)
 *                  - 0x41: ESP8266_CON_0_CONNECT (connessione 1 connessa)
 *                  - 0x42: ESP8266_CON_0_CONNECT (connessione 2 connessa)
 *                  - 0x43: ESP8266_CON_0_CONNECT (connessione 3 connessa)
 *                  - 0x44: ESP8266_CON_0_CONNECT (connessione 4 connessa)
 *                  - 0x22: ESP8266_RESP_ALREADY_CON (connessione gi esistente)
 *                  - 0x23: ESP8266_RESP_DNS_FAIL (indirizzo non trovato)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 * Overview:        Si connette via TCP al sito richiesto
 * Preconditions:   - Il modulo ESP8266 deve essere impostata in modalit stazione o Stazione + Soft AP (ESP8266_set_mode())
 *                  - Il modulo ESP8266 deve essere in modalit connessioni multiple (ESP8266_multiple_con())
 *                  - Il modulo ESP8266 deve essere connesso ad un Acces Point
 * Note:            Il comando pu essere usato anche in modalit connessione singola, ma non  implementato. Usare solo la modalit connessioni
 *                  multiple
/* ============================================================================================================================================ */
unsigned char ESP8266_open_tcp_connection(unsigned char _connection, const unsigned char _domain[], const unsigned char _port[], const unsigned char _time[], unsigned int _timeout){
    unsigned char _return = 0;
    unsigned char _temp = 0;
    
    _ESP8266_USART_send_string("AT+CIPSTART=");
    _ESP8266_USART_send_char('0' + _connection);
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_char(0x22);     //" 
    _ESP8266_USART_send_string("TCP");
    _ESP8266_USART_send_char(0x22);     //"
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_char(0x22);     //"
    _ESP8266_USART_send_string(_domain);
    _ESP8266_USART_send_char(0x22);     //"
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_string(_port);
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_string(_time);
    _ESP8266_USART_send_string("\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);
    
    if ((_return == ESP8266_RESP_CON_0) || (_return == ESP8266_RESP_CON_1) ||
        (_return == ESP8266_RESP_CON_2) || (_return == ESP8266_RESP_CON_3) ||
        (_return == ESP8266_RESP_CON_4)){
            
        _return = esp8266_buffer[0] - '0';
        ESP8266_clean_buffer(2);
            
        _temp = ESP8266_decode_resp();
            
        switch (_temp){
            case ESP8266_RESP_CONNECT_FAIL: _return |= ESP8266_CON_FAIL_OFFSET;
                break;
            case ESP8266_RESP_CONNECT:      _return |= ESP8266_CON_CONNECT_OFFSET;
                break;
        } 
        
    }
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_open_udp_connection
 * Input:           _connection: numero della connessione da usare 0~4 (unsigned char)
 *                  _domain[]: dominio a cui connettersi, indirizzo o IP (stringa)
 *                  _port[]: porta di connessione (stringa)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (impostare > 10000) (unsigned int)          
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x30: ESP8266_CON_0_FAIL (connessione 0 fallita)
 *                  - 0x31: ESP8266_CON_1_FAIL (connessione 1 fallita)
 *                  - 0x32: ESP8266_CON_2_FAIL (connessione 2 fallita)
 *                  - 0x33: ESP8266_CON_3_FAIL (connessione 3 fallita)
 *                  - 0x34: ESP8266_CON_4_FAIL (connessione 4 fallita)
 *                  - 0x40: ESP8266_CON_0_CONNECT (connessione 0 connessa)
 *                  - 0x41: ESP8266_CON_0_CONNECT (connessione 1 connessa)
 *                  - 0x42: ESP8266_CON_0_CONNECT (connessione 2 connessa)
 *                  - 0x43: ESP8266_CON_0_CONNECT (connessione 3 connessa)
 *                  - 0x44: ESP8266_CON_0_CONNECT (connessione 4 connessa)
 *                  - 0x22: ESP8266_RESP_ALREADY_CON (connessione gi esistente)
 *                  - 0x23: ESP8266_RESP_DNS_FAIL (indirizzo non trovato)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 * Overview:        Si connette via UDP al sito richiesto
 * Preconditions:   - Il modulo ESP8266 deve essere impostata in modalit stazione o Stazione + Soft AP (ESP8266_set_mode())
 *                  - Il modulo ESP8266 deve essere in modalit connessioni multiple (ESP8266_multiple_con())
 *                  - Il modulo ESP8266 deve essere connesso ad un Acces Point
 * Note:            Il comando pu essere usato anche in modalit connessione singola, ma non  implementato. Usare solo la modalit connessioni
 *                  multiple
/* ============================================================================================================================================ */
unsigned char ESP8266_open_udp_connection(unsigned char _connection, const unsigned char _domain[], const unsigned char _port[], unsigned int _timeout){
    unsigned char _return = 0;
    unsigned char _temp = 0;
    
    _ESP8266_USART_send_string("AT+CIPSTART=");
    _ESP8266_USART_send_char('0' + _connection);
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_char(0x22);     //" 
    _ESP8266_USART_send_string("UDP");
    _ESP8266_USART_send_char(0x22);     //"
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_char(0x22);     //"
    _ESP8266_USART_send_string(_domain);
    _ESP8266_USART_send_char(0x22);     //"
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_string(_port);
    _ESP8266_USART_send_string("\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);
    
    if ((_return == ESP8266_RESP_CON_0) || (_return == ESP8266_RESP_CON_1) ||
        (_return == ESP8266_RESP_CON_2) || (_return == ESP8266_RESP_CON_3) ||
        (_return == ESP8266_RESP_CON_4)){
            
        _return = esp8266_buffer[0] - '0';
        ESP8266_clean_buffer(2);
            
        _temp = ESP8266_decode_resp();
            
        switch (_temp){
            case ESP8266_RESP_CONNECT_FAIL: _return |= ESP8266_CON_FAIL_OFFSET;
                break;
            case ESP8266_RESP_CONNECT:      _return |= ESP8266_CON_CONNECT_OFFSET;
                break;
        } 
        
    }
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_close_connection
 * Input:           _connection: numero della connessione da chiudere 0~5 (5 = tutte le connessioni) (unsigned char)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (impostare > 10000) (unsigned int)          
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x24: ESP8266_RESP_UNLINK (connessione inesistente)
 *                  - 0x30: ESP8266_CON_0_FAIL (connessione 0 fallita)
 *                  - 0x31: ESP8266_CON_1_FAIL (connessione 1 fallita)
 *                  - 0x32: ESP8266_CON_2_FAIL (connessione 2 fallita)
 *                  - 0x33: ESP8266_CON_3_FAIL (connessione 3 fallita)
 *                  - 0x34: ESP8266_CON_4_FAIL (connessione 4 fallita)
 *                  - 0x50: ESP8266_CON_0_CLOSE (connessione 0 chiusa)
 *                  - 0x51: ESP8266_CON_1_CLOSE (connessione 1 chiusa)
 *                  - 0x52: ESP8266_CON_2_CLOSE (connessione 2 chiusa)
 *                  - 0x53: ESP8266_CON_3_CLOSE (connessione 3 chiusa)
 *                  - 0x54: ESP8266_CON_4_CLOSE (connessione 4 chiusa)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Chiude la connessione TCP o UDP selezionata
 * Preconditions:   - Il modulo ESP8266 deve essere impostata in modalit stazione o Stazione + Soft AP (ESP8266_set_mode())
 *                  - Il modulo ESP8266 deve essere in modalit connessioni multiple (ESP8266_multiple_con())
 *                  - Il modulo ESP8266 deve essere connesso ad un Acces Point
 * Note:            In disconnessione di tutte le connessioni torna in successione 0,CLOSE 2,CLOSE relativo alle connessioni disconnesse, in questo caso
 *                  leggere la risposta direttamente in esp8266_buffer
/* ============================================================================================================================================ */
unsigned char ESP8266_close_connection(unsigned char _connection, unsigned int _timeout){
    unsigned char _return = 0;
    unsigned char _temp = 0;
    
    _ESP8266_USART_send_string("AT+CIPCLOSE=");
    _ESP8266_USART_send_char('0' + _connection);
    _ESP8266_USART_send_string("\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);
    
    if ((_return == ESP8266_RESP_CON_0) || (_return == ESP8266_RESP_CON_1) ||
        (_return == ESP8266_RESP_CON_2) || (_return == ESP8266_RESP_CON_3) ||
        (_return == ESP8266_RESP_CON_4)){
            
        _return = esp8266_buffer[0] - '0';
        ESP8266_clean_buffer(2);
            
        _temp = ESP8266_decode_resp();
            
        switch (_temp){
            case ESP8266_RESP_CONNECT_FAIL: _return |= ESP8266_CON_FAIL_OFFSET;
                break;
            case ESP8266_RESP_CLOSE:        _return |= ESP8266_CON_CLOSE_OFFSET;
                break;
        } 
        
    }
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_send_data_over_con_l
 * Input:           _connection: numero della connessione alla quale inviare i dati 0~4 (unsigned char)
 *                  _length[]: lunghezza della stringa da inviare (stringa)
 *                  _string[]: stringa da inviare (stringa)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)              
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: Dati inviati correttamente
 *                  - 0x1F: ESP8266_RESP_SEND_FAIL (invio dati fallito)
 *                  - 0x20: ESP8266_RESP_LINK_NOT_VALID (connessione non valida)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Invia una stringa di dati dalla lunghezza fissa alla connessione selezionata.
 * Preconditions:   - Il modulo ESP8266 deve essere in modalit connessioni multiple (ESP8266_multiple_con())
 *                  - La connessione deve essere attiva
 * Note:            - Se si invia una stringa pi corta rispetto all lunghezza dichiarata il modulo resta bloccato finch non vengono inviati tutti i dati
 *                  - Impostare un timeout relativamente alto per assicurarsi di ricevere una risposta
/* ============================================================================================================================================ */
unsigned char ESP8266_send_data_over_con_l(unsigned char _connection, const unsigned char _length[], const unsigned char _string[], unsigned int _timeout){
    unsigned char _return = 0;
    unsigned int _length_int = 0;
    
    _length_int = atoi (_length);
    
    _ESP8266_USART_send_string("AT+CIPSEND=");   
    _ESP8266_USART_send_char('0' + _connection);
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_string(_length);
    _ESP8266_USART_send_string("\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);
    
    if (_return == ESP8266_RESP_OK){                                                    // primo OK
        ESP8266_clean_buffer(2);
        _return = ESP8266_decode_resp();                                                // >
        if (_return == ESP8266_RESP_WRAP_RET)
            _ESP8266_USART_send_n_chars(_string, _length_int);                          // invio stringa
    }
    _return =  ESP8266_wait_resp(_timeout);                                             // qui ricevo quanti dati ho inviato, informazione trascurata
    _return =  ESP8266_wait_decode_resp(_timeout);                                      // qui decodifico SEND OK o SEND FAIL o altri messaggi d'errore              
    
    if (_return == ESP8266_RESP_SEND_OK)
        _return = 1;

    ESP8266_reset_buffer();                                                             // resetto il buffer dopo l'invio
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_send_data_over_con
 * Input:           _connection: numero della connessione alla quale inviare i dati 0~4 (unsigned char)
 *                  _length[]: lunghezza della stringa da inviare (stringa)
 *                  _string[]: stringa da inviare (stringa)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)              
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: Dati inviati correttamente
 *                  - 0x1F: ESP8266_RESP_SEND_FAIL (invio dati fallito)
 *                  - 0x20: ESP8266_RESP_LINK_NOT_VALID (connessione non valida)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Invia una stringa di dati alla connessione selezionata. La trasmissione verr terminata a fine stringa o alla lunghezza impostata.
 * Preconditions:   - Il modulo ESP8266 deve essere in modalit connessioni multiple (ESP8266_multiple_con())
 *                  - La connessione deve essere attiva
 * Note:            - Assicurarsi che la lunghezza impostata copra la lunghezza massima della stringa da inviare
 *                  - Impostare un timeout relativamente alto per assicurarsi di ricevere una risposta
/* ============================================================================================================================================ */
unsigned char ESP8266_send_data_over_con(unsigned char _connection, const unsigned char _length[], const unsigned char _string[], unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+CIPSENDEX=");   
    _ESP8266_USART_send_char('0' + _connection);
    _ESP8266_USART_send_char(',');
    _ESP8266_USART_send_string(_length);
    _ESP8266_USART_send_string("\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);
    
    if (_return == ESP8266_RESP_OK){
        ESP8266_clean_buffer(2);
        _return = ESP8266_decode_resp();
        if (_return == ESP8266_RESP_WRAP_RET)
            _ESP8266_USART_send_string(_string);
            _ESP8266_USART_send_string("\\0");
    }
    
    _return =  ESP8266_wait_resp(_timeout);                                             // qui ricevo quanti dati ho inviato, informazione trascurata
    _return =  ESP8266_wait_decode_resp(_timeout);                                      //qui decodifico SEND OK o SEND FAIL o altri messaggi d'errore      
    
    if (_return == ESP8266_RESP_SEND_OK)
        _return = 1;

    ESP8266_reset_buffer();
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_check_connection_status
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)              
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0xC2: ESP8266_CON_AP_AND_IP (stazione connessa ad accesso point ed indirizzo IP ottenuto)
 *                  - 0xC3: ESP8266_STA_TCP_UDP_TRA (connessione TCP o UDP attiva)
 *                  - 0xC4: ESP8266_STA_TCP_UDP_TRA_DIS (connessione TCP o UDP disconnessa)
 *                  - 0xC5: ESP8266_STA_NOT_CON_AP (stazione non connessa ad Access Point)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Controlla lo stato della connessione
 * Note:            In esp8266_buffer torna eventuali valori della connessione
 *                  es: <link ID>, <type>, <remote_IP>, <remote_port>, <local_port>, <tetype>    
/* ============================================================================================================================================ */
unsigned char ESP8266_check_connection_status(unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+CIPSTATUS\r\n");
    _return =  ESP8266_wait_decode_resp(_timeout);
    
    if (_return == ESP8266_RESP_STATUS){
        _return = (esp8266_buffer[7] - '0') + ESP8266_STAUTS_OFFSET;
        ESP8266_clean_buffer(19);
        
    }
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_ping
 * Input:           _domain[]: dominio a cui connettersi, indirizzo o IP (stringa)
 *                  _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)              
 * Output:          - 0x00: ESP8266_RESP_ERROR (il modulo ESP8266 ha restituito ERROR)
 *                  - 0x01: Ping eseguito
 *                  - 0x25: ESP8266_RESP_PING_TO (timeout ping)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 *
 * Overview:        Esegue un ping all'indirizzo richiesto
 * Preconditions:   Il modulo ESP8266 deve essere connesso ad un Acces Point
 * Note:            In esp8266_buffer torna il tempo del ping in ms:
 *                  es:214        
/* ============================================================================================================================================ */
unsigned char ESP8266_ping(const unsigned char _domain[], unsigned int _timeout){
    unsigned char _return = 0;
    
    _ESP8266_USART_send_string("AT+PING=");
    _ESP8266_USART_send_char(0x22);     //"   
    _ESP8266_USART_send_string(_domain);   
    _ESP8266_USART_send_char(0x22);     //"   
    _ESP8266_USART_send_string("\r\n");
    
    _return =  ESP8266_wait_decode_resp(_timeout);
    
    if (_return == ESP8266_RESP_PING){
        ESP8266_clean_buffer(1);
        _return = 1;
    }
    return _return;
} 
   
/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_check_incoming_data
 * Output:          - 0x30: ESP8266_CON_0_FAIL (connessione 0 fallita)
 *                  - 0x31: ESP8266_CON_1_FAIL (connessione 1 fallita)
 *                  - 0x32: ESP8266_CON_2_FAIL (connessione 2 fallita)
 *                  - 0x33: ESP8266_CON_3_FAIL (connessione 3 fallita)
 *                  - 0x34: ESP8266_CON_4_FAIL (connessione 4 fallita)
 *                  - 0x40: ESP8266_CON_0_CONNECT (connessione 0 connessa)
 *                  - 0x41: ESP8266_CON_0_CONNECT (connessione 1 connessa)
 *                  - 0x42: ESP8266_CON_0_CONNECT (connessione 2 connessa)
 *                  - 0x43: ESP8266_CON_0_CONNECT (connessione 3 connessa)
 *                  - 0x44: ESP8266_CON_0_CONNECT (connessione 4 connessa)
 *                  - 0x50: ESP8266_CON_0_CONNECT (connessione 0 chiusa)
 *                  - 0x51: ESP8266_CON_0_CONNECT (connessione 1 chiusa)
 *                  - 0x52: ESP8266_CON_0_CONNECT (connessione 2 chiusa)
 *                  - 0x53: ESP8266_CON_0_CONNECT (connessione 3 chiusa)
 *                  - 0x54: ESP8266_CON_0_CONNECT (connessione 4 chiusa)
 *                  - 0x60: ESP8266_CON_0_SEND (dati ricevuti da connessione 0)
 *                  - 0x61: ESP8266_CON_0_SEND (dati ricevuti da connessione 1)
 *                  - 0x62: ESP8266_CON_0_SEND (dati ricevuti da connessione 2)
 *                  - 0x63: ESP8266_CON_0_SEND (dati ricevuti da connessione 3)
 *                  - 0x64: ESP8266_CON_0_SEND (dati ricevuti da connessione 4)
 *                  - 0xF0: ESP8266_NO_INCOMING_DATA (non sono stati ricevuti dati)
 *                  - 0xFF: ESP8266_RESP_NOT_HANDLED (risposta ricevuta ma non gestita)
 * Overview:        Verifica la presenza di nuovi dati in arrivo
 * Note:            In caso di nuovi dati in arrivo diversi dallo stato della connessione (CONNECT/FAIL/CLOSE) i dati ricevuti sono in esp8266_buffer
/* ============================================================================================================================================ */
unsigned char ESP8266_check_incoming_data(void){
    unsigned char _return = ESP8266_NO_INCOMING_DATA;
    unsigned char _temp = 0;

    if (esp8266_string_completed) {
        _return = ESP8266_decode_resp();
        
        if ((_return == ESP8266_RESP_CON_0) || (_return == ESP8266_RESP_CON_1) ||       // se i dati ricevuti sono 0, 1, 2, 3, 4,
            (_return == ESP8266_RESP_CON_2) || (_return == ESP8266_RESP_CON_3) ||
            (_return == ESP8266_RESP_CON_4)){
            
            _return = esp8266_buffer[0] - '0';                                          // formo il byte di ritorno, i 4 bit bassi identificano il numero della connessione
            ESP8266_clean_buffer(2);
            
            _temp = ESP8266_decode_resp();
            
            switch (_temp){                                                             // i 4 bit alti identificano la connessione e disconnessione
                case ESP8266_RESP_CONNECT_FAIL: _return |= ESP8266_CON_FAIL_OFFSET;
                    break;
                case ESP8266_RESP_CONNECT:      _return |= ESP8266_CON_CONNECT_OFFSET;
                    break;
                case ESP8266_RESP_CLOSE:        _return |= ESP8266_CON_CLOSE_OFFSET;
                    break;
            } 
        
        }
    
        if (_return == ESP8266_RESP_IPD){                                               // se la risposta  IPD sono arrivati dei dati
            _return = esp8266_buffer[5] - '0' + ESP8266_CON_SEND_OFFSET;                // torno il numero della connessione che ha inviato
            ESP8266_clean_buffer_until_char(':');                                       // e nel buffer rimangono i dati inviati
        }
    }
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_wait_resp
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * Output:          - 0xFB: ESP8266_RESP_RECEIVED (risposta ricevuta)
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)
 *
 * Overview:        Resetta il buffer di ricezione e attende il tempo di timeout o il flag di risposta ricevuta
/* ============================================================================================================================================ */
unsigned char ESP8266_wait_resp(unsigned int _timeout){
    unsigned char _return = ESP8266_RESP_TIMEOUT;
    unsigned int _counter = 0;
    
    ESP8266_reset_buffer();
        
// Attesa timeout o risposta ricevuta
    do {                                                                                // ciclo di delay fino a _timeout raggiunto o fino a esp8266_string_completed = 1 (impostata in interrupt di ricezione)
        Delay_ms(1);
        _counter ++;
    }  while ((_counter < _timeout) && (esp8266_string_completed == 0));   
    
    if (esp8266_string_completed)
        _return = ESP8266_RESP_RECEIVED;

    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_decode_resp                  
 * Output:          tabella excel
 *
 * Overview:        Decodifica la risposta ricevuta
/* ============================================================================================================================================ */
unsigned char ESP8266_decode_resp(void){
    unsigned char _return = ESP8266_RESP_NOT_HANDLED;

    const unsigned char _resp_2_check[RESP_HANDLED_1][RESP_MAX_LENGTH_1] = {RESPONSES_1}; // array di confronto
    unsigned char _resp_length = 0;
    
    // Pulisco la stringa eliminando tutti /r /n
    if (!esp8266_buffer_cleaned){                                                       // pulisco solo 1 volta nei casi in cui la risposta viene decodificata pi volte (per esempio dopo cancellazione parziale del buffer)
        for (unsigned char _i = 0; esp8266_buffer[_i] != '\0'; _i++) {
            if(esp8266_buffer[_i] == '\r' || esp8266_buffer[_i] == '\n'){
                for(unsigned char _l = _i; esp8266_buffer[_l] != '\0'; _l++){
                    esp8266_buffer[_l] = esp8266_buffer[_l+1];
                } 
                _i--;
            }
        }
    esp8266_buffer_cleaned = 1;
    }
    
#ifdef DEBUG_RESEND_ON_UART_CLEANED
    _ESP8266_deb_USART_send_string(esp8266_buffer);
#endif  

    for (unsigned char _i = 0; _i <RESP_HANDLED_1; _i++) {                              // ciclo per tutte le risposte nell'array di confronto
        for (_resp_length = 0; _resp_2_check[_i][_resp_length] != '\0'; _resp_length++);// conto i caratteri che compongono la stringa di confronto
        
        for (unsigned char _l=0; _l<_resp_length; _l++){                                // ciclo fino a fine stringa di confronto                     
            if (esp8266_buffer[_l] != _resp_2_check[_i][_l])                            // al primo byte diverso si interrompe
                break;                           
            if (_l == _resp_length-1) {                                                 // verifico se numero di caratteri corretti = lunghezza stringa
                    _return = _i;                                                       // se tutto verificato torno il numero della stringa verificata
            }
        }
        if (_return != ESP8266_RESP_NOT_HANDLED)                                        // se ho trovato una corrispondenza interrompo il ciclo di confronto
            break;
    }
                                                                                        // se nel primo array di risposte non ho trovato corrispondenze
    if (_return == ESP8266_RESP_NOT_HANDLED){                                           // ciclo di controllo secondo array risposte

        const unsigned char _resp_2_check[RESP_HANDLED_2][RESP_MAX_LENGTH_2] = {RESPONSES_2};
        
        for (unsigned char _i = 0; _i <RESP_HANDLED_2; _i++) {                          // ciclo per tutte le risposte nell'array di confronto
            for (_resp_length = 0; _resp_2_check[_i][_resp_length] != '\0'; _resp_length++);    // conto i caratteri che compongono la stringa di confronto
            for (unsigned char _l=0; _l<_resp_length; _l++){                            // ciclo fino a fine stringa di confronto                          
                if (esp8266_buffer[_l] != _resp_2_check[_i][_l])                        // al primo byte diverso si interrompe
                    break;                           
                if (_l == _resp_length-1) {                                             // verifico numero di caratteri corretti = lunghezza stringa
                        _return = _i;                                                   // se tutto verificato torno il numero della stringa verificata
                }
            }
            if (_return != ESP8266_RESP_NOT_HANDLED){
                _return += RESP_HANDLED_1;                                              // sommo offset risposte secondo array
                break;
            }
        }
    
    }
    return _return;   
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_wait_decode_resp
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)
 * Output:          Codifica ritorno risposte decodificate da ESP8266_decode_resp()
 *
 * Overview:        Attende il tempo di timeout o il flag di risposta ricevuta, a risposta ricevuta pulisce la stringa da /r/n e spazi e decodifica la risposta.
 *                  se la risposta  busy p... attendo un'altra risposta
/* ============================================================================================================================================ */
unsigned char ESP8266_wait_decode_resp(unsigned int _timeout){
    unsigned char _return = 0;
    
    do {
        _return = ESP8266_wait_resp(_timeout);
        if (_return == ESP8266_RESP_RECEIVED)
            _return = ESP8266_decode_resp();
    } while (_return == ESP8266_RESP_BUSY);                                             // se la risposta  busy attendo nuova risposta
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_check_ready
 * Input:           _timeout: tempo massimo in millisecondi di attesa della risposta da parte del modulo (unsigned int)                  
 * Output:          - 0xFC: EPS8266_NOT_READY (modulo ESP8266 ha dato risposta ma non  stata trovata la parola ready)
 *                  - 0xFD: EPS8266_READY (modulo ESP8266 pronto)         
 *                  - 0xFE: ESP8266_RESP_TIMEOUT (risposta non ricevuta, timeout)

 *
 * Overview:        Cerca la parola ready alla fine del buffer di ricezione
/* ============================================================================================================================================ */
unsigned char ESP8266_check_ready(unsigned int _timeout){
    unsigned char _return = ESP8266_RESP_TIMEOUT;
    unsigned int _counter = 0;
        
    do {                                                                                // ciclo di delay fino a _timeout raggiunto o fino a esp8266_string_completed = 1 (impostata in interrupt di ricezione)
        Delay_ms(1);
        _counter ++;
        
    }  while (_counter < _timeout);
    
    if (esp8266_string_completed){
        if ( esp8266_buffer[esp8266_buffer_index-1] == 0x0A &&                          // fine stringa ready
             esp8266_buffer[esp8266_buffer_index-2] == 0x0D &&
             esp8266_buffer[esp8266_buffer_index-3] == 'y' &&
             esp8266_buffer[esp8266_buffer_index-4] == 'd')
            _return = ESP8266_READY;
        else 
            _return = ESP8266_NOT_READY;
    }
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_reset_buffer
 *
 * Overview:        Azzera il buffer di ricezione
/* ============================================================================================================================================ */
void ESP8266_reset_buffer(void){
    esp8266_buffer_index = 0;                                                           // azzero indice buffer e flag data_ready
    esp8266_string_completed = 0;
    esp8266_buffer_cleaned = 0;
    for (unsigned int _i = 0; _i < ESP8266_BUFFER_LENGTH; _i++)                         // azzero l'array della risposta ricevuta
    {
        esp8266_buffer[_i] = 0;
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_clean_buffer
 * Input:           _n: numero di byte da cancellare all'inizio del buffer di ricezione (unsigned char)
 *
 * Overview:        Cancella parte del buffer di ricezione spostando all'inizio i byte rimanenti
/* ============================================================================================================================================ */
void ESP8266_clean_buffer(unsigned char _n){
    for (unsigned char _i = 0;  esp8266_buffer[_i] !='\0'; _i++){
        esp8266_buffer[_i] = esp8266_buffer[_i + _n];     
    }
}  

/* ============================================================================================================================================= /
 * FUNCTION:        ESP8266_clean_buffer_until_char
 * Input:           _char: ultimo carattere da cancellare (unsigned char)
 *
 * Overview:        Cancella il buffer di ricezione fino al carattere indicato spostando all'inizio i byte rimanenti (carattere compreso)
/* ============================================================================================================================================ */
void ESP8266_clean_buffer_until_char(unsigned char _char){
    unsigned char _counter = 0;
    
    for(_counter = 0; esp8266_buffer[_counter] != _char; _counter++);                   // conto quanti caratteri ci sono prima del primo carattere valido
    _counter++;
    
    ESP8266_clean_buffer(_counter);                                                     // cancello il buffer fino al carattere valido
}





/* ============================================================================================================================================= /
 * FUNCTION:        Ciclo interrupt da integrare in main
 *
 * Overview:        Alla ricezione di un byte sulla linea UART dedicata salva il byte nel buffer di ricezione esp8266_buffer e il timer dedicato
 *                  viene avviato/resettato.
 *                  Se il timer genera un interrupt significa che non si stanno pi ricevendo dati da 5ms quindi la stringa  completa, viene
 *                  settato il flag esp8266_string_completed e il timer si ferma
 *                  
/* ============================================================================================================================================ */
//void high_priority interrupt inter_high(void) {
//    if (ESP8266_INTERRUPT_IE && ESP8266_INTERRUPT_IF) {                               // se l'interrupt  stato causato da una ricezione dati USART
//        
//        esp8266_buffer[esp8266_buffer_index] = _ESP8266_USART_receive();                // salvo i dati ricevuti nell'array usart_received
//        
//#ifdef DEBUG_RESEND_ON_UART_RAW
//    _ESP8266_deb_USART_send_char(esp8266_buffer[esp8266_buffer_index]);
//#endif  
//        
//        esp8266_buffer_index++;
//        
//        if (esp8266_buffer_index > ESP8266_BUFFER_LENGTH)                               // se supero la dimensione massima resetto l'indice
//            esp8266_buffer_index = 0;    
// 
//        ESP8266_TMR = ESP8266_TIMER_PRELOAD;                                            // Resetto e avvio il timer
//        ESP8266_TIMER_ON = ON;                                                          // avvio il timer (se spento: primo byte ricevuto)
//        
//        //        ESP8266_INTERRUPT_IF = 0;                                                // Resetta gi USART_init
//    }
//    
//    else if (ESP8266_TIMER_INT_IE && ESP8266_TIMER_INT_IF) {                            // interrupt timer significa che non ricevo byte da 5ms   
//        ESP8266_TMR = ESP8266_TIMER_PRELOAD;
//        
//        esp8266_string_completed = 1;                                                   // setto il flag di ricezione completata
//        ESP8266_TIMER_ON = OFF;                                                         // spengo timer
//        ESP8266_TIMER_INT_IF = 0;
//    }
//}


#endif
