/* ============================================================================================================================================= /
 * LIBRARY:         Decodifica meteo con chip HKW581
 * FILE:            HKW581.c
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         04/10/2016
 * UPDATED:         02/11/2016
 *
 * DESCRIP:         Decodifica del i bit meteo del segnale radio orario DCF77, da usare con libreria DCF77.c
 *
 * FUNCTIONS:              
/* ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.0               02/11/2016          - Prima release
  1.0               04/10/2016          - Inizio sviluppo
//! [History]
/* ============================================================================================================================================ */

#ifndef HKW581
#define HKW581

/* VARIABILI GLOBALI LIBRERIA ================================================================================================================= */
/** \brief struttura contenente i dati meteo decriptati */
typedef struct {
    unsigned char day_fc;               // previsioni meteo giorno
    unsigned char night_fc;             // previsioni meteo notte
    unsigned char extreme_w_wind_dir;   // dato tempo pesante (trasmissioni con t.max) e direzione del vento (trasmissioni con t.min)
    unsigned char rain_pr_wind_str;     // dato probabilit pioggia (trasmissioni con t.max) e forza del vento (trasmissioni con t.min)
    unsigned char anomaly;              // anomalia meteo  (bit 15)
    signed char temp_fc;                // temperatura massima (trasmissioni con t.max) e minima (trasmissioni con t.min)
    unsigned char decoder_status;       // decoder status, 0x01 = OK
}hwk581_decoded_data;

/* INCLUDES =================================================================================================================================== */
#include    "HKW581_settings.h"
#include    "HKW581.h"
#include    "HKW581_decode_tables.h"

/* ============================================================================================================================================= /
 * FUNCTION:    HKW581_init
 *
 * Overview:    Inizializzazione delle linee clock e data
/* ============================================================================================================================================ */
void HKW581_init(void){
    HKW581_CLK_IN = 0;
    HKW581_DATA_IN = 0;
    
#ifdef HKW581_RST    
    HKW581_RST = 0;
    Delay_ms(100);
    HKW581_RST = 1;
#endif
}

/* ============================================================================================================================================= /
 * FUNCTION:    HKW581_stop
 *
 * Overview:    Resetta le linee di clock e data e disabilita il chip
/* ============================================================================================================================================ */
void HKW581_stop(void){
    HKW581_CLK_IN = 0;
    HKW581_DATA_IN = 0;
#ifdef HKW581_RST     
    HKW581_RST = 0;
#endif
}

/* ============================================================================================================================================= /
 * FUNCTION:    HKW581_read_brf
 * Output:      0: chip non pronto
 *              1: chip pronto
 *
 * Overview:    Legge lo stato del PIN BRF, torna a 1 quando BRF va a 0 per indicare che il chip  pronto a ricevere i dati
/* ============================================================================================================================================ */
unsigned char HKW581_read_brf(void){
    unsigned char _return = 0;
    
    if (HKW581_BRF == 0) {
        Delay_ms(1);                                                                    // filtro anti rimbalzo, ho notato degli spike a massa ogni 2 secondi.
        if (HKW581_BRF == 0)
            _return = 1;
    }
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    HKW581_decode_meteo_data
 * Input:       - _meteo_bits_1: 14 bit meteo del primo minuto (unsigned int)
 *              - _meteo_bits_2: 14 bit meteo del secondo minuto (unsigned int)
 *              - _meteo_bits_3: 14 bit meteo del terzo minuto (unsigned int)
 *              - _minute: minuto (secondo minuto) (unsigned char)
 *              - _hours: ora (unsigned char)
 *              - _day: giorno (unsigned char)
 *              - _month: mese (unsigned char)
 *              - _day_week: giorno della settimana (unsigned char)
 *              - _year: anno (unsigned char)
 * Output:      restituisce una struct di variabili contenente i dati del meteo decodificati (dcf77_decoded_data)
 * Note:        i dati di ritorno vanno interpretati a seconda dell'ora di ricezione (probabilit pioggia, meteo estremo e vento condividono le stesse variabili di ritorni dati meteo per diversi giorni sono inviati in fasce orarie diverse)
 * Overview:    Invia i dati in ingresso al chip, torna il dato decodificato dal chip
/* ============================================================================================================================================ */
hwk581_decoded_data HKW581_decode_meteo_data(unsigned int _meteo_bits_1,unsigned int _meteo_bits_2,unsigned int _meteo_bits_3, unsigned char _minute, unsigned char _hour, unsigned char _day, unsigned char _month, unsigned char _day_w, unsigned char _year) {
    unsigned long _decoded_data_raw = 0;
    hwk581_decoded_data _decoded_data;
    
    while (!HKW581_read_brf());                                                         // attendo che il chip sia pronto a ricevere (Bloccante)
    
    // i dati meteo arrivano gi in ordine giusto (cio come sono stati ricevuti)
    // gli altri dati vanno riportati in ordine di ricezione e vanno aggiunti 0 per fare 8 bit o 5 bit
    
    // Inverto i dati di minuto, ora, giorno anno per riportarli allo stato originale dopo la ricezione
    _minute = Invert_8(_minute);
    _hour = Invert_8(_hour);
    _day = Invert_8(_day);
    _year = Invert_8(_year);
    
    // inverto mese e giorno settimane ed elimino i bit in pi
    _month = Invert_8(_month) >> 3;
    _day_w = Invert_8(_day_w) >> 5;
    
    // invio i pacchetti all' HKW581, totale 82 bit
    HKW581_send_data_packet(_meteo_bits_1, 14);
    HKW581_send_data_packet(_meteo_bits_2, 14);
    HKW581_send_data_packet(_meteo_bits_3, 14);
    HKW581_send_data_packet(_minute, 8);
    HKW581_send_data_packet(_hour, 8);
    HKW581_send_data_packet(_day, 8);
    HKW581_send_data_packet(_month, 5);
    HKW581_send_data_packet(_day_w, 3);
    HKW581_send_data_packet(_year, 8);

    // ricevo i dati dall'HKW581
    for (unsigned char _i = 0; _i < 24; _i++){ 
        while (!HKW581_Q_OUT);
        _decoded_data_raw |= (unsigned long) (HKW581_DATA_OUT & 0x01) << _i ;           // Primo bit ricevuto  LSB, in questo modo ho poi tutti i dati gi in ordine corretto
        HKW581_CLK_IN = 1;
        while (HKW581_Q_OUT);
        HKW581_CLK_IN = 0;
    }
    
   // decodifico i dati ricevuti nelle variabili della struct
    _decoded_data.day_fc = _decoded_data_raw & 0b00001111;
    _decoded_data.night_fc = (_decoded_data_raw >> 4) & 0b00001111;
    _decoded_data.extreme_w_wind_dir = (_decoded_data_raw >> 8) & 0b00001111;
    _decoded_data.rain_pr_wind_str = (_decoded_data_raw >> 12) & 0b00000111;
    _decoded_data.anomaly = (_decoded_data_raw >> 15) & 0b00000001;
    _decoded_data.temp_fc = ((_decoded_data_raw >> 16) & 0b00111111) -22;
    _decoded_data.decoder_status = (_decoded_data_raw >> 22) & 0b00000011;
    
     return _decoded_data;                                                              // torno la struct
}

/* ============================================================================================================================================= /
 * FUNCTION:    Invert_8
 * Input:       _data: byte da invertire (unsigned char)
 * Output       byte con bit invertiti (unsigned char)
 *
 * Overview:    Inverte i bit MSB->LSB e viceversa di 1 byte
/* ============================================================================================================================================ */
unsigned char Invert_8(unsigned char _data){
    unsigned char _inverted = 0;
    _inverted = (_data & 0b10000000)  >> 7;
    _inverted |= (_data & 0b01000000) >> 5;
    _inverted |= (_data & 0b00100000) >> 3;
    _inverted |= (_data & 0b00010000) >> 1;
    _inverted |= (_data & 0b00001000) << 1;
    _inverted |= (_data & 0b00000100) << 3;
    _inverted |= (_data & 0b00000010) << 5;
    _inverted |= (_data & 0b00000001) << 7;
    
    return _inverted;
}

/* ============================================================================================================================================= /
 * FUNCTION:    HKW581_send_data_packet
 * Input        _data: dato da inviare (unsigned int, o char)
 *              _bits: quantit di bit da inviare (unsigned char)
 *
 * Overview:    Invia i dati dall0HKW581
/* ============================================================================================================================================ */
void HKW581_send_data_packet(unsigned int _data, unsigned char _bits) {
    
    // invio i singoli bit, MSB first
    for (unsigned char _i = 0; _i < _bits; _i++) {
        HKW581_DATA_IN = (_data >> _bits-1 - _i) & 0x01;
        HKW581_CLK_IN = 1;
        while (!HKW581_Q_OUT);
        HKW581_CLK_IN = 0;
        while (HKW581_Q_OUT);
    }
}  

#endif          