/* ============================================================================================================================================= /
 * LIBRARY:         RTCC modulo interno PIC18F6x_8x
 * FILE:            RTCC_PIC18F6x_8x.c
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F6xxxx / PIC18F8xxxx
 * CREATED:         07/03/2017
 * UPDATED:         09/03/2017
 *
 * DESCRIP:         Gestione real time clock calendar modulo interno PIC18F6xxxx / PIC18F8xxxx
 *
 * FUNCTIONS:       RTCC_init()
                    RTCC_osc()
                    RTCC_osc_trim()
                    RTCC_read_all()
                    RTCC_write_all()
                    RTCC_read_seconds()
                    RTCC_read_minutes()
                    RTCC_read_hours()
                    RTCC_read_day_w()
                    RTCC_read_day()
                    RTCC_read_month()
                    RTCC_read_year()
                    RTCC_write_seconds()
                    RTCC_write_minutes()
                    RTCC_write_hours()
                    RTCC_write_day_w()
                    RTCC_write_day()
                    RTCC_write_month()
                    RTCC_write_year()
                    RTCC_alarm()
                    RTCC_alarm_mode()
                    RTCC_write_all_al()
                    RTCC_read_seconds_al()
                    RTCC_read_minutes_al()
                    RTCC_read_hours_al()
                    RTCC_read_day_w_al()
                    RTCC_read_day_al()
                    RTCC_read_month_al()
                    RTCC_write_seconds_al()
                    RTCC_write_minutes_al()
                    RTCC_write_hours_al()
                    RTCC_write_day_w_al()
                    RTCC_write_day_al()
                    RTCC_write_month_al()
/* ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.0               09/03/2017          - Prima build completa.
  1.0               07/03/2017          - Inizio studio
//! [History]
/* ============================================================================================================================================ */

/*
 * NOTA SUGLI ALLARMI:
 * i bit di settaggio allarmi non si sommano tra di loro ma sono ad esclusione, per esempio se imposto ALMxMSK si 010 l'interrupt scatter  ad ogni
 * ora specificata di ogni giorno e data e rester  attivo fino all'ora successiva.
 * Per impostare un all'arme all'ora, minuto, secondi va sempre impostato ALMxMSK a 111 avendo cura di impostare di volta in volta il giorno e il mese
 */

#ifndef RTCC_PIC18F6x_8x
#define RTCC_PIC18F6x_8x

/* VARIABILI GLOBALI LIBRERIA================================================================================================================== */
typedef struct {                                                                        // struct valori raw
    unsigned char seconds;
    unsigned char minutes;
    unsigned char hours;
    unsigned char day_w;
    unsigned char day;                  // giorno mese
    unsigned char month;                // mese
    unsigned char year;                 // anno
} rtcc_data; 


/* INCLUDES =================================================================================================================================== */
//#include "RTCC_PIC18F6x_8x_settings.h"
#include "RTCC_PIC18F6x_8x.h"
#include <xc.h>   

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_init
 * Input:           _out_mode: modalit di uscita del pin dedicato: RTCC_OUT_CLOCK_SOURCE/RTCC_OUT_SECOND/RTCC_OUT_ALARM/RTCC_OUT_DISABLED (unsigned char)
 *                  _al_interrupt: abilitazione e priorit interrupt allarme: RTCC_AL_INTERRUPT_PR_HIGH/RTCC_AL_INTERRUPT_PR_LOW/RTCC_AL_INTERRUPT_DISABLED (unsigned char)
 *
 * Overview:        Inizializza il modulo RTCC ed imposta la modalit di funzionamento del pin di uscita
/* ============================================================================================================================================ */
void RTCC_init(unsigned char _out_mode, unsigned char _al_interrupt){
    
    // configurazione pin di uscita
    if (_out_mode == RTCC_OUT_DISABLED){
        RTCCFGbits.RTCOE = 0;
    } else {
        PADCFG1bits.RTSECSEL = _out_mode;
        RTCCFGbits.RTCOE = 1;
    }
    
    // configurazione interrupt
    if (_al_interrupt == RTCC_AL_INTERRUPT_DISABLED) {
        PIE3bits.RTCCIE = 0;    
    } else {
        PIR3bits.RTCCIF = 0;
        IPR3bits.RTCCIP = _al_interrupt;
        PIE3bits.RTCCIE = 1;   
    }
    
    // impostazione registri ora e allarme a valori standard
    RTCC_write_all(0x28,0x03,0x43,0x00,0x00,0x01,0x00);
    RTCC_write_all_al(0,0,0,0,0,0);
    ALRMCFG = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_osc
 * Input:           _on_off: RTCC abilitato o disabilitato: ON/OFF (unsigned char)
 *
 * Overview:        Avvia l'oscillatore o ferma l'RTCC
 * Preconditions:   Impostare la sorgente del clock nei registri di configurazione RTCOSC bit (CONFIG3L<0>).
/* ============================================================================================================================================ */
void RTCC_osc(signed char _on_off){
    EECON2 = 0x55;                                                                      // abilito la scrittura di RTCWREN
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    RTCCFGbits.RTCEN = _on_off;                                                         // attivo o disattivo
    
    EECON2 = 0x55;                                                                      // disabilito la scrittura
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_osc_trim
 * Input:           _factor: parametro di calibrazione -127 ~ +127 (si aggiungono o tolgono 4 cicli di clock al minuto per bit) (signed char)
 *
 * Overview:        Imposta il registro di calibrazione
/* ============================================================================================================================================ */
void RTCC_osc_trim(signed char _factor){
    RTCCAL = _factor;
} 

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_all
 * Output:          Struct contenente i valori di data e ora codificati in packed BCD
 *
 * Overview:        Legge data e ora dal RTCC
/* ============================================================================================================================================ */
rtcc_data RTCC_read_all(void){
    rtcc_data _return;
    unsigned char _flush;
    
    RTCCFGbits.RTCPTR = RTCC_PTR_YEAR;                                                  // puntatore si auto decrementa ogni volta che leggo RTCVALH 
    _return.year = RTCVALL;
    _flush = RTCVALH;                                                                   // leggo solo per far decrementare il puntatore
    _return.day = RTCVALL;
    _return.month = RTCVALH;
    _return.hours = RTCVALL;
    _return.day_w = RTCVALH;
    _return.seconds = RTCVALL;
    _return.minutes = RTCVALH;
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_write_all
 * Input:           _day: valore del giorno da scrivere nell' RTCC (packed BCD)
 *                  _month: valore del mese da scrivere nell' RTCC (packed BCD)
 *                  _year: valore dell'anno da scrivere nell' RTCC (packed BCD)
 *                  _day_w: valore del giorno della settimana da scrivere nell' RTCC (packed BCD)
 *                  _hours: valore delle ora da scrivere nell' RTCC (packed BCD)
 *                  _minutes: valore dei minuti da scrivere nell' RTCC (packed BCD)
 *                  _seconds: valore dei secondi da scrivere nell' RTCC (packed BCD)
 *
 * Overview:        Scrive i valori di data e ora nel RTCC
/* ============================================================================================================================================ */
void RTCC_write_all(unsigned char _day, unsigned char _month, unsigned char _year, unsigned char _day_w,  unsigned char _hours,  unsigned char _minutes, unsigned char _seconds){
    unsigned char _flush;
    
    EECON2 = 0x55;                                                                      // abilito la scrittura dei registri
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    RTCCFGbits.RTCPTR = RTCC_PTR_YEAR;                                                  // puntatore si auto decrementa ogni volta che scrivo RTCVALH
    
    RTCVALL = _year;
    _flush = RTCVALH;                                                                   // leggo solo per far decrementare il puntatore
    RTCVALL = _day;
    RTCVALH = _month;
    RTCVALL = _hours;
    RTCVALH = _day_w;
    RTCVALL = _seconds;
    RTCVALH = _minutes;
    
    EECON2 = 0x55;                                                                      // disabilito la scrittura dei registri
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_seconds
 * Output:          valore dei secondi (packed BCD)
 *
 * Overview:        Legge i secondi dal RTCC
/* ============================================================================================================================================ */
unsigned char RTCC_read_seconds(void) {
    RTCCFGbits.RTCPTR = RTCC_PTR_SEC_MIN;
    return RTCVALL & 0b01111111;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_minutes
 * Output:          valore dei minuti (packed BCD)
 *
 * Overview:        Legge i minuti dal RTCC 
/* ============================================================================================================================================ */
unsigned char RTCC_read_minutes(void) {
    RTCCFGbits.RTCPTR = RTCC_PTR_SEC_MIN;
    return RTCVALH & 0b01111111;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_hours
 * Output:          valore delle ore (packed BCD)
 *
 * Overview:        Legge le ore dal RTCC 
/* ============================================================================================================================================ */
unsigned char RTCC_read_hours(void) {
    RTCCFGbits.RTCPTR = RTCC_PTR_HOU_WD;
    return RTCVALL & 0b00111111;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_day_w
 * Output:          valore del giorno della settimana (packed BCD)
 *
 * Overview:        Legge il giorno della settimana dal RTCC 
/* ============================================================================================================================================ */
unsigned char RTCC_read_day_w(void) {
    RTCCFGbits.RTCPTR = RTCC_PTR_HOU_WD;
    return RTCVALH & 0b00000111;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_day
 * Output:          valore del giorno (packed BCD)
 *
 * Overview:        Legge il giorno dal RTCC 
/* ============================================================================================================================================ */
unsigned char RTCC_read_day(void) {
    RTCCFGbits.RTCPTR = RTCC_PTR_DAY_MON;
    return RTCVALL & 0b00111111;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_month
 * Output:          valore del mese (packed BCD)
 *
 * Overview:        Legge il mese dal RTCC 
/* ============================================================================================================================================ */
unsigned char RTCC_read_month(void) {
    RTCCFGbits.RTCPTR = RTCC_PTR_DAY_MON;
    return RTCVALH & 0b00111111;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_year
 * Output:          valore dell'anno (packed BCD)
 *
 * Overview:        Legge l'anno dal RTCC 
/* ============================================================================================================================================ */
unsigned char RTCC_read_year(void) {
    RTCCFGbits.RTCPTR = RTCC_PTR_YEAR;
    return RTCVALL;
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_seconds
 * Input:       _seconds: valore dei secondi da scrivere nell' RTCC (packed BCD)
 * 
 * Overview:    Scrive il valore dei secondi nell' RTCC
/* ============================================================================================================================================ */
void RTCC_write_seconds(unsigned char _seconds) {
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    RTCCFGbits.RTCPTR = RTCC_PTR_SEC_MIN;
    RTCVALL = _seconds & 0b01111111;
    
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_minutes
 * Input:       _minutes: valore dei minuti da scrivere nell' RTCC (packed BCD)
 * 
 * Overview:    Scrive il valore dei minuti nell' RTCC
/* ============================================================================================================================================ */
void RTCC_write_minutes(unsigned char _minutes) {
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    RTCCFGbits.RTCPTR = RTCC_PTR_SEC_MIN;
    RTCVALH = _minutes & 0b01111111;
    
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_hours
 * Input:       _hours: valore delle ore da scrivere nell' RTCC (packed BCD)
 * 
 * Overview:    Scrive il valore delle ore nell' RTCC
/* ============================================================================================================================================ */
void RTCC_write_hours(unsigned char _hours) {
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    RTCCFGbits.RTCPTR = RTCC_PTR_HOU_WD;
    RTCVALL = _hours & 0b00111111;
    
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_day_w
 * Input:       _day_w: valore del giorno della settimana da scrivere nell' RTCC (packed BCD)
 * 
 * Overview:    Scrive il valore del giorno della settimana nell' RTCC
/* ============================================================================================================================================ */
void RTCC_write_day_w(unsigned char _day_w) {
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    RTCCFGbits.RTCPTR = RTCC_PTR_HOU_WD;
    RTCVALH = _day_w & 0b00000111;
    
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_day
 * Input:       _day: valore del giorno da scrivere nell' RTCC (packed BCD)
 * 
 * Overview:    Scrive il valore del giorno nell' RTCC
/* ============================================================================================================================================ */
void RTCC_write_day(unsigned char _day) {
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    RTCCFGbits.RTCPTR = RTCC_PTR_DAY_MON;
    RTCVALL = _day & 0b00111111;
    
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_month
 * Input:       _month: valore del mese da scrivere nell' RTCC (packed BCD)
 * 
 * Overview:    Scrive il valore del mese nell' RTCC
/* ============================================================================================================================================ */
void RTCC_write_month(unsigned char _month) {
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    RTCCFGbits.RTCPTR = RTCC_PTR_DAY_MON;
    RTCVALH = _month & 0b00111111;
    
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_year
 * Input:       _year: valore dell'anno da scrivere nell' RTCC (packed BCD)
 * 
 * Overview:    Scrive il valore dell'anno nell' RTCC
/* ============================================================================================================================================ */
void RTCC_write_year(unsigned char _year) {
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    RTCCFGbits.RTCPTR = RTCC_PTR_YEAR;
    RTCVAL = _year;
    
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_alarm
 * Input:       _on_off: Allarme abilitato o disabilitato: ON/OFF (unsigned char)
 * 
 * Overview:    Abilita o disabilita l'allarme
/* ============================================================================================================================================ */
void RTCC_alarm(unsigned char _on_off){
    ALRMCFGbits.ALRMEN = _on_off;                                                      // attivo o disattivo
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_alarm_mode
 * Input:       _mode: Modalit allarme: RTCC_AL_HALF_SEC/RTCC_AL_SEC/RTCC_AL_10_SEC/RTCC_AL_MIN/RTCC_AL_10_MIN/RTCC_AL_HOU/RTCC_AL_DAY/
 *                     RTCC_AL_WEEK/RTCC_AL_MONTH/RTCC_AL_YEAR (unsigned char)
 *              _repeat: Ripetizione allarme: RTCC_AL_REPEAT/RTCC_AL_ONE_TIME (unsigned char)
 * 
 * Overview:    Impostazioni allarme
 * Note:        Non viene considerato il registro ALRMRPT, se viene impostato l'allarme in modalit REPEAT verr ripetuto ogni volta che si verifica
 *              la condizione
/* ============================================================================================================================================ */
void RTCC_alarm_mode(unsigned char _mode, unsigned char _repeat){
    ALRMCFGbits.AMASK = _mode;
    ALRMCFGbits.CHIME = _repeat;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_all_al
 * Output:          Struct contenente i valori di data e ora allarme codificati in packed BCD
 *
 * Overview:        Legge data e ora dell'allarme dal RTCC
 * Note:            l'anno sar sempre 0
/* ============================================================================================================================================ */
rtcc_data RTCC_read_all_al(void){
    rtcc_data _return;
    unsigned char _flush;
    
    ALRMCFGbits.ALRMPTR = RTCC_PTR_DAY_MON;                                             // puntatore si auto decrementa ogni volta che leggo ALRMVALH 
    _return.year = 0;
    
    _return.day = ALRMVALL;
    _return.month = ALRMVALH;
    _return.hours = ALRMVALL;
    _return.day_w = ALRMVALH;
    _return.seconds = ALRMVALL;
    _return.minutes = ALRMVALH;
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_write_all_al
 * Input:           _day: valore del giorno dell'allarme da scrivere nell' RTCC (packed BCD)
 *                  _month: valore del mese dell'allarme da scrivere nell' RTCC (packed BCD)
 *                  _day_w: valore del giorno dell'allarme della settimana da scrivere nell' RTCC (packed BCD)
 *                  _hours: valore delle ora dell'allarme da scrivere nell' RTCC (packed BCD)
 *                  _minutes: valore dei minuti dell'allarme da scrivere nell' RTCC (packed BCD)
 *                  _seconds: valore dei secondi dell'allarme da scrivere nell' RTCC (packed BCD)
 *
 * Overview:        Scrive i valori di data e ora dell'allarme nel RTCC
/* ============================================================================================================================================ */
void RTCC_write_all_al(unsigned char _day, unsigned char _month, unsigned char _day_w,  unsigned char _hours,  unsigned char _minutes, unsigned char _seconds){
    unsigned char _flush;
    
    EECON2 = 0x55;                                                                      // abilito la scrittura dei registri
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    ALRMCFGbits.ALRMPTR = RTCC_PTR_DAY_MON;                                             // puntatore si auto decrementa ogni volta che scrivo ALRMVALH
    
    ALRMVALL = _day;
    ALRMVALH = _month;
    ALRMVALL = _hours;
    ALRMVALH = _day_w;
    ALRMVALL = _seconds;
    ALRMVALH = _minutes;
    
    EECON2 = 0x55;                                                                      // disabilito la scrittura dei registri
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_seconds_al
 * Output:          valore dei secondi (packed BCD)
 *
 * Overview:        Legge i secondi dal RTCC
/* ============================================================================================================================================ */
unsigned char RTCC_read_seconds_al(void) {
    ALRMCFGbits.ALRMPTR = RTCC_PTR_SEC_MIN;
    return ALRMVALL & 0b01111111;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_minutes_al
 * Output:          valore dei minuti (packed BCD)
 *
 * Overview:        Legge i minuti dal RTCC 
/* ============================================================================================================================================ */
unsigned char RTCC_read_minutes_al(void) {
    ALRMCFGbits.ALRMPTR = RTCC_PTR_SEC_MIN;
    return ALRMVALH & 0b01111111;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_hours_al
 * Output:          valore delle ore (packed BCD)
 *
 * Overview:        Legge le ore dal RTCC 
/* ============================================================================================================================================ */
unsigned char RTCC_read_hours_al(void) {
    ALRMCFGbits.ALRMPTR = RTCC_PTR_HOU_WD;
    return ALRMVALL & 0b00111111;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_day_w_al
 * Output:          valore del giorno della settimana (packed BCD)
 *
 * Overview:        Legge il giorno della settimana dal RTCC 
/* ============================================================================================================================================ */
unsigned char RTCC_read_day_w_al(void) {
    ALRMCFGbits.ALRMPTR = RTCC_PTR_HOU_WD;
    return ALRMVALH & 0b00000111;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_day_al
 * Output:          valore del giorno (packed BCD)
 *
 * Overview:        Legge il giorno dal RTCC 
/* ============================================================================================================================================ */
unsigned char RTCC_read_day_al(void) {
    ALRMCFGbits.ALRMPTR = RTCC_PTR_DAY_MON;
    return ALRMVALL & 0b00111111;
}

/* ============================================================================================================================================= /
 * FUNCTION:        RTCC_read_month_al
 * Output:          valore del mese (packed BCD)
 *
 * Overview:        Legge il mese dal RTCC 
/* ============================================================================================================================================ */
unsigned char RTCC_read_month_al(void) {
    ALRMCFGbits.ALRMPTR = RTCC_PTR_DAY_MON;
    return ALRMVALH & 0b00111111;
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_seconds_al
 * Input:       _seconds: valore dei secondi da scrivere nell' RTCC (packed BCD)
 * 
 * Overview:    Scrive il valore dei secondi nell' RTCC
/* ============================================================================================================================================ */
void RTCC_write_seconds_al(unsigned char _seconds) {
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    ALRMCFGbits.ALRMPTR = RTCC_PTR_SEC_MIN;
    ALRMVALL = _seconds & 0b01111111;
    
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_minutes_al
 * Input:       _minutes: valore dei minuti da scrivere nell' RTCC (packed BCD)
 * 
 * Overview:    Scrive il valore dei minuti nell' RTCC
/* ============================================================================================================================================ */
void RTCC_write_minutes_al(unsigned char _minutes) {
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    ALRMCFGbits.ALRMPTR = RTCC_PTR_SEC_MIN;
    ALRMVALH = _minutes & 0b01111111;
    
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_hours_al
 * Input:       _hours: valore delle ore da scrivere nell' RTCC (packed BCD)
 * 
 * Overview:    Scrive il valore delle ore nell' RTCC
/* ============================================================================================================================================ */
void RTCC_write_hours_al(unsigned char _hours) {
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    ALRMCFGbits.ALRMPTR = RTCC_PTR_HOU_WD;
    ALRMVALL = _hours & 0b00111111;
    
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_day_w_al
 * Input:       _day_w: valore del giorno della settimana da scrivere nell' RTCC (packed BCD)
 * 
 * Overview:    Scrive il valore del giorno della settimana nell' RTCC
/* ============================================================================================================================================ */
void RTCC_write_day_w_al(unsigned char _day_w) {
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    ALRMCFGbits.ALRMPTR = RTCC_PTR_HOU_WD;
    ALRMVALH = _day_w & 0b00000111;
    
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_day_al
 * Input:       _day: valore del giorno da scrivere nell' RTCC (packed BCD)
 * 
 * Overview:    Scrive il valore del giorno nell' RTCC
/* ============================================================================================================================================ */
void RTCC_write_day_al(unsigned char _day) {
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    ALRMCFGbits.ALRMPTR = RTCC_PTR_DAY_MON;
    ALRMVALL = _day & 0b00111111;
    
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    RTCC_write_month_al
 * Input:       _month: valore del mese da scrivere nell' RTCC (packed BCD)
 * 
 * Overview:    Scrive il valore del mese nell' RTCC
/* ============================================================================================================================================ */
void RTCC_write_month_al(unsigned char _month) {
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 1;
    
    ALRMCFGbits.ALRMPTR = RTCC_PTR_DAY_MON;
    ALRMVALH = _month & 0b00111111;
    
    EECON2 = 0x55;
    EECON2 = 0xAA;
    RTCCFGbits.RTCWREN = 0;
}

#endif