/* ============================================================================================================================================= /
 * LIBRARY:         SPI
 * FILE:            SPI_init.c
 * LAST REV:        2.5
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         17/04/2014
 * UPDATED:         22/03/2017
 *
 * DESCRIP:         gestione del bus SPI primario e secondario (se disponibile).
 *                  Al momento sono state sviluppate solo le funzionalit  per dispositivo MASTER.        
 *
 * FUNCTIONS:       - SPI_enable
 *                  - SPI_disable
 *                  - SPI_send_byte
 *                  - SPI_receive_byte
 *                  - SPI_exchange_byte
 *                  - SPI2_enable
 *                  - SPI2_disable
 *                  - SPI2_send_byte
 *                  - SPI2_receive_byte
 *                  - SPI2_exchange_byte
 *                  
/* ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  2.5               22/03/2017          - Aggiunte funzioni SPI_exchange_byte e SPI2_exchange_byte
  2.4               17/02/2017          - Modifica da  PIR3bits.SSP2IF a SSP2IF per compatibilit con PIC18F67K22 (si trova su PIR2 invece che su PIR3)
  2.3               16/12/2016          - Compatibilit con PIC18F26K22 e PIC18F67K22   
  2.2               07/11/2016          - Rivisti cicli invio e ricezione 
  2.1               28/08/2016          - Adattamento per poter utilizzare 2 moduli contemporaneamente, compatibilit con PIC18F46K22
                                        - Linee SPI impostate automaticamente a seconda del PIC in uso e del modulo selezionato in SPI_init_settings (gestione in I2C_init.h)
  2.0               19/07/2016          - Modificata gestione per PIC diversi
                                        - Implementata gestione secondo modulo SPI per PIC con 2 moduli SPI
                                        - Spostati settaggi in SPI_init_settings
  1.0               17/03/2014          - Prima release
 //! [History]
/* ============================================================================================================================================ */

#ifndef SPI
#define SPI

/* INCLUDES =================================================================================================================================== */
#include "SPI_init_settings.h"
#include "SPI_init.h"

/* ============================================================================================================================================= /
 * FUNCTION:    SPI_enable
 * Input:       _smp:   SAMPLE_AT_END       = Input data sampled at end of data output time
 *                      SAMPLE_AT_MIDDLE    = Input data sampled at middle of data output time
 *              _cke:   TRA_ACTIVE_TO_IDLE  = Transmit occurs on transition from active to Idle clock state
 *                      TRA_IDLE_TO_ACTIVE  = Transmit occurs on transition from Idle to active clock state
 *              _ckp:   IDLE_IS_HIGH        = Idle state for clock is a high level
 *                      IDLE_IS_LOW         = Idle state for clock is a low level
 *              _ssmp:  MASTER_FOSC_4       = SPI Master mode, clock = FOSC/4
 *                      MASTER_FOSC_16      = SPI Master mode, clock = FOSC/16
 *                      MASTER_FOSC_64      = SPI Master mode, clock = FOSC/64
 *                      MASTER_TMR2_2       = SPI Master mode, clock = TMR2 output/2 (impostare timer 2 per ottenere la frequenza di clock desiderata)
 *                      SLAVE_WITH_SS       = SPI Slave mode, clock = SCK pin, SS pin control enabled
 *                      SLAVE_WO_SS         = SPI Slave mode, clock = SCK pin, SS pin control disabled, SS can be used as I/O pin
 *
 * Overview:    Abilita il protocollo SPI sul modulo primario con i parametri desiderati
/* ============================================================================================================================================ */ 
#ifdef SPI_MODULE_1
void SPI_enable(unsigned char _smp, unsigned char _cke, unsigned char _ckp, unsigned char _sspm) {

    unsigned char _flush;
   
#ifdef _12F1840
    #ifdef SPI_SDO
        if (SPI_SDO_RA == 0)                                                            // TX su RA0
            APFCONbits.SDOSEL = 0;    
        else if (SPI_SDO_RA == 4)                                                       // TX su RA4
            APFCONbits.SDOSEL = 1;   
    #endif
    #ifdef SPI_SS
        if (SPI_SS_RA == 3)                                                             // RX su RA1
            APFCONbits.SSSEL = 0;
        else if (SPI_SS_RA == 0)                                                        // RX su RA5    
            APFCONbits.SSSEL = 1;
    #endif
#endif
    
#ifdef SPI_SDI                                                                          // se SDI  usato e definito
    SPI_SDI_TRIS = IN;                                                                  // imposto TRIS in ingresso
#endif
    
#ifdef SPI_SDO                                                                          // se SDO  usato e definito
    SPI_SDO_TRIS = OUT;                                                                 // imposto SDO in uscita
#endif
    
    if (_sspm == SLAVE_WO_SS) {                                                         // modalit  SLAVE senza SS
        SPI_SCK_TRIS = IN;                                                              // imposto clock clock in ingresso
    } else if (_sspm == SLAVE_WITH_SS) {                                                // modalit  SLAVE con SS
        SPI_SCK_TRIS = IN;                                                              // Imposto clock clock in ingresso
#ifdef SPI_SS                                                                           // e se definito imposto SS in ingresso
        SPI_SS_TRIS = IN;                                                               // imposto SS in ingresso
#endif
    } else {                                                                            // per tutte le modalit  MASTER
         SPI_SCK_TRIS = OUT;                                                            // imposto clock in uscita
    }
    
    _flush = SSPBUF;                                                                    // pulisco i registri
    PIR1bits.SSPIF = 0;
    SSPCON1bits.WCOL = 0;
    SSPCON1bits.SSPOV = 0;             
    
    SSPSTATbits.SMP = _smp;                                                             // 1 = Input data sampled at end of data output time
                                                                                        // 0 = Input data sampled at middle of data output time
    SSPSTATbits.CKE = _cke;                                                             // 1 = Transmit occurs on transition from active to Idle clock state
                                                                                        // 0 = Transmit occurs on transition from Idle to active clock state
    SSPCON1bits.CKP = _ckp;                                                             // 1 = Idle state for clock is a high level
                                                                                        // 0 = Idle state for clock is a low level
    SSPCON1bits.SSPM = _sspm;                                                           // 0101 = SPI Slave mode, clock = SCK pin, SS pin control disabled, SS can be used as I/O pin(3)
                                                                                        // 0100 = SPI Slave mode, clock = SCK pin, SS pin control enabled(3)
                                                                                        // 0011 = SPI Master mode, clock = TMR2 output/2(3)
                                                                                        // 0010 = SPI Master mode, clock = FOSC/64(3)
                                                                                        // 0001 = SPI Master mode, clock = FOSC/16(3)
                                                                                        // 0000 = SPI Master mode, clock = FOSC/4(3)
    SSPCON1bits.SSPEN = 1;                                                              // abilito SPI
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    SPI2_enable
 * Input:       _smp:   SAMPLE_AT_END       = Input data sampled at end of data output time
 *                      SAMPLE_AT_MIDDLE    = Input data sampled at middle of data output time
 *              _cke:   TRA_ACTIVE_TO_IDLE  = Transmit occurs on transition from active to Idle clock state
 *                      TRA_IDLE_TO_ACTIVE  = Transmit occurs on transition from Idle to active clock state
 *              _ckp:   IDLE_IS_HIGH        = Idle state for clock is a high level
 *                      IDLE_IS_LOW         = Idle state for clock is a low level
 *              _ssmp:  MASTER_FOSC_4       = SPI Master mode, clock = FOSC/4
 *                      MASTER_FOSC_16      = SPI Master mode, clock = FOSC/16
 *                      MASTER_FOSC_64      = SPI Master mode, clock = FOSC/64
 *                      MASTER_TMR2_2       = SPI Master mode, clock = TMR2 output/2 (impostare timer 2 per ottenere la frequenza di clock desiderata)
 *                      SLAVE_WITH_SS       = SPI Slave mode, clock = SCK pin, SS pin control enabled
 *                      SLAVE_WO_SS         = SPI Slave mode, clock = SCK pin, SS pin control disabled, SS can be used as I/O pin(3)
 *
 * Overview:    Abilita il protocollo SPI sul modulo secondario con i parametri desiderati
/* ============================================================================================================================================ */
#ifdef SPI_MODULE_2                                                                     // compilo solo se il secondo modulo  effettivamente presente ed usato
void SPI2_enable(unsigned char _smp, unsigned char _cke, unsigned char _ckp, unsigned char _sspm) { 
                                                                                        
    unsigned char _flush;       
    
#ifdef SPI2_SDI                                                                         // se SDI  usato e definito
    SPI2_SDI_TRIS = IN;                                                                 // imposto TRIS in ingresso
#endif
    
#ifdef SPI2_SDO                                                                         // se SDO  usato e definito
    SPI2_SDO_TRIS = OUT;                                                                // imposto SDO in uscita
#endif
    
    if (_sspm == SLAVE_WO_SS) {                                                         // modalit  SLAVE senza SS
        SPI2_SCK_TRIS = IN;                                                             // imposto clock clock in ingresso
    } else if (_sspm == SLAVE_WITH_SS) {                                                // modalit  SLAVE con SS
        SPI2_SCK_TRIS = IN;                                                             // Imposto clock clock in ingresso
#ifdef SPI2_SS                                                                          // e se definito imposto SS in ingresso
        SPI2_SS_TRIS = IN;                                                              // imposto SS in ingresso
#endif
    } else {                                                                            // per tutte le modalit  MASTER
         SPI2_SCK_TRIS = OUT;                                                           // imposto clock in uscita
    }
    
    _flush = SSP2BUF;                                                                   // pulisco i registri
//    PIR3bits.SSP2IF = 0;
    SSP2IF = 0;
    SSP2CON1bits.WCOL = 0;
    SSP2CON1bits.SSPOV = 0;   
    
    SSP2STATbits.SMP = _smp;                                                            // 1 = Input data sampled at end of data output time
                                                                                        // 0 = Input data sampled at middle of data output time
    SSP2STATbits.CKE = _cke;                                                            // 1 = Transmit occurs on transition from active to Idle clock state
                                                                                        // 0 = Transmit occurs on transition from Idle to active clock state
    SSP2CON1bits.CKP = _ckp;                                                            // 1 = Idle state for clock is a high level
                                                                                        // 0 = Idle state for clock is a low level
    SSP2CON1bits.SSPM = _sspm;                                                          // 0101 = SPI Slave mode, clock = SCK pin, SS pin control disabled, SS can be used as I/O pin(3)
                                                                                        // 0100 = SPI Slave mode, clock = SCK pin, SS pin control enabled(3)
                                                                                        // 0011 = SPI Master mode, clock = TMR2 output/2(3)
                                                                                        // 0010 = SPI Master mode, clock = FOSC/64(3)
                                                                                        // 0001 = SPI Master mode, clock = FOSC/16(3)
                                                                                        // 0000 = SPI Master mode, clock = FOSC/4(3)
    SSP2CON1bits.SSPEN = 1;                                                             // abilito SPI
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    SPI_disable
 *
 * Overview:    Disabilita il protocollo SPI del modulo primario
/* ============================================================================================================================================ */
#ifdef SPI_MODULE_1
void SPI_disable(void) {
    SSPCON1bits.SSPEN = 0;                                                              // disailito SPI
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:    SPI2_disable
 *
 * Overview:    Disabilita il protocollo SPI del modulo secondario
/* ============================================================================================================================================ */
#ifdef SPI_MODULE_2                                                                     // compilo solo se secondo modulo presente ed usato
void SPI2_disable(void) {
    SSP2CON1bits.SSPEN = 0;                                                             // disabilito SPI
}
#endif      

/* ============================================================================================================================================= /
 * FUNCTION:        SPI_send_byte
 * Input:           _byte: dato da inviare (unsigned char)
 * Output:          -1 in caso di bus collision
 *
 * Overview:        invia il dato via SPI sul modulo primario
/* ============================================================================================================================================ */
#ifdef SPI_MODULE_1  
signed char SPI_send_byte(unsigned char _byte) {
    unsigned char _flush;
    
    _flush = SSPBUF;                                                                    // resetto precedente trasmissione
    PIR1bits.SSPIF = 0;        
    SSPCON1bits.WCOL = 0;
                                                                 
    SSPBUF = _byte;
    
    if (SSPCON1bits.WCOL == 0x01){
        return -1;   
    } 
    
    while (!PIR1bits.SSPIF);                                                            // attendo che la trasmissione sia stata completata
    
    return 0;
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:        SPI2_send_byte
 * Input:           _byte: dato da inviare (unsigned char)
 * Output:          -1 in caso di bus collision
 *
 * Overview:        invia il dato via SPI  sul modulo secondario
/* ============================================================================================================================================ */
#ifdef SPI_MODULE_2   
signed char SPI2_send_byte(unsigned char _byte) {                                              // compilo solo se secondo modulo presente ed usato
                                                                                        
    unsigned char _flush;   
    
    _flush = SSP2BUF;                                                                   // resetto precedente trasmissione
//    PIR3bits.SSP2IF = 0;                                                                // resetto precedente trasmissione
    SSP2IF = 0;
    SSP2CON1bits.WCOL = 0;
    
    SSP2BUF = _byte;
    
    if (SSP2CON1bits.WCOL == 0x01){
        return (-1);   
    }
    
//    while (!PIR3bits.SSP2IF);                                                           // attendo che la trasmissione sia stata completata
    while (!SSP2IF);                                                           // attendo che la trasmissione sia stata completata
    
    return 0;
}
#endif


/* ============================================================================================================================================= /
 * FUNCTION:        SPI_receive_byte
 * Output:          dato ricevuto (unsigned char)
 *
 * Overview:        riceve il dato via SPI sul modulo primario in polling sul bit BF
/* ============================================================================================================================================ */
#ifdef SPI_MODULE_1
unsigned char SPI_receive_byte(void) {
    unsigned char _flush;
    
    _flush = SSPBUF;                                                                    // resetto registri
    PIR1bits.SSPIF = 0;
    SSPBUF = 0;
    
    while (!PIR1bits.SSPIF);                                                            // attendo che venga ricevuto un byte
        
    return SSPBUF;                                                                      // ritorno il valore
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:        SPI2_receive_byte
 * Output:          dato ricevuto (unsigned char)
 *
 * Overview:        riceve il dato via SPI sul modulo secondario in polling sul bit BF 
/* ============================================================================================================================================ */
#ifdef SPI_MODULE_2
unsigned char SPI2_receive_byte(void) {                                                 // compilo solo se secondo modulo presente ed usato
    
    unsigned char _flush;
    
    _flush = SSP2BUF;                                                                   // resetto registri
//    PIR3bits.SSP2IF = 0;
    SSP2IF = 0;
    SSP2BUF = 0;
    
//    while (!PIR3bits.SSP2IF);                                                            // attendo che venga ricevuto un byte
    while (!SSP2IF);                                                            // attendo che venga ricevuto un byte
                                                                      
    
    return SSP2BUF;                                                                     // ritorno il valore
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:        SPI_exchange_byte
 * Input:           _data: dato da inviare (unsigned char)
 * Output:          dato ricevuto (unsigned char)
 *
 * Overview:        invia e riceve 1 byte via SPI sul modulo primario
/* ============================================================================================================================================ */
#ifdef SPI_MODULE_1
unsigned char SPI_exchange_byte(unsigned char _data){
    
    SSP2CON1bits.WCOL = 0;
SSP1BUF = _data;

    while(SSP1STATbits.BF == 0)
    {
    }

    return (SSP1BUF);
}
#endif

/* ============================================================================================================================================= /
 * FUNCTION:        SPI2_exchange_byte
 * Input:           _data: dato da inviare (unsigned char)
 * Output:          dato ricevuto (unsigned char)
 *
 * Overview:        invia e riceve 1 byte via SPI sul modulo secondario
/* ============================================================================================================================================ */
#ifdef SPI_MODULE_2
unsigned char SPI2_exchange_byte(unsigned char _data)
{

    SSP2CON1bits.WCOL = 0;
    SSP2BUF = _data;

    while(SSP2STATbits.BF == 0)
    {
    }

    return (SSP2BUF);
}
#endif
#endif
