/* ============================================================================================================================================= /
 * LIBRARY:         Gestione TFT ILI9325
 * FILE:            TFT_ILI9325.c
 * LAST REV:        1.4
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         17/10/2016
 * UPDATED:         19/07/2017 
 *
 * DESCRIP:         Gestione display TFT con controller ILI9325 320*240
 *
 * FUNCTIONS:       PRIVATE FUNCTION
 *                  - TFT_write_register_address()
 *                  - TFT_write_register_value()
 *                  - TFT_write_register()
 *                  - TFT_set_gram_address()
 *                  - TFT_set_window()
 *                  - TFT_set_window_dir()
 *                  - TFT_Plot8()
 *                  - TFT_write_char_w()
 *                  - TFT_write_space_btw_chars()
 *                  
 *                  PUBLIC FUNCTION
 *                  - TFT_read_register()
 *                  - TFT_init()
 *                  - TFT_clear()
 *                  - TFT_idle()
 *                  - TFT_sleep()
 *                  - TFT_standby()
 *                  - TFT_off()
 *                  - TFT_active()
 *                  - TFT_draw_pixel()
 *                  - TFT_draw_image()
 *                  - TFT_draw_image_monocr()
 *                  - TFT_draw_line_x()
 *                  - TFT_draw_line_y()
 *                  - TFT_draw_line_p2p()
 *                  - TFT_draw_line_angle()
 *                  - TFT_draw_circle()
 *                  - TFT_draw_rect()
 *                  - TFT_fill_rect()
 *                  - TFT_fill_rounded_rect()
 *                  - TFT_set_font()
 *                  - TFT_write_char()
 *                  - TFT_write_string()
 *                  - TFT_write_unsint()
 *                  - TFT_write_sint()
 *                  - TFT_write_PBCD()
 *                  - TFT_write_HEX()
 *                  - TFT_write_bit_string()
 *                  - TFT_write_float()
 *                  - TFT_RGB_convert()
 *                  - TFT_draw_image_sd()
 *                  
 * ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes 
  1.4               19/07/2017          - Velogizzata scrittura immagini da SD con variabili _buffer_sd_x_2 e _pixel_x_2 al posto di eseguire le moltiplicazioni
                                          ad ogni ciclo.
  1.3               08/04/2017          - Aggiunto ciclo di controllo per scrittura immagini da scheda SD con larghezza di pixel dispari.
                                        - Aggiunto controllo carattere valido nella funzione TFT_write_char (if (_character >= 32 &&  _character <= 151))
  1.2               26/03/2017          - Modificato nome funzioni _block in _rect
                                        - Aggiunta funzione TFT_draw_rounded_rect()
                                        - TFT_fill_rounded_rect()
                                        - Aggiunta funzione TFT_set_window_dir()
                                        - Aggiunta funzione TFT_draw_image_sd()
  1.1               06/11/2016          - Aggiunta funzione TFT_draw_image_monocr()
  1.0               27/10/2016          - Prima release completa
  1.0               17/10/2016          - Inizio studio TFT
//! [History]
/* ============================================================================================================================================ */

#ifndef TFT_ILI9325
#define TFT_ILI9325

/* INCLUDES =================================================================================================================================== */
#include    "TFT_ILI9325_settings.h"
#include    "TFT_ILI9325.h"

#include     <stdlib.h>
#include     <math.h>

/* Variabili globali libreria ================================================================================================================= */
    unsigned char font_width = 0;                                                       // larghezza del font 
    unsigned char font_height = 0;                                                      // altezza del font
    unsigned char font_space = 0;                                                       // spazio tra le lettere di una parola
    unsigned char font_first_char_offset = 0;                                           // offset per allineare il carattere char alla tabella in base al contenuto del font
    unsigned char font_vertical_byte = 0;                                               // numero di byte per altezza del carattere
    const unsigned char (*font_pointer) = 0;                                            // puntatore all'array
    unsigned int font_offset_btw_char = 0;                                              // offset tra i caratteri (serve per font con pi byte per l'altezza)

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_write_register_address
 * Input:       _addr: indirizzo del registro (unsigned char)
 *
 * Overview:    Imposta il registro da leggere o scrivere
/* ============================================================================================================================================ */
void TFT_write_register_address (unsigned char _addr){
    TFT_CS = 0;
    TFT_RS = 0;
    
#ifdef TFT_8_BIT_DATA
    TFT_DATA_D10_17_WRITE = (0x00);                                                     // primo byte dell'indirizzo  sempre 0x00
    TFT_WR = 0;                                                                         // WR pulse
    TFT_WR = 1;    
    TFT_DATA_D10_17_WRITE = (_addr);
    TFT_WR = 0;
    TFT_WR = 1;
#elif defined TFT_16_BIT_DATA
    TFT_DATA_D10_17_WRITE = (0x00);
    TFT_DATA_D1_8_WRITE = (_addr);
    TFT_WR = 0;
    TFT_WR = 1;
#endif               

    TFT_RS = 1;
    TFT_CS = 1;         
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_write_register_value
 * Input:       _value: valore da scrivere nel registro (unsigned int)
 *
 * Overview:    Scrive il valore nel registro precedentemente indirizzato tramite funzione TFT_write_register_address() 
/* ============================================================================================================================================ */
void TFT_write_register_value (unsigned int _value){
    TFT_CS = 0;
    
#ifdef TFT_8_BIT_DATA    
    TFT_DATA_D10_17_WRITE = (_value >> 8);                                              // Invio MSB
    TFT_WR = 0;
    TFT_WR = 1;
    
    TFT_DATA_D10_17_WRITE = (_value);                                                   // Invio LSB
    TFT_WR = 0;
    TFT_WR = 1;

#elif defined TFT_16_BIT_DATA
    TFT_DATA_D10_17_WRITE = (_value >> 8);
    TFT_DATA_D1_8_WRITE = (_value);            
    TFT_WR = 0;
    TFT_WR = 1;
#endif
    TFT_CS = 1;
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_write_register
 * Input:       _addr: indirizzo del registro (unsigned char)
 *              _value: valore da scrivere nel registro (unsigned int)
 *
 * Overview:    Scrive il valore nel registro all'indirizzo dichiarato
/* ============================================================================================================================================ */
void TFT_write_register (unsigned char _addr,unsigned int _value){           
    TFT_write_register_address(_addr);
    TFT_write_register_value(_value);    
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_read_register
 * Input:       _addr: indirizzo del registro (unsigned char)
 * Return:      valore letto nel registro (unsigned int)
 *
 * Overview:    Legge il valore nel registro all'indirizzo dichiarato
/* ============================================================================================================================================ */
#ifdef TFT_RD
unsigned int TFT_read_register (unsigned char _addr){
    unsigned int _readed = 0;
    
    TFT_write_register_address(_addr);
    TFT_DATA_D10_17_DIR = TFT_DATA_INPUT;
    
#ifdef TFT_16_BIT_DATA
     TFT_DATA_D1_8_DIR = TFT_DATA_INPUT;
#endif

    TFT_CS = 0;
    TFT_RD = 0;

#ifdef TFT_8_BIT_DATA     
    _readed =  TFT_DATA_D10_17_READ;
    TFT_RD = 1;
    _readed = _readed <<8;
    TFT_RD = 0;
    _readed |= TFT_DATA_D10_17_READ;

#elif defined TFT_16_BIT_DATA
    _readed = ((unsigned int)TFT_DATA_D10_17_READ << 8) | TFT_DATA_D1_8_READ; 
#endif   
    
    TFT_RD = 1;
    TFT_CS = 1;
    
    TFT_DATA_D10_17_DIR = TFT_DATA_OUTPUT;

#ifdef TFT_16_BIT_DATA
    TFT_DATA_D1_8_DIR = TFT_DATA_OUTPUT;
#endif
    
    return _readed;
}

#endif

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_init
 * 
 * Overview:    Inizializza il display TFT
/* ============================================================================================================================================ */
void TFT_init (void){  // poi passa parametro rotazione da qui
    
    // tutte le linee di controllo sono attive basse, metto tutto a 1
    TFT_CS = 1;          
    TFT_RS = 1;               
    TFT_WR = 1; 
    TFT_RST = 1;
    
#ifdef TFT_RD
    TFT_RD = 1;
#endif

    TFT_DATA_D10_17_DIR = TFT_DATA_OUTPUT;
    
#ifdef TFT_16_BIT_DATA
     TFT_DATA_D1_8_DIR = TFT_DATA_OUTPUT;
#endif
    
    // impulso sul reset  dopo il power on per fare un hardware reset
    Delay_ms(200);
    TFT_RST=0;
    Delay_ms(50);
    TFT_RST=1;
    Delay_ms(200);
    
//    TFT_write_register(0xE5, 0x78F0);                                                   // set SRAM internal timing   
    
    // Display off settings
    TFT_write_register(DISPLAY_CTRL_1,0x0000);                                          // 0x07 reset to power on status (DTE=0, D[1:0]=00, GON=0)
    TFT_write_register(POWER_CONTROL_3,0x0000);                                         // 0x12 reset to power on status (PON=0)   
    TFT_write_register(POWER_CONTROL_3,0x0000);                                         // 0x12
    TFT_write_register(POWER_CONTROL_4,0x0000);                                         // 0x13 
    
// POWER SUPPLY INITIAL SETTING BEFORE POWER SUPPLY STARTUP //
    TFT_write_register(POWER_CONTROL_2,0x0017);                                         // 0x11, DC1[2:0]=0000 (frequ. step-up 2 = fosc/4) DC0[2:0]=0001 (frequ. step-up 1 = fosc/2), VC[2:0]=111 (VCC ratio = 1xVCC)
    TFT_write_register(POWER_CONTROL_3,0x00BC);                                         // 0x12, VCIRE=1 (int.ref 2.5V), PON=0 (VGL out enabled), VRH[3:0]= 1100 (VREG1 OUT = 2.5V x 1.80 = 4.500V)
    TFT_write_register(POWER_CONTROL_4,0x1800);                                         // 0x13, VDV[4:0]=11000 (x1.10)
    TFT_write_register(POWER_CONTROL_7,0x0023);                                         // 0x29, VCM[5:0]=100111 (VcomH = VREG1OUT x 0.880)
    TFT_write_register(POWER_CONTROL_1,0x0000);                                         // 0x10
    
    Delay_ms(100);
    
// POWER SUPPLY OPERATION SETTING //
    TFT_write_register(POWER_CONTROL_1,0x0290);                                         // 0x10, SAP=0 (surce driver enaled), BT[2:0]=2 (step-up factor), APE=1 (power enabled), AP[2:0]=001 (1.0/1.0), SLP = 0 (non sleep), STB=0 (non stand by)
    TFT_write_register(POWER_CONTROL_2,0x0007);                                         // 0x11, DC1[2:0]=0 (frequ. step-up 2 = fosc/4) DC0[2:0]=0 (frequ. step-up 1 = fosc), VC[2:0]=111 (VCC ratio = 1xVCC)
      
    TFT_write_register(POWER_CONTROL_3,0x009C);                                         // 0x12, VCIRE=1 (int.ref 2.5V), PON=1 (VGL out enabled), VRH[3:0]= 1100 (VREG1 OUT = 2.5V x 1.80 = 4.500V)                        
                        
    Delay_ms(100);
    
// SET THE OTHER REGISTERS
#if defined TFT_PORTRAIT || defined TFT_LANDSCAPE
    TFT_write_register(DRIVER_OUTPUT_CTRL,0x0100);                                      // 0x01, SS=1 (shift direction of outputs is from S720 to S1) SM=0
#elif defined TFT_PORTRAIT_180 || defined TFT_LANDSCAPE_180    
    TFT_write_register(DRIVER_OUTPUT_CTRL,0x0000);                                      // 0x01, SS=0 (shift direction of outputs is from SS1 to S720) SM=0
#endif
    
    TFT_write_register(DRIVING_WAVE_CTRL, 0x0700);                                      // 0x02, B/C=1 EOR=1 (Line inversion) 

#if defined TFT_LANDSCAPE || defined TFT_LANDSCAPE_180
    TFT_write_register(ENTRY_MODE,0x1030);                                              // 0x03, AM=0, ID[1:0]=11 (h: incr, v: incr), BGR=?1? (Swap the RGB data to BGR in writing into GRAM), TRI=0 (8 bit 2 transfer or 16 bit 1 transfer (limit 65.536 color)  
#elif defined TFT_PORTRAIT || defined TFT_PORTRAIT_180
    TFT_write_register(ENTRY_MODE,0x1038);                                              // 0x03, AM=1, ID[1:0]=11 (h: incr, v: incr), BGR=?1? (Swap the RGB data to BGR in writing into GRAM), TRI=0 (8 bit 2 transfer or 16 bit 1 transfer (limit 65.536 color)  
#endif
    TFT_write_register(RESIZING_CTRL,0x0000);                                           // 0x04, no resize
    TFT_write_register(DISPLAY_CTRL_2,0x020E);                                          // 0x08, FP=10 (2 line) e BF=1110 (14 line)
    TFT_write_register(DISPLAY_CTRL_3,0x0000);                                          // 0x09, set non-display area refresh cycle ISC[3:0] 
    TFT_write_register(DISPLAY_CTRL_4,0x0000);                                          // 0xA0, FMARK function disabled.
    TFT_write_register(RGB_DISPLAY_INT_CTRL_1,0x0003);                                  // 0x03, disabilitato RGB interface setting (RGB NOT USED)
    TFT_write_register(FRAME_MARKER_POSITION,0x0000);                                   // 0x0D, Frame marker Position 
    TFT_write_register(RGB_DISPLAY_INT_CTRL_2,0x0000);                                  // 0x0F, RGB interface polarity (RGB NOT USED)
    TFT_write_register(FRAME_RATE_AND_COLOR_CRTL,0x000D);                               // 0x2B, FRS[3:0]=1101 (frame rate 93)
    TFT_write_register(GRAM_HORIZONTAL_ADDRESS,0x0000);                                 // reset GRAM indirizzi orizzontale e verticale
    TFT_write_register(GRAM_VERTICAL_ADDRESS,0x0000);                                   // per scrivere in GRAM puntare il primo indirizzo, poi AC si incrementa automaticamente ad ogni scrittura delle GRAM (all'interno della finestra settata)

// GAMMA CURVE ADJUSTMENT //  
    TFT_write_register(GAMMA_CTRL_30H,0x0000);
    TFT_write_register(GAMMA_CTRL_31H,0x0104);
    TFT_write_register(GAMMA_CTRL_32H,0x0000);
    TFT_write_register(GAMMA_CTRL_35H,0x0203);
    TFT_write_register(GAMMA_CTRL_36H,0x0405);
    TFT_write_register(GAMMA_CTRL_37H,0x0000);
    TFT_write_register(GAMMA_CTRL_38H,0x0203);
    TFT_write_register(GAMMA_CTRL_39H,0x0000);
    TFT_write_register(GAMMA_CTRL_3CH,0x0203);
    TFT_write_register(GAMMA_CTRL_3DH,0x0405);
    
//    TFT_write_register(GAMMA_CTRL_30H,0x0007);
//    TFT_write_register(GAMMA_CTRL_31H,0x0707);
//    TFT_write_register(GAMMA_CTRL_32H,0x0006);
//    TFT_write_register(GAMMA_CTRL_35H,0x0704);
//    TFT_write_register(GAMMA_CTRL_36H,0x1f04);
//    TFT_write_register(GAMMA_CTRL_37H,0x0004);
//    TFT_write_register(GAMMA_CTRL_38H,0x0000);
//    TFT_write_register(GAMMA_CTRL_39H,0x0706);
//    TFT_write_register(GAMMA_CTRL_3CH,0x0701);
//    TFT_write_register(GAMMA_CTRL_3DH,0x000f);


// GRAM AREA  //
    TFT_write_register(HORIZONTAL_GRAM_START, 0x0000);                                  // Horizontal GRAM Start Address
    TFT_write_register(HORIZONTAL_GRAM_END, 0x00EF);                                    // Horizontal GRAM End Address 
    TFT_write_register(VERTICAL_GRAM_START, 0x0000);                                    // Vertical GRAM Start Address 
    TFT_write_register(VERTICAL_GRAM_END, 0x013F);                                      // Vertical GRAM Start Address  

#if defined TFT_LANDSCAPE || defined TFT_PORTRAIT_180
    TFT_write_register(GATE_SCAN_CTRL_60H,0x2700);                                      // GS=0, SCN[5:0]=000000 (Gate Scan Line start 0), NL[5:0]=100111 (numbers of lines = 320 lines) 
#elif defined TFT_PORTRAIT || defined TFT_LANDSCAPE_180 
    TFT_write_register(GATE_SCAN_CTRL_60H,0xA700);                                      // GS=1, SCN[5:0]=000000 (Gate Scan Line start 0), NL[5:0]=100111 (numbers of lines = 320 lines) 
#endif
    TFT_write_register(GATE_SCAN_CTRL_61H,0x0001);                                      // NDL=0, VLE=0, REV=1???   
    TFT_write_register(GATE_SCAN_CTRL_6AH,0x0000);                                      // set scrolling line  

// PARTIAL DISPLAY CONTROL //      
    TFT_write_register(PARTIAL_IMAGE_1_POS,0x0000);
    TFT_write_register(PARTIAL_IMAGE_1_START_ADDR,0x0000);
    TFT_write_register(PARTIAL_IMAGE_1_END_ADDR,0x0000);
    TFT_write_register(PARTIAL_IMAGE_2_POS,0x0000);
    TFT_write_register(PARTIAL_IMAGE_2_START_ADDR,0x0000);
    TFT_write_register(PARTIAL_IMAGE_2_END_ADDR,0x0000);

// PANEL CONTROL
    TFT_write_register(PANEL_INTERFACE_CTRL_1,0x0010);                                  // 0x90, DIVI[1:0]=00 (fosc/1) RTNI[4:0]=10000 (16 clocks)
    TFT_write_register(PANEL_INTERFACE_CTRL_2,0x0000);                                  // 0x92, NOWI[2:0]=000 (0 clocks)
    TFT_write_register(PANEL_INTERFACE_CTRL_4,0x0000);                                  // 0x95, clock for RGB interface (RGB INTERFACE NOT USED) 
    TFT_write_register(PANEL_INTERFACE_CTRL_5,0x0000);                                  // 0x97, NOWE[3:0]=0000 (0 clock) (RGB INTERFACE NOT USED)     
    
    
// ENDING POWER ON    
    TFT_write_register(POWER_CONTROL_1,0x1290);                                         // 0x10, SAP=1 (surce driver enaled), BT[2:0]=2 (step-up factor), APE=1 (power enabled), AP[2:0]=001 (1.0/1.0), SLP = 0 (non sleep), STB=0 (non stand by)
    TFT_write_register(DISPLAY_CTRL_1,0x0001);                                          // 0x07, D[1:0]=01 (only internal operation)  
    TFT_clear (0xFFFF);                                                                 // prima di attivare il display
    TFT_write_register(DISPLAY_CTRL_1,0x0133);                                          // 0x07, PTDE[1:0]=00 (no partial images), BASEE=1 (base image displayed), GON=1 & DTE=1 (normal display), CL=0 (256k colors or 65k depends on transfer method), D1=1 D0=1
    
    Delay_ms(100);
    
#ifdef TFT_BACKILL
    TFT_BACKILL = ON;
#endif
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_set_gram_address
 * Input:       _x: coordinata x del pixel da accendere: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y: coordinata y del pixel da accendere: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)
 *
 * Overview:    Imposta l'indirizzo del pixel da scrivere
/* ============================================================================================================================================ */
void TFT_set_gram_address (unsigned int _x, unsigned int _y){
#if defined TFT_LANDSCAPE || defined TFT_LANDSCAPE_180                                  // inverto _x e _y se sono in TFT_LANDSCAPE
    unsigned int _temp;
    _temp = _y;
    _y = _x;
    _x = _temp;
#endif  

    TFT_write_register(GRAM_HORIZONTAL_ADDRESS, _x-1);                                  // imposto l'indirizzo sottraendo 1 per riportare a 0~239 - 0~319
    TFT_write_register(GRAM_VERTICAL_ADDRESS, _y-1);
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_set_window
 * Input:       _x_start: coordinata x del primo pixel della finestra: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y_start: coordinata y del primo pixel della finestra: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)
 *              _x_end: coordinata x dell'ultimo pixel della finestra: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y_end: coordinata y dell'ultimo pixel della finestra: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *
 * Overview:    Imposta la finestra da considerare per l'auto incremento degli indirizzi GRAM, serve in TFT_clear(), TFT_fill_rect(), 
 *              TFT_write_char(), TFT_write_string() per velocizzare le operazioni di scrittura dei pixel contigui.
/* ============================================================================================================================================ */
void TFT_set_window(unsigned int _x_start, unsigned int _y_start, unsigned int _x_end, unsigned int _y_end){
    TFT_set_gram_address(_x_start, _y_start);                                           // imposto il primo pixel per auto incremento (il  -1 e l'orientamento sono nella funzione)

#if defined TFT_LANDSCAPE || defined TFT_LANDSCAPE_180                                  // se TFT_LANDSCAPE inverto _x e _y
    unsigned int _temp;
    _temp = _y_start;
    _y_start = _x_start;
    _x_start = _temp;
    
    _temp = _y_end;
    _y_end = _x_end;
    _x_end = _temp;
#endif 

    // limita la finestra alla risoluzione, se per errore si imposta una finestra pi grande viene mantenuta alla risoluzione massima
    if (_x_end > H_RESOLUTION)
        _x_end = H_RESOLUTION;
    if (_y_end > V_RESOLUTION) 
        _y_end = V_RESOLUTION;
    
    TFT_write_register(HORIZONTAL_GRAM_START, _x_start-1);                              // scrivo gli indirizzi nei registri
    TFT_write_register(HORIZONTAL_GRAM_END, _x_end-1);                                  // -1 per riportare a 0~239 - 0~319
    TFT_write_register(VERTICAL_GRAM_START, _y_start-1);
    TFT_write_register(VERTICAL_GRAM_END, _y_end-1);    
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_set_window_dir
 * Input:       _mode: modalit di incremento del contatore nella finestra: W_DEFAULT, W_IMG, W_BMP (unsigned char)
 *
 * Overview:    Imposta la modalit di incremento del puntatore in GRAM, da cambiare se si usano funzioni di stampa immagini
 * Note:        Dopo aver stampato l'immagine si deve ripristinare la modalit di incremento di default
/* ============================================================================================================================================ */
void TFT_set_window_dir(unsigned char _mode){    
    if (_mode == W_IMG){
        #if defined TFT_PORTRAIT
        TFT_write_register(DRIVER_OUTPUT_CTRL,0x0100);  //SS = 1
        TFT_write_register(ENTRY_MODE,0x1030);          //AM 0 ID1-0 11
        TFT_write_register(GATE_SCAN_CTRL_60H,0xA700);  //GS1
        #elif defined TFT_LANDSCAPE
        TFT_write_register(DRIVER_OUTPUT_CTRL,0x0100);  //SS = 1
        TFT_write_register(ENTRY_MODE,0x1038);          //AM 1 ID1-0 11
        TFT_write_register(GATE_SCAN_CTRL_60H,0x2700);  //GS0
        #elif defined TFT_PORTRAIT_180
        TFT_write_register(DRIVER_OUTPUT_CTRL,0x0000);  //SS = 0    
        TFT_write_register(ENTRY_MODE,0x1010);          //AM 0 ID1-0 01
        TFT_write_register(GATE_SCAN_CTRL_60H,0x2700);  //GS0
        #elif defined TFT_LANDSCAPE_180
        TFT_write_register(DRIVER_OUTPUT_CTRL,0x0000);  //SS = 0
        TFT_write_register(ENTRY_MODE,0x1038);          //AM 1 ID1-0 11
        TFT_write_register(GATE_SCAN_CTRL_60H,0xA700);  //GS1
        #endif
    }else if (_mode == W_BMP){
        #if defined TFT_PORTRAIT
        TFT_write_register(DRIVER_OUTPUT_CTRL,0x0100);  //SS = 1
        TFT_write_register(ENTRY_MODE,0x1010);          //AM 0 ID1-0 01
        TFT_write_register(GATE_SCAN_CTRL_60H,0xA700);  //GS1
        #elif defined TFT_LANDSCAPE
        TFT_write_register(DRIVER_OUTPUT_CTRL,0x0100);  //SS = 1
        TFT_write_register(ENTRY_MODE,0x1028);          //AM 1 ID1-0 10
        TFT_write_register(GATE_SCAN_CTRL_60H,0x2700);  //GS0
        #elif defined TFT_PORTRAIT_180
        TFT_write_register(DRIVER_OUTPUT_CTRL,0x0000);  //SS = 0
        TFT_write_register(ENTRY_MODE,0x1030);          //AM 0 ID1-0 11
        TFT_write_register(GATE_SCAN_CTRL_60H,0x2700);  //GS0
        #elif defined TFT_LANDSCAPE_180
        TFT_write_register(DRIVER_OUTPUT_CTRL,0x0000);  //SS = 0
        TFT_write_register(ENTRY_MODE,0x1028);  //AM 1 ID1-0 10
        TFT_write_register(GATE_SCAN_CTRL_60H,0xA700);  //GS1 
        #endif
    } else {
        #if defined TFT_PORTRAIT || defined TFT_LANDSCAPE
        TFT_write_register(DRIVER_OUTPUT_CTRL,0x0100);                                  // 0x01, SS=1 (shift direction of outputs is from S720 to S1) SM=0
        #elif defined TFT_PORTRAIT_180 || defined TFT_LANDSCAPE_180    
        TFT_write_register(DRIVER_OUTPUT_CTRL,0x0000);                                  // 0x01, SS=0 (shift direction of outputs is from SS1 to S720) SM=0
        #endif

        #if defined TFT_LANDSCAPE || defined TFT_LANDSCAPE_180
        TFT_write_register(ENTRY_MODE,0x1030);                                          // 0x03, AM=0, ID[1:0]=11 (h: incr, v: incr), BGR=?1? (Swap the RGB data to BGR in writing into GRAM), TRI=0 (8 bit 2 transfer or 16 bit 1 transfer (limit 65.536 color)  
        #elif defined TFT_PORTRAIT || defined TFT_PORTRAIT_180
        TFT_write_register(ENTRY_MODE,0x1038);                                          // 0x03, AM=1, ID[1:0]=11 (h: incr, v: incr), BGR=?1? (Swap the RGB data to BGR in writing into GRAM), TRI=0 (8 bit 2 transfer or 16 bit 1 transfer (limit 65.536 color)  
        #endif

        #if defined TFT_LANDSCAPE || defined TFT_PORTRAIT_180
        TFT_write_register(GATE_SCAN_CTRL_60H,0x2700);                                  // GS=0, SCN[5:0]=000000 (Gate Scan Line start 0), NL[5:0]=100111 (numbers of lines = 320 lines) 
        #elif defined TFT_PORTRAIT || defined TFT_LANDSCAPE_180 
        TFT_write_register(GATE_SCAN_CTRL_60H,0xA700);                                  // GS=1, SCN[5:0]=000000 (Gate Scan Line start 0), NL[5:0]=100111 (numbers of lines = 320 lines) 
        #endif
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_clear
 * Input:       _color: colore di riempimento di tutto lo schermo (unsigned int)
 *
 * Overview:    Colora tutto lo schermo, sfrutta l'auto incremento dell'indirizzo GRAM
/* ============================================================================================================================================ */
void TFT_clear(unsigned int _color){
                                                                                        
    TFT_set_window (1,1,X_MAX,Y_MAX);                                                   // imposto la finestra a seconda del verso di orientamento del display

    TFT_write_register_address(GRAM_DATA);                                              // scrivo l'indirizzo per passare i dati GRAM
    for (unsigned char _i = 0; _i < H_RESOLUTION; _i++){                                // il contatore degli indirizzi si incrementa in automatico
        for(unsigned int _j = 0; _j < V_RESOLUTION; _j++)
            TFT_write_register_value(_color);                                           // scrivo il colore
    }   
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_idle
 *
 * Overview:    Disattiva la visualizzazione, il display riceve ed esegue comandi
/* ============================================================================================================================================ */
void TFT_idle(void){
    TFT_write_register(DISPLAY_CTRL_1,0x0130);
} 

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_sleep
 *
 * Overview:    Ferma tutte le operazioni del Display tranne l'oscillatore. Aggiornamento della GRAM e operazioni non verranno eseguiti
/* ============================================================================================================================================ */
void TFT_sleep(void){
    TFT_write_register(DISPLAY_CTRL_1,0x0100);                                          // 0x07, PTDE[1:0]=00 (no partial images), BASEE=1 (base image displayed), GON=0 & DTE=0 (VGH), CL=0 (256k colors or 65k depends on transfer method), D1=0 D0=0
    TFT_write_register(POWER_CONTROL_1,0x1292);                                         // 0x10, SAP=1 (surce driver enaled), BT[2:0]=2 (step-up factor), APE=1 (power enabled), AP[2:0]=001 (1.0/1.0), SLP = 1 (sleep), STB=0 (non stand by)
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_standby
 *
 * Overview:    Ferma tutte le operazioni del Display tranne l'alimentazione della GRAM. Aggiornamento della GRAM e operazioni non verranno eseguiti
/* ============================================================================================================================================ */
void TFT_standby(void){
    TFT_write_register(DISPLAY_CTRL_1,0x0100);                                          // 0x07, PTDE[1:0]=00 (no partial images), BASEE=1 (base image displayed), GON=0 & DTE=0 (VGH), CL=0 (256k colors or 65k depends on transfer method), D1=0 D0=0
    TFT_write_register(POWER_CONTROL_1,0x1291);                                         // 0x10, SAP=1 (surce driver enaled), BT[2:0]=2 (step-up factor), APE=1 (power enabled), AP[2:0]=001 (1.0/1.0), SLP = 0 (non sleep), STB=1 (stand by)
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_off
 *
 * Overview:    Disattiva tutto il display e la retroilluminazione se gestita tramite I/O digitale
/* ============================================================================================================================================ */
void TFT_off(void){
    TFT_write_register(DISPLAY_CTRL_1,0x0100);                                          // 0x07, PTDE[1:0]=00 (no partial images), BASEE=1 (base image displayed), GON=0 & DTE=0 (VGH), CL=0 (256k colors or 65k depends on transfer method), D1=0 D0=0
    TFT_write_register(POWER_CONTROL_1,0x0080);                                         // 0x10, SAP=0 (surce driver enaled), BT[2:0]=2 (step-up factor), APE=1 (power enabled), AP[2:0]=000 (Halt), SLP = 0 (non sleep), STB=0 (non stand by)
    TFT_write_register(POWER_CONTROL_3,0x008C);                                         // 0x12, VCIRE=1 (int.ref 2.5V), PON=0 (VGL out disabled), VRH[3:0]= 1100 (VREG1 OUT = 2.5V x 1.80 = 4.500V)

#ifdef TFT_BACKILL
    TFT_BACKILL = OFF;
#endif
}              

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_active
 *
 * Overview:    Attiva il display dopo idle, sleep, standby e off
/* ============================================================================================================================================ */
void TFT_active(void){
TFT_write_register(POWER_CONTROL_1,0x1290);                                             // 0x10, SAP=1 (surce driver enaled), BT[2:0]=2 (step-up factor), APE=1 (power enabled), AP[2:0]=001 (1.0/1.0), SLP = 0 (non sleep), STB=0 (non stand by)
    Delay_ms(100);
    TFT_write_register(DISPLAY_CTRL_1,0x0133); 

#ifdef TFT_BACKILL
    TFT_BACKILL = ON;
#endif
}              

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_draw_pixel
 * Input:       _x: coordinata x del pixel da accendere: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y: coordinata y del pixel da accendere: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)   
 *              _color: colore del pixel (unsigned int)
 *
 * Overview:    Accende 1 pixel del colore selezionato all'indirizzo xy sul display
/* ============================================================================================================================================ */
void TFT_draw_pixel (unsigned int _x, unsigned int _y, unsigned int _color){
    TFT_set_gram_address(_x, _y);                                                       // imposto l'indirizzo GRAM del pixel
    TFT_write_register_address(GRAM_DATA);                                              // imposto l'indirizzo del buffer per passare i dati da scrivere all'ondirizzo GRAM precedentemente settato
    TFT_write_register_value(_color);                                                   // invio il colore da scrivere all'indirizzo GRAM
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_draw_image
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)
 *              _image: puntatore al primo indirizzo in flash dell'array contenente l'immagine
 *
 * Overview:    Scrive un immagine salvata in array sul Display
 * Note:        I primi 3 int dell'array devono contenere i dati di lunghezza, altezza e numero di colori (0xFFFF per colori a 16 bit)
/* ============================================================================================================================================ */
void TFT_draw_image(unsigned int _x,unsigned int _y, const unsigned int *_image) {
    TFT_set_window_dir(W_IMG);                                                          // imposto orientamento display e incremento puntatore per immagini
    
    TFT_set_window(_x, _y, _x + _image[0] -1 , _y + _image[1] -1);                      // setto la finestra da xy a lunghezza e altezza dell'immagine.
    TFT_write_register_address(GRAM_DATA);                                              // scrivo l'indirizzo del buffer per passare i valori alla GRAM
    
    for (unsigned int _i=0; _i < _image[0] * _image[1]; _i++){                          // ciclo di scrittura continua dei pixel
        TFT_write_register_value(_image[_i+3]);
    }
    
    TFT_set_window_dir(W_DEFAULT);                                                      // imposto orientamento display e incremento puntatore default
    TFT_set_window (1,1,X_MAX,Y_MAX);                                                   // resetto finestra a tutto display
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_draw_image_monocr
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)
 *              _image: puntatore al primo indirizzo in flash dell'array contenente l'immagine
 *              _color: colore dell'immagine (unsigned int)
 *              _bk_color: colore dello sfondo (unsigned int)
 *
 * Overview:    Scrive un immagine monocromatica salvata in array sul Display
 * Note:        I primi 5 char dell'array devono contenere i dati di lunghezza, altezza e byte altezza
/* ============================================================================================================================================ */
void TFT_draw_image_monocr(unsigned int _x,unsigned int _y, const unsigned char *_image, unsigned int _color, unsigned int _bk_color) {
    unsigned char _temp = 0;
    unsigned char _byte_height = _image[4];
    unsigned int _l = _image[0];
    _l = _l << 8;
    _l |= _image[1];
    
    unsigned int _h = _image[2];
    _h = _h << 8;
    _h |= _image[3];
    
    TFT_set_window(_x, _y, _x + _l, _y + _h -1);                                        // setto la finestra da xy a lunghezza e altezza dell'immagine.
    TFT_write_register_address(GRAM_DATA);                                              // scrivo l'indirizzo del buffer per passare i valori alla GRAM
        
    for (unsigned int _i=0; _i < (_l) * _byte_height; _i++){                            // ciclo di scrittura continua dei pixel
        _temp = _image[_i+5];
        
        for (unsigned char _j=0; _j < 8; _j++) {
            if (_temp & 0x01) 
                TFT_write_register_value(_color);                                       // scrivere pixel colorato
            else
                TFT_write_register_value(_bk_color);                                    // oppure colore di sfondo                
            _temp = _temp >> 1; 
        }
    }
    TFT_set_window (1,1,X_MAX,Y_MAX);                                                   // resetto finestra a tutto display
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_draw_line_x
 * Input:       _x_start: coordinata x di partenza: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _x_end: coordinata x finale: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y: coordinata y: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)
 *              _color: colore della linea (unsigned int)
 *
 * Overview:    Disegna una linea orizzontale su display da punto _x_start a punto _x_end compresi su riga _y
/* ============================================================================================================================================ */   
void TFT_draw_line_x (unsigned int _x_start, unsigned int _x_end, unsigned _y, unsigned int _color){
    unsigned int _i = 0;
    
    do{
        TFT_draw_pixel(_x_start+_i, _y, _color);
        _i++;
    }while ((_x_start +_i) <= _x_end);
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_draw_line_y
 * Input:       _y_start: coordinata y di partenza: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)
 *              _y_end: coordinata y finale: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)
 *              _x: coordinata x: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _color: colore della linea (unsigned int)
 *
 * Overview:    Disegna una linea verticale su display da punto _y_start a punto _y_end compresi su colonna _x
/* ============================================================================================================================================ */ 
void TFT_draw_line_y (unsigned int _y_start, unsigned int _y_end, unsigned _x, unsigned int _color){
    unsigned int _i = 0;
    do{
        TFT_draw_pixel(_x, _y_start+_i, _color);
        _i++;
    }while ((_y_start +_i) <= _y_end);
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_draw_line_p2p
 * Input:       _x_start: coordinata x di partenza: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y_start: coordinata y di partenza: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)             
 *              _x_end: coordinata x finale: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y_end: coordinata y finale: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)
 *              _color: colore della linea (unsigned int)
 *
 * Overview:    Disegna una linea da punto a punto su display da punto _x/_y_start a punto x/_y_end compresi
/* ============================================================================================================================================ */
void TFT_draw_line_p2p (unsigned int _x_start, unsigned int _y_start, unsigned int _x_end, unsigned int _y_end, unsigned int _color){
    unsigned int _x = _x_start;
    unsigned int _y = _y_start;
    signed int _x_inc1 = 0;
    signed int _x_inc2 = 0;
    signed int _y_inc1 = 0;
    signed int _y_inc2 = 0;
    unsigned int _den = 0;
    unsigned int _num = 0;
    unsigned int _numadd = 0;
    unsigned int _numpixels = 0;
    unsigned int _curpixel = 0;
    unsigned int _delta_x = 0;
    unsigned int _delta_y = 0;

    if (_y_start == _y_end) {                                                           // Se i 2 punti y sono uguali  devo disegnare una linea orizzontale
        if (_x_end > _x_start){                                                         // controllo la direzione
            TFT_draw_line_x(_x_start, _x_end, _y_start, _color);
        } else {
            TFT_draw_line_x(_x_end, _x_start, _y_start, _color);
        }
    } else if (_x_start == _x_end) {                                                    // Se i 2 punti x sono uguali devo disegnare una linea verticale
        if (_y_end > _y_start) {                                                        // controllo la direzione
            TFT_draw_line_y(_y_start, _y_end, _x_start, _color);
        } else {
            TFT_draw_line_y(_y_end, _y_start, _x_start, _color);
        }
    } else {                                                                            // altrimenti linea da punto a punto (routine trovata su internet e adattata)
        if (_x_end > _x_start) {                                                        // calcolo delta ed incrementi
            _delta_x = _x_end - _x_start;
            _x_inc1 = 1;
            _x_inc2 = 1;
        } else {
            _delta_x = _x_start - _x_end;
            _x_inc1 = -1;
            _x_inc2 = -1;
        }

        if (_y_end > _y_start) {
            _delta_y = _y_end - _y_start;
            _y_inc1 = 1;
            _y_inc2 = 1;
        } else {
            _delta_y = _y_start - _y_end;
            _y_inc1 = -1;
            _y_inc2 = -1;
        }

        if (_delta_x >= _delta_y) {
          _x_inc1 = 0;                                                                  // Don't change the x when numerator >= denominator
          _y_inc2 = 0;                                                                  // Don't change the y for every iteration
          _den = _delta_x;
          _num = _delta_x / 2;
          _numadd = _delta_y;
          _numpixels = _delta_x;                                                        // There are more x-values than y-values
        } else {                                                                        // There is at least one y-value for every x-value
          _x_inc2 = 0;                                                                  // Don't change the x for every iteration
          _y_inc1 = 0;                                                                  // Don't change the y when numerator >= denominator
          _den = _delta_y;
          _num = _delta_y / 2;
          _numadd = _delta_x;
          _numpixels = _delta_y;                                                        // There are more y-values than x-values
        }

        for (_curpixel = 0; _curpixel <= _numpixels; _curpixel++)
        {
          TFT_draw_pixel (_x, _y, _color);
          _num = _num + _numadd;                                                        // Increase the numerator by the top of the fraction
          if (_num >= _den)                                                             // Check if numerator >= denominator
          {
            _num -= _den;                                                               // Calculate the new numerator value
            _x += _x_inc1;                                                              // Change the x as appropriate
            _y += _y_inc1;                                                              // Change the y as appropriate
          }
          _x += _x_inc2;                                                                // Change the x as appropriate
          _y += _y_inc2;                                                                // Change the y as appropriate
        }
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_draw_line_angle
 * Input:       _x_start: coordinata x di partenza: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y_start: coordinata y di partenza: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int) 
 *              _length: lunghezza (unsigned int) 
 *              _angle: angolo: 0-360 (unsigned int)
 *              _color: colore della linea (unsigned int)
 *
 * Overview:    Disegna una linea da punto origine di lunghezza stabilita con angolo su display
/* ============================================================================================================================================ */
void TFT_draw_line_angle (unsigned int _x_start, unsigned int _y_start, unsigned char _length,  unsigned int _angle, unsigned int _color) {
    TFT_draw_line_p2p(_x_start, _y_start,(signed int) (_x_start + _length * cos(_angle * 0.017453292519)), (signed int) (_y_start + _length * sin(_angle * 0.017453292519)), _color);
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_draw_circle
 * Input:       _x_centr: coordinata x del centro del cerchio: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y_centr: coordinata y del centro del cerchio: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int) 
 *              _r: raggio unsigned int)
 *              _color: colore della linea (unsigned int)
 *
 * Overview:    Disegna un cerchio dato centro e raggio su display
/* ============================================================================================================================================ */
void TFT_draw_circle(unsigned int _x_centr, unsigned int _y_centr, unsigned int _r, unsigned int _color){                 // routine trovata su internet e adattata
    unsigned int _x = 0;
    unsigned int _y = _r;
    signed int _p = 0;

    _p= -(_r/2);
    TFT_Plot8(_x_centr, _y_centr, _x, _y, 0x0F, _color);
    while(_x<_y){
        _x++;
        if(_p<0)
            _p=_p+2*_x+1;
        else{
            _y--;
            _p=_p+2*(_x-_y)+1;
        }
        TFT_Plot8(_x_centr,_y_centr, _x, _y, 0x0F, _color);
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_draw_arc
 * Input:       _x_centr: coordinata x del centro del cerchio: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y_centr: coordinata y del centro del cerchio: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int) 
 *              _r: raggio (unsigned int)
 *              _corner: angolo da disegnare
 *              _color: colore della linea (unsigned int)
 *
 * Overview:    Disegna un arco dato centro, raggio e angolo su display
/* ============================================================================================================================================ */
void TFT_draw_arc(unsigned int _x_centr, unsigned int _y_centr, unsigned int _r, unsigned char _corner, unsigned int _color){                 // routine trovata su internet e adattata
    unsigned int _x = 0;
    unsigned int _y = _r;
    signed int _p = 0;

    _p= -(_r/2);
    TFT_Plot8(_x_centr, _y_centr, _x, _y, _corner, _color);
    while(_x<_y){
        _x++;
        if(_p<0)
            _p=_p+2*_x+1;
        else{
            _y--;
            _p=_p+2*(_x-_y)+1;
        }
        TFT_Plot8(_x_centr,_y_centr, _x, _y, _corner, _color);
    }
}

void TFT_Plot8(unsigned int _x, unsigned int _y, unsigned int _dx, unsigned int _dy, unsigned char _corner, unsigned int _color){
    if (_corner & 0b0001){
        TFT_draw_pixel (_x+_dx, _y-_dy, _color);    // 1 
        TFT_draw_pixel (_x+_dy, _y-_dx, _color);    //
    }

    if (_corner & 0b0010){
        TFT_draw_pixel (_x+_dx, _y+_dy, _color);    // 2
        TFT_draw_pixel (_x+_dy, _y+_dx, _color);    //
    }
    
    if (_corner & 0b0100){
        TFT_draw_pixel (_x-_dx, _y+_dy, _color);    // 3 
        TFT_draw_pixel (_x-_dy, _y+_dx, _color);    //    
    }
    
    if (_corner & 0b1000){
        TFT_draw_pixel (_x-_dx, _y-_dy, _color);    // 4
        TFT_draw_pixel (_x-_dy, _y-_dx, _color);    //
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_fill_circle
 * Input:       _x_centr: coordinata x del centro del cerchio: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y_centr: coordinata y del centro del cerchio: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int) 
 *              _r: raggio unsigned int)
 *              _color: colore della linea (unsigned int)
 *
 * Overview:    Disegna un cerchio pieno dato centro e raggio su display
/* ============================================================================================================================================ */
void TFT_fill_circle(unsigned int _x_centr,unsigned int _y_centr ,unsigned int _r, unsigned int _color){
signed int _xoff =0;
signed int _yoff = _r;
signed int _balance = -_r;
unsigned int _p0, _p1, _w0, _w1;

    while (_xoff <= _yoff) {
         _p0 = _x_centr - _xoff;
         _p1 = _x_centr - _yoff;
         _w0 = _xoff + _xoff;
         _w1 = _yoff + _yoff;

         TFT_draw_line_x(_p0,_p0+_w0,_y_centr+_yoff,_color);
         TFT_draw_line_x(_p0,_p0+_w0,_y_centr-_yoff,_color);
         TFT_draw_line_x(_p1,_p1+_w1,_y_centr+_xoff,_color);
         TFT_draw_line_x(_p1,_p1+_w1,_y_centr-_xoff,_color);

        if ((_balance += _xoff++ + _xoff)>= 0) {
            _balance  -= --_yoff+_yoff;
        }
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_draw_rect
 * Input:       _x_start: coordinata x di partenza: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y_start: coordinata y di partenza: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)             
 *              _l: lunghezza: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _h: altezza: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)
 *              _color: colore del rettangolo (unsigned int)
 *
 * Overview:    Disegna una rettangolo su display
/* ============================================================================================================================================ */
void TFT_draw_rect(unsigned int _x_start, unsigned int _y_start, unsigned int _l, unsigned int _h, unsigned int _color){
    _l = _x_start + _l -1;                                                              // -1 per togliere il pixel iniziale dalla lunghezza e dall'altezza totali
    _h = _y_start + _h -1;
    
    TFT_draw_line_x (_x_start, _l, _y_start, _color);
    TFT_draw_line_x (_x_start, _l, _h, _color);
    TFT_draw_line_y (_y_start, _h, _x_start, _color);
    TFT_draw_line_y (_y_start, _h, _l, _color);
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_draw_rounded_rect
 * Input:       _x_start: coordinata x di partenza: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y_start: coordinata y di partenza: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)             
 *              _l: lunghezza: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _h: altezza: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)
 *              _r: raggio degli angoli: 1~255 (unsigned int)
 *              _color: colore del rettangolo (unsigned int)
 *
 * Overview:    Disegna una rettangolo con angoli arrotondati su display
/* ============================================================================================================================================ */
void TFT_draw_rounded_rect(unsigned int _x_start, unsigned int _y_start, unsigned int _l, unsigned int _h, unsigned int _r, unsigned int _color){
    _l = _x_start + _l -1 - _r;                                                              // -1 per togliere il pixel iniziale dalla lunghezza e dall'altezza totali
    _h = _y_start + _h -1 - _r;
    
    TFT_draw_line_x (_x_start +_r, _l, _y_start, _color);
    TFT_draw_line_x (_x_start +_r, _l, _h+_r, _color);
    TFT_draw_line_y (_y_start +_r, _h, _x_start, _color);
    TFT_draw_line_y (_y_start +_r, _h, _l+_r, _color);
    
    TFT_draw_arc(_l, _y_start + _r, _r, 1, _color);
    TFT_draw_arc(_l, _h, _r, 2, _color);
    TFT_draw_arc(_x_start + _r, _h, _r, 4, _color);
    TFT_draw_arc(_x_start + _r, _y_start + _r, _r, 8, _color);
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_fill_rect
 * Input:       _x_start: coordinata x di partenza: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y_start: coordinata y di partenza: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)             
 *              _l: lunghezza: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _h: altezza: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)
 *              _color: colore del rettangolo (unsigned int)
 *
 * Overview:    Disegna un rettangolo pieno su display, sfrutta la finestra e l'auto incremento dell'indirizzo GRAM
/* ============================================================================================================================================ */
void TFT_fill_rect(unsigned int _x_start, unsigned int _y_start, unsigned int _l, unsigned int _h, unsigned int _color){
    _l = _x_start + _l-1;                                                               // -1 per togliere il pixel iniziale dalla lunghezza e dall'altezza totali
    _h = _y_start + _h-1;                                                               // per velocizzare lo riempimento creo una finestra e sfrutto il puntatore
                                                                                        // auto incrementante                                                                            
    TFT_set_window(_x_start, _y_start, _l, _h);                                         // setto la finestra
    
    TFT_write_register_address(GRAM_DATA);                                              // scrivo l'indirizzo del buffer per passare i valori alla GRAM
    for (unsigned int _i = 0; _i <= _l; _i++){
        for(unsigned int _j = 0; _j <= _h; _j++)
            TFT_write_register_value(_color);                                           // scrivo il colore
    }
    TFT_set_window (1,1,X_MAX,Y_MAX);                                                   // resetto finestra a tutto display
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_fill_rounded_rect
 * Input:       _x_start: coordinata x di partenza: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y_start: coordinata y di partenza: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)             
 *              _l: lunghezza: 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _h: altezza: 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)
 *              _r: raggio degli angoli: 1~255 (unsigned int)
 *              _color: colore del rettangolo (unsigned int)
 *
 * Overview:    Disegna una rettangolo pieno con angoli arrotondati su display
/* ============================================================================================================================================ */
void TFT_fill_rounded_rect(unsigned int _x_start, unsigned int _y_start, unsigned int _l, unsigned int _h, unsigned int _r, unsigned int _color){
    
    TFT_fill_rect(_x_start, _y_start + _r, _l, _h-(2*_r), _color);
    TFT_fill_rect(_x_start + _r, _y_start, _l-(2*_r), _r, _color);
    TFT_fill_rect(_x_start + _r, _y_start + _h -_r, _l-(2*_r), _r, _color);
    
    TFT_fill_circle(_x_start + _r, _y_start +_r, _r, _color);
    TFT_fill_circle(_x_start + _r, _y_start +_h -_r-1, _r, _color);
    TFT_fill_circle(_x_start + _l -_r-1, _y_start +_h -_r-1, _r, _color);
    TFT_fill_circle(_x_start + _l -_r-1,  _y_start +_r, _r, _color);
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_set_font
 * Input:       *_font_pointer: puntatore all'array del font
 *
 * Overview:    Imposta le variabili globali per il font selezionato
/* ============================================================================================================================================ */
void TFT_set_font(const unsigned char *_font_pointer){
    font_width = _font_pointer[0];
    font_height = _font_pointer[1];
    font_space = _font_pointer[2];
    font_first_char_offset = _font_pointer[3];
    font_vertical_byte = _font_pointer[4];
        
    font_pointer = _font_pointer;
    font_offset_btw_char = font_width * font_vertical_byte;                             // calcolo di quanti byte devo spostarmi nella tabella per ogni carattere
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_write_char
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)             
 *              _character: carattere da scrivere (unsigned char)
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 *              _color: colore del carattere (unsigned int)
 *              _bk_color: colore dello sfondo (unsigned int)
 * Output:      valore di x effettivamente scritti (unsigned char)              
 *
 * Overview:    Scrive un singlo carattere su Display e colora lo sfondo con i colori selezionati.
 * Precond:     Impostare il font tramite la funzione TFT_set_font()
 * Note:        La scrittura del carattere viene eseguita tramite finestra di dimensioni impostate tramite le dimensioni del font. In questo modo
 *              la scrittura  pi veloce e vengono scritti anche tutti i pixel che non rappresentano il carattere (sfondo) cancellando quindi
 *              in automatico eventuali scritte precedenti  (fino alla grandezza del font).
 *              Se si devono scrivere caratteri variabili usare SAVESPACE_OFF
/* ============================================================================================================================================ */
unsigned char TFT_write_char(unsigned int _x, unsigned int _y, unsigned char _character, unsigned char _savespace, unsigned int _color, unsigned int _bk_color){
    unsigned char _effective_x = 0;
    
    TFT_set_window(_x, _y, _x + font_width-1, _y + font_height-1);                      // setto la finestra  (-1 per calcolare anche il primo pixel)
    TFT_write_register_address(GRAM_DATA);                                              // scrivo l'indirizzo del buffer per passare i valori alla GRAM
       
    if (_character >= 33 &&  _character <= 151) {                                       // se carattere valido 
        _effective_x = TFT_write_char_w(_character, _savespace, _color, _bk_color);     // scrivo carattere nella finestra
    } else {
        for (unsigned char _m = 0; _m < font_width; _m++){                              // scrivo solo sfondo, larghezza massima font anche se SAVESPACE = ON
            for (unsigned char _k = 0; _k<font_height; _k++){
                TFT_write_register_value(_bk_color);
            }
            _effective_x++;
        }
    }

    TFT_set_window (1,1,X_MAX,Y_MAX);                                                   // resetto finestra a tutto display
    return _effective_x;
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_write_char_w
 * Input:       _character: carattere da scrivere (unsigned char)
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 *              _color: colore del carattere (unsigned int)
 *              _bk_color: colore dello sfondo (unsigned int)
 * Output:      valore di x effettivamente scritti (unsigned char)              
 *
 * Overview:    Scrive caratteri nella finestra selezionata, da usare solo dopo aver impostato la finestra (usato da TFT_write_char() e TFT_write_string())
 * Precond:     Impostare il font tramite la funzione TFT_set_font()
/* ============================================================================================================================================ */
unsigned char TFT_write_char_w(unsigned char _character, unsigned char _savespace, unsigned int _color, unsigned int _bk_color){
    unsigned char _data[10] = {0};                                                      // buffer temporaneo per i pixel in altezza, aumentare dimensione se si vogliono usare font con altezza > 10 byte

    signed char _b = 0;
    unsigned char _effective_x = 0;
    unsigned char _y_counter = 0;
    unsigned int _index;
    
    _index = _character - font_first_char_offset;                                       // tolgo l'offset dei caratteri dell'array 
    _index *= font_offset_btw_char;                                                     // calcolo il byte da prendere in base al carattere e all'offset tra i caratteri
    
    
    for (unsigned char _i = 0; _i < font_width; _i++){                                  // ciclo per larghezza
        _b = 0;
        
        do{                                                                             // creo array temporaneo di byte per pixel in altezza del carattere
            // _character = byte nell'array, _i * font_vertical_byte = colonna di byte che formano l'altezza del carattere, _b = byte della colonna, 5 = primi 5 byte contenenti le informazioni del font  
//            volatile unsigned int calc = (_index) + (_i * font_vertical_byte) + _b + 5;
            _data[_b] = font_pointer[(_index) + (_i * font_vertical_byte) + _b + 5];
            _b++;
        } while (_b < font_vertical_byte);
        
        _b = 0;
        _y_counter = 0;
        
        
        if (_data[0]|_data[1]|_data[2]|_data[3]|_data[4]|_data[5]|_data[6]|_data[7]|_data[8]|_data[9] != 0){    // se la colonna ha dei valori li scrivo su display
            for (unsigned char _j = 0; _j < font_height; _j++) {                        // ciclo per bit per vedere se
                if (_data[_b] & 0x01)                                                   
                    TFT_write_register_value(_color);                                   // scrivere pixel colorato per il carattere  
                else 
                    TFT_write_register_value(_bk_color);                                // oppure colore di sfondo
                
                _data[_b] = _data[_b] >> 1;                                             
                _y_counter++;                                                           // contatore per passare al byte successivo dell'array colonna
                if (_y_counter==8){
                    _y_counter = 0;
                    _b++;
                }
            }
            _effective_x++;
        } else if (_savespace == SAVESPACE_OFF){                                        // se non vanno scritti valori del carattere ma savespaCe  disattivato
            _effective_x++;                                                             // incremento comunque x
            for (unsigned char _k = 0; _k < font_height; _k++)                          // e riempio la colonna con il colore dello sfondo
                TFT_write_register_value(_bk_color);
        }
    }
    
    return (_effective_x);                                                              // torno valore di x effettivamente scritte
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_write_space_btw_chars
 * Input:       _bk_color: colore dello sfondo (unsigned int)
 * Output:      valore di x effettivamente scritti (unsigned char)              
 *
 * Overview:    Riempie lo spazio tra i caratteri con il colore sfondo e torna il valore di x effettivamente scritte.
 * Precond:     Impostare il font tramite la funzione TFT_set_font()
/* ============================================================================================================================================ */
unsigned char TFT_write_space_btw_chars (unsigned int _bk_color){
    unsigned char _effective_x = 0;
    for (unsigned char _m = 0; _m < font_space; _m++){                                  // aggiungo spazio a fine carattere
        for (unsigned char _k = 0; _k<font_height; _k++){
            TFT_write_register_value(_bk_color);
        }
        _effective_x++;
    }
    return _effective_x;
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_write_string
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)    
 *              _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _string[]: puntatore ad array di caratteri da scrivere, byte finale \0 (unsigned char)
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 *              _color: colore del carattere (unsigned int)
 *              _bk_color: colore dello sfondo (unsigned int)
 *
 * Overview:    Scrive una stringa di caratteri su Display e colora lo sfondo con i colori selezionati.
 * Note:        La scrittura della stringa viene eseguita tramite finestra di dimensioni impostate tramite le dimensioni del font e la variabile
 *              in ingresso _max_x. In questo modo a scrittura  pi veloce e vengono scritti anche tutti i pixel che non rappresentano il carattere (sfondo)
 *              cancellando quindi in automatico eventuali scritte precedenti (fino a _x_max).
/* ============================================================================================================================================ */
void TFT_write_string (unsigned int _x, unsigned int _y, unsigned int _max_x, const unsigned char _string[], unsigned char _savespace, unsigned int _color, unsigned int _bk_color){
    unsigned char _i = 0;
    unsigned int _incremental_x_pos = 0;                                                // variabile statica per il conteggio delle x effettivamente scritte
        
    TFT_set_window(_x, _y, _x + _max_x - 1, _y + font_height-1);                        // setto la finestra
    TFT_write_register_address(GRAM_DATA);                                              // scrivo l'indirizzo del buffer per passare i valori alla GRAM
    
    while (_string[_i]!='\0') {                                                         // per ogni carattere della stringa fino a carattere nullo
        if (_string[_i] == ' ') {                                                       // se carattere spazio
            for (unsigned char _m = 0; _m < 4 * font_space; _m++){                      // riempio tante colonne in relazione al font_space (3*font_space)
                for (unsigned char _k = 0; _k<font_height; _k++){
                    TFT_write_register_value(_bk_color);
                }
            }
            _incremental_x_pos += 4 * font_space;                                       // e aumento di conseguenza il contatore
        } else {                                                                        // altrimenti scrivo il carattere e aumento di conseguenza la posizione x
            _incremental_x_pos += TFT_write_char_w(_string[_i], _savespace, _color, _bk_color);
            _incremental_x_pos += TFT_write_space_btw_chars(_bk_color);
        }
        _i++;
    }
    
    while (_x + _incremental_x_pos < _x + _max_x ){                                     // riempio la parte restante della finestra per cancellare eventuali vecchie scritte
        for (unsigned char _k = 0; _k<font_height; _k++){
            TFT_write_register_value(_bk_color);
        }
        _incremental_x_pos++;        
    }   
    TFT_set_window (1,1,X_MAX,Y_MAX);                                                   // resetto finestra a tutto display
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_write_unsint
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)    
 *              _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _unsint: valore da scrivere (unsigned int)
 *              _number_digits: numero di cifre da scrivere (1~5, unsigned char) 
 *              _zero_cleaning: scrive o cancella zero iniziali (ZERO_CLEANING_ON/ZERO_CLEANING_OFF)
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 *              _color: colore del carattere (unsigned int)
 *              _bk_color: colore dello sfondo (unsigned int)
 *
 * Overview:    Scrive un numero unsigned int su Display dopo averlo convertito in array stringa di caratteri ASCII 
/* ============================================================================================================================================ */
void TFT_write_unsint(unsigned int _x, unsigned int _y, unsigned int _max_x, unsigned int _unsint, unsigned char _number_digits, unsigned char _zero_cleaning, unsigned char _savespace, unsigned int _color, unsigned int _bk_color) {
    unsigned char _i=0;
    unsigned int _denom = 0;
    unsigned char _converted[5] = {0};
    unsigned char _converted_format[6] = {0};
    unsigned char _first_non_zero = 0;
    
    if (_number_digits > 5)                                                             // limito il numero massimo di cifre a 5 (unsigned int valore massimo  65535)
        _number_digits = 5;
    
    for (_i=0; _i<4; _i++) {                                                            // converto unsigned int
        if (_i==0) {_denom = 10000;}                                                    
        if (_i==1) {_denom = 1000;}                                                     
        if (_i==2) {_denom = 100;}                                                      
        if (_i==3) {_denom = 10;}                                                       

         _converted[_i] = _unsint/_denom;                                               // in array da 5 caratteri
        if (_converted[_i]) {                                                           // se risultato != 0
            _unsint = _unsint - (_denom * _converted[_i]);                              // sottraggo il valore gi  scritto
        }
    }
    _converted[4]= _unsint;                                                             // riporto il resto nell'array

    // in array _converted ho il dato completo di 0 iniziali

    _i = 0;
    for (unsigned char _c = 0; _c < _number_digits-1 ; _c++) {                          // ciclo per numero di cifre -1 (l'ultima la scrivo sempre)

        if (_converted[_c + (5 -_number_digits)] != 0) {
            _converted_format[_i] = _converted[_c + (5 -_number_digits)] + 0b00110000;
            _first_non_zero = 1;
        } else {
            if (_zero_cleaning == ZERO_CLEANING_ON && _first_non_zero == 0)
                _i--;
            else
                _converted_format[_i] = '0';
        }
        _i++;
    }

    _converted_format[_i] =  _converted[4] + 0b00110000;
    _converted_format[_i+1] = '\0';                                                     // aggiungo valore di fine stringa
    TFT_write_string(_x, _y, _max_x, _converted_format, _savespace, _color, _bk_color); // scrivo la stringa dei valori
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_write_sint
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)    
 *              _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _sint: valore da scrivere (signed int)
 *              _number_digits: numero di cifre da scrivere (1~5, unsigned char) 
 *              _zero_cleaning: scrive o cancella zero iniziali (ZERO_CLEANING_ON/ZERO_CLEANING_OFF)
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 *              _color: colore del carattere (unsigned int)
 *              _bk_color: colore dello sfondo (unsigned int)
 *
 * Overview:    Scrive un numero signed int su Display dopo averlo convertito in array stringa di caratteri ASCII 
/* ============================================================================================================================================ */
void TFT_write_sint (unsigned int _x, unsigned int _y, unsigned int _max_x, signed int _sint, unsigned char _number_digits, unsigned char _zero_cleaning, unsigned char _savespace, unsigned int _color, unsigned int _bk_color){
    unsigned int _incremental_x_pos = 0;
    
    TFT_set_window(_x, _y, _x + font_width + font_space -1, _y + font_height-1);                      // setto la finestra  (-1 per calcolare anche il primo pixel)
    TFT_write_register_address(GRAM_DATA);                                              // scrivo l'indirizzo del buffer per passare i valori alla GRAM
    
    if (_sint <0) {                                                                     // se < 0
        _incremental_x_pos = TFT_write_char_w('-', SAVESPACE_ON, _color, _bk_color);            // scrivo il segno -
        _incremental_x_pos += TFT_write_space_btw_chars(_bk_color);
        TFT_write_unsint((_x + _incremental_x_pos), _y, _max_x, ~(_sint-1), _number_digits, _zero_cleaning, _savespace, _color, _bk_color);    // scrivo sul display il complemento a 1 + 1
    } else {                                                                            // se positivo
        _incremental_x_pos = TFT_write_char_w('+', SAVESPACE_ON, _color, _bk_color);            // scrivo il segno -
        _incremental_x_pos += TFT_write_space_btw_chars(_bk_color);
        TFT_write_unsint(_x + _incremental_x_pos, _y, _max_x + font_width , _sint, _number_digits, _zero_cleaning, _savespace, _color, _bk_color);                       // tratto come se fosse un unsigned int
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_write_PBCD
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)    
 *              _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _pbcd: valore numerico codificato in packed BCD (unsigned char)
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 *              _color: colore del carattere (unsigned int)
 *              _bk_color: colore dello sfondo (unsigned int)
 *
 * Overview:    Scrive un numero codificato in 1 byte packed BCD su Display 
/* ============================================================================================================================================ */
void TFT_write_PBCD(unsigned int _x, unsigned int _y, unsigned int _max_x, unsigned char _pbcd, unsigned char _savespace, unsigned int _color, unsigned int _bk_color){
    unsigned char _string[3];
    
    _string[0] = ((_pbcd & 0b11110000) >> 4) + '0';                                     // decine = 4 bit alti del dato in ingresso
    _string[1] = (_pbcd & 0b00001111) + '0';                                            // unit  = 4 bit bassi del byte in ingresso;
    _string[2] = '\0';
    
    TFT_write_string(_x, _y, _max_x, _string, _savespace, _color, _bk_color);           // scrivo la stringa dei valori
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_write_HEX
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)    
 *              _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _hex: valore da scrivere (unsigned char)    
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 *              _color: colore del carattere (unsigned int)
 *              _bk_color: colore dello sfondo (unsigned int)
 *
 * Overview:    Scrive un byte HEX su display
/* ============================================================================================================================================ */
void TFT_write_HEX(unsigned int _x, unsigned int _y, unsigned int _max_x, unsigned char _hex, unsigned char _savespace, unsigned int _color, unsigned int _bk_color){
    unsigned char _H = 0;                                                               // decine
    unsigned char _L = 0;                                                               // unit 
    unsigned char _string[3];
    
    _L = _hex & 0b00001111;                                                             // unit  = 4 bit bassi del byte in ingresso
    _H = (_hex & 0b11110000) >> 4;                                                      // decine = 4 bit alti del dato in ingresso

    if (_H < 10)
        _string[0] = _H | 0b00110000;
    else
        _string[0] = _H + 0b00110111;
    
    if (_L < 10)
        _string[1] = _L | 0b00110000;
    else
        _string[1] = _L + 0b00110111;
    
    _string[2] = '\0';
    
    TFT_write_string(_x, _y, _max_x, _string, _savespace, _color, _bk_color);           // scrivo la stringa dei valori
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_write_bit_string
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)    
 *              _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _byte: valore da scrivere (unsigned char)
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 *              _color: colore del carattere (unsigned int)
 *              _bk_color: colore dello sfondo (unsigned int)
 *
 * Overview:    Scrive un byte come stringa di bit 1-0 su display
/* ============================================================================================================================================ */
void TFT_write_bit_string (unsigned int _x, unsigned int _y, unsigned int _max_x, unsigned char _byte, unsigned char _savespace, unsigned int _color, unsigned int _bk_color) {
    unsigned char _string[9] = {0};
    
    for (unsigned char _i = 0; _i<8; _i++){
        if (_byte & 0x80)
            _string[_i] = '1';
        else 
            _string[_i] = '0';
        
        _byte <<= 1;
    }
    
    _string[8] = '\0';
    
    TFT_write_string(_x, _y, _max_x, _string, _savespace, _color, _bk_color);           // scrivo la stringa dei valori
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_write_float
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)    
 *              _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _float: valore da scrivere (float)
 *              _digits_int: numero di cifre interi da scrivere (1~5, unsigned char) 
 *              _digits_part: numero di cifre decimali da scrivere (1~10, unsigned char)
 *              _zero_cleaning: scrive o cancella zero iniziali (ZERO_CLEANING_ON/ZERO_CLEANING_OFF)
 *              _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 *              _color: colore del carattere (unsigned int)
 *              _bk_color: colore dello sfondo (unsigned int)
 *
 * Overview:    Scrive un valore float su display
/* ============================================================================================================================================ */
void TFT_write_float (unsigned int _x, unsigned int _y, unsigned int _max_x, float _float, unsigned char _digits_int, unsigned char _digits_part, unsigned char _zero_cleaning, unsigned char _savespace, unsigned int _color, unsigned int _bk_color) {
    signed int _int = 0;
    float _part = 0;
    unsigned char _converted[5] = {0};
    unsigned char _string_part[10] = {0};
    unsigned char _string_compl[18] = {0};                                              // stringa finale segno + max 5 interi + , + max 10 decimali + carattere di fine stringa
    unsigned char _string_index = 0;
    
    unsigned int _denom = 0;
    unsigned char _first_non_zero = 0;
    
    if (_digits_int > 5)                                                                // valore massimo di interi
        _digits_int = 5;
    
    if (_digits_part > 10)                                                              // valore massimo di decimali = 10
        _digits_part = 10;
        
    if (_float < 0){                                                                    // valore negativo
        _float *= -1;                                                                   // porto positivo
       _string_compl[0] = '-';                                                          // e scrivo segno meno nella stringa finale
       _string_index++;
    }
    
    _int = (int) _float;                                                                // estraggo la parte intera
    _part = _float - (float)_int;                                                       // lascio solo la parte decimale
    
    // converto decimali in stringa
    for (unsigned char _i = 0; _i < _digits_part +1; _i++ ){                            // per ogni decimale richiesto +1 per calcolo approssimazione
        _part = _part *10;                                                              // moltiplico per 10
        _string_part[_i] = (unsigned char)_part;                                        // estraggo la parte intera
        _part = _part - (float) _string_part[_i];                                       // dalla parte decimale sottraggo la nuova intera
                                                     
    }    
    
    // arrotondamento decimali
    if (_digits_part == 0) {                                                            // 0 cifre decimali
        if (_string_part[0] > 4){                                                       // se la prima decimale  maggiore di 4 aggiungo 1 agli interi
            if (_int >= 0)                                                              // se positivo
                _int++;
            else                                                                        // altrimenti sottraggo 1
                _int--;    
        }
    } else {                                                                            // se ho pi cifre decimali
        if (_string_part[_digits_part] > 4){                                            // se l'ultima decimale (aggiunta solo per l'arrotondamento)  maggiore di 4
                _string_part[_digits_part - 1]++;                                       // aggiungo uno alla decimale precedente
        }
        
        for (unsigned char _i = 0; _i < _digits_part -1; _i++ ){                        // controlla dalla penultima decimale alla prima
            if (_string_part[_digits_part - 1 - _i] > 9){                               // se maggiore di 9
                _string_part[_digits_part - 1 - _i] = 0;                                // la porto a 0
                _string_part[_digits_part - 2 - _i]++;                                  // e aggiungo uno alla precedente
            }
        }

        if (_string_part[0] > 9){                                                       // se la prima decimale  maggiore di 9
            _string_part[0] = 0;                                                        // la azzero e incremento o decremento gli interi
            if (_int >= 0)
                _int++;
            else
                _int--;
        }
    
    }
    
     // conversione intero in stringa
    for (unsigned char _i=0; _i<4; _i++) {                                              // converto unsigned int
        if (_i==0) {_denom = 10000;}                                                    
        if (_i==1) {_denom = 1000;}                                                     
        if (_i==2) {_denom = 100;}                                                      
        if (_i==3) {_denom = 10;}                                                       

        _converted[_i] = _int/_denom;                                                   // in array da 5 caratteri
        if (_converted[_i]) {                                                           // se risultato != 0
            _int = _int - (_denom * _converted[_i]);                                    // sottraggo il valore gi  scritto
        }
    }
    _converted[4]= _int;                                                                // riporto il resto nell'array

                                                                                        // in array _converted ho il dato completo di 0 iniziali
                                                                                        // sposto in stringa finale solo le cifre massime definite da _digits_int e da _zero_cleaning 
    for (unsigned char _c = 0; _c < _digits_int-1 ; _c++) {                             // ciclo per numero di cifre -1 (l'ultima la scrivo sempre)

        if (_converted[_c + (5 -_digits_int)] != 0) {
            _string_compl[_string_index] = _converted[_c + (5 -_digits_int)] + 0b00110000;
            _first_non_zero = 1;
        } else {
            if (_zero_cleaning == ZERO_CLEANING_ON && _first_non_zero == 0)
                _string_index--;
            else
                _string_compl[_string_index] = '0';
        }
        _string_index++;
    }
    _string_compl[_string_index] =  _converted[4] + 0b00110000;
    
    _string_index++;
    _string_compl[_string_index] = ',';                                                 // salvo carattere , nella stringa
    _string_index++;
    
    for (unsigned char _i = 0; _i < _digits_part; _i++){
        _string_compl[_string_index] = _string_part[_i] |= 0b00110000;                  // sposto i valori dall'array _string_part a _string_compl convertendolo in carattere ASCII
        _string_index++;
    }
    
    _string_compl[_string_index] = '\0';                                                // aggiungo carattere di fine stringa
    
    
    TFT_write_string(_x , _y, _max_x, _string_compl, _savespace, _color, _bk_color);    // scrivo la stringa dei valori
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_RGB_convert
 * Input:       _r: valore del colore rosso 0 ~ 0xFF (unsigned char)
 *              _g: valore del colore verde 0 ~ 0xFF (unsigned char)
 *              _b: valore del colore blu 0 ~ 0xFF (unsigned char)
 * Output:      valore del colore RGB formattato in 5-6-5 bit (unsigned int)              
 *
 * Overview:    Compone il colore RGB in intero a 5-6-5 bit (8-8-8 a 5-6-5)
/* ============================================================================================================================================ */
unsigned int TFT_RGB_convert(unsigned char _r, unsigned char _g, unsigned char _b){
    
    _r = (_r & 0x1F) << 11; 
    _g = (_g & 0x3F) << 5; 
    _b = _b & 0x1F;
    
    return _r | _g | _b;
}

/* ============================================================================================================================================= /
 * FUNCTION:    TFT_draw_image_sd
 * Input:       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 *              _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)
 *              _image: stringa contenente il nome del file con estensione (stringa)
 *
 * Overview:    Scrive un immagine salvata su memoria SD sul Display
 * Precond;     - Dichiarare le variabili per lettura SD in main.c
 *              - Impostare il SD_BUFFER in TFT_ILI9325_settings.h per velocizzare la scrittura di grosse immagini 
 * Note:        L'immagine deve essere salvata in BMP Windows V3, in modalit 16 bit 5r6g5b
 *              Se l'immagine ha una larghezza dispari i programmi aggiungono un pixel nero,  stato aggiunto un ciclo di controllo che in caso di 
 *              immagini con numero di pixel di larghezza dispari non scrive su display il pixel nero aggiunto.
/* ============================================================================================================================================ */
#ifdef _FATFS
void TFT_draw_image_sd(unsigned int _x,unsigned int _y, const unsigned char *_image){
    unsigned int _sd_buffer_x_2 = SD_BUFFER * 2; 
    static unsigned char _buffer[SD_BUFFER * 2] = {0};
    unsigned long _height = 0;
    unsigned long _width = 0;
    unsigned int _width_counter = 0;
    unsigned int _start = 0;
    unsigned int _color = 0;
    unsigned long _compres = 0;
    unsigned long _pixel = 0;
    unsigned long _pixel_x_2 = 0;
    unsigned char _end = 0;
    unsigned char _even_odd = 0;
    
    UINT bw;
    extern FATFS sd_card;	/* FatFs work area needed for each volume */
    extern FIL file;		/* File object needed for each open file */
    
    f_mount(&sd_card, "", 1);                                                           // monto SD
    f_open(&file, _image, FA_OPEN_EXISTING | FA_READ);                                  // apro file con diritti di lettura
    
    f_lseek(&file, 0x1C);                                                               // sposto il cursore all'indirizzo per lettura
    f_read(&file,&_color,2,&bw);    // deve essere 16                                   // colori, 2 byte
    f_read(&file,&_compres,4,&bw);  // deve essere 0 o 3 (per i miei file 3)            // compressione, 4 byte
    
    f_lseek(&file, 0x00);                                                               // sposto cursore all'inizio del file
    f_read(&file,_buffer,2,&bw);                                                       // leggo i primi 2 byte
    if ((_buffer[0] == 'B') && (_buffer[1] == 'M') && (_color == 16) && ((_compres == 0)|(_compres==3))){      // verifico che sia un BMP con le caratteristiche richieste
        
        f_lseek(&file, 0x0A);                                                           // sposto il cursore
        f_read(&file,&_start,2,&bw);                                                    // leggo indirizzo start
        f_lseek(&file, 0x12);                                                           // 
        f_read(&file,&_width,4,&bw);                                                    // leggo larghezza
        f_lseek(&file, 0x16);                                                           
        f_read(&file,&_height,4,&bw);                                                   // leggo altezza

        if ((_width%2) == 0)
            _even_odd = 0;
        else
            _even_odd = 1; // dispari
            

        
        TFT_set_window_dir(W_BMP);                                                      // imposto la direzione della finestra
        TFT_set_window (_x,_y,_x+_width-1,_y+_height-1);                                // imposto la finestra
        
        TFT_set_gram_address(_x, _y+_height-1);                                         // imposto il primo pixel per auto incremento (il  -1 e l'orientamento sono nella funzione)
        
        TFT_write_register_address(GRAM_DATA);                                          // scrivo l'indirizzo del buffer per passare i valori alla GRAM
        
        f_lseek(&file, _start);                                                         // sposto il cursore al primo byte dell'immagine

        if (_even_odd == 0){    // pari
            _pixel = _width * _height;                                                  // totale pixel immagine        
            
            do{
                if (_pixel >= SD_BUFFER){                                               // leggo i byte richiesti, salvo in buffer e scrivo su display
                    f_read(&file,_buffer,_sd_buffer_x_2,&bw);
                    for (unsigned int _l = 0; _l < _sd_buffer_x_2; _l = _l+2){
                        TFT_write_register_value((unsigned int)_buffer[_l]|(unsigned int)_buffer[_l+1]<<8);
                    }

                    _pixel -= SD_BUFFER;  
                } else {                                                                // leggo e scrivo ultimi byte
                    _pixel_x_2 = _pixel*2;
                    f_read(&file,_buffer,_pixel_x_2,&bw);
                    for (unsigned int _l = 0; _l < _pixel_x_2; _l = _l+2){
                        TFT_write_register_value((unsigned int)_buffer[_l]|(unsigned int)_buffer[_l+1]<<8);
                    }

                    _end = 1;
                }
            }while (_end == 0);
        } else {                                                                        // dispari
            _pixel = (_width +1) * _height;                                             // conteggio pixel con larghezza + 1 bit aggiunto nero non  da scrivere ma comunque da leggere dal file
            do{
                if (_pixel >= SD_BUFFER){                                               // leggo i byte richiesti, salvo in buffer e scrivo su display
                    f_read(&file,_buffer,_sd_buffer_x_2,&bw);
                    for (unsigned int _l = 0; _l < _sd_buffer_x_2; _l = _l+2){

                        if (_width_counter < _width){                                   // controllo per saltare il pixel nero aggiunto in immagini con larghezza dispari
                            TFT_write_register_value((unsigned int)_buffer[_l]|(unsigned int)_buffer[_l+1]<<8);
                            _width_counter++;
                        } else {
                            _width_counter = 0;
                        }
                    }

                    _pixel -= SD_BUFFER;  
                    
                } else {                                                                // leggo e scrivo ultimi byte
                    _pixel_x_2 = _pixel*2;
                    f_read(&file,_buffer,_pixel_x_2,&bw);
                    for (unsigned int _l = 0; _l < _pixel_x_2; _l = _l+2){

                        if (_width_counter < _width){
                            TFT_write_register_value((unsigned int)_buffer[_l]|(unsigned int)_buffer[_l+1]<<8);
                            _width_counter++;
                        } else {
                            _width_counter = 0;
                        }
                    }

                    _end = 1;
                }
            }while (_end == 0);
        }
        
        TFT_set_window_dir(W_DEFAULT);                                                  // reimposto finestra e impostazioni orientamento di default
        TFT_set_window (1,1,X_MAX,Y_MAX);    
    }
    
    f_close(&file);	
}
#endif
#endif