/* ============================================================================================================================================= /
 * LIBRARY:         Gestione Touch screen con controller XPT2046
 * FILE:            XPT2046.c
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         06/11/2016
 * UPDATED:         
 *
 * DESCRIP:         Gestione Touch screen con controller XPT2046
 *
 * FUNCTIONS:       XPT2046_enable()
 *                  XPT2046_disable()
 *                  XPT2046_read_point_raw()
 *                  XPT2046_calc_scaling_factor()
 *                  XPT2046_scale_point()
 *                  XPT2046_read_point_scaled()
 * ============================================================================================================================================= /
//! [History]
  Change Log:
  Rev.              Date                Changes
  1.1               13/07/2017          - Clock SPI impostabile in XPT2046_settings.h
  1.0               10/11/2016          - Prima release completa
  1.0               06/11/2016          - Inizio studio touch screen
//! [History]
/* ============================================================================================================================================ */

#ifndef XPT2046
#define XPT2046

/* Variabili globali libreria ================================================================================================================= */
typedef struct {
    unsigned int x;        
    unsigned int y;
}xpt2046_point;

float x_scale_factor = 0;
float y_scale_factor = 0;

unsigned int xpt2046_raw_x_min = 0;
unsigned int xpt2046_raw_y_min = 0;
unsigned int xpt2046_raw_x_max = 0;
unsigned int xpt2046_raw_y_max = 0;

/* INCLUDES =================================================================================================================================== */
#include    "XPT2046_settings.h"
#include    "XPT2046.h"
/* ============================================================================================================================================= /
 * FUNCTION:    XPT2046_enable
 *
 * Overview:    Abilita il modulo SPI e ne imposta i parametri
/* ============================================================================================================================================ */
void XPT2046_enable(void){   
    _XPT2046_SPI_disable();
    _XPT2046_SPI_enable(SAMPLE_AT_MIDDLE,TRA_ACTIVE_TO_IDLE,IDLE_IS_LOW,XPT2046_SPI_CLOCK);
    XPT2046_CS = XPT2046_CS_HIGH;
}

/* ============================================================================================================================================= /
 * FUNCTION:    XPT2046_disable
 *
 * Overview:    Disabilita il modulo SPI
/* ============================================================================================================================================ */    
void XPT2046_disable(void){              
    _XPT2046_SPI_disable();
    XPT2046_CS = XPT2046_CS_HIGH;
}

/* ============================================================================================================================================= /
 * FUNCTION:    XPT2046_read_point_raw
 * Output:      torna struct con variabili x e y raw (0~4096) (unsigned int)
 *
 * Overview:    Legge le coordinate del punto premuto sul touch screen 
 * Note:        Impostare la costante XPT2046_AVG_FACTOR in XPT2046_settings.h per selezionare numero di letture da fare per singolo punto. 
 *              (1-2-4-5-16-32-64-128-256) da tenere in considerazione che un numero maggiore di campioni equivale a pi tempo per l'acquisizione
 *              di un singolo punto. (Clock @64MHz SPI Clock @1MHz: AVG = 1 TAQ = 56uS, AVG = 8 TAQ = 0.450ms, AVG = 32 TAQ = 1mS, AVG = 256 TAQ = 10ms)
/* ============================================================================================================================================ */   
xpt2046_point XPT2046_read_point_raw(void){
    xpt2046_point _touched_point_raw;
    unsigned int _temp = 0;
    unsigned long _x_temp = 0;
    unsigned long _y_temp = 0;
    
    _touched_point_raw.x = 0;  // prova a commentare
    _touched_point_raw.y = 0;
    
    for (unsigned int _i = 0; _i < XPT2046_AVG_FACTOR; _i++) {                          // ciclo di lettura X e Y per media
        
        unsigned char _power_var = XPT2046_POWER_ON_BTW_CONV;                           // power ON tra le conversioni 
        if (_i == XPT2046_AVG_FACTOR-1)                                                 // ultima conversione imposta power OFF
            _power_var = XPT2046_POWER_OFF_BTW_CONV;
        
        XPT2046_CS = XPT2046_CS_LOW;                                                               
        _XPT2046_SPI_send_byte(XPT2046_X | _power_var);                                // X, 90 spegne, 91 tiene attivo
        Delay_200ns();                                                                  // Delay massimo di busy, non avendo il pin per controllarlo direttamente imposto delay massimo

        _temp = _XPT2046_SPI_receive_byte();
        _temp << = 8;
        _temp |= _XPT2046_SPI_receive_byte();

        XPT2046_CS = XPT2046_CS_HIGH;

        _temp >> = 3;                                                                   // forse  4

        _x_temp += _temp;

        XPT2046_CS = XPT2046_CS_LOW;
        _XPT2046_SPI_send_byte(XPT2046_Y | _power_var);                                // Y, 90 spegne, 91 tiene attivo
        Delay_200ns();

        _temp = _XPT2046_SPI_receive_byte();
        _temp << = 8;
        _temp |= _XPT2046_SPI_receive_byte();

        XPT2046_CS = XPT2046_CS_HIGH;

        _temp >> = 3;
        
        _y_temp += _temp;
    }
    
    _x_temp >> = XPT2046_AVG_DIVISOR;                                                   // divido per AVG
    _y_temp >> = XPT2046_AVG_DIVISOR;

    _touched_point_raw.x = _x_temp;
    _touched_point_raw.y = _y_temp;

// adattamento valori all'orientamento del display
#ifdef TFT_PORTRAIT
    _temp = _touched_point_raw.x;
    _touched_point_raw.x = _touched_point_raw.y;
    _touched_point_raw.y = 4096 - _temp;
#endif
    
#ifdef TFT_PORTRAIT_180
    _temp = _touched_point_raw.x;
    _touched_point_raw.x = 4096 - _touched_point_raw.y;
    _touched_point_raw.y = _temp;
#endif    

#ifdef TFT_LANDSCAPE_180
    _touched_point_raw.x = 4096 - _touched_point_raw.x;
    _touched_point_raw.y = 4096 - _touched_point_raw.y;
#endif    

    return _touched_point_raw;
}

/* ============================================================================================================================================= /
 * FUNCTION:    XPT2046_calc_scaling_factor
 * Input:       _point_1: punto toccato pi in alto a sinistra per calcolare i valori minimi (struct xpt2046_point)
 *              _point_2: punto toccato pi in basso a destra per calcolare i valori massimi (struct xpt2046_point)
 *
 * Overview:    Calcola i fattori di scaling (lineare senza tener conto di errori vari) per adattare il touch alla risoluzione del display 
/* ============================================================================================================================================ */   
void XPT2046_calc_scaling_factor(xpt2046_point _point_1, xpt2046_point _point_2){

    xpt2046_raw_x_min = _point_1.x;
    xpt2046_raw_y_min = _point_1.y;
    xpt2046_raw_x_max = _point_2.x;
    xpt2046_raw_y_max = _point_2.y;

    x_scale_factor = (float)X_MAX / ((float)xpt2046_raw_x_max - (float)xpt2046_raw_x_min);
    y_scale_factor = (float)Y_MAX / ((float)xpt2046_raw_y_max - (float)xpt2046_raw_y_min);

}

/* ============================================================================================================================================= /
 * FUNCTION:    XPT2046_scale_point
 * Input:       _touched_point_raw: punto toccato (struct xpt2046_point)
 * Output:      struct xpt2046_point contenente le coordinate del punto toccato scalate alla risoluzione del display 
 *
 * Overview:    Calcola le coordinate del punto scalate alla risoluzione del display 
/* ============================================================================================================================================ */   
xpt2046_point XPT2046_scale_point(xpt2046_point _touched_point_raw){
    float _x_scaled_temp = 0;
    float _y_scaled_temp = 0;
    
    xpt2046_point _touched_point_scaled;         
    
    _x_scaled_temp = ((float)_touched_point_raw.x - (float)xpt2046_raw_x_min) * x_scale_factor;
    _y_scaled_temp = ((float)_touched_point_raw.y - (float)xpt2046_raw_y_min) * y_scale_factor;
    
    // calcoli in float per non perdere approssimazione
    // andrebbe fatta approssimazione prima di riconvertire in int le coordinate
    
    if (_x_scaled_temp > X_MAX)
        _x_scaled_temp = X_MAX;
    else if (_x_scaled_temp < 0)
        _x_scaled_temp = 0;
    if (_y_scaled_temp > Y_MAX)
        _y_scaled_temp = Y_MAX;
    else if (_y_scaled_temp < 0)
        _y_scaled_temp = 0;        
    
    _touched_point_scaled.x = (unsigned int)_x_scaled_temp;
    _touched_point_scaled.y = (unsigned int)_y_scaled_temp; 
    
    return _touched_point_scaled;
}

/* ============================================================================================================================================= /
 * FUNCTION:    XPT2046_read_point_scaled
 * Input:       void
 * Output:      struct xpt2046_point contenente le coordinate del punto toccato scalate alla risoluzione del display 
 *
 * Overview:    Legge le coordinate del punto premuto sul touch screen  e le scala alla risoluzione del display 
/* ============================================================================================================================================ */ 
xpt2046_point XPT2046_read_point_scaled(void){
    return XPT2046_scale_point(XPT2046_read_point_raw());
}

#endif