/* ============================================================================================================================================= /
 * LIBRARY:         Decodifica DCF77
 * FILE:            DCF77.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         28/09/2016
 * UPDATED:         
 *
 * DESCRIP:         parametri e prototipi per libreria DCF77.c
/* ============================================================================================================================================= /
 *
/* CONTROLLI DI COMPILAZIONE ================================================================================================================== */
    
#ifdef DCF77_ACTIVE_HIGH
    #define DCF77_SIGNAL_ACTIVE     1 
    #define DCF77_SIGNAL_IDLE       0 
#elif defined DCF77_ACTIVE_LOW
    #define DCF77_SIGNAL_ACTIVE     0
    #define DCF77_SIGNAL_IDLE       1
#else
    #error Definire la logica del segnale DCF77
#endif

#ifdef DCF77_PDN
    #ifdef DCF77_PDN_ACTIVE_LOW
        #define DCF77_ON  0
        #define DCF77_OFF 1
#elif defined DCF77_PDN_ACTIVE_HIGH
        #define DCF77_ON  1
        #define DCF77_OFF 0
    #else
        #error  Definire la logica del pin di attivazione
    #endif
#endif

#ifndef DCF77_SIGNAL
    #error Definire il pin di connessione del segnale DCF77 (RB0~RB3)
#endif

#define ON                  1
#define OFF                 0

#define DCF77_CET   0b00001000
#define DCF77_CEST  0b00000100


/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file DCF77.c
 * Libreria per la decodifica del segnale radio orario DCF77
 * - Per un corretto funzionamento:
 *  + Includere la libreria Delay.c
 *  + Se si utilizza la ricezione dei dati via interrupt collegare il segnale su un input con interrupt su rising edge e disabilitare la resistenza di pullup del PIC
 *  + Se si utilizza la ricezione dei dati via interrupt  necesario associare al DCF77 un timer con interrupt relativo nel file DCF77_settings.h.
 * - Utilizzo:
 *  + Utilizzo tramite funzioni
 *      - richiamare la funzione DCF77_init();
 *      - richiamare la funzione DCF77_search_minute_mark() (il programma rester bloccato finch non verr trovato il minute mark)
 *      - richiamare la funzione DCF77_receive_data() (il programma rester bloccato per 1 minuto)
 *      - richiamare la funzione DCF77_decode_data() per decodificare il dato ricevuto e salvarlo nella struct voluta
 *  + Utilizzo tramite interrupt
 *      - impostare correttamente le linee e il timer usato nel file DCF77_settings.h
 *      - richiamare la funzione DCF77_start_receive_via_interrupt();
 *      - a dato ricevuto il flag data_ready va a 1
 *      - richiamare la funzione DCF77_decode_data() per decodificare il dato ricevuto e salvarlo nella struct voluta
 *      - se non si vogliono ricevere altri dati fermare la ricezione tramite la funzione DCF77_stop_receive_via_interrupt();
 * - Files:
 *  + DCF77.c: Codice funzioni 
 *  + DCF77.h: Parametri e prototipi
 *  + DCF77_settings.h: Impostazioni specifiche per progetto
 */

/**
 * Inizializzazione ed avvio del modulo, da usare solo se si usa il modulo DCF77 tramite funzioni e con pin di abilitazione
 * @param   void
 * @return  void 
 */
void DCF77_init(void);

/**
 * Attende il minute mark nella trasmissione, quando lo trova torna 1
 * @param   void
 * @return  1 quando il minute mark viene rilevato 
 */
unsigned char DCF77_search_minute_mark(void);

/**
 * Riceve il dato (58 secondi) e lo salva nell'array _dcf77_buffer
 * @param   void
 * @return  void
 * @note    decodificare il dato tramite la funzione  DCF77_decode_data per salvare il dato in una struct
 */
void DCF77_receive_data(void);

/**
 * Decodifica il dato contenuto nell'array di ricezione _dcf77_buffer, effettua i controlli di parit e valori massimi su ore,
 * minuti e data.
 * @param   void
 * @return  struct di variabili contenente i dati decodificati (dcf77_decoded_data)
 * @note    Se la parit non  corretta imposta il valore a F1.   \n
 *          Se il dato  fuori dai valori massimi imposta il valore a F0.
 */
dcf77_decoded_data DCF77_decode_data(void);    

/**
 * Abilita interrupt e relativo timer di ricezione. 
 *    - I dati vengono ricevuti in interrupt e salvati nel buffer di ricezione _dcf77_buffer
 *    - A sincronizzazione avvenuta viene settata la variabile globale dcf77_sync 
 *    - A ricezione completata viene settata la variabile dcf77_data_ready
 *    - Ad ogni bit ricevuto viene settata la variabile globale dcf77_seconds_pulse (pulsa ad ogni secondo) 
 * @param   void
 * @return  void
 */
void DCF77_start_receive_via_interrupt(void);

/**
 * Disabilita interrupt e relativo timer di ricezione. (attenzione, disabilita tutti gli interrupt on change su PORTB)
 * @param   void
 * @return  void
 */
void DCF77_stop_receive_via_interrupt(void);

/**
 * Routine di interrupt per la ricezione dei dati DCF77, se necessario commentare e spostare nel main del programma.
 *    - I dati vengono salvati nel buffer di ricezione _dcf77_buffer 
 *    - A sincronizzazione avvenuta viene settata la variabile globale dcf77_sync 
 *    - A ricezione completata viene settata la variabile dcf77_data_ready
 *    - Ad ogni bit ricevuto viene settata la variabile globale dcf77_seconds_pulse (pulsa ad ogni secondo) 
 * @param   void
 * @return  void
 */
interrupt high_priority void DCF77_receive_via_interrupt(void);