/* ============================================================================================================================================= /
 * LIBRARY:         EEPROM I2C
 * FILE:            EEPROM_I2C.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         11/08/2016
 * UPDATED:         
 *
 * DESCRIP:         Parametri e prototipi per libreria EEPROM_I2C.c
 *
/* ============================================================================================================================================= /
 * 
/* CONTROLLI DI COMPILAZIONE ================================================================================================================== */

#ifndef I2C
    #error  Includere I2C_init.c
#endif

#ifdef EEPROM_I2C_MODULE_1                                                              // Ridefinisco le chiamate alle funzioni a seconda del modulo I2C usato
    #ifndef I2C_MODULE_1
        #error I2C_MODULE_1 non abilitato in I2C_init_settings.h
    #endif
    #define _EEPROM_I2C_2ByteAddr_1ByteData_read  I2C_2ByteAddr_1ByteData_read
    #define _EEPROM_I2C_2ByteAddr_1ByteData_write I2C_2ByteAddr_1ByteData_write
#elif defined EEPROM_I2C_MODULE_2
    #ifndef I2C_MODULE_2
        #error I2C_MODULE_2 non abilitato in I2C_init_settings.h
    #endif
    #define _EEPROM_I2C_2ByteAddr_1ByteData_read  I2C2_2ByteAddr_1ByteData_read
    #define _EEPROM_I2C_2ByteAddr_1ByteData_write I2C2_2ByteAddr_1ByteData_write

#else
    #error Definire il modulo I2C da utilizzare per EEPROM_I2C
#endif

/* INDIRIZZI REGISTRI ========================================================================================================================= */
#define EEPROM_I2C_ADDR           0xA0

/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file EEPROM_I2C.c
 * Libreria per la gestione di memorie EEPROM via I2C, adatta per memorie con indirizzi a 2 byte (es.24LC32).
 * - Per un corretto funzionamento:
 *  + Includere la libreria I2C_init.c prima delle libreria EEPROM_I2C.c
 *  + Impostare l'indirizzo hardware della memoria in EEPROM_I2C_settings.h
 *  + Impostare il modulo I2C da utilizzare per la EEPROM I2C in EEPROM_I2C_settings.h
 *  + Abilitare il modulo I2C utilizzato in I2C_init_settings.h
 *  + Avviare il modulo I2C nel programma principale richiamando la funzione I2C_init() o I2C2_init()
 * - Files:
 *  + EEPROM_I2C.c: Codice funzioni 
 *  + EEPROM_I2C.h: Parametri e prototipi
 *  + EEPROM_I2C_settings.h: Impostazioni specifiche per progetto
 */
 
/**
 * Legge il dato all'indirizzo EEprom richiesto e torna il valore
 * @param   _dev_addr: indirizzo impostato sui pin A0-A2 (unsigned char)
 * @param   _addr: indirizzo da leggere (unsigned int)
 * @return  dato letto all'indirizzo richiesto (unsigned char)
 * @note    In caso di errori di comunicazione potrebbero tornare i valori (0xFD, 0xFE, 0xFF) (-3, -2, -1)
 */
unsigned char EEPROM_I2C_read (signed char _dev_addr, unsigned int _addr);

/**
 * Scrive il dato all'indirizzo EEprom richiesto
 * @param   _dev_addr: indirizzo impostato sui pin A0-A2 (unsigned char)
 * @param   _addr: indirizzo da scrivere (unsigned int)
 * @param   _data: dato da scrivere (unsigned char)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision        
 */
signed char EEPROM_I2C_write (signed char _dev_addr, unsigned int _addr, unsigned char _data);

/**
 * Scrive il dato all'indirizzo EEprom richiesto e verifica che sia stato scritto correttamente
 * @param   _dev_addr: indirizzo impostato sui pin A0-A2 (unsigned char)
 * @param   _addr: indirizzo da scrivere (unsigned int)
 * @param   _data: valore da scrivere (unsigned char)
 * @return  
 *          - 0: Dato scritto correttamente
 *          - -1: Bus collision
 *          - -2: ACK non ricevuto
 *          - -3: Write collision        
 *          - -4: Dato da salvare e dato salvato non corrispondono
 */
signed char EEPROM_I2C_write_e_check (signed char _dev_addr, unsigned int _addr, unsigned char _data);

/**
 * Legge il dato all'indirizzo EEprom richiesto e torna il valore valore intero letto da 2 indirizzi consecutivi
 * @param   _dev_addr: indirizzo impostato sui pin A0-A2 (unsigned char)
 * @param   _addr: indirizzo da leggere (unsigned int)
 * @return  dato letto in _addr e _addr+1 (unsigned int)
 * @note    Non torna eventuali errori di comunicazione I2C
 */
unsigned int EEPROM_I2C_read_int(unsigned char _dev_addr, unsigned int _addr);

/**
 * Scrive il dato nell'indirizzo dichiarato e in quello successivo (2 byte)
 * @param   _dev_addr: indirizzo impostato sui pin A0-A2 (unsigned char)
 * @param   _addr: indirizzo da leggere (unsigned int)
 * @param   _data: dato da scrivere (unsigned int)
 * @return  void
 * @note    Non torna eventuali errori di comunicazione I2C
 */
void EEPROM_I2C_write_int(unsigned char _dev_addr, unsigned int _addr, unsigned int _data);

/** 
 * Scrive il dato nell'indirizzo dichiarato e in quello successivo (2 byte) e verifica che sia stato scritto correttamente
 * @param   _dev_addr: indirizzo impostato sui pin A0-A2 (unsigned char)
 * @param   _addr: indirizzo da scrivere (unsigned int)
 * @param   _data: dato da scrivere (unsigned char)
 * @return  
 *          - 0: Dato scritto correttamente     
 *          - -4: Dato da salvare e dato salvato non corrispondono
 * @note    Non torna eventuali errori di comunicazione I2C          
 */
signed char EEPROM_I2C_write_int_e_check(unsigned char _dev_addr, unsigned int _addr, unsigned int _data);
