/* ============================================================================================================================================= /
 * LIBRARY:         Decodifica meteo con chip HKW581
 * FILE:            HKW581.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         04/10/2016
 * UPDATED:         
 *
 * DESCRIP:         parametri e prototipi per libreria HKW581.c
/* ============================================================================================================================================= /
 *
/* CONTROLLI DI COMPILAZIONE ================================================================================================================== */

/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file HKW581.c
 * Libreria per la decodifica del i bit meteo del segnale radio orario DCF77, da usare con libreria DCF77.c
 * - Per un corretto funzionamento:
 *  + Includere la libreria Delay.c prima della libreria HKW581.c
 *  + Includere la libreria DCF77.c prima della libreria HKW581.c
 *  + Disabilitare eventuali resistenze di pullup sulle linee
 * - Utilizzo:
 *  + Abilitare il chip tramite la funzione HKW581_init().
 *  + Attendere che il chip sia pronto (BRF = 0) tramite la funzione HKW581_read_brf().
 *  + Inviare i dati da decodificare raccolti nei 3 minuti tramite la funzione HKW581_decode_meteo_data().
 *  + Per inviare altri dati attendere che il chip sia pronto (BRF = 0) tramite la funzione HKW581_read_brf().
 *  + Nota: BRF va a 0 dopo 30~40s dall'avvio o dalla decodifica di un dato.
 * - Files:
 *  + HKW581.c: Codice funzioni 
 *  + HKW581.h: Parametri e prototipi
 *  + HKW581_settings.h: Impostazioni specifiche per progetto
 *  + HKW581_decode_tables.h: Tabelle di decodifica da dati numerici a stringhe di testo.
 */

/**
 * Inizializzazione delle linee clock e data
 * @param   void
 * @return  void 
 */
void HKW581_init(void);

/**
 * Resetta le linee di clock e data e disabilita il chip
 * @param   void
 * @return  void 
 */
void HKW581_stop(void);

/**
 * Legge lo stato del PIN BRF, torna a 1 quando BRF va a 0 per indicare che il chip  pronto a ricevere i dati
 * @param   void
 * @return  
 *          - 0: chip non pronto
 *          - 1: chip pronto 
 */
unsigned char HKW581_read_brf(void);

/**
 * Invia i dati in ingresso al chip, torna il dato decodificato dal chip
 * @param   _meteo_bits_1: 14 bit meteo del primo minuto (unsigned int)
 * @param   _meteo_bits_2: 14 bit meteo del secondo minuto (unsigned int)
 * @param   _meteo_bits_3: 14 bit meteo del terzo minuto (unsigned int)
 * @param   _minute: minuto (secondo minuto) (unsigned char)
 * @param   _hours: ora (unsigned char)
 * @param   _day: giorno (unsigned char)
 * @param   _month: mese (unsigned char)
 * @param   _day_week: giorno della settimana (unsigned char)
 * @param   _year: anno (unsigned char)
 * @return  struct di variabili contenente i dati del meteo decodificati (dcf77_decoded_data)
 * @note    i dati di ritorno vanno interpretati a seconda dell'ora di ricezione (probabilit pioggia, meteo estremo e vento condividono le stesse variabili di ritorni dati meteo per diversi giorni sono inviati in fasce orarie diverse)
 */
hwk581_decoded_data HKW581_decode_meteo_data(unsigned int _meteo_bits_1,unsigned int _meteo_bits_2,unsigned int _meteo_bits_3, unsigned char _minute, unsigned char _hour, unsigned char _day, unsigned char _month, unsigned char _day_w, unsigned char _year);

/**
 * Inverte i bit MSB->LSB e viceversa di 1 byte
 * @param   _data: byte da invertire (unsigned char)
 * @return  byte con bit invertiti (unsigned char)
 */
unsigned char Invert_8(unsigned char _data);

/**
 * Invia i dati dall0HKW581
 * @param   _data: dato da inviare (unsigned int, o char)
 * @param   _bits: quantit di bit da inviare (unsigned char)
 * @return  void
 */
void HKW581_send_data_packet(unsigned int _data, unsigned char _bits);
