/* ============================================================================================================================================= /
 * LIBRARY:         RTCC modulo interno PIC18F6x_8x
 * FILE:            RTCC_PIC18F6x_8x.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         PIC18F6xxxx / PIC18F8xxxx
 * CREATED:         07/03/2017
 *
 * DESCRIP:         Parametri e prototipi per libreria RTCC_PIC18F6x_8x.c
/* ============================================================================================================================================ */

/* COSTANTI =================================================================================================================================== */
#define RTCC_PTR_SEC_MIN                0
#define RTCC_PTR_HOU_WD                 1
#define RTCC_PTR_DAY_MON                2
#define RTCC_PTR_YEAR                   3

#define RTCC_OUT_CLOCK_SOURCE           2
#define RTCC_OUT_SECOND                 1
#define RTCC_OUT_ALARM                  0
#define RTCC_OUT_DISABLED               3

#define RTCC_AL_INTERRUPT_PR_HIGH       1
#define RTCC_AL_INTERRUPT_PR_LOW        0
#define RTCC_AL_INTERRUPT_DISABLED      2

#define ON                              1
#define OFF                             0

#define RTCC_AL_HALF_SEC                0b0000
#define RTCC_AL_SEC                     0b0001
#define RTCC_AL_10_SEC                  0b0010
#define RTCC_AL_MIN                     0b0011
#define RTCC_AL_10_MIN                  0b0100
#define RTCC_AL_HOU                     0b0101
#define RTCC_AL_DAY                     0b0110
#define RTCC_AL_WEEK                    0b0111
#define RTCC_AL_MONTH                   0b1000
#define RTCC_AL_YEAR                    0b1001

#define RTCC_AL_REPEAT                  1
#define RTCC_AL_ONE_TIME                0

#define MIN_SEC                         0x00                                            // valori minimi e massimi per ogni dato
#define MAX_SEC                         0x59
#define MIN_MIN                         0x00
#define MAX_MIN                         0x59
#define MIN_HOU                         0x00
#define MAX_HOU                         0x23
#define MIN_DAY                         0x01
#define MAX_DAY                         0x31
#define MIN_MONTH                       0x01
#define MAX_MONTH                       0x12
#define MIN_YEAR                        0x00
#define MAX_YEAR                        0x99
#define MIN_DAY_WEEK                    0x00                                            // Domenica
#define MAX_DAY_WEEK                    0x06                                            // Sabato

/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file RTCC_PIC18F6x_8x.c
 * Libreria per la gestione real time clock calendar modulo interno PIC18F6xxxx / PIC18F8xxxx.
 * - Per un corretto funzionamento:
 *  - Impostare la sorgente del clock nei registri di configurazione RTCOSC bit (CONFIG3L<0>)
 *  - Inizializzare il modulo tramite la funzione RTCC_init()
 *  - Impostare l'ora tramite le apposite funzioni
 *  - Avviare l'oscillatore tramite la funzione RTCC_alarm(ON)
 *  - E' consigliato fermare l'oscillatore prima di modificare i valori di data e ora.
 * - Files:
 *  + RTCC_PIC18F6x_8x.c: Codice funzioni 
 *  + RTCC_PIC18F6x_8x.h: Parametri e prototipi
  */

/**
 * Inizializza il modulo RTCC ed imposta la modalit di funzionamento del pin di uscita
 * @param   _out_mode: modalit di uscita del pin dedicato (unsigned char):   
 *              - RTCC_OUT_CLOCK_SOURCE: uscita Clock
 *              - RTCC_OUT_SECOND: uscita 1Hz     
 *              - RTCC_OUT_ALARM: uscita interrupt allarmi
 *              - RTCC_OUT_DISABLED: nessuna uscita, il pin viene usato come I/O normale
 * @param   _al_interrupt: abilitazione e priorit interrupt allarme (unsigned char)
 *              - RTCC_AL_INTERRUPT_PR_HIGH
 *              - RTCC_AL_INTERRUPT_PR_LOW
 *              - RTCC_AL_INTERRUPT_DISABLED     
 * @return  void     
 */
void RTCC_init(unsigned char _out_mode, unsigned char _al_interrupt);

/**
 * Avvia l'oscillatore o ferma L'RTCC
 * @param   _on_off: RTCC abilitato o disabilitato: ON/OFF (unsigned char)
 * @return  void     
 * @pre     Impostare la sorgente del clock nei registri di configurazione RTCOSC bit (CONFIG3L<0>).
 */
void RTCC_osc(signed char _on_off);

/**
 * Imposta il registro di calibrazione
 * @param   _factor: parametro di calibrazione -127 ~ +127 (si aggiungono o tolgono 4 cicli di clock al minuto per bit) (signed char)
 * @return  void     
 */
void RTCC_osc_trim(signed char _factor);

/**
 * Legge data e ora dal RTCC
 * @param   void
 * @return  Struct contenente i valori di data e ora codificati in packed BCD     
 */
rtcc_data RTCC_read_all(void);

/**
 * Scrive i valori di data e ora nel RTCC
 * @param   _day: valore del giorno da scrivere nell' RTCC (packed BCD)
 * @param   _month: valore del mese da scrivere nell' RTCC (packed BCD)
 * @param   _year: valore dell'anno da scrivere nell' RTCC (packed BCD)
 * @param   _day_w: valore del giorno della settimana da scrivere nell' RTCC (packed BCD)
 * @param   _day_w: valore del giorno della settimana da scrivere nell' RTCC (packed BCD)
 * @param   _minutes: valore dei minuti da scrivere nell' RTCC (packed BCD)
 * @param   _seconds: valore dei secondi da scrivere nell' RTCC (packed BCD)
 * @return  void    
 */
void RTCC_write_all(unsigned char _day, unsigned char _month, unsigned char _year, unsigned char _day_w,  unsigned char _hours,  unsigned char _minutes, unsigned char _seconds);

/**
 * Legge i secondi dal RTCC
 * @param   void
 * @return  valore dei secondi (packed BCD)
 */
unsigned char RTCC_read_seconds(void);

/**
 * Legge i minuti dal RTCC
 * @param   void
 * @return  valore dei minuti (packed BCD)
 */
unsigned char RTCC_read_minutes(void);

/**
 * Legge le ore dal RTCC
 * @param   void
 * @return  valore delle ore (packed BCD)
 */
unsigned char RTCC_read_hours(void);

/**
 * Legge il giorno della settimana dal RTCC
 * @param   void
 * @return  valore del giorno della settimana (packed BCD)
 */
unsigned char RTCC_read_day_w(void);

/**
 * Legge il giorno dal RTCC
 * @param   void
 * @return  valore del giorno codificato (packed BCD)
 */
unsigned char RTCC_read_day(void);

/**
 * Legge il mese dal RTCC
 * @param   void
 * @return  valore del mese (packed BCD)
 */
unsigned char RTCC_read_month(void);

/**
 * Legge l'anno dal RTCC
 * @param   void
 * @return  valore dell'anno codificato (packed BCD)
 */
unsigned char RTCC_read_year(void);

/**
 * Scrive il valore dei secondi nell' RTCC
 * @param   _seconds: valore dei secondi da scrivere nell' RTCC (packed BCD)
 * @return  void     
 */
void RTCC_write_seconds(unsigned char _seconds);

/**
   Scrive il valore dei minuti nell' RTCC
 * @param   _minutes : valore dei minuti da scrivere nell' RTCC (packed BCD)
 * @return  void     
 */
void RTCC_write_minutes(unsigned char _minutes);

/**
 * Scrive il valore delle ore nell' RTCC
 * @param   _hours : valore delle ore da scrivere nell' RTCC (packed BCD)
 * @return  void      
 */
void RTCC_write_hours(unsigned char _hours);

/**
 * Scrive il valore del giorno della settimana nell' RTCC
 * @param   _day_w : valore del giorno della settimana da scrivere nell' RTCC (packed BCD)
 * @return  void       
 */
void RTCC_write_day_w(unsigned char _day_w);

/**
 * Scrive il valore del giorno nell' RTCC
 * @param   _day : valore del giorno da scrivere nell' RTCC (packed BCD)
 * @return  void    
 */
void RTCC_write_day(unsigned char _day);

/**
 * Scrive il valore del mese nell' RTCC
 * @param   _day : valore del mese da scrivere nell' RTCC (packed BCD)
 * @return  void      
 */
void RTCC_write_month(unsigned char _month);

/**
 * Scrive il valore dell'anno nell' RTCC
 * @param   _year : valore dell'anno da scrivere nell' RTCC (packed BCD)
 * @return  void       
 */
void RTCC_write_year(unsigned char _year);

/**
 * Abilita o disabilita l'allarme
 * @param   _on_off: Allarme abilitato o disabilitato: ON/OFF (unsigned char)
 * @return  void       
 */
void RTCC_alarm(unsigned char _on_off);

/**
 * Impostazioni allarme
 * @param   _mode: Modalit allarme (unsigned char):
 *              - RTCC_AL_HALF_SEC
 *              - RTCC_AL_SEC
 *              - RTCC_AL_10_SEC
 *              - RTCC_AL_MIN
 *              - RTCC_AL_10_MIN
 *              - RTCC_AL_HOU
 *              - RTCC_AL_DAY
 *              - RTCC_AL_WEEK
 *              - RTCC_AL_MONTH
 *              - RTCC_AL_YEAR
 * @param   _repeat: Ripetizione allarme (unsigned char):
 *              - RTCC_AL_REPEAT
 *              - RTCC_AL_ONE_TIME           
 * @return  void 
 * @note    Non viene considerato il registro ALRMRPT, se viene impostato l'allarme in modalit REPEAT verr ripetuto ogni volta che si verifica
 *          la condizione
 */
void RTCC_alarm_mode(unsigned char _mode, unsigned char _repeat);

/**
 * Legge data e ora dell'allarme dal RTCC
 * @param   void
 * @return  Struct contenente i valori di data e ora allarme codificati in packed BCD
 * @note    l'anno sar sempre 0
 */
 rtcc_data RTCC_read_all_al(void);
 
 /**
 * Scrive i valori di data e ora dell'allarme nel RTCC
 * @param   _day: valore del giorno dell'allarme da scrivere nell' RTCC (packed BCD)
 * @param   _month: valore del mese dell'allarme  da scrivere nell' RTCC (packed BCD)
 * @param   _day_w: valore del giorno dell'allarme  della settimana da scrivere nell' RTCC (packed BCD)
 * @param   _day_w: valore del giorno dell'allarme  della settimana da scrivere nell' RTCC (packed BCD)
 * @param   _minutes: valore dei minuti dell'allarme  da scrivere nell' RTCC (packed BCD)
 * @param   _seconds: valore dei secondi dell'allarme  da scrivere nell' RTCC (packed BCD)
 * @return  void    
 */
 void RTCC_write_all_al(unsigned char _day, unsigned char _month, unsigned char _day_w,  unsigned char _hours,  unsigned char _minutes, unsigned char _seconds);

/**
 * Legge i secondi dell'allarme dal RTCC
 * @param   void
 * @return  valore dei secondi (packed BCD)
 */
unsigned char RTCC_read_seconds_al(void);

/**
 * Legge i minuti dell'allarme dal RTCC
 * @param   void
 * @return  valore dei minuti (packed BCD)
 */
unsigned char RTCC_read_minutes_al(void);

/**
 * Legge le ore dell'allarme dal RTCC
 * @param   void
 * @return  valore delle ore (packed BCD)
 */
unsigned char RTCC_read_hours_al(void);

/**
 * Legge il giorno della settimana dell'allarme dal RTCC
 * @param   void
 * @return  valore del giorno della settimana (packed BCD)
 */
unsigned char RTCC_read_day_w_al(void);

/**
 * Legge il giorno dell'allarme dal RTCC
 * @param   void
 * @return  valore del giorno codificato (packed BCD)
 */
unsigned char RTCC_read_day_al(void);

/**
 * Legge il mese dell'allarme dal RTCC
 * @param   void
 * @return  valore del mese (packed BCD)
 */
unsigned char RTCC_read_month_al(void);

/**
 * Scrive il valore dei secondi dell'allarme nell' RTCC
 * @param   _seconds: valore dei secondi da scrivere nell' RTCC (packed BCD)
 * @return  void     
 */
void RTCC_write_seconds_al(unsigned char _seconds);

/**
   Scrive il valore dei minuti dell'allarme nell' RTCC
 * @param   _minutes : valore dei minuti da scrivere nell' RTCC (packed BCD)
 * @return  void     
 */
void RTCC_write_minutes_al(unsigned char _minutes);

/**
 * Scrive il valore delle dell'allarme ore nell' RTCC
 * @param   _hours : valore delle ore da scrivere nell' RTCC (packed BCD)
 * @return  void      
 */
void RTCC_write_hours_al(unsigned char _hours);

/**
 * Scrive il valore del giorno della settimana dell'allarme nell' RTCC
 * @param   _day_w : valore del giorno della settimana da scrivere nell' RTCC (packed BCD)
 * @return  void       
 */
void RTCC_write_day_w_al(unsigned char _day_w);

/**
 * Scrive il valore del giorno dell'allarme nell' RTCC
 * @param   _day : valore del giorno da scrivere nell' RTCC (packed BCD)
 * @return  void    
 */
void RTCC_write_day_al(unsigned char _day);

/**
 * Scrive il valore del mese dell'allarme nell' RTCC
 * @param   _day : valore del mese da scrivere nell' RTCC (packed BCD)
 * @return  void      
 */
void RTCC_write_month_al(unsigned char _month);