/* ============================================================================================================================================= /
 * LIBRARY:         Gestione TFT ILI9325
 * FILE:            TFT_ILI9325.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         17/10/2016
 * UPDATED:         
 *
 * DESCRIP:         Parametri e prototipi per libreria TFT_ILI9325.c
/* ============================================================================================================================================ */

/* CONTROLLI DI COMPILAZIONE ================================================================================================================== */
#if defined TFT_PORTRAIT || defined TFT_PORTRAIT_180
    #define X_MAX   240
    #define Y_MAX   320
#elif defined TFT_LANDSCAPE || defined TFT_LANDSCAPE_180
    #define X_MAX   320
    #define Y_MAX   240
#else
    #error Definire l'orientamento del display
#endif

#ifdef TFT_8_BIT_DATA
#elif defined TFT_16_BIT_DATA
#else
    #error definire la modalit i invio dati 8/16 bit
#endif

#ifndef XTAL_FREQ
    #error Definire XTAL_FREQ
#endif

#ifndef DELAY_C
    #error  includere libreria Delay.c in Main.c
#endif

/* COSTANTI  ================================================================================================================================== */
//#define INPUT               0b11111111                                                  // costanti per direzione dati PIC I/O
//#define OUTPUT              0b00000000
#define TFT_DATA_INPUT      0b11111111
#define TFT_DATA_OUTPUT     0b00000000
#define ZERO_CLEANING_ON    0b01000001
#define ZERO_CLEANING_OFF   0b01000000
#define SAVESPACE_ON        0b10000001
#define SAVESPACE_OFF       0b10000000
#define W_DEFAULT           0
#define W_IMG               1
#define W_BMP               2


/* COLORI  ================================================================================================================================== */
#define BLACK       0x0000
#define WHITE       0xFFFF
#define RED         0xF800
#define BLUE        0x00FF
#define GREEN       0x07E0
#define YELLOW      0xFFE0
#define BROWN       0x7800
#define LIGHT_BROWN 0x2090
#define PURPLE      0x780F
#define GRAY        0xC618
#define CYAN        0x07FF
#define ORANGE      0xFD20


/* INDIRIZZI REGISTRI ========================================================================================================================= */
#define ID_CODE                     0x00
#define DRIVER_OUTPUT_CTRL          0x01
#define DRIVING_WAVE_CTRL           0x02
#define ENTRY_MODE                  0x03
#define RESIZING_CTRL               0x04
#define DISPLAY_CTRL_1              0x07
#define DISPLAY_CTRL_2              0x08
#define DISPLAY_CTRL_3              0x09
#define DISPLAY_CTRL_4              0x0A
#define RGB_DISPLAY_INT_CTRL_1      0x0C 
#define FRAME_MARKER_POSITION       0x0D
#define RGB_DISPLAY_INT_CTRL_2      0x0F
#define POWER_CONTROL_1             0x10
#define POWER_CONTROL_2             0x11
#define POWER_CONTROL_3             0x12
#define POWER_CONTROL_4             0x13

#define GRAM_HORIZONTAL_ADDRESS     0x20
#define GRAM_VERTICAL_ADDRESS       0x21
#define GRAM_DATA                   0x22

#define POWER_CONTROL_7             0x29
#define FRAME_RATE_AND_COLOR_CRTL   0x2B

#define GAMMA_CTRL_30H              0x30
#define GAMMA_CTRL_31H              0x31
#define GAMMA_CTRL_32H              0x32
#define GAMMA_CTRL_35H              0x35
#define GAMMA_CTRL_36H              0x36
#define GAMMA_CTRL_37H              0x37
#define GAMMA_CTRL_38H              0x38
#define GAMMA_CTRL_39H              0x39
#define GAMMA_CTRL_3CH              0x3C
#define GAMMA_CTRL_3DH              0x3D

#define HORIZONTAL_GRAM_START       0x50
#define HORIZONTAL_GRAM_END         0x51
#define VERTICAL_GRAM_START         0x52
#define VERTICAL_GRAM_END           0x53

#define GATE_SCAN_CTRL_60H          0x60
#define GATE_SCAN_CTRL_61H          0x61
#define GATE_SCAN_CTRL_6AH          0x6A

#define PARTIAL_IMAGE_1_POS         0x80
#define PARTIAL_IMAGE_1_START_ADDR  0x81
#define PARTIAL_IMAGE_1_END_ADDR    0x82

#define PARTIAL_IMAGE_2_POS         0x83
#define PARTIAL_IMAGE_2_START_ADDR  0x84
#define PARTIAL_IMAGE_2_END_ADDR    0x85

#define PANEL_INTERFACE_CTRL_1      0x90
#define PANEL_INTERFACE_CTRL_2      0x92
#define PANEL_INTERFACE_CTRL_4      0x95
#define PANEL_INTERFACE_CTRL_5      0x97

/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file TFT_ILI9325.c
 * Libreria per la gestione di display tft 320x240 pixel con controller ILI9325. Tutte le funzioni sono implementate per gestire pixel con colori
 * a 16bit (65536 colori).
 * - Per un corretto funzionamento:
 *  + Includere la libreria Delay.c prima delle libreria TFT_ILI9325.c
 *  + Definire la frequenza del clock in MHz (XTAL_FREQ)
 *  + Impostare i PIN di uscita in TFT_ILI9325_settings.h
 *  + Impostare l'orientamento del display in TFT_ILI9325_settings.h
 *  + Inizializzare il Display tramite la funzione TFT_init();
 *  + Informazioni approfondite sulla libreria si possono trovare nel file <a href="md__c_1__working_space__progetti__librerie__c__t_f_t__i_l_i9325_readme.html" target="_self"><b>TFT_ILI9325_readme.mb</b></a>
 * - Files:
 *  + TFT_ILI9325.c: Codice funzioni
 *  + TFT_ILI9325.h: Parametri e prototipi
 *  + TFT_ILI9325_settings.h: Impostazioni specifiche per progetto
 */

/** 
 * Imposta il registro da leggere o scrivere
 * @param   _addr: indirizzo del registro (unsigned char)
 * @return  void
 */
void TFT_write_register_address (unsigned char _addr);

/** 
 * Scrive il valore nel registro precedentemente indirizzato tramite funzione TFT_write_register_address()
 * @param   _value: valore da scrivere nel registro (unsigned int)
 * @return  void
 */
void TFT_write_register_value (unsigned int _value);

/** 
 * Scrive il valore nel registro all'indirizzo dichiarato 
 * @param   _addr: indirizzo del registro (unsigned char)
 * @param   _value: valore da scrivere nel registro (unsigned int)
 * @return  void
 */
void TFT_write_register (unsigned char _addr,unsigned int _value);

/** 
 * Legge il valore nel registro all'indirizzo dichiarato
 * @param   _addr: indirizzo del registro (unsigned char)
 * @return  valore letto nel registro (unsigned int)
 */
unsigned int TFT_read_register (unsigned char _addr); 

/** 
 * Inizializza il display TFT
 * @param   void
 * @return  void
 */
void TFT_init (void);

/** 
 * Imposta l'indirizzo del pixel da scrivere
 * @param   _x: coordinata x del pixel da accendere: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param   _y: coordinata y del pixel da accendere: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @return  void
 */
void TFT_set_gram_address (unsigned int _x, unsigned int _y);

/** 
 * Imposta la finestra da considerare per l'auto incremento degli indirizzi GRAM, serve in TFT_clear(), TFT_fill_rect(), 
 * TFT_write_char() e TFT_write_string() per velocizzare le operazioni di scrittura dei pixel contigui.
 * @param   _x_start: coordinata x del primo pixel della finestra: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param   _y_start: coordinata y del primo pixel della finestra: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param   _x_end: coordinata x dell'ultimo pixel della finestra: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param   _y_end: coordinata y dell'ultimo pixel della finestra: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @return  void
 */
void TFT_set_window(unsigned int _x_start, unsigned int _y_start, unsigned int _x_end, unsigned int _y_end);

/** 
 * Imposta la modalit di incremento del puntatore in GRAM, da cambiare se si usano funzioni di stampa immagini
 * @param   _mode: modalit di incremento del contatore nella finestra (unsigned char): 
 *              - W_DEFAULT (per tutte le funzioni standard)
 *              - W_IMG (per immagini in memoria programma)
 *              - W_BMP (per immagini BMP su memoria esterna es. SD card)
 * @return  void
 * @note    Dopo aver stampato l'immagine si deve ripristinare la modalit di incremento di default 
 */
void TFT_set_window_dir(unsigned char _mode);

/** 
 * Colora tutto lo schermo
 * @param   _color: colore di riempimento di tutto lo schermo (unsigned int)
 * @return  void
 */
void TFT_clear(unsigned int _color);

/** 
 * Disattiva la visualizzazione, il display riceve ed esegue comandi
 * @param   void
 * @return  void
 */
void TFT_idle(void);

/** 
 * Ferma tutte le operazioni del Display tranne l'oscillatore. Aggiornamento della GRAM e operazioni non verranno eseguiti
 * @param   void
 * @return  void
 */
void TFT_sleep(void);

/** 
 * Ferma tutte le operazioni del Display tranne l'alimentazione della GRAM. Aggiornamento della GRAM e operazioni non verranno eseguiti
 * @param   void
 * @return  void
 */
void TFT_standby(void);

/** 
 * Disattiva tutto il display e la retroilluminazione se gestita tramite I/O digitale
 * @param   void
 * @return  void
 */
void TFT_off(void);

/** 
 * Attiva il display dopo idle, sleep, standby e off
 * @param   void
 * @return  void
 */
void TFT_active(void);

/** 
 * Accende 1 pixel del colore selezionato all'indirizzo xy sul display
 * @param   _x: coordinata x del pixel da accendere: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param   _y: coordinata y del pixel da accendere: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int)  
 * @param   _color: colore del pixel (unsigned int)
 * @return  void
 */
void TFT_draw_pixel (unsigned int _x, unsigned int _y, unsigned int _color);                            

/** 
 * Scrive un immagine salvata in array sul Display
 * @param   _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param   _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int)
 * @param   _image: puntatore al primo indirizzo in flash dell'array contenente l'immagine
 * @return  void
 * @note    I primi 3 int dell'array devono contenere i dati di lunghezza, altezza e numero di colori (0xFFFF per colori a 16 bit)
 */
void TFT_draw_image(unsigned int _x,unsigned int _y, const unsigned int *_image);

/** 
 * Scrive un immagine monocromatica salvata in array sul Display
 * @param   _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param   _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int)
 * @param   _image: puntatore al primo indirizzo in flash dell'array contenente l'immagine
 * @param   _color: colore dell'immagine (unsigned int)
 * @param   _bk_color: colore dello sfondo (unsigned int)
 * @return  void
 * @note    I primi 5 char dell'array devono contenere i dati di lunghezza, altezza e byte altezza
 */
void TFT_draw_image_monocr(unsigned int _x,unsigned int _y, const unsigned char *_image, unsigned int _color, unsigned int _bk_color) ;

/** 
 * Disegna una linea orizzontale su display da punto _x_start a punto _x_end compresi su riga _y
 * @param   _x_start: coordinata x di partenza: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param   _x_end: coordinata x finale: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param   _y: coordinata y: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param   _color: colore della linea (unsigned int)
 * @return  void
 */  
void TFT_draw_line_x (unsigned int _x_start, unsigned int _x_end, unsigned _y, unsigned int _color);

/** 
 * Disegna una linea verticale su display da punto _y_start a punto _y_end compresi su colonna _x
 * @param       _y_start: coordinata y di partenza: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _y_end: coordinata y finale: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _x: coordinata x: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _color: colore della linea (unsigned int)
 * @return      void
 */
 void TFT_draw_line_y (unsigned int _y_start, unsigned int _y_end, unsigned _x, unsigned int _color);

/**
 * Disegna una linea da punto a punto su display da punto _x/_y_start a punto x/_y_end compresi
 * @param       _x_start: coordinata x di partenza: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y_start: coordinata y di partenza: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int)          
 * @param       _x_end: coordinata x finale: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y_end: coordinata y finale: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _color: colore della linea (unsigned int)
 * @return      void
 */
void TFT_draw_line_p2p (unsigned int _x_start, unsigned int _y_start, unsigned int _x_end, unsigned int _y_end, unsigned int _color);

/**
 * Disegna una linea da punto origine di lunghezza stabilita con angolo su display
 * @param       _x_start: coordinata x di partenza: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y_start: coordinata y di partenza: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _length: lunghezza (unsigned int) 
 * @param       _angle: angolo: 0-360 (unsigned int)
 * @param       _color: colore della linea (unsigned int)
 * @return      void 
 */
void TFT_draw_line_angle (unsigned int _x_start, unsigned int _y_start, unsigned char _length,  unsigned int _angle, unsigned int _color);

/**
 * Disegna un cerchio dato centro e raggio su display
 * @param       _x_centr: coordinata x del centro del cerchio: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y_centr: coordinata y del centro del cerchio: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _r: raggio (unsigned int)
 * @param       _color: colore della linea (unsigned int)
 * @return      void
 */
void TFT_draw_circle(unsigned int _x_centr, unsigned int _y_centr, unsigned int _r, unsigned int _color);

/**
 *  Disegna un arco dato centro, raggio e angolo su display
 * @param       _x_centr: coordinata x del centro del cerchio: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y_centr: coordinata y del centro del cerchio: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _r: raggio (unsigned int)
 * @param       _corner: angolo da disegnare
 * @param       _color: colore della linea (unsigned int)
 * @return      void
 */
void TFT_draw_arc(unsigned int _x_centr, unsigned int _y_centr, unsigned int _r, unsigned char _corner, unsigned int _color);

/*
 * Funzione privata per scrittura pixel cerchi e angoli
 */
void TFT_Plot8(unsigned int _x, unsigned int _y, unsigned int _dx, unsigned int _dy,  unsigned char _corner, unsigned int _color);

/**
 * Disegna un cerchio pieno dato centro e raggio su display
 * @param       _x_centr: coordinata x del centro del cerchio: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y_centr: coordinata y del centro del cerchio: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _r: raggio (unsigned int)
 * @param       _color: colore della linea (unsigned int)
 * @return      void
 */
void TFT_fill_circle(unsigned int _x_centr,unsigned int _y_centr ,unsigned int _r, unsigned int _color);

/**
 * Disegna una rettangolo su display
 * @param       _x_start: coordinata x di partenza: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y_start: coordinata y di partenza: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _l: lunghezza: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _h: altezza: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _color: colore del rettangolo (unsigned int)
 * @return      void
 */
void TFT_draw_rect(unsigned int _x_start, unsigned int _y_start, unsigned int _l, unsigned int _h, unsigned int _color);

/**
 * Disegna una rettangolo con angoli arrotondati su display
 * @param       _x_start: coordinata x di partenza: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y_start: coordinata y di partenza: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _l: lunghezza: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _h: altezza: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _r: raggio degli angoli: 1~255 (unsigned int)
 * @param       _color: colore del rettangolo (unsigned int)
 * @return      void
 */
void TFT_draw_rounded_rect(unsigned int _x_start, unsigned int _y_start, unsigned int _l, unsigned int _h, unsigned int _r, unsigned int _color);

/**
 * Disegna un rettangolo pieno su display
 * @param       _x_start: coordinata x di partenza: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y_start: coordinata y di partenza: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _l: lunghezza: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _h: altezza: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _color: colore del rettangolo (unsigned int)
 * @return      void
 * Overview:    
 */
void TFT_fill_rect(unsigned int _x_start, unsigned int _y_start, unsigned int _l, unsigned int _h, unsigned int _color);

/**
 * Disegna una rettangolo pieno con angoli arrotondati su display
 * @param       _x_start: coordinata x di partenza: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y_start: coordinata y di partenza: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _l: lunghezza: 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _h: altezza: 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int) 
 * @param       _color: colore del rettangolo (unsigned int)
 * @return      void
 * Overview:    
 */
void TFT_fill_rounded_rect(unsigned int _x_start, unsigned int _y_start, unsigned int _l, unsigned int _h, unsigned int _r, unsigned int _color);

/**
 * Imposta le variabili globali per il font selezionato
 * @param       *_font_pointer: puntatore all'array del font
 * @return      void
 */
void TFT_set_font(const unsigned char *_font_pointer);

/**
 * Scrive un singlo carattere su Display e colora lo sfondo con i colori selezionati.
 * @param       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int)             
 * @param       _character: carattere da scrivere (unsigned char)
 * @param       _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 * @param       _color: colore del carattere (unsigned int)
 * @param       _bk_color: colore dello sfondo (unsigned int)
 * @return      valore di x effettivamente scritti (unsigned char)
 * @pre         Impostare il font tramite la funzione TFT_set_font()
 * @note        La scrittura del carattere viene eseguita tramite finestra di dimensioni impostate tramite le dimensioni del font. In questo modo
 *              la scrittura  pi veloce e vengono scritti anche tutti i pixel che non rappresentano il carattere (sfondo) cancellando quindi
 *              in automatico eventuali scritte precedenti  (fino alla grandezza del font).
 *              Se si devono scrivere caratteri variabili usare SAVESPACE_OFF              
 */
unsigned char TFT_write_char(unsigned int _x, unsigned int _y, unsigned char _character,  unsigned char _savespace, unsigned int _color, unsigned int _bk_color);

/**
 * Scrive caratteri nella finestra selezionata, da usare solo dopo aver impostato la finestra (usato da TFT_write_char() e TFT_write_string())
 * @param       _character: carattere da scrivere (unsigned char)
 * @param       _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 * @param       _color: colore del carattere (unsigned int)
 * @param       _bk_color: colore dello sfondo (unsigned int)
 * @return      valore di x effettivamente scritti (unsigned char)
 * @pre         Impostare il font tramite la funzione TFT_set_font()
*/
unsigned char TFT_write_char_w(unsigned char _character, unsigned char _savespace, unsigned int _color, unsigned int _bk_color);

/**
 * Riempie lo spazio tra i caratteri con il colore sfondo e torna il valore di x effettivamente scritte.
 * @param       _bk_color: colore dello sfondo (unsigned int)
 * @return      valore di x effettivamente scritti (unsigned char)
 */
unsigned char TFT_write_space_btw_chars (unsigned int _bk_color);

/**
 * Scrive una stringa di caratteri su Display e colora lo sfondo con i colori selezionati.
 * @param       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int)    
 * @param       _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _string[]: puntatore ad array di caratteri da scrivere, byte finale \0 (unsigned char)
 * @param       _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 * @param       _color: colore del carattere (unsigned int)
 * @param       _bk_color: colore dello sfondo (unsigned int)
 * @return      void
 * @pre         Impostare il font tramite la funzione TFT_set_font()
 * @note        La scrittura della stringa viene eseguita tramite finestra di dimensioni impostate tramite le dimensioni del font e la variabile
 *              in ingresso _max_x. In questo modo a scrittura  pi veloce e vengono scritti anche tutti i pixel che non rappresentano il carattere (sfondo)
 *              cancellando quindi in automatico eventuali scritte precedenti (fino a _x + _x_max).
 */
void TFT_write_string (unsigned int _x, unsigned int _y, unsigned int _max_x, const unsigned char _string[], unsigned char _savespace, unsigned int _color, unsigned int _bk_color);

/**
 * Scrive un numero unsigned int su Display dopo averlo convertito in array stringa di caratteri ASCII
 * @param       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int)    
 * @param       _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _unsint: valore da scrivere (unsigned int)
 * @param       _number_digits: numero di cifre da scrivere (1~5, unsigned char) 
 * @param       _zero_cleaning: scrive o cancella zero iniziali (ZERO_CLEANING_ON/ZERO_CLEANING_OFF)
 * @param       _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 * @param       _color: colore del carattere (unsigned int)
 * @param       _bk_color: colore dello sfondo (unsigned int)
 * @return      void
 */
void TFT_write_unsint(unsigned int _x, unsigned int _y, unsigned int _max_x, unsigned int _unsint, unsigned char _number_digits, unsigned char _zero_cleaning, unsigned char _savespace, unsigned int _color, unsigned int _bk_color);

/**
 * Scrive un numero signed int su Display dopo averlo convertito in array stringa di caratteri ASCII
 * @param       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int)    
 * @param       _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _sint: valore da scrivere (signed int)
 * @param       _number_digits: numero di cifre da scrivere (1~5, unsigned char) 
 * @param       _zero_cleaning: scrive o cancella zero iniziali (ZERO_CLEANING_ON/ZERO_CLEANING_OFF)
 * @param       _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 * @param       _color: colore del carattere (unsigned int)
 * @param       _bk_color: colore dello sfondo (unsigned int)
 * @return      void
 */
void TFT_write_sint (unsigned int _x, unsigned int _y, unsigned int _max_x, signed int _sint, unsigned char _number_digits, unsigned char _zero_cleaning, unsigned char _savespace, unsigned int _color, unsigned int _bk_color);

/**
 * Scrive un numero codificato in 1 byte packed BCD su Display
 * @param       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int)    
 * @param       _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _pbcd: valore numerico codificato in packed BCD (unsigned char)
 * @param       _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 * @param       _color: colore del carattere (unsigned int)
 * @param       _bk_color: colore dello sfondo (unsigned int)
 * @return      void
 */
void TFT_write_PBCD(unsigned int _x, unsigned int _y, unsigned int _max_x, unsigned char _pbcd, unsigned char _savespace, unsigned int _color, unsigned int _bk_color);

/**
 * Scrive un byte HEX su display
 * @param       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int)    
 * @param       _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _hex: valore da scrivere (unsigned char)
 * @param       _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 * @param       _color: colore del carattere (unsigned int)
 * @param       _bk_color: colore dello sfondo (unsigned int)
 * @return      void
 */
void TFT_write_HEX(unsigned int _x, unsigned int _y, unsigned int _max_x, unsigned char _hex, unsigned char _savespace, unsigned int _color, unsigned int _bk_color);

/**
 * Scrive un byte come stringa di bit 1-0 su display
 * @param       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int)    
 * @param       _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _byte: valore da scrivere (unsigned char)
 * @param       _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 * @param       _color: colore del carattere (unsigned int)
 * @param       _bk_color: colore dello sfondo (unsigned int)
 * @return      void
 */
void TFT_write_bit_string (unsigned int _x, unsigned int _y, unsigned int _max_x, unsigned char _byte, unsigned char _savespace, unsigned int _color, unsigned int _bk_color);

/**
 * Scrive un valore float su display
 * @param       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se PORTRAIT, 1~240 se LANDSCAPE (unsigned int)    
 * @param       _max_x: lunghezza massima della stringa (lunghezza partendo da _x): 1~240 se PORTRAIT, 1~320 se LANDSCAPE (unsigned int)
 * @param       _float: valore da scrivere (float)
 * @param       _digits_int: numero di cifre interi da scrivere (1~5, unsigned char) 
 * @param       _digits_part: numero di cifre decimali da scrivere (1~10, unsigned char)
 * @param       _zero_cleaning: scrive o cancella zero iniziali (ZERO_CLEANING_ON/ZERO_CLEANING_OFF)
 * @param       _savespace: attiva o disattiva rimozione spazzi vuoti dai caratteri (SAVESPACE_ON/SAVESPACE_OFF) 
 * @param       _color: colore del carattere (unsigned int)
 * @param       _bk_color: colore dello sfondo (unsigned int)
 * @return      void
 */
void TFT_write_float (unsigned int _x, unsigned int _y, unsigned int _max_x, float _float, unsigned char _digits_int, unsigned char _digits_part, unsigned char _zero_cleaning, unsigned char _savespace, unsigned int _color, unsigned int _bk_color);

/**
 * Compone il colore RGB in intero a 5-6-5 bit (8-8-8 a 5-6-5)
 * @param       _r: valore del colore rosso 0 ~ 0xFF (unsigned char)
 * @param       _g: valore del colore verde 0 ~ 0xFF (unsigned char)
 * @param       _b: valore del colore blu 0 ~ 0xFF (unsigned char)
 * @return      valore del colore RGB formattato in 5-6-5 bit     
 */
unsigned int TFT_RGB_convert(unsigned char _r, unsigned char _g, unsigned char _b);

/**
 * Scrive un immagine salvata su memoria SD sul Display
 * @param       _x: coordinata x di partenza (primo pixel a sinistra della scritta): 1~240 se TFT_PORTRAIT, 1~320 se TFT_LANDSCAPE (unsigned int)
 * @param       _y: coordinata y di partenza (primo pixel a in alto della scritta): 1~320 se TFT_PORTRAIT, 1~240 se TFT_LANDSCAPE (unsigned int)
 * @param       _image: stringa contenente il nome del file con estensione (stringa)
 * @return      void
 * @pre         - Dichiarare le variabili per lettura SD in main.c
 *              - Impostare il SD_BUFFER in TFT_ILI9325_settings.h per velocizzare la scrittura di grosse immagini 
 * @note        L'immagine deve essere salvata in BMP Windows V3, in modalit 16 bit 5r6g5b \n
 *              Se l'immagine ha una larghezza dispari i programmi aggiungono un pixel nero,  stato aggiunto un ciclo di controllo che in caso di 
 *              immagini con numero di pixel di larghezza dispari non scrive su display il pixel nero aggiunto.
 */
void TFT_draw_image_sd(unsigned int _x,unsigned int _y, const unsigned char *_image);
