/* ============================================================================================================================================= /
 * LIBRARY:         Gestione Touch screen con controller XPT2046
 * FILE:            XPT2046.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         06/11/2016
 * UPDATED:          
 *
 * DESCRIP:         Parametri e prototipi per libreria XPT2046.c
/* ============================================================================================================================================ */

/* CONTROLLI DI COMPILAZIONE ================================================================================================================== */

#ifndef SPI
    #error  Includere SPI_init.c
#endif

#ifdef XPT2046_SPI_MODULE_1                                                             // Ridefinisco le chiamate alle funzioni a seconda del modulo I2C usato
    #ifndef SPI_MODULE_1
        #error SPI_MODULE_1 non abilitato in SPI_init_settings.h
    #endif
    
    #define _XPT2046_SPI_enable         SPI_enable
    #define _XPT2046_SPI_disable        SPI_disable
    #define _XPT2046_SPI_send_byte      SPI_send_byte
    #define _XPT2046_SPI_receive_byte   SPI_receive_byte

#elif defined XPT2046_SPI_MODULE_2
    #ifndef SPI_MODULE_2
        #error SPI_MODULE_2 non abilitato in SPI_init_settings.h
    #endif
    
    #define _XPT2046_SPI_enable         SPI2_enable
    #define _XPT2046_SPI_disable        SPI2_disable
    #define _XPT2046_SPI_send_byte      SPI2_send_byte
    #define _XPT2046_SPI_receive_byte   SPI2_receive_byte

#else
    #error Definire il modulo SPI da utilizzare per XPT2046
#endif
         
// impostazione divisore (numero di shift a sinistra) per media            
#if (XPT2046_AVG_FACTOR == 1)
    #define XPT2046_AVG_DIVISOR 0
#elif (XPT2046_AVG_FACTOR == 2)
    #define XPT2046_AVG_DIVISOR 1
#elif (XPT2046_AVG_FACTOR == 4)
    #define XPT2046_AVG_DIVISOR 2
#elif (XPT2046_AVG_FACTOR == 8)
    #define XPT2046_AVG_DIVISOR 3
#elif (XPT2046_AVG_FACTOR == 16)
    #define XPT2046_AVG_DIVISOR 4
#elif (XPT2046_AVG_FACTOR == 32)
    #define XPT2046_AVG_DIVISOR 5
#elif (XPT2046_AVG_FACTOR == 64)
    #define XPT2046_AVG_DIVISOR 6
#elif (XPT2046_AVG_FACTOR == 128)
    #define XPT2046_AVG_DIVISOR 7
#elif (XPT2046_AVG_FACTOR == 256)
    #define XPT2046_AVG_DIVISOR 8
#else
    #define XPT2046_AVG_FACTOR  4
    #define XPT2046_AVG_DIVISOR  2
#endif
            
// macro delay 200ns @64MHz
#define Delay_200ns(); Nop(); Nop(); Nop(); Nop();


/* COSTANTI  ================================================================================================================================== */
#define XPT2046_CS_HIGH 1;
#define XPT2046_CS_LOW 0;

/* INDIRIZZI REGISTRI ========================================================================================================================= */
#define XPT2046_X      0x90                                                            // configurazione per X 12 bit modalit differenziale power down 
#define XPT2046_Y      0xD0                                                            // configurazione per Y 12 bit modalit differenziale power down 
#define XPT2046_POWER_OFF_BTW_CONV 0x00
#define XPT2046_POWER_ON_BTW_CONV  0x01            


/* PROTOTIPI E DOCUMENTAZIONE ================================================================================================================= */

/** @file XPT2046.c
 * Libreria per la gestione di pannelli touch screen con controller XPT2046.
 * - Per un corretto funzionamento:
 *    + Includere la libreria SPI_init.c prima delle libreria XPT2046.c
 *    + Impostare il modulo SPI da utilizzare per l'XPT2046 in XPT2046_settings.h
 *    + Abilitare il modulo SPI utilizzato in SPI_init_settings.h
 *    + Impostare il clock SPI e l'averaging factor in XPT2046_settings.h 
 *    + Se si utilizzano pi device in SPI disabilitare il modulo SPI tramite funzione XPT2046_disable() e riabilitarlo per il nuovo modulo
 *    (i parametri di comunicazione potrebbero essere diversi). In alternativa impostare direttamente nel programma principale il modulo SPI con le funzioni
 *    SPI_enable() e SPI2_enable()
 *    + Leggere le coordinate dei punti minimi e massimi
 *    + Calcolare il fattore di scaling tramite la funzione XPT2046_calc_scaling_factor()
 *    + Se XPT2046_IRQ va a 0 leggere il punto toccato tramite funzione XPT2046_read_point_scaled();  
 * - Files:
 *  + XPT2046.c: Codice funzioni
 *  + XPT2046.h: Parametri e prototipi
 *  + XPT2046_settings.h: Impostazioni specifiche per progetto
 */

/** 
 * Abilita il modulo SPI e ne imposta i parametri
 * @param   void
 * @return  void
 */  
void XPT2046_enable(void);

/** 
 * Disabilita il modulo SPI e ne imposta i parametri
 * @param   void
 * @return  void
 */ 
void XPT2046_disable(void);   

/** 
 * Legge le coordinate del punto premuto sul touch screen
 * @param   void
 * @return  struct con variabili x e y raw (0~4096) (unsigned int)
 * Note:    Impostare la costante XPT2046_AVG_FACTOR in XPT2046_settings.h per selezionare numero di letture da fare per singolo punto. 
 *          (1-2-4-5-16-32-64-128-256) da tenere in considerazione che un numero maggiore di campioni equivale a pi tempo per l'acquisizione
 *          di un singolo punto. (Clock @64MHz SPI Clock @1MHz: AVG = 1 TAQ = 56uS, AVG = 8 TAQ = 0.450ms, AVG = 32 TAQ = 1mS, AVG = 256 TAQ = 10ms)
 */ 
xpt2046_point XPT2046_read_point_raw(void);

/** 
 * Calcola i fattori di scaling per adattare il touch alla risoluzione del display 
 * @param   _point_1: punto toccato pi in alto a sinistra per calcolare i valori minimi (struct xpt2046_point)
 * @param   _point_2: punto toccato pi in basso a destra per calcolare i valori massimi (struct xpt2046_point)
 * @return  void
 * @note    Salva variabili globali x_scale_factor e y_scale_factor
 */    
void XPT2046_calc_scaling_factor(xpt2046_point _point_1, xpt2046_point _point_2);
 
/** 
 * Calcola le coordinate del punto scalate alla risoluzione del display 
 * @param   _touched_point_raw: punto toccato (struct xpt2046_point)
 * @return  struct xpt2046_point contenente le coordinate del punto toccato scalate alla risoluzione del display 
 */ 
xpt2046_point XPT2046_scale_point(xpt2046_point _touched_point_raw);

/** 
 * Legge le coordinate del punto premuto sul touch screen  e le scala alla risoluzione del display 
 * @param   void
 * @return  struct xpt2046_point contenente le coordinate del punto toccato scalate alla risoluzione del display 
 */ 
xpt2046_point XPT2046_read_point_scaled(void);