/* ============================================================================================================================================= /
 * PROJECT:         Stazione Meteo
 * FILE:            Main.c
 * LAST REV:        1.3
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F67K22
 * CREATED:         16/02/2017
 * UPDATED:         02/10/2017
 *
 * DESCRIP:         Stazione meteo
 *
 * ============================================================================================================================================= /
 * Change History:
 * Rev.             Date                Changes
 * 1.3              02/10/2017          - Correzione bug calcolo fase lunare
 *                                      - Implementato invio file LOG.TXT da SD via WiFi
 *                                      - Migliorato ciclo controllo dati DCF77 ricevuti
 * 1.2              02/08/2017          - Correzioni Bug
 *                                      - Aggiunto setup CET/CEST in setup orario manuale
 *                                      - Migliorata esposizione LOG debug                                                  
 *                                      - dcf77_received_counter come variabile globale e resettato se dcf77 fermato in fase di ricezione
 * 1.1              10/08/2017          Prima release pubblica
 *                                      - ottimizzazioni e correzioni bug
 * 1.0              15/07/2017          Prima release completa
 *                  16/02/2017          Inizio sviluppo firmware complessivo
 *                                      
 /* ============================================================================================================================================ */
 /* INCLUDES =================================================================================================================================== */
 
#include <xc.h>
#include "Main.h"

#include "Delay.c"
#include "I2C_init.c"
#include "SPI_init.c"
#include "USART_init.c"
#include "EEPROM.c"
#include "EEPROM_map.h"

#include "ff.c"

#include "HDC1080.c"
#include "LPS25HB.c"
#include "VEML6075.c"
#include "EEPROM_I2C.c"

#include "TFT_ILI9325.c"
#include "XPT2046.c"

#include "ESP8266.c"
#include "HMTRP.c"

#include "DCF77.c"
#include "HKW581.c"

#include "RTCC_PIC18F6x_8x.c"

#include "math.h"

#include "Graphics_def.h"
#include "Prototipi.h"

/* GLOBAL VARIABLES =========================================================================================================================== */

// variabili per modulo remoto
unsigned char remote_battery_level = 0;
bit remote_connected = 0;

// struttura valori sensori
typedef struct {
    float temp_hdc1080;
    float humidity;
    float press;
    float temp_lps25;
    veml6075_uv uv_readed;    
} sensors_values;

typedef struct {
    unsigned int temp_hdc1080_raw;
    unsigned int humidity_raw;
    unsigned long press_raw;
    signed int temp_lps25_raw;
    veml6075_uv_raw uv_readed_raw;
} sensors_raw_values;

// variabili sensori
sensors_values local_actual;
sensors_values local_min;
sensors_values local_max;
sensors_values remote_actual;
sensors_values remote_min;
sensors_values remote_max;

sensors_raw_values sensors_local_raw_values;
sensors_raw_values sensors_remote_raw_values;

// variabili per libreria FaFs
FATFS sd_card;	                                                                        // FatFs work area needed for each volume
FIL file;		                                                                        // File object needed for each open file
unsigned int bw;

// variabili personalizzabili utente e salvate in EEPROM interna
unsigned int local_altitude = 0;
signed char time_zone_utc = 0;                                                          // 1 = utc +1
signed char meteo_zone = 0;                                                             // zona meteo (milano = 40)
bit press_to_write = 0;                                                                 // 0 = pressione assoluta, 1 = pressione normalizzata al livello del mare
bit dcf77_user_enable = 0;                                                              // 0 = dcf77 disabilitato, 1 = dcf77 abilitato

// variabili per retroilluminazione personalizzabili utente e salvate in EEPROM interna
unsigned char ccpr4l = 0;       
unsigned char backill_on_h = 0;
unsigned char backill_on_m = 0;
unsigned char backill_off_h = 0;
unsigned char backill_off_m = 0;

// impostazione utente tempo refresh sensori salvate in EEPROM
unsigned int time_refresh_remote_sensors = 0; 
unsigned int time_refresh_local_sensors = 0;    

// variabili data e ora
dcf77_decoded_data dcf77_time;
dcf77_decoded_data dcf77_time_0;
dcf77_decoded_data dcf77_time_1;
dcf77_decoded_data dcf77_time_2;
bit cet_cest_flag = 0;              // 0 = CET, 1 = CEST
rtcc_data rtcc_time;
unsigned char dcf77_received_counter = 0;


// variabili meteo
hwk581_decoded_data meteo_fc_decoded;

typedef struct {
    unsigned char day_fc;
    unsigned char night_fc;
    unsigned char extreme_w;
    unsigned char rain_pr;
    unsigned char anomaly;
    unsigned char wind_dir;
    unsigned char wind_str;
    signed char temp_min;
    signed char temp_max;
}meteo_fc_compl;

meteo_fc_compl meteo_fc_day_1;
meteo_fc_compl meteo_fc_day_2;
meteo_fc_compl meteo_fc_day_3;
meteo_fc_compl meteo_fc_day_4;
meteo_fc_compl * meteo_fc_day_pointer; 
rtcc_data meteo_update_time; 

// variabili per orari meteo avvio DCF77
unsigned char hour_meteo[8];
unsigned char minute_meteo = 0;

// variabili wifi
bit wifi_on_off = 0;                                                                    // 0 = wifi spento, 1 = wifi attivo
unsigned char server_status;                                                            // 0 = non creato, 1 = server creato
bit wifi_connected = 0;                                                                 // 0 = wifi non connesso, 1 = wifi connesso
bit wifi_trans_icon_drawn = 0;                                                          // icona di connessioni in ingresso attive
unsigned char ssid[32];                                                                 // buffer contentente l'ssid selezionato
unsigned char password[32];                                                             // buffer contentente la password per l'ssid selezionato
const unsigned char port[] = "80";
signed char active_conn = 0;                                                          // numero di connessioni attive (0~5)

// variabili di controllo
unsigned char first_sync = 0;                                                           // controllo azioni aggiuntive dopo prima sincronizzazione DCF77
unsigned char meteo_time = 0;                                                           // conteggio dato meteo a seconda della fascia oraria
bit dcf77_status = DCF77_DISABLED;                                                      // 0 = DCF77 disabilitato, 1 = DCF77 abilitato
bit dcf77_icon_drawn = 0;
//bit valid_time = 0;                                                                     // flag data e ora validi ricevuti
bit new_sec = 0;                                                                        // flag nuovo secondo
char second_rt = 0;                                                                     // valore dei secondi per azioni basate sui secondi
bit rem_con_icon_drawn = 0;
bit meteo_page = 0;                                                                     // 0 = giorno attuale, 1 = previsione 2 giorni seguenti
bit new_local_sensors_values = 0;                                                       // 1 = nuovi dati letti dai sensori
bit new_remote_sensors_values = 0;
bit sensor_page = 0;                                                                    // 0 = locali, 1 = remoto
bit record_log = 0;                                                                     // 0 = valori dei sensori non salvati, 1 = salva i valori dei sensori in EEPROM
bit new_log = 0;                                                                        // 1 = avviata nuova registrazione log sensori, 0:registrazione log in corso (impostare a 1 prima di avviare nuova registrazione log)
unsigned char eeprom_log_index = 0;                                                     // indice univoco incrementa ad ogni avvio salvataggio log, e viene salvato in eeprom prima dei dati rilevati
unsigned char chart_2_plot = 0;
volatile unsigned int hmtrp_timeout_counter = 0;                                        // contatore per timeout ricezione da modulo remoto
volatile unsigned int counter_refresh_local_sensors = 1;                                // contatore secondi trascorsi per acquisizione nuovi valori sensori locali

bit new_day = 0;
unsigned char dcf77_minute_sync = 0;                                                    // conta i minuti trascorsi in fase di sincronizzazione dcf77
bit dcf77_stopped = 0;                                                                  // segnala che la sincronizzazione dcf77  stata interrotta dopo 8 minuti senza dati
bit server_closed_for_dcf77_sync = 0;

bit no_plot_chart = 0;                                                                  // controllo extra per pagina log sensori altrimenti ad ogni tocco invalido si vede che ridisegna lo stesso grafico
unsigned char log_next_allowed = 0;                                                     // abilitazione per scrolling grafico
unsigned char log_prev_allowed = 0;
    
unsigned char page = 0;                                                                 // pagina visualizzata

// variabili per statistiche
rtcc_data rtcc_time_start;
unsigned int    dcf77_success_sync_counter = 0;
unsigned int    dcf77_fail_sync_counter = 0;
unsigned int    meteo_success_sync_counter = 0;
unsigned int    meteo_fail_sync_counter = 0;
unsigned int    wifi_incoming_con_counter = 0;
unsigned int    wifi_real_time_send_counter = 0;
unsigned int    wifi_log_send_counter = 0;
unsigned int    remote_discon_counter = 0;

// array nomi giorni
const unsigned char * name_day[7] = {"Domenica", "Lunedi", "Martedi", "Mercoledi", "Giovedi", "Venerdi", "Sabato"};
// array sensori
const unsigned char * label[8] = {" Sensori locali", "  Remoto", "Temp.", "RH       %", "Pre. hPa", "UVA  mW", "UVB cmq", "UVI"};
// array testi pulsanti
const unsigned char * sw_label[21] = {"LOG","SET","STA","ILL", "REC", "STO", "EXP", "LOC", "REM", "TEM", "RH", "PRE", "UVA", "UVB", "UVI", "+", "-", "OK", "EXT", "RSC", "CON"};
// array messaggi vari
//                                    0                              1                      2                               3                 4           5           6               7           8            9         
const unsigned char * messages[36] = {"Calibrazione touch screen, ", "Tocca i punti rossi", "Tocca il display per uscire.", "HARDWARE CHECK", "HDC1080:", "Trovato.", "Non trovato.", "LPS25HB:", "VEML6075:", "24LC1025 B1:",
//                                  10              11          12        13                 14             15             16             17             18      19          20           21                   22          23
                                    "24LC1025 B2:", "ESP8266:", "HMTRP:", "Tocca per iniz.", "HMTRP CONF:", "Led accesi.", "HMTRP NORM:", "Led Spenti.", "Fine.", "REMOTO:", "Accendere", "Nessuna risposta.", "Saltato.", "DCF77:",
//                                  24         25                 26             27                28         29         30    31     32      33                 34                        35
                                    "HKW581:", "Attesa BRF a 0.", "Invio Dati,", "Decodifica ok.", "Errore.", "Buzzer:", "ON", "OFF", "TFT:", "Tocca per iniz.", "HARDWARE CHECK FINITO,", "Riavviare il modulo remoto"};

// array citt meteo            
const unsigned char * city[60] = {"F - Bordeaux", "F - La Rochelle","F - Paris", "F - Brest", "F - Clermont-Ferrand",  "F - Beziers", "B - Bruxelles", "F - Dijon", "F - Marseille", "F - Lyon",            // 0 - 9
                                  "F - Grenoble", "CH - La Chaux de F.", "D - Frankfurt am M.", "D - Trier" , "D - Duisburg", "GB - Swansea", "GB - Manchester", "F - le Havre", "GB - London", "D - Bremerhaven",   // 10 - 19
                                  "DK - Herning", "DK - Arhus", "D - Hannover", "DK - Kobenhavn", "D - Rostock", "D - Ingolstadt", "D - Munchen", "I - Bolzano", "D - Nurnberg", "D - Leipzig",             // 20 - 29
                                  "D - Erfurt", "CH - Lausanne", "CH - Zurich", "CH - Adelboden", "CH - Sion", "CH -Glarus", "CH - Davos", "D - Kassel", "CH - Locarno", "I - Sestriere",                   // 30 - 39
                                  "I - Milano", "I - Roma", "NL - Amsterdam", "I - Genova", "I - Venezia", "F - Strasbourg", "A - Klagenfurt", "A - Innsbruck", "A - Salzburg", "SK - Bratislava",          // 40 - 49
                                  "CZ - Praha", "CZ - Decin", "D - Berlin", "S - Goteborg", "S - Stockholm", "S - Kalmar", "S - Jonkoping", "D - Donaueschingen", "N - Oslo", "D - Stuttgart"};             // 50 - 59
// array menu setup                    
const unsigned char * setup_items_1[9] = {"DATA/ORA MANUALE", "FUSO ORARIO","REGIONE METEO", "ALLARMI", "SENSORI", "WIFI", "LUMINOSITA", "", "pagina 2 ->"};
const unsigned char * setup_items_2[9] = {"CALIBRAZIONE TOUCH", "TEST HARDWARE","LOOP SENSORI","TEST DCF77","","","","","<- pagina 1"};
const unsigned char * setup_messages[32] = {"Impostazione fuso orario UTC", "Impostazione citta per", "le previsioni meteo", "Impostazioni illuminazione", "Livello", "Ora accensione", "Ora Spegnimento", "Impostazione ora manuale", 
                                            "Giorno", "Data", "Ora", "Impostazioni sensori", "Locale:", "Remoto:", "Pressione:", "Altitudine (m.):","Impostazioni WiFi", "Connessione WiFi:","SSID:","Stato:","IP:", "Server:", "Porta:" , 
                                            "Cerca rete", "Ricerca reti in corso", "Inserire la password per","Connesso","Non Connesso","Connessione in corso", "Nessuna rete trovata", "DCF77:", "CET/CEST:"};
const unsigned char * others_messages[2] = {"Rete WiFi valida salvata in","memoria, tentativo di "};
const unsigned char * sensor_time[9] = {"","5 sec.","10 sec.", "30 sec.", "1 min.", "5 min.","10 min.","20 min.","30 min."};
const unsigned char * press_descr[2] = {"assoluta", "normalizzata"};
const unsigned char * on_off[2] = {"OFF", "ON"};
const unsigned char * cet_cest[2] = {"CET", "CEST"};

const unsigned char * encryption[5] = {"Open", "WEP" , "WPA_PSK", "WPA2_PSK", "WPA_WPA2_PSK"};

const unsigned char * stat_items[9] = {"Attivo da: ", "DCF77 validi: ", "DCF77 invalidi: ", "Meteo validi: ", "Meteo invalidi: ", "Conn. avvenute: ", "Invio RT: ", "Invio LOG: ", "Discon. remoto: "};

/* DEBUG ====================================================================================================================================== */
#define DEBUG_LOG_SD                // salva file log su SD
//#define DEBUG_LOG_SD_METEO_TIME     // salva gli orari a cui far partire il DCF77 per il meteo 
//#define BYPASS_DCF77_AT_STARTUP     // non attende la prima sincronizzazione DCF77
#define DEBUG_SENSOR_ANOMALY        // valori sensori insoliti

#ifdef DEBUG_LOG_SD
#include "Debug.c"
#endif

/* INTERRUPT ================================================================================================================================== */
void high_priority interrupt inter_high(void) {
    volatile static unsigned int _dcf77_timer_counter = 0;
    volatile static unsigned char _dcf77_bit_counter = 0;
    volatile static unsigned char _dcf77_signal_check = 0;
    
/* INTERRUPT PER DCF77 */     
    if (DCF77_INT_IE && DCF77_INT_IF){
        if (_dcf77_timer_counter > 185) {                                               // minute mark con 185 va benissimo
                _dcf77_timer_counter = 0;
                DCF77_TMR = DCF77_TIMER_PRELOAD;
                _dcf77_bit_counter = 0;   
                dcf77_sync = 1;
                dcf77_seconds_pulse = 1;
                DCF77_INT_IE = 0;                                                       // disabilito interrupt INT
        } else if ((_dcf77_timer_counter >= 90) && (_dcf77_timer_counter <= 110) ){     // impulso ad inizio secondo valido
            _dcf77_timer_counter = 0;
            DCF77_TMR = DCF77_TIMER_PRELOAD;
            _dcf77_bit_counter++;
            dcf77_seconds_pulse = 1;
            DCF77_INT_IE = 0;                                                           // disabilito interrupt INT
        } 
        
        DCF77_TIMER_ON = ON;                                                            // avvio timer (dopo primo interrupt)
        DCF77_TIMER_INT_IF = 0;
        DCF77_TIMER_INT_IE= ON;
        DCF77_INT_IF = 0;
    }
      
    if (DCF77_TIMER_INT_IE && DCF77_TIMER_INT_IF) {
        DCF77_TMR = DCF77_TIMER_PRELOAD;                                                // resetto il preload
        
        _dcf77_timer_counter++;  
        
        if (dcf77_seconds_pulse == 1){                                                  // se ho gi un impulso ad inizio secondo valido, se arriva un impulso prima di 130ms non viene considerato e il timer continua
            if ((_dcf77_timer_counter >=13) && (_dcf77_timer_counter <=17)) {           // controllo il segnale a 130ms, 140ms, 150ms, 160ms, 170ms,
                if (DCF77_SIGNAL == DCF77_SIGNAL_ACTIVE)                                // e ne salvo il valore
                    _dcf77_signal_check++;
            } else if (_dcf77_timer_counter == 18) {
                                           
                if (_dcf77_signal_check > 3)                                            // se ho trovato piu 1 salvo 1 altrimenti 0
                    _dcf77_buffer[_dcf77_bit_counter] = 1;    
                else
                    _dcf77_buffer[_dcf77_bit_counter] = 0;    

                dcf77_seconds_pulse = 0;
                _dcf77_signal_check = 0;                                                
                
                if (_dcf77_bit_counter == 58 && dcf77_sync) {                           // se il contatore  arrivato a 58 ed il minute_mark  stato trovato almeno una volta
                    dcf77_data_ready = 1;                                               // imposto data ready
                    dcf77_sync = 0;                                                     // azzero per attesa nuovo minute mark, serve anche in caso di second leap
                }
            }
        } else {                                                                        
            if (_dcf77_timer_counter == 85){                                            // Abilito interrupt INT in attesa del secondo
                DCF77_INT_IF = 0;
                DCF77_INT_IE = 1; 
            }
        }
        DCF77_TIMER_INT_IF = 0;
    }
    if (ESP8266_INTERRUPT_IE && ESP8266_INTERRUPT_IF) {                                 // se l'interrupt  stato causato da una ricezione dati USART
        
        esp8266_buffer[esp8266_buffer_index] = _ESP8266_USART_receive();                // salvo i dati ricevuti nell'array usart_received
        esp8266_buffer_index++;
        
        if (esp8266_buffer_index > ESP8266_BUFFER_LENGTH)                               // se supero la dimensione massima resetto l'indice
            esp8266_buffer_index = 0;    
 
        ESP8266_TMR = ESP8266_TIMER_PRELOAD;                                            // Resetto e avvio il timer
        ESP8266_TIMER_ON = ON;                                                          // avvio il timer (se spento: primo byte ricevuto)
        
        //        ESP8266_INTERRUPT_IF = 0;                                                // Resetta gi USART_init
    }
    
    if (ESP8266_TIMER_INT_IE && ESP8266_TIMER_INT_IF) {                            // interrupt timer significa che non ricevo byte da 5ms   
        ESP8266_TMR = ESP8266_TIMER_PRELOAD;
        
        esp8266_string_completed = 1;                                                   // setto il flag di ricezione completata
        ESP8266_TIMER_ON = OFF;                                                         // spengo timer
        ESP8266_TIMER_INT_IF = 0;
    }
}

void low_priority interrupt inter_low(void) {
/* INTERRUPT PER HMTRP */    
    if (HMTRP_INTERRUPT_IE && HMTRP_INTERRUPT_IF) {                                // se l'interrupt  stato causato da una ricezione dati USART
        hmtrp_buffer[hmtrp_buffer_index] = HMTRP_receive();                             // salvo i dati ricevuti nell'array usart_received
        
        if (hmtrp_buffer_index>0){
            if ((hmtrp_buffer[hmtrp_buffer_index]== 0x0A) && (hmtrp_buffer[hmtrp_buffer_index-1])== 0x0D){
                hmtrp_string_received = 1;
                hmtrp_buffer[hmtrp_buffer_index] = 0x00;                                // tolgo caratteri di fine stringa
                hmtrp_buffer[hmtrp_buffer_index-1] = 0x00;
            }
        }
        hmtrp_buffer_index++;
        
        if (hmtrp_buffer_index > HMTRP_BUFFER_LENGTH)                                   // se supero la dimensione massima resetto l'indice
            hmtrp_buffer_index = 0;    
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:        Main
 *
 * Overview:
/* ============================================================================================================================================ */

void main(void) {
    
/* INIZIALIZZAZIONI */
    Define_IO();
    Module_settings();

    XPT2046_enable();                                                                   // Inizializzo SPI per XPT2046    
    TFT_init();                                                                         // Inizializzo TFT
    TFT_off();
    
    HMTRP_init();                                                                       // Inizializzo modulo HMTRP
    USART2_enable(USART_BAUDR_9600);                                                    // Inizializzo USART  per HMTRP

    USART_enable(USART_BAUDR_115200);                                                   // Inizializzo USART  per ESP8266
    ESP8266_init(ESP8266_DEF_TIMEOUT);                                                  // Inizializzo modulo ESP8266
    
    DCF77_init();                                                                       // Inizializzo modulo DCF77
    HKW581_init();                                                                      // Inizializzo HKW581
    
//    HMTRP_mode(HMTRP_SLEEP_MODE);                                                       // disattivo WIFI e HMTRP per evitare interferenze con DCF77
    HMTRP_mode(HMTRP_NORMAL_MODE);                                                       
    ESP8266_power(OFF, 10);
    
    I2C_init_master (400);                                                              // Inizializzo I2C modulo 1 per HDC1080, LPS25HB, VEML6075, 24LC1025  
    
    RTCC_init(RTCC_OUT_DISABLED, RTCC_AL_INTERRUPT_DISABLED);                           // RTCC senza uscita interrupt e senza interrupt interni
    RTCC_alarm_mode(RTCC_AL_MIN, RTCC_AL_REPEAT);                                       // allarme si attiva ogni minuto, ripetizione sempre
    RTCC_alarm(ON);                                                                     // allarme attivo
    
    // LOG AVVIO AVVENUTO E RCON    
    #ifdef DEBUG_LOG_SD
    Write_log(BOOT);
    #endif
   
    /* IMPOSTAZIONE VALORI DI PARTENZA PRIMO AVVIO DOPO PROGRAMMAZIONE */  
    if (EEPROM_read(EEPROM_START_CHECK) != 0x70){                                       // se il byte di controllo non  0x70
        Save_eeprom_default_variables();                                                // salvo i valori di partenza in EEPROM
        
        // LOG PRIMO AVVIO   
        #ifdef DEBUG_LOG_SD
        Write_log(FIRST_START);
        #endif    
    }

    /* LETTURA VALORI AVVIO */
    Load_eeprom_variables();                                                            // carico i valori dalla EEPROM ed imposto le variabili globali                 
    
    // LOG DUMP VARIABILI EEPROM   
    #ifdef DEBUG_LOG_SD
    Write_log(EEPROM_VARIABLES);
    #endif
    
    Init_meteo_data();                                                                 // metto tutte le variabili meteo a 0x7F
    
    /* IMPOSTAZIONE SENSORI STAZIONE */    
    HDC1080_set(HDC1080_ONE_AQ,HDC1080_TEMP_RES_14bit,HDC1080_HUMI_RES_14bit);
    LPS25HB_set_resolution(LPS25HB_DEV_ADDR,LPS25HB_TEMP_RES_8,LPS25HB_PRES_RES_8);  
    LPS25HB_set_mode(LPS25HB_DEV_ADDR,LSP25HB_ODR_ONE_SHOT);                            
    LPS25HB_set_onepoint_cal(LPS25HB_DEV_ADDR, LPS25HB_ONE_POINT_CAL);
    VEML6075_set(VEML6075_ONE_AQ,VEML6075_NORMAL_DR,VEML6075_IT_50MS);
    
/* TEST AREA  */
// inserire qui funzioni in test   
    
/* !!!!!!!!!! */    

    /* PRIMO AVVIO TIME E DISPLAY */
    TFT_clear(BACK_COLOR);
    page = MAIN_PAGE;
    TFT_set_font(font_lucida_sans_10x16);                                               // imposto font principale
    
    // se wifi abilitato all'avvio ed  stato salvato un SSID valido
    // tenta la connessione alla rete
    if (wifi_on_off == ON && ssid[0] != '\0'){
        TFT_active();
//        TFT_set_font(font_lucida_sans_10x16);
        
        TFT_write_string(10,20,230,others_messages[0],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);  // "Rete valida salvata in"
        TFT_write_string(10,40,230,others_messages[1],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);  // "memoria, tentadivo di "
        TFT_write_string(10,60,230,setup_messages[28],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);  // "Connessione in corso"
        
        ESP8266_power(ON,ESP8266_DEF_TIMEOUT);                                          // abilito modulo wifi      
        Wifi_connect(ssid, password);                                                   // connessione
        if (dcf77_user_enable == 1)
            Wifi_close_server();                                                        // chiudo subito il server per ricezione dati DCF77    
    }
    
    Get_local_sensors_values();                                                         // leggo sensori locali
    new_local_sensors_values = 1;
    
    Draw_layout(MAIN_PAGE);                                                             // pagina principale con valori iniziali di RTCC e sensori locali
    Draw_switch(SW_INIT_4);                                                             // disegno i pulsanti
    
    TFT_active();                                                                       // attivo il display (se non  gi attivo da connessione wifi)

    if (dcf77_user_enable){
        #ifdef DEBUG_LOG_SD
        Write_log(START_DCF);
        #endif   
        dcf77_status = DCF77_ENABLED;                                                       // abilito DCF77 per prima sincronizzazione
        DCF77_start_receive_via_interrupt();                                                // avvio ricezione DCF77
    }

    RTCC_osc(ON);                                                                       // avvio oscillatore
    
// salta la prima sincronizzazione DCF77 se BYPASS_DCF77_AT_STARTUP definito
#ifdef BYPASS_DCF77_AT_STARTUP
    dcf77_status = DCF77_DISABLED;                                                      // disabilito DCF77 e fermo ricezione
    DCF77_stop_receive_via_interrupt();
    HMTRP_mode(HMTRP_NORMAL_MODE);                                                      // abilito ricezione modulo remoto
    first_sync = 1;                                                                     // forzo flag prima sincronizzazione
    RTCC_write_all(0x30,0x07,0x17,0,0x13,0x33,0x01);                                    // data fittizia per simulare sincronizzazione
    
    meteo_fc_day_1.day_fc = 1;
    meteo_fc_day_1.night_fc = 1;
    meteo_fc_day_1.extreme_w = 0;
    meteo_fc_day_1.rain_pr = 1;
    meteo_fc_day_1.wind_dir = 0;
    meteo_fc_day_1.wind_str = 1;
    meteo_fc_day_1.temp_min = 22;
    meteo_fc_day_1.temp_max = 29;
    
    meteo_fc_day_2.day_fc = 1;
    meteo_fc_day_2.night_fc = 1;
    meteo_fc_day_2.extreme_w = 0;
    meteo_fc_day_2.rain_pr = 0;
    meteo_fc_day_2.wind_dir = 1;
    meteo_fc_day_2.wind_str = 1;
    meteo_fc_day_2.temp_min = 24;
    meteo_fc_day_2.temp_max = 30;
    
    meteo_fc_day_3.day_fc = 1;
    meteo_fc_day_3.night_fc = 1;
    meteo_fc_day_3.extreme_w = 0;
    meteo_fc_day_3.rain_pr = 0;
    meteo_fc_day_3.wind_dir = 2;
    meteo_fc_day_3.wind_str = 2;
    meteo_fc_day_3.temp_min = 26;
    meteo_fc_day_3.temp_max = 33;
    
    meteo_update_time.day = 0x30;
    meteo_update_time.month = 0x07;
    meteo_update_time.hours = 0x14;
    meteo_update_time.minutes = 0x02;
    
    Update_time();
#endif
    
    while(1){
        // operazioni da fare ogni secondo
        if (RTCCFGbits.HALFSEC == 0 && new_sec == 0){
            new_sec = 1;
            second_rt = RTCC_read_seconds();

            // contatori 
            hmtrp_timeout_counter++;
            counter_refresh_local_sensors++;
                    
            /* controllo livello wifi ogni 30 secondi solo se wifi abilitato e se non ci sono connessioni
               in ingresso attive
               se arriva una connessione e poi viene chiesta subito la scansione della rete
               perdo il comando in ingresso
             */
            if ((second_rt == 0x32 || second_rt == 0x02) && (wifi_on_off == ON) && (active_conn == 0)){
                Wifi_check_con(0);
            }
            
        }else if (RTCCFGbits.HALFSEC == 1 && new_sec == 1){ 
            new_sec = 0;
        } 

        // lettura valori sensori locali        
        if(counter_refresh_local_sensors == time_refresh_local_sensors){                // se sono passati i secondi selezionati dall'ultima lettura
            counter_refresh_local_sensors = 0;
            Get_local_sensors_values();                                                 // leggo i sensori e salva in EEPROM se registrazione log  attiva
            new_local_sensors_values = 1;
        }   

        // ricezione valori sensori remoti
        if (hmtrp_string_received && hmtrp_buffer[0] == 0xFA){
            
            Get_remote_sensors_values();                                                // converto i valori ricevuti salvo variabili e LOG se abilitato
            new_remote_sensors_values = 1;
            hmtrp_timeout_counter = 0;
            
            #ifdef  DEBUG_LOG_SD
            if (remote_connected == 0)
                Write_log(REMOTE_CONNECT);            
            #endif
            
            remote_connected = 1;                                                       // flag modulo remoto connesso   
        } else if (hmtrp_string_received){                                              // pulisco la stringa (accensione modulo remoto manda F0 ma qui non serve)
            HMTRP_reset_buffer();
        }

        // controllo timeout ricezione da modulo remoto
        if (remote_connected && hmtrp_timeout_counter > time_refresh_remote_sensors) {
            hmtrp_timeout_counter = 0;
            remote_connected = 0;
            
            if (page != MAIN_PAGE)                                                      // se timeout avviene fuori da pagina main
                rem_con_icon_drawn = 0;                                                 // cancello il flag di icona disegnata per non ridisegnarla quando
                                                                                        // torno in pagina main
            remote_discon_counter++;
            
            #ifdef  DEBUG_LOG_SD
            Write_log(REMOTE_DISCON);            
            #endif
        }
        
        if (wifi_connected)                                                             // se wifi connesso 
            Wifi_check_incoming();                                                      // polling connessioni e dati in arrivo
        
        // ciclo di gestione segnale DCF77 ora e meteo, se ricezione attivata
        if (dcf77_user_enable == 1 && dcf77_status == DCF77_ENABLED)
            DCF77_manage_loop();

        // operazioni da fare solo dopo la prima sincronizzazione DCF77
        if (first_sync == 1){
            first_sync = 2;

            rtcc_time = RTCC_read_all();                                                // leggo l'orario dal RTCC    
            rtcc_time_start = rtcc_time;                                                // salvo ora avvio

            // leggo i valori meteo da EEPROM solo se almeno un giorno di salvataggio  uguale al giorno attuale
            if (EEPROM_read(EEPROM_FC_D1_ADDRESS) == rtcc_time.day || EEPROM_read(EEPROM_FC_D2_ADDRESS) == rtcc_time.day ||
                EEPROM_read(EEPROM_FC_D3_ADDRESS) == rtcc_time.day || EEPROM_read(EEPROM_FC_D4_ADDRESS) == rtcc_time.day)                     
                Load_meteo_data_eeprom(); 
            
            if (page == MAIN_PAGE){                                                     // se sono in pagina main
                meteo_page = 0;                                                     
                Draw_meteo(meteo_page);                                                 // scrivo il meteo su display e luna, se il meteo non  stato caricato da eeprom verranno scritti i valori a -- in attesa di valori validi
            }
        }
        
        // Allarme da RTCC ogni minuto per aggiornamento dati su display e azioni basate sull'orario
        if (PIR3bits.RTCCIF){
            PIR3bits.RTCCIF = 0;
            
            rtcc_time = RTCC_read_all();                                                // leggo l'orario dal RTCC    
            
            if (page == MAIN_PAGE)
                Update_time();                                                          // Aggiorno data e ora su display se sono in pagina MAIN

            // se abilitata ricezione DCF77 incremento contatore ad ogni minuto per limitare i tentativi di sincronizzazione
            if (dcf77_status == DCF77_ENABLED){
                dcf77_minute_sync++;
                
                // se superato il limite ferma la ricezione e apre il server se connesso a wifi
                if (dcf77_minute_sync > DCF77_MINUTE_LIMIT){
                    DCF77_stop_receive_via_interrupt();
                    dcf77_status = DCF77_DISABLED;
                    dcf77_minute_sync = 0;
                    dcf77_stopped = 1;
                    dcf77_received_counter = 0;
                    
                    #ifdef DEBUG_LOG_SD
                    Write_log(DCF77_STOPPED);
                    #endif
                    
                    if (server_closed_for_dcf77_sync == 1)    
                        Wifi_open_server();
                }
            }

            if (dcf77_user_enable)
                Check_time_2_sync();                                                    // Controllo se devo avviare la ricezione DCF77
            
            // a mezzanotte porto avanti le previsioni meteo e azzero i minimi e massimi dei sensori
            if (rtcc_time.hours == 0x00 && rtcc_time.minutes == 0x00){                  
                meteo_fc_day_1 = meteo_fc_day_2;
                meteo_fc_day_2 = meteo_fc_day_3;
                meteo_fc_day_3 = meteo_fc_day_4;

                Save_meteo_data_eeprom(1);                                              // salvo i nuovi dati in EEPROM (1 = tutti i dati meteo)
                
                local_min = local_actual;                                               // reset sensori minimi e massimi al valore attuale
                local_max = local_actual;
                remote_min = remote_actual;
                remote_max = remote_actual;

                new_day = 1;                                                            // flag per aggiornare meteo e sensori alla mezzanotte
                
                #ifdef  DEBUG_LOG_SD
                Write_log(NEW_DAY);            
                Write_log(METEO_EEPROM);
                #endif
            } 

//            // alle 6:00 aggiorno la fase lunare
//            else if (rtcc_time.hours == 0x06 && rtcc_time.minutes == 0x00 && meteo_page == 0)
//                Draw_moon_phase();

        }
        
        /* GESTIONE PAGINA MAIN */
        if (page == MAIN_PAGE){
            
            // scrittura icona antenna lampeggiante con la ricezione DCF77
            if (dcf77_status == DCF77_ENABLED){
                if (dcf77_seconds_pulse == 1 && dcf77_icon_drawn == 0){                                    
                    TFT_draw_image_sd(DCF77_ICON_X,DCF77_ICON_Y,"DCF_on.bmp");
                    dcf77_icon_drawn = 1;
                }
                else if (dcf77_seconds_pulse == 0 && dcf77_icon_drawn == 1){
                    TFT_draw_image_sd(DCF77_ICON_X,DCF77_ICON_Y,"DCF_off.bmp");
                    dcf77_icon_drawn = 0;
                } 
            } else if (dcf77_status == DCF77_DISABLED && dcf77_seconds_pulse == 1 && dcf77_user_enable == 1) {
                TFT_fill_rect(DCF77_ICON_X,DCF77_ICON_Y,DCF77_W,DCF77_H,BACK_COLOR);    // cancello l'icona dell'antenna
                dcf77_seconds_pulse = 0;
            }
            
            // icona DCF_off in caso in cui la ricezione sia stata fermata dopo 8 minuti senza segnale valido
            if (dcf77_stopped == 1){
                TFT_draw_image_sd(DCF77_ICON_X,DCF77_ICON_Y,"DCF_off.bmp");
                dcf77_stopped = 0;
            }

            if (new_day) {                                                              // nuovo giorno
                new_day = 0;
                Draw_meteo(meteo_page);                                                 // aggiorno il meteo sul display e la fase lunare                

                if (sensor_page == 0)                                                   // scrivo i valori dei sensori selezionati su display
                    Write_sensor_values(&local_actual, &local_min, &local_max);
                else
                    Write_sensor_values(&remote_actual, &remote_min, &remote_max);

            }
            
            // accensione o spegnimento della retroilluminazione all'orario impostato                
            if (rtcc_time.hours == backill_off_h && rtcc_time.minutes == backill_off_m)
                CCPR4L = 0;
            else if (rtcc_time.hours == backill_on_h && rtcc_time.minutes == backill_on_m)
                CCPR4L = ccpr4l;  

            /* Scrittura sensori locali su display se pagina 0 selezionata */        
            if (new_local_sensors_values) {                                             // se sono stati ricevuti nuovi dati
                if (sensor_page == 0)                                                   // se pagina 0 selezionata
                    Write_sensor_values(&local_actual, &local_min, &local_max);        // scrivo i valori
                new_local_sensors_values = 0;
            }
            
            // disegno l'icona solo se non  gi disegnata
            if (remote_connected && rem_con_icon_drawn == 0){
                TFT_draw_image_sd(REMO_ICON_X,REMO_ICON_Y,"REM_CON.bmp");
                rem_con_icon_drawn = 1;
            // se il modulo remoto si  disconnesso cancello l'icona e la batteria
            } else if (remote_connected == 0 && rem_con_icon_drawn == 1){
                TFT_fill_rect(REMO_ICON_X, REMO_ICON_Y,52,15,BACK_COLOR);               
                rem_con_icon_drawn = 0;
            }

            // aggiorno sempre il livello della batteria alla ricezione dei dati
            if (remote_connected && new_remote_sensors_values){
                switch (remote_battery_level){
                    case 0: TFT_draw_image_sd(BAT_ICON_X,BAT_ICON_Y,"BAT_0.bmp");
                        break;
                    case 1: TFT_draw_image_sd(BAT_ICON_X,BAT_ICON_Y,"BAT_1.bmp");
                        break;
                    case 2: TFT_draw_image_sd(BAT_ICON_X,BAT_ICON_Y,"BAT_2.bmp");
                        break;
                    case 3: TFT_draw_image_sd(BAT_ICON_X,BAT_ICON_Y,"BAT_3.bmp");
                        break;
                    case 4: TFT_draw_image_sd(BAT_ICON_X,BAT_ICON_Y,"DC.bmp");
                        break;
                }
            }
            
            // se sono stati ricevuti nuovi valori dai sensori remoti scrivo i valori se pagina 1 selezionata         
            if (new_remote_sensors_values){
                if (sensor_page == 1)                                                   
                    Write_sensor_values(&remote_actual, &remote_min, &remote_max);     
                
                new_remote_sensors_values = 0;
            }

            // gestione touch screen pagina main
            XPT2046_enable();                                                           // per settare parametri corretti SPI (SD  impostata + veloce)
            if (XPT2046_IRQ == 0) {                                                     // se  stato toccato lo schermo

                switch (Touch_screen_manage_loop(MAIN_PAGE)){                           // leggo il punto toccato
                    case T_HOUR:                                                        // Orario toccato, aggiornamento sync DCF77
                        if (dcf77_user_enable){
                            dcf77_status = DCF77_ENABLED;                               // avvio ricezione DCF77
                            DCF77_start_receive_via_interrupt();
                            first_sync = 0;                                             // riporto a 0 per ricaricare i valori meteo se corretti e ricalcolare la fase lunare
                        }
                        break;
                    case T_METEO:                                                       // meteo toccato
                        meteo_page = !meteo_page;                                       // cambio pagina e
                        Draw_meteo(meteo_page);                                         // scrivo la nuova pagina su display
                        break;
                    case T_LOCAL_SENSOR:                                                // sensori locali
//                        TFT_set_font(font_lucida_sans_10x16);
                        TFT_write_string(SENSOR_TITLE_1_X,SENSOR_TITLE_Y,115,label[0],SAVESPACE_ON,LOCAL_SENS_P_LABEL_COLOR,SENSOR_FRAME_COLOR);    // "Sensori locali"
                        TFT_write_string(SENSOR_TITLE_2_X,SENSOR_TITLE_Y,62,label[1],SAVESPACE_ON,REMOTE_SENS_LABEL_COLOR,BACK_COLOR);              // "Remoto"
                        Write_sensor_values(&local_actual, &local_min, &local_max);
                        sensor_page = 0;
                        break;
                    case T_REMOTE_SENSOR:
//                        TFT_set_font(font_lucida_sans_10x16);
                        TFT_write_string(SENSOR_TITLE_1_X,SENSOR_TITLE_Y,115,label[0],SAVESPACE_ON,REMOTE_SENS_LABEL_COLOR,BACK_COLOR);             // "Sensori locali"
                        TFT_write_string(SENSOR_TITLE_2_X,SENSOR_TITLE_Y,62,label[1],SAVESPACE_ON,REMOTE_SENS_P_LABEL_COLOR,SENSOR_FRAME_COLOR);    // "Remoto"
                        Write_sensor_values(&remote_actual, &remote_min, &remote_max);
                        sensor_page = 1;
                        break;
                    case T_SW_1:                                                        // PAGINA LOG
                        Draw_switch (T_SW_1);
                        Draw_layout(LOG_PAGE);                                          // cancella solo e non disegna nulla
                        Draw_switch(SW_LOG_PAGE);
                        page = LOG_PAGE;
                        break;
                    case T_SW_2:
                        Draw_switch (T_SW_2);
                        Draw_layout(SETUP_1_PAGE);
                        page = SETUP_1_PAGE;
                        break;
                    case T_SW_3:                                                        // PAGINA STATISTICHE
                        Draw_switch (T_SW_3);
                        Draw_layout(STA_PAGE);
                        chart_2_plot = 0;
                        page = STA_PAGE;
                        break;
                    case T_SW_4:                                                        // ACCENSIONE E SPEGNIMENTO RETROILLUMINAZIONE
                        if (CCPR4L == 0) 
                            CCPR4L = ccpr4l;
                        else 
                            CCPR4L = 0;
                        
                        Delay_ms(200);
                        break;

                }
            }
        } 
          
        /* GESTIONE PAGINA SETUP 1 */
        else if (page == SETUP_1_PAGE){
            XPT2046_enable();                                                           // per settare parametri corretti SPI (SD  impostata + veloce)
            if (XPT2046_IRQ == 0) {                                                     // se  stato toccato lo schermo

                switch (Touch_screen_manage_loop(TOUCH_SETUP_MENU)){
                    case T_SETUP_R_1:
                        Setup_time_manual();                                            // DATA ORA MANUALE
                        Back_2(SETUP_1_PAGE);
                        break;
                    case T_SETUP_R_2:
                        Setup_utc();                                                    // FUSO ORARIO
                        Back_2(SETUP_1_PAGE);
                        break;
                    case T_SETUP_R_3:
                        Setup_meteo_region();                                           // REGIONE METEO
                        Back_2(SETUP_1_PAGE);
                        break;
//                    case T_SETUP_R_4:
//                        Setup_alarm();                                                  // ALLARMI
//                        break;       
                    case T_SETUP_R_5:
                        Setup_sensors();                                                // SENSORI
                        Back_2(SETUP_1_PAGE);
                        break;
                    case T_SETUP_R_6:
                        Setup_wifi();                                                   // WIFI
                        Back_2(SETUP_1_PAGE);
//                        if (record_log)
//                            TFT_draw_image_sd(LOG_ICON_X,LOG_ICON_Y,"LOG.bmp");
                        break;
                    case T_SETUP_R_7:
                        Setup_light();                                                  // LUMINOSITA
                        Back_2(SETUP_1_PAGE);                                                 
                        break;
//                    case T_SETUP_R_8:
//                        break;
                    case T_SETUP_R_9:
                        Draw_layout(SETUP_2_PAGE);                                      // CAMBIO PAGINA SETUP
//                        XPT2046_enable();
                        page = SETUP_2_PAGE;
                        break;
                    case T_SW_1:                                                        // PAGINA LOG
                        Draw_switch (T_SW_1);
                        Draw_layout(LOG_PAGE);
                        Draw_switch(SW_LOG_PAGE);
                        page = LOG_PAGE;
                        break;
                    case T_SW_2:                                                        // USCITA DA SETUP PER RITORNO IN MAIN
                        Draw_switch (0);
                        Draw_layout(MAIN_PAGE);
                        page = MAIN_PAGE;
                        break;
                    case T_SW_3:                                                        // PAGINA STATISTICHE
                        Draw_switch (T_SW_3);
                        Draw_layout(STA_PAGE);
                        chart_2_plot = 0;
                        page = STA_PAGE;
                        break;
//                    case T_SW_4:
//                        break;                        
                }
            }
            
        /* GESTIONE PAGINA SETUP 2 */
        } else if (page == SETUP_2_PAGE){
            XPT2046_enable();                                                           // per settare parametri corretti SPI (SD  impostata + veloce)
            if (XPT2046_IRQ == 0) {                                                     // se  stato toccato lo schermo

                switch (Touch_screen_manage_loop(TOUCH_SETUP_MENU)){
                    case T_SETUP_R_1:                                                   // CALIBRAZIONE TOUCH
                        Calibrate_touch();
                        Back_2(SETUP_2_PAGE);
                        break;
                    case T_SETUP_R_2:                                                   // CHECK HARDWARE
                        Hardware_check();
                        Back_2(SETUP_2_PAGE);
                        break;
                    case T_SETUP_R_3:                                                   // LOOP SENSORI
                        Test_sensors();
                        Back_2(SETUP_2_PAGE);
                        break;
                    case T_SETUP_R_4:                                                  
                        DCF77_check();
                        Back_2(SETUP_2_PAGE);
                        break;
                    case T_SETUP_R_6:                                                   // DISEGNO
                        Drawing();
                        Back_2(SETUP_2_PAGE);
                        break;
                    case T_SETUP_R_9:
                        Draw_layout(SETUP_1_PAGE);                                      // CAMBIO PAGINA SETUP
                        page = SETUP_1_PAGE;
                        break;
                    case T_SW_1:                                                        // PAGINA LOG
                        Draw_switch (T_SW_1);
                        Draw_layout(LOG_PAGE);
                        Draw_switch(SW_LOG_PAGE);
                        page = LOG_PAGE;
                        break;
                    case T_SW_2:                                                        // USCITA DA SETUP PER RITORNO IN MAIN
                        Draw_switch (0);            
                        Draw_layout(MAIN_PAGE);
                        page = MAIN_PAGE;
                        break;
                    case T_SW_3:                                                        // PAGINA STATISTICHE
                        Draw_switch (T_SW_3);
                        Draw_layout(STA_PAGE);
                        chart_2_plot = 0;
                        page = STA_PAGE;
                        break;
//                    case T_SW_4:
//                        break;                      
                    
                }
            }
        
        /* GESTIONE PAGINA LOG */    
        } else if (page == LOG_PAGE){
            
            no_plot_chart = 0;
            XPT2046_enable();                                                           // per settare parametri corretti SPI (SD  impostata + veloce)
            if (XPT2046_IRQ == 0) {                                                     // se  stato toccato lo schermo

                switch (Touch_screen_manage_loop(TOUCH_LOG)){

                    case T_SW_1:                                                        // ritorno in pagina main
                        Draw_switch (0);                    
                        Draw_layout(MAIN_PAGE);
                        chart_2_plot = 0;
                        page = MAIN_PAGE;
                        break;
                    case T_SW_2:                                                        // PAGINA SETUP
                        Draw_switch (T_SW_2);
                        Draw_layout(SETUP_1_PAGE);
                        chart_2_plot = 0;
                        page = SETUP_1_PAGE;
                        break;
                    case T_SW_3:                                                        // PAGINA STATISTICHE
                        Draw_switch (T_SW_3);
                        Draw_layout(STA_PAGE);
                        chart_2_plot = 0;
                        page = STA_PAGE;
                        break;
//                    case T_SW_4:
//                        if (CCPR4L == 0) 
//                            CCPR4L = ccpr4l;
//                        else 
//                            CCPR4L = 0;
//                        
//                        Delay_ms(200);
//                        break;
                    case T_SW_5:                                                        // AVVIO SALVATAGGIO DATI SENSORI
                        Draw_switch (T_SW_5);
                        record_log = 1;
                        new_log = 1;
                        no_plot_chart = 1;
                        TFT_draw_image_sd(LOG_ICON_X,LOG_ICON_Y,"LOG.bmp");
                        break;
                    case T_SW_6:                                                        // STOP SALVATAGGIO DATI SENSORI
                        Draw_switch (T_SW_6);
                        record_log = 0;
                        no_plot_chart = 1;
                        TFT_fill_rect(LOG_ICON_X,LOG_ICON_Y,13,15,BACK_COLOR);
                        break;
                    case T_SW_7:                                                        // RESTART LOG SENSORI MANTENENDO ANCHE I VECCHI DATI
                        Draw_switch (T_SW_7);
                        record_log = 1;
                        no_plot_chart = 1;
                        TFT_draw_image_sd(LOG_ICON_X,LOG_ICON_Y,"LOG.bmp");
                        break;
                   case T_SW_8:                                                         // ESPORTAZIONE CSV
                        Draw_switch (T_SW_8);
                        no_plot_chart = 1;
                        Export_sensors_log("EXPORT.csv");
                        Draw_switch (T_SW_8);
                        break;
                   case T_SW_9:                                                         // SENSORI LOCALI
                        Draw_switch (T_SW_9);
                        chart_2_plot &= 0x0F;                                           // cancello precedente byte dati locali/remoto
                        chart_2_plot |= 0x10;                                           // aggiungo byte dati locali
                        break;
                   case T_SW_10:                                                        // SENSORI REMOTI
                        Draw_switch (T_SW_10);
                        chart_2_plot &= 0x0F;                                           // cancello precedente byte dati locali/remoto
                        chart_2_plot |= 0x20;                                           // aggiungo byte dati remoti
                        break;
                   case T_SW_11:                                                        // TEMPERATURA
                        Draw_switch (T_SW_11);
                        chart_2_plot &= 0xF0;                                           // cancello precedente byte tipo di dati
                        chart_2_plot |= 0x01;                                           // aggiungo byte tipo di dati
                        break;
                   case T_SW_12:                                                        // UMIDITA
                        Draw_switch (T_SW_12);
                        chart_2_plot &= 0xF0;                                           // cancello precedente byte tipo di dati
                        chart_2_plot |= 0x02;                                           // aggiungo byte tipo di dati
                        break;
                   case T_SW_13:                                                        // PRESSIONE
                        Draw_switch (T_SW_13);
                        chart_2_plot &= 0xF0;                                           // cancello precedente byte tipo di dati
                        chart_2_plot |= 0x03;                                           // aggiungo byte tipo di dati
                        break;
                   case T_SW_14:                                                        // UVA
                        Draw_switch (T_SW_14);
                        chart_2_plot &= 0xF0;                                           // cancello precedente byte tipo di dati
                        chart_2_plot |= 0x04;                                           // aggiungo byte tipo di dati
                        break;
                   case T_SW_15:                                                        // UVB
                        Draw_switch (T_SW_15);
                        chart_2_plot &= 0xF0;                                           // cancello precedente byte tipo di dati
                        chart_2_plot |= 0x05;                                           // aggiungo byte tipo di dati
                        break;
                   case T_SW_16:                                                        // UVI
                        Draw_switch (T_SW_16);
                        chart_2_plot &= 0xF0;                                           // cancello precedente byte tipo di dati
                        chart_2_plot |= 0x06;                                           // aggiungo byte tipo di dati
                        break;   
                    case T_ARROW_NEXT:                                                  // PROSSIMI DATI
                        if (log_next_allowed)
                            chart_2_plot = 0x30;
                        else 
                            no_plot_chart = 1;
                        break;
                    case T_ARROW_PREV:                                                  // DATI PRECEDENTI
                        if (log_prev_allowed)
                            chart_2_plot = 0x40;
                        else 
                            no_plot_chart = 1;
                        break;
                    default: no_plot_chart = 1;
                }
                // Disegno il grafico solo se c' una combinazione valida e se il display  stato toccato in uno dei punti stabiliti
                if (((chart_2_plot >= 0x11 && chart_2_plot <= 0x16) || (chart_2_plot >= 0x21 && chart_2_plot <= 0x26) || chart_2_plot == 0x30 || chart_2_plot == 0x40) && no_plot_chart == 0)
                    chart_2_plot = Plot_log_chart(chart_2_plot);                                                                                             
            }
            
        /* GESTIONE PAGINA STATISTICHE */    
        } else if (page == STA_PAGE){
        
            // scrittura dei valori delle variabili statistiche
            TFT_write_PBCD(STAT_VALUE_1_X, STAT_ITEM_1_Y, 22, rtcc_time_start.day, SAVESPACE_OFF, STAT_ITEM_COLOR, STAT_BK_COLOR);
            TFT_write_char(STAT_VALUE_1_X+22,STAT_ITEM_1_Y,'/',SAVESPACE_ON,STAT_ITEM_COLOR,STAT_BK_COLOR);
            TFT_write_PBCD(STAT_VALUE_1_X+32,STAT_ITEM_1_Y,22,rtcc_time_start.month,SAVESPACE_OFF,STAT_ITEM_COLOR,STAT_BK_COLOR);
            TFT_write_char(STAT_VALUE_1_X+54,STAT_ITEM_1_Y,'/',SAVESPACE_ON,STAT_ITEM_COLOR,STAT_BK_COLOR);
            TFT_write_PBCD(STAT_VALUE_1_X+64,STAT_ITEM_1_Y,22,rtcc_time_start.year,SAVESPACE_OFF,STAT_ITEM_COLOR,STAT_BK_COLOR);
            TFT_write_PBCD(STAT_VALUE_1_X+90,STAT_ITEM_1_Y,22,rtcc_time_start.hours,SAVESPACE_OFF,STAT_ITEM_COLOR,STAT_BK_COLOR);
            TFT_write_char(STAT_VALUE_1_X+112,STAT_ITEM_1_Y,':',SAVESPACE_ON,STAT_ITEM_COLOR,STAT_BK_COLOR);
            TFT_write_PBCD(STAT_VALUE_1_X+116,STAT_ITEM_1_Y,22,rtcc_time_start.minutes,SAVESPACE_OFF,STAT_ITEM_COLOR,STAT_BK_COLOR);
                
            TFT_write_unsint(STAT_VALUE_2_X, STAT_ITEM_2_Y, 70, dcf77_success_sync_counter, 5 ,ZERO_CLEANING_ON, SAVESPACE_ON,STAT_ITEM_COLOR, STAT_BK_COLOR);
            TFT_write_unsint(STAT_VALUE_3_X, STAT_ITEM_3_Y, 70, dcf77_fail_sync_counter, 5 ,ZERO_CLEANING_ON, SAVESPACE_ON,STAT_ITEM_COLOR, STAT_BK_COLOR);
            TFT_write_unsint(STAT_VALUE_4_X, STAT_ITEM_4_Y, 70, meteo_success_sync_counter, 5 ,ZERO_CLEANING_ON, SAVESPACE_ON,STAT_ITEM_COLOR, STAT_BK_COLOR);
            TFT_write_unsint(STAT_VALUE_5_X, STAT_ITEM_5_Y, 70, meteo_fail_sync_counter, 5 ,ZERO_CLEANING_ON, SAVESPACE_ON,STAT_ITEM_COLOR, STAT_BK_COLOR);
            TFT_write_unsint(STAT_VALUE_6_X, STAT_ITEM_6_Y, 70, wifi_incoming_con_counter, 5 ,ZERO_CLEANING_ON, SAVESPACE_ON,STAT_ITEM_COLOR, STAT_BK_COLOR);
            TFT_write_unsint(STAT_VALUE_7_X, STAT_ITEM_7_Y, 70, wifi_real_time_send_counter, 5 ,ZERO_CLEANING_ON, SAVESPACE_ON,STAT_ITEM_COLOR, STAT_BK_COLOR);
            TFT_write_unsint(STAT_VALUE_8_X, STAT_ITEM_8_Y, 70, wifi_log_send_counter, 5 ,ZERO_CLEANING_ON, SAVESPACE_ON,STAT_ITEM_COLOR, STAT_BK_COLOR);
            TFT_write_unsint(STAT_VALUE_9_X, STAT_ITEM_9_Y, 70, remote_discon_counter, 5 ,ZERO_CLEANING_ON, SAVESPACE_ON,STAT_ITEM_COLOR, STAT_BK_COLOR);
            
            XPT2046_enable();                                                           // per settare parametri corretti SPI (SD  impostata + veloce)
            if (XPT2046_IRQ == 0) {                                                     // se  stato toccato lo schermo

                switch (Touch_screen_manage_loop(TOUCH_STA)){

                    case T_SW_1:                                                        // PAGINA LOG
                        Draw_switch (T_SW_1);
                        Draw_layout(LOG_PAGE);
                        Draw_switch(SW_LOG_PAGE);
                        page = LOG_PAGE;
                        break;
                    case T_SW_2:                                                        // PAGINA SETUP
                        Draw_switch (T_SW_2);
                        Draw_layout(SETUP_1_PAGE);
                        chart_2_plot = 0;
                        page = SETUP_1_PAGE;
                        break;
                    case T_SW_3:                                                        // ritorno in pagina MAIN
                        Draw_switch (0);                    
                        Draw_layout(MAIN_PAGE);
                        chart_2_plot = 0;
                        page = MAIN_PAGE;
                        break;
                    case T_SW_4:
                        if (CCPR4L == 0) 
                            CCPR4L = ccpr4l;
                        else 
                            CCPR4L = 0;
                        
                        Delay_ms(200);
                        break; 
                }
            }
        }                     
    }
} 

/* ============================================================================================================================================= /
 * FUNCTION:    Back_to
 * Input:       _page: pagina da mostrare
 *
 * Overview:    Mostra su display la pagina setup di ritorno da menu setup interni. Riscrive icone di stato che potrebbero essere state cancellate
/* ============================================================================================================================================ */
void Back_2 (unsigned char _page){
    Draw_switch(SW_INIT_4);
    Draw_switch(T_SW_2 );
    Draw_layout(_page);
    
    Wifi_check_con(1);
    if (wifi_trans_icon_drawn)
        TFT_draw_image_sd(TRAN_ICON_X,TRAN_ICON_Y,"TRAN.bmp");
    if (record_log)
        TFT_draw_image_sd(LOG_ICON_X,LOG_ICON_Y,"LOG.bmp");
}

/* ============================================================================================================================================= /
 * FUNCTION:    Draw_layout
 * Input:       _page: pagina da disegnare (unsigned char)
 *
 * Overview:    Disegna il layout statico delle pagine
/* ============================================================================================================================================ */
void Draw_layout(unsigned char _page){
//    TFT_clear(BACK_COLOR);

    TFT_fill_rect(1,17,240,280, BACK_COLOR);                                            // cancello tutto tranne le icone in alto e i pulsanti in basso
    TFT_fill_rect(TIME_X,TIME_Y,TIME_X+90,TIME_Y+50, BACK_COLOR);
    
    /* PAGINA MAIN */ 
    if (_page == MAIN_PAGE){
        
        rtcc_time = RTCC_read_all();                                                    // leggo l'orario dal RTCC
        Update_time();
        Draw_meteo(meteo_page);                                                         // scrivo il meteo su display e luna (tutto --)
        
        // griglia sensori
        TFT_draw_rect(SENSORS_X, SENSORS_Y, SENSORS_W, SENSOR_H, SENSOR_FRAME_COLOR);
        TFT_draw_rect(SENSORS_X+1, SENSORS_Y+1, SENSORS_W-2, SENSOR_H-2, SENSOR_FRAME_COLOR);
        TFT_draw_line_x(SENSORS_X,SENSORS_X+SENSORS_W, SENSORS_Y+18,SENSOR_FRAME_COLOR);  
        TFT_draw_line_x(SENSORS_X,SENSORS_X+SENSORS_W, SENSORS_Y+19,SENSOR_FRAME_COLOR);
        
        // nomi sensori
//        TFT_set_font(font_lucida_sans_10x16);
        TFT_write_string(SENSOR_TITLE_1_X,SENSOR_TITLE_Y,115,label[0],SAVESPACE_ON,LOCAL_SENS_P_LABEL_COLOR,SENSOR_FRAME_COLOR);    // "Sensori locali"
        TFT_write_string(SENSOR_TITLE_2_X,SENSOR_TITLE_Y,62,label[1],SAVESPACE_ON,REMOTE_SENS_LABEL_COLOR,BACK_COLOR);              // "Remoto"

        TFT_write_string(SENSORS_LABEL_X,SENSORS_R1_Y,40,label[2],SAVESPACE_ON,TEMP_LABEL_COLOR,TEMP_LABEL_BK_COLOR);               // "Temp."
        TFT_write_char(SENSORS_LABEL_X+43,SENSORS_R1_Y,127,SAVESPACE_ON,TEMP_LABEL_COLOR,TEMP_LABEL_BK_COLOR); 
        TFT_write_char(SENSORS_LABEL_X+48,SENSORS_R1_Y,'C',SAVESPACE_ON,TEMP_LABEL_COLOR,TEMP_LABEL_BK_COLOR); 
        TFT_write_string(SENSORS_LABEL_X,SENSORS_R2_Y,60,label[3],SAVESPACE_ON,RH_LABEL_COLOR,RH_LABEL_BK_COLOR);                   // "RH       %"
        TFT_write_string(SENSORS_LABEL_X,SENSORS_R3_Y,60,label[4],SAVESPACE_ON,PRES_LABEL_COLOR,PRES_LABEL_BK_COLOR);               // "Pre. hPa"
        TFT_write_string(SENSORS_LABEL_X,SENSORS_R4_Y,60,label[5],SAVESPACE_ON,UVA_LABEL_COLOR,UVA_LABEL_BK_COLOR);                 // "UVA mW"
        TFT_write_string(SENSORS_LABEL_X,SENSORS_R5_Y,60,label[6],SAVESPACE_ON,UVB_LABEL_COLOR,UVB_LABEL_BK_COLOR);                 // "UVA cm"    
        TFT_write_string(SENSORS_LABEL_X,SENSORS_R6_Y,60,label[7],SAVESPACE_ON,UVI_LABEL_COLOR,UVI_LABEL_BK_COLOR);                 // "UVI"
        
        TFT_draw_line_x (SENSORS_LABEL_X+33,SENSORS_LABEL_X+60,SENSORS_R5_Y,UVA_LABEL_COLOR);
        
        sensor_page = 0;                                                                // ripristino pagina sensori iniziale
        new_local_sensors_values = 1;                                                   // forzo riscrittura valori sensori locali
        new_remote_sensors_values = 1;
        rem_con_icon_drawn = 0;        
    
        if(dcf77_user_enable == 0)
            TFT_draw_image_sd(DCF77_ICON_X,DCF77_ICON_Y,"DCF_dis.bmp");
    }                                                                                                                                                                                                     
    
    /* PAGINA SETUP 1 */ 
    else if (_page == SETUP_1_PAGE){
        // icone
        TFT_draw_image_sd(SETUP_ICON_1_X,SETUP_ICON_1_Y,"TIME.bmp");
        TFT_draw_image_sd(SETUP_ICON_3_X,SETUP_ICON_2_Y,"UTC.bmp");
        TFT_draw_image_sd(SETUP_ICON_4_X,SETUP_ICON_3_Y,"EARTH.bmp");
//        TFT_draw_image_sd(SETUP_ICON_5_X,SETUP_ICON_4_Y,"ALARM.bmp");
        TFT_draw_image_sd(SETUP_ICON_6_X,SETUP_ICON_5_Y,"SENS.bmp");
        TFT_draw_image_sd(SETUP_ICON_7_X,SETUP_ICON_6_Y + 3,"WIFI_S.bmp");
        TFT_draw_image_sd(SETUP_ICON_7_X,SETUP_ICON_7_Y,"ILL.bmp");
//        TFT_draw_image_sd(SETUP_ICON_8_X,SETUP_ICON_8_Y,".bmp");

        // scritte menu
//        TFT_set_font(font_lucida_sans_10x16);
        TFT_write_string(SETUP_ITEM_1_X,SETUP_ITEM_1_Y,200,setup_items_1[0],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // "DATA/ORA MANUALE"
        TFT_write_string(SETUP_ITEM_2_X,SETUP_ITEM_2_Y,200,setup_items_1[1],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // "FUSO ORARIO"
        TFT_write_string(SETUP_ITEM_3_X,SETUP_ITEM_3_Y,200,setup_items_1[2],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // "REGIONE METEO"
//        TFT_write_string(SETUP_ITEM_4_X,SETUP_ITEM_4_Y,200,setup_items_1[3],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // "ALLARMI"
        TFT_write_string(SETUP_ITEM_5_X,SETUP_ITEM_5_Y,200,setup_items_1[4],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // "SENSORI"
        TFT_write_string(SETUP_ITEM_6_X,SETUP_ITEM_6_Y,200,setup_items_1[5],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // "WIFI"
        TFT_write_string(SETUP_ITEM_7_X,SETUP_ITEM_7_Y,200,setup_items_1[6],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // "LUMINOSITA"
//        TFT_write_string(SETUP_ITEM_8_X,SETUP_ITEM_8_Y,200,setup_items_1[7],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // ""
        TFT_write_string(SETUP_ITEM_9_X,SETUP_ITEM_9_Y,200,setup_items_1[8],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // "pagina 2 ->"
        
        // linee di separazione
        TFT_draw_line_x(SETUP_LINE_1_X, SETUP_LINE_X_END, SETUP_LINE_1_Y, SETUP_LINE_COLOR);
        TFT_draw_line_x(SETUP_LINE_2_X, SETUP_LINE_X_END, SETUP_LINE_2_Y, SETUP_LINE_COLOR);
        TFT_draw_line_x(SETUP_LINE_3_X, SETUP_LINE_X_END, SETUP_LINE_3_Y, SETUP_LINE_COLOR);
        TFT_draw_line_x(SETUP_LINE_4_X, SETUP_LINE_X_END, SETUP_LINE_4_Y, SETUP_LINE_COLOR);
        TFT_draw_line_x(SETUP_LINE_5_X, SETUP_LINE_X_END, SETUP_LINE_5_Y, SETUP_LINE_COLOR);
        TFT_draw_line_x(SETUP_LINE_6_X, SETUP_LINE_X_END, SETUP_LINE_6_Y, SETUP_LINE_COLOR);
        TFT_draw_line_x(SETUP_LINE_7_X, SETUP_LINE_X_END, SETUP_LINE_7_Y, SETUP_LINE_COLOR);
//        TFT_draw_line_x(SETUP_LINE_8_X, SETUP_LINE_X_END, SETUP_LINE_8_Y, SETUP_LINE_COLOR);
        TFT_draw_line_x(SETUP_LINE_9_X, SETUP_LINE_X_END, SETUP_LINE_9_Y, SETUP_LINE_COLOR);
    }
    /* PAGINA SETUP 2 */ 
    else if (_page == SETUP_2_PAGE){
        // icone
        TFT_draw_image_sd(SETUP_ICON_1_X + 3,SETUP_ICON_1_Y,"TOUCH.bmp");
        TFT_draw_image_sd(SETUP_ICON_2_X,SETUP_ICON_2_Y,"HW.bmp");
        TFT_draw_image_sd(SETUP_ICON_3_X,SETUP_ICON_3_Y,"LOOP.bmp");
        TFT_draw_image_sd(SETUP_ICON_4_X,SETUP_ICON_4_Y,"DCF_on.bmp");
//        TFT_draw_image_sd(SETUP_ICON_5_X,SETUP_ICON_5_Y,".bmp");
//        TFT_draw_image_sd(SETUP_ICON_6_X,SETUP_ICON_6_Y,".bmp");
//        TFT_draw_image_sd(SETUP_ICON_7_X,SETUP_ICON_7_Y," .bmp");
//        TFT_draw_image_sd(SETUP_ICON_8_X,SETUP_ICON_8_Y,".bmp");
    
        // scritte menu
//        TFT_set_font(font_lucida_sans_10x16);
        TFT_write_string(SETUP_ITEM_1_X,SETUP_ITEM_1_Y,200,setup_items_2[0],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // "CALIBRAZIONE TOUCH"
        TFT_write_string(SETUP_ITEM_2_X,SETUP_ITEM_2_Y,200,setup_items_2[1],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // "TEST HARDWARE"
        TFT_write_string(SETUP_ITEM_3_X,SETUP_ITEM_3_Y,200,setup_items_2[2],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // "LOOP SENSORI"
        TFT_write_string(SETUP_ITEM_4_X,SETUP_ITEM_4_Y,200,setup_items_2[3],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // "TEST DCF77" 
//        TFT_write_string(SETUP_ITEM_5_X,SETUP_ITEM_5_Y,200,setup_items_2[4],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // 
//        TFT_write_string(SETUP_ITEM_6_X,SETUP_ITEM_6_Y,200,setup_items_2[5],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // 
//        TFT_write_string(SETUP_ITEM_7_X,SETUP_ITEM_7_Y,200,setup_items_2[6],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // 
//        TFT_write_string(SETUP_ITEM_8_X,SETUP_ITEM_8_Y,200,setup_items_2[7],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // 
        TFT_write_string(SETUP_ITEM_9_X,SETUP_ITEM_9_Y,200,setup_items_2[8],SAVESPACE_ON,SETUP_ITEM_COLOR,SETUP_BK_COLOR);            // "<- pagina 1"
        
        // linee di separazione
        TFT_draw_line_x(SETUP_LINE_1_X, SETUP_LINE_X_END, SETUP_LINE_1_Y, SETUP_LINE_COLOR);
        TFT_draw_line_x(SETUP_LINE_2_X, SETUP_LINE_X_END, SETUP_LINE_2_Y, SETUP_LINE_COLOR);
        TFT_draw_line_x(SETUP_LINE_3_X, SETUP_LINE_X_END, SETUP_LINE_3_Y, SETUP_LINE_COLOR);
        TFT_draw_line_x(SETUP_LINE_4_X, SETUP_LINE_X_END, SETUP_LINE_4_Y, SETUP_LINE_COLOR);
//        TFT_draw_line_x(SETUP_LINE_5_X, SETUP_LINE_X_END, SETUP_LINE_5_Y, SETUP_LINE_COLOR);
//        TFT_draw_line_x(SETUP_LINE_6_X, SETUP_LINE_X_END, SETUP_LINE_6_Y, SETUP_LINE_COLOR);
//        TFT_draw_line_x(SETUP_LINE_7_X, SETUP_LINE_X_END, SETUP_LINE_7_Y, SETUP_LINE_COLOR);
//        TFT_draw_line_x(SETUP_LINE_8_X, SETUP_LINE_X_END, SETUP_LINE_8_Y, SETUP_LINE_COLOR);
        TFT_draw_line_x(SETUP_LINE_9_X, SETUP_LINE_X_END, SETUP_LINE_9_Y, SETUP_LINE_COLOR);
    }
    
    else if (_page == STA_PAGE){
        // linee di separazione
        TFT_draw_line_x(STAT_LINE_1_X, STAT_LINE_X_END, STAT_LINE_1_Y, STAT_LINE_COLOR);
        TFT_draw_line_x(STAT_LINE_2_X, STAT_LINE_X_END, STAT_LINE_2_Y, STAT_LINE_COLOR);
        TFT_draw_line_x(STAT_LINE_3_X, STAT_LINE_X_END, STAT_LINE_3_Y, STAT_LINE_COLOR);
        TFT_draw_line_x(STAT_LINE_4_X, STAT_LINE_X_END, STAT_LINE_4_Y, STAT_LINE_COLOR);
        TFT_draw_line_x(STAT_LINE_5_X, STAT_LINE_X_END, STAT_LINE_5_Y, STAT_LINE_COLOR);
        TFT_draw_line_x(STAT_LINE_6_X, STAT_LINE_X_END, STAT_LINE_6_Y, STAT_LINE_COLOR);
        TFT_draw_line_x(STAT_LINE_7_X, STAT_LINE_X_END, STAT_LINE_7_Y, STAT_LINE_COLOR);
        TFT_draw_line_x(STAT_LINE_8_X, STAT_LINE_X_END, STAT_LINE_8_Y, STAT_LINE_COLOR);
        TFT_draw_line_x(STAT_LINE_9_X, STAT_LINE_X_END, STAT_LINE_9_Y, STAT_LINE_COLOR);
        
        // scritte statiche
//        TFT_set_font(font_lucida_sans_10x16);
        TFT_write_string(STAT_ITEM_1_X,STAT_ITEM_1_Y,200,stat_items[0],SAVESPACE_ON,STAT_ITEM_COLOR,STAT_BK_COLOR);            // "Attivo da: "
        TFT_write_string(STAT_ITEM_2_X,STAT_ITEM_2_Y,200,stat_items[1],SAVESPACE_ON,STAT_ITEM_COLOR,STAT_BK_COLOR);            // "DCF77 validi: "
        TFT_write_string(STAT_ITEM_3_X,STAT_ITEM_3_Y,200,stat_items[2],SAVESPACE_ON,STAT_ITEM_COLOR,STAT_BK_COLOR);            // "DCF77 invalidi: "
        TFT_write_string(STAT_ITEM_4_X,STAT_ITEM_4_Y,200,stat_items[3],SAVESPACE_ON,STAT_ITEM_COLOR,STAT_BK_COLOR);            // "Meteo validi: "
        TFT_write_string(STAT_ITEM_5_X,STAT_ITEM_5_Y,200,stat_items[4],SAVESPACE_ON,STAT_ITEM_COLOR,STAT_BK_COLOR);            // "Meteo invalidi: "
        TFT_write_string(STAT_ITEM_6_X,STAT_ITEM_6_Y,200,stat_items[5],SAVESPACE_ON,STAT_ITEM_COLOR,STAT_BK_COLOR);            // "Connessioni avvenute: "
        TFT_write_string(STAT_ITEM_7_X,STAT_ITEM_7_Y,200,stat_items[6],SAVESPACE_ON,STAT_ITEM_COLOR,STAT_BK_COLOR);            // "Invio RT: "
        TFT_write_string(STAT_ITEM_8_X,STAT_ITEM_8_Y,200,stat_items[7],SAVESPACE_ON,STAT_ITEM_COLOR,STAT_BK_COLOR);            // "Invio LOG: "
        TFT_write_string(STAT_ITEM_9_X,STAT_ITEM_9_Y,200,stat_items[8],SAVESPACE_ON,STAT_ITEM_COLOR,STAT_BK_COLOR);            // "Discon. remoto: "
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Draw_switch
 * Input:       _pressed: numero del pulsante premuto 0 = nessun pulsante premuto (unsigned char)
 *
 * Overview:    Disegna i pulsanti premuti o non premuti
/* ============================================================================================================================================ */
void Draw_switch(unsigned char _pressed){
    static unsigned char _prev_pressed = 0;
    TFT_set_font(font_segoe_20x24);
    
    // inizializzazioni: disegno i rettangoli vuoti
    switch (_pressed){
        case SW_INIT_4:
            TFT_draw_image_sd(SW_1_X,SW_1_Y,"SW.bmp");    
            TFT_draw_image_sd(SW_2_X,SW_2_Y,"SW.bmp");
            TFT_draw_image_sd(SW_3_X,SW_3_Y,"SW.bmp");
            TFT_draw_image_sd(SW_4_X,SW_4_Y,"SW.bmp");
            break;
        case SW_SETUP_INCR:
            TFT_draw_image_sd(SW_1_X,SW_1_Y,"SW.bmp");    
            TFT_draw_image_sd(SW_2_X,SW_2_Y,"SW.bmp");
            TFT_draw_image_sd(SW_3_X,SW_3_Y,"SW.bmp");
            TFT_draw_image_sd(SW_4_X,SW_4_Y,"SW.bmp");
            break; 
        case SW_SETUP_OK_EXT:
            TFT_fill_rect(SW_1_X,SW_1_Y,120,25, BACK_COLOR);
            TFT_draw_image_sd(SW_3_X,SW_3_Y,"SW.bmp");
            TFT_draw_image_sd(SW_4_X,SW_4_Y,"SW.bmp");
            break;
        case SW_RESC_ESC:
            TFT_fill_rect(SW_1_X,SW_1_Y,120,25, BACK_COLOR);
            TFT_draw_image_sd(SW_3_X,SW_3_Y,"SW.bmp");
            TFT_draw_image_sd(SW_4_X,SW_4_Y,"SW.bmp");
            break;            
        case SW_LOG_PAGE:
            TFT_draw_image_sd(SW_5_X,SW_5_Y,"SW_P.bmp");    
            TFT_draw_image_sd(SW_6_X,SW_6_Y,"SW_P.bmp");
            TFT_draw_image_sd(SW_7_X,SW_7_Y,"SW.bmp");
            TFT_draw_image_sd(SW_8_X,SW_8_Y,"SW_Y.bmp");
            TFT_draw_image_sd(SW_9_X,SW_9_Y,"SW_G.bmp");    
            TFT_draw_image_sd(SW_10_X,SW_10_Y,"SW_G.bmp");
            TFT_draw_image_sd(SW_11_X,SW_11_Y,"SW_LG.bmp");
            TFT_draw_image_sd(SW_12_X,SW_12_Y,"SW_LG.bmp");
            TFT_draw_image_sd(SW_13_X,SW_13_Y,"SW_LG.bmp");    
            TFT_draw_image_sd(SW_14_X,SW_14_Y,"SW_LG.bmp");
            TFT_draw_image_sd(SW_15_X,SW_15_Y,"SW_LG.bmp");
            TFT_draw_image_sd(SW_16_X,SW_16_Y,"SW_LG.bmp");
            break; 
    }

    //
    if (_pressed <= T_SW_4 || _pressed == SW_INIT_4){  // da 0 a 4 e SW_INIT
        // scrivo sempre le scritte come non premute
        TFT_write_string(SW_1_TEXT_X,SW_1_TEXT_Y,43,sw_label[0],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR);
        TFT_write_string(SW_2_TEXT_X,SW_2_TEXT_Y,40,sw_label[1],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR);
        TFT_write_string(SW_3_TEXT_X,SW_3_TEXT_Y,42,sw_label[2],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR);
        TFT_write_string(SW_4_TEXT_X,SW_4_TEXT_Y,30,sw_label[3],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR);
        
        // Scrivo il tasto premuto
        switch (_pressed){
            case T_SW_1:
                TFT_write_string(SW_1_TEXT_X,SW_1_TEXT_Y,40,sw_label[0],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_BACK_COLOR);
                break;
            case T_SW_2:
                TFT_write_string(SW_2_TEXT_X,SW_2_TEXT_Y,40,sw_label[1],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_BACK_COLOR);
                break;
            case T_SW_3:
                TFT_write_string(SW_3_TEXT_X,SW_3_TEXT_Y,42,sw_label[2],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_BACK_COLOR);
                break;
//            case T_SW_4: 
//                TFT_write_string(SW_4_TEXT_X,SW_4_TEXT_Y,30,sw_label[3],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_BACK_COLOR);
//                break;
        }
        
    // pulsanti per impostazioni incrementali    
    } else if (_pressed == SW_SETUP_INCR){
        TFT_write_string(SW_1_TEXT_X +12,SW_1_TEXT_Y,20,sw_label[15],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR); // "+"
        TFT_write_string(SW_2_TEXT_X +12,SW_2_TEXT_Y,20,sw_label[16],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR); // "-"
        TFT_write_string(SW_3_TEXT_X +5,SW_3_TEXT_Y,30,sw_label[17],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR);  // "OK"
        TFT_write_string(SW_4_TEXT_X -8,SW_4_TEXT_Y,40,sw_label[18],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR);  // "ESC"
    
    // pulsanti ok exit
    } else if (_pressed == SW_SETUP_OK_EXT){
        TFT_write_string(SW_3_TEXT_X +5,SW_3_TEXT_Y,30,sw_label[17],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR);  // "OK"
        TFT_write_string(SW_4_TEXT_X -8,SW_4_TEXT_Y,40,sw_label[18],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR);  // "ESC"
        
    } else if (_pressed == SW_RESC_ESC) {
        TFT_write_string(SW_3_TEXT_X +2,SW_3_TEXT_Y,40,sw_label[19],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR);  // "RSC"
        TFT_write_string(SW_4_TEXT_X -8,SW_4_TEXT_Y,40,sw_label[18],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR);  // "ESC"
    // scritte pulsanti log iniziali

    }else if (_pressed == SW_LOG_PAGE){
        TFT_write_string(SW_5_TEXT_X,SW_5_TEXT_Y,40,sw_label[4],SAVESPACE_ON,SW_TEXT_COLOR,SW_P_BACK_COLOR); // "REC"
        TFT_write_string(SW_6_TEXT_X,SW_6_TEXT_Y,45,sw_label[5],SAVESPACE_ON,SW_TEXT_COLOR,SW_P_BACK_COLOR); // "STO"
        TFT_write_string(SW_7_TEXT_X,SW_7_TEXT_Y,45,sw_label[20],SAVESPACE_ON,SW_TEXT_COLOR,SW_P_BACK_COLOR); // "CON"
        TFT_write_string(SW_8_TEXT_X,SW_8_TEXT_Y,40,sw_label[6],SAVESPACE_ON,SW_TEXT_COLOR,SW_Y_BACK_COLOR); // "EXP"
        TFT_write_string(SW_9_TEXT_X,SW_9_TEXT_Y,40,sw_label[7],SAVESPACE_ON,SW_TEXT_COLOR,SW_G_BACK_COLOR); // "LOC"
        TFT_write_string(SW_10_TEXT_X,SW_10_TEXT_Y,45,sw_label[8],SAVESPACE_ON,SW_TEXT_COLOR,SW_G_BACK_COLOR); // "REM"
        TFT_write_string(SW_11_TEXT_X,SW_11_TEXT_Y,40,sw_label[9],SAVESPACE_ON,SW_TEXT_COLOR,SW_LG_BACK_COLOR); // "TEM"
        TFT_write_string(SW_12_TEXT_X,SW_12_TEXT_Y,35,sw_label[10],SAVESPACE_ON,SW_TEXT_COLOR,SW_LG_BACK_COLOR); // "RH"
        TFT_write_string(SW_13_TEXT_X,SW_13_TEXT_Y,40,sw_label[11],SAVESPACE_ON,SW_TEXT_COLOR,SW_LG_BACK_COLOR); // "PRE"
        TFT_write_string(SW_14_TEXT_X,SW_14_TEXT_Y,45,sw_label[12],SAVESPACE_ON,SW_TEXT_COLOR,SW_LG_BACK_COLOR); // "UVA"
        TFT_write_string(SW_15_TEXT_X,SW_15_TEXT_Y,40,sw_label[13],SAVESPACE_ON,SW_TEXT_COLOR,SW_LG_BACK_COLOR); // "UVB"
        TFT_write_string(SW_16_TEXT_X,SW_16_TEXT_Y,40,sw_label[14],SAVESPACE_ON,SW_TEXT_COLOR,SW_LG_BACK_COLOR); // "UVI"
    
    }else if (_pressed >= T_SW_11 && _pressed <= T_SW_16){
        TFT_write_string(SW_11_TEXT_X,SW_11_TEXT_Y,40,sw_label[9],SAVESPACE_ON,SW_TEXT_COLOR,SW_LG_BACK_COLOR); // "TEM"
        TFT_write_string(SW_12_TEXT_X,SW_12_TEXT_Y,35,sw_label[10],SAVESPACE_ON,SW_TEXT_COLOR,SW_LG_BACK_COLOR); // "RH"
        TFT_write_string(SW_13_TEXT_X,SW_13_TEXT_Y,40,sw_label[11],SAVESPACE_ON,SW_TEXT_COLOR,SW_LG_BACK_COLOR); // "PRE"
        TFT_write_string(SW_14_TEXT_X,SW_14_TEXT_Y,45,sw_label[12],SAVESPACE_ON,SW_TEXT_COLOR,SW_LG_BACK_COLOR); // "UVA"
        TFT_write_string(SW_15_TEXT_X,SW_15_TEXT_Y,40,sw_label[13],SAVESPACE_ON,SW_TEXT_COLOR,SW_LG_BACK_COLOR); // "UVB"
        TFT_write_string(SW_16_TEXT_X,SW_16_TEXT_Y,40,sw_label[14],SAVESPACE_ON,SW_TEXT_COLOR,SW_LG_BACK_COLOR); // "UVI"
     switch(_pressed){
            case T_SW_11: TFT_write_string(SW_11_TEXT_X,SW_11_TEXT_Y,40,sw_label[9],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_LG_BACK_COLOR); // "TEM"
                          break;
            case T_SW_12: TFT_write_string(SW_12_TEXT_X,SW_12_TEXT_Y,35,sw_label[10],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_LG_BACK_COLOR); // "RH"
                          break;
            case T_SW_13: TFT_write_string(SW_13_TEXT_X,SW_13_TEXT_Y,40,sw_label[11],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_LG_BACK_COLOR); // "PRE"
                          break;
            case T_SW_14: TFT_write_string(SW_14_TEXT_X,SW_14_TEXT_Y,45,sw_label[12],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_LG_BACK_COLOR); // "UVA"
                          break;
            case T_SW_15: TFT_write_string(SW_15_TEXT_X,SW_15_TEXT_Y,40,sw_label[13],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_LG_BACK_COLOR); // "UVB"
                          break;
            case T_SW_16: TFT_write_string(SW_16_TEXT_X,SW_16_TEXT_Y,40,sw_label[14],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_LG_BACK_COLOR); // "UVI"
                          break;
        }
    }
    
    switch(_pressed){
        case T_SW_PLUS:
            TFT_write_string(SW_1_TEXT_X +12,SW_1_TEXT_Y,20,sw_label[15],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_BACK_COLOR); // "+"
            Delay_ms(100);
            TFT_write_string(SW_1_TEXT_X +12,SW_1_TEXT_Y,20,sw_label[15],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR); // "+"
            break;
        case T_SW_LESS:
            TFT_write_string(SW_2_TEXT_X +12,SW_2_TEXT_Y,20,sw_label[16],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_BACK_COLOR); // "-"
            Delay_ms(100);
            TFT_write_string(SW_2_TEXT_X +12,SW_2_TEXT_Y,20,sw_label[16],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR); // "-"
            break;
        case T_SW_OK:
            TFT_write_string(SW_3_TEXT_X +5,SW_3_TEXT_Y,30,sw_label[17],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_BACK_COLOR);  // "OK"
            Delay_ms(100);
            TFT_write_string(SW_3_TEXT_X +5,SW_3_TEXT_Y,30,sw_label[17],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR);  // "OK"
            break;
        case T_SW_EXIT:
            TFT_write_string(SW_4_TEXT_X -8,SW_4_TEXT_Y,40,sw_label[18],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_BACK_COLOR);  // "ESC"
            Delay_ms(100);
            TFT_write_string(SW_4_TEXT_X -8,SW_4_TEXT_Y,40,sw_label[18],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR);  // "ESC"
            break;
        case T_SW_RSC:
            TFT_write_string(SW_3_TEXT_X +2,SW_3_TEXT_Y,40,sw_label[19],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_BACK_COLOR);  // "RSC"
            Delay_ms(100);
            TFT_write_string(SW_3_TEXT_X +2,SW_3_TEXT_Y,40,sw_label[19],SAVESPACE_ON,SW_TEXT_COLOR,SW_BACK_COLOR);  // "RSC"
            break;                      
        case T_SW_5:
            TFT_write_string(SW_5_TEXT_X,SW_5_TEXT_Y,40,sw_label[4],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_P_BACK_COLOR); // "REC"
            Delay_ms(100);
            TFT_write_string(SW_5_TEXT_X,SW_5_TEXT_Y,40,sw_label[4],SAVESPACE_ON,SW_TEXT_COLOR,SW_P_BACK_COLOR); // "REC"
            break;
        case T_SW_6:
            TFT_write_string(SW_6_TEXT_X,SW_6_TEXT_Y,45,sw_label[5],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_P_BACK_COLOR); // "STO"
            Delay_ms(100);
            TFT_write_string(SW_6_TEXT_X,SW_6_TEXT_Y,45,sw_label[5],SAVESPACE_ON,SW_TEXT_COLOR,SW_P_BACK_COLOR); // "STO"
            break;
        case T_SW_7:
            TFT_write_string(SW_7_TEXT_X,SW_7_TEXT_Y,45,sw_label[20],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_P_BACK_COLOR); // "CON"
            Delay_ms(100);
            TFT_write_string(SW_7_TEXT_X,SW_7_TEXT_Y,45,sw_label[20],SAVESPACE_ON,SW_TEXT_COLOR,SW_P_BACK_COLOR); // "CON"
            break;
        case T_SW_8:
            if (_prev_pressed != T_SW_8){
                TFT_write_string(SW_8_TEXT_X,SW_8_TEXT_Y,40,sw_label[6],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_Y_BACK_COLOR); // "EXP"
                _prev_pressed = T_SW_8;
            } else {
                TFT_write_string(SW_8_TEXT_X,SW_8_TEXT_Y,40,sw_label[6],SAVESPACE_ON,SW_TEXT_COLOR,SW_Y_BACK_COLOR); // "EXP"
                _prev_pressed = 0;
            }
            break;
        case T_SW_9: 
            TFT_write_string(SW_9_TEXT_X,SW_9_TEXT_Y,40,sw_label[7],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_G_BACK_COLOR); // "LOC"
            TFT_write_string(SW_10_TEXT_X,SW_10_TEXT_Y,45,sw_label[8],SAVESPACE_ON,SW_TEXT_COLOR,SW_G_BACK_COLOR); // "REM"
            break;
        case T_SW_10: 
            TFT_write_string(SW_9_TEXT_X,SW_9_TEXT_Y,40,sw_label[7],SAVESPACE_ON,SW_TEXT_COLOR,SW_G_BACK_COLOR); // "LOC"
            TFT_write_string(SW_10_TEXT_X,SW_10_TEXT_Y,45,sw_label[8],SAVESPACE_ON,SW_P_TEXT_COLOR,SW_G_BACK_COLOR); // "REM"
            break;                 
    }
    TFT_set_font(font_lucida_sans_10x16);                                               // ripristino font principale
} 

/* ============================================================================================================================================= /
 * FUNCTION:    Draw_meteo
 * Input:       _meteo_page: pagina meteo da disegnare
 *
 * Overview:    Disegna il meteo a seconda della pagina selezionata
/* ============================================================================================================================================ */
void Draw_meteo(unsigned char _meteo_page){
    unsigned char _day_w = 0;

//    TFT_set_font(font_lucida_sans_10x16);
    TFT_fill_rect(1,METEO_ICON_D1_Y,240,SENSORS_Y-METEO_ICON_D1_Y,BACK_COLOR);

    // giorno attuale
    if (_meteo_page == 0){                      
        
        switch (meteo_fc_day_1.day_fc){                                                 // icona meteo previsioni giorno
            case 1: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_D_1.bmp");
                    break;
            case 2: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_D_2.bmp");
                    break;
            case 3: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_3.bmp");
                    break;
            case 4: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_4.bmp");
                    break;
            case 5: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_D_5.bmp");
                    break;
            case 6: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_6.bmp");
                    break;
            case 7: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_7.bmp");
                    break;
            case 8: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_D_8.bmp");
                    break;
            case 9: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_9.bmp");
                    break;
            case 10: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_D_10.bmp");
                    break;
            case 11: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_11.bmp");
                    break;
            case 12: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_D_12.bmp");
                    break;
            case 13: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_13.bmp");
                    break;
            case 14: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_14.bmp");
                    break;
            case 15: TFT_draw_image_sd(METEO_ICON_D1_X,METEO_ICON_D1_Y,"M_D_15.bmp");
                    break;
        }
        
        switch (meteo_fc_day_1.night_fc){                                               // icona meteo previsioni notte
            case 1: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_N_1.bmp");
                    break;
            case 2: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_N_2.bmp");
                    break;
            case 3: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_3.bmp");
                    break;
            case 4: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_4.bmp");
                    break;
            case 5: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_N_5.bmp");
                    break;
            case 6: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_6.bmp");
                    break;
            case 7: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_7.bmp");
                    break;
            case 8: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_N_8.bmp");
                    break;
            case 9: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_9.bmp");
                    break;
            case 10: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_N_10.bmp");
                    break;
            case 11: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_11.bmp");
                    break;
            case 12: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_N_12.bmp");
                    break;
            case 13: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_13.bmp");
                    break;
            case 14: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_14.bmp");
                    break;
            case 15: TFT_draw_image_sd(METEO_ICON_N1_X,METEO_ICON_N1_Y,"M_N_15.bmp");
                    break;
        }
        
        switch (meteo_fc_day_1.extreme_w){                                              // flag allerta meteo
            case 1: TFT_draw_image_sd(EXTREME_ICON_D1_X, EXTREME_ICON_D1_Y,"EXTREME.bmp");
                    TFT_draw_image_sd(EXTREME_ICON_N1_X, EXTREME_ICON_N1_Y,"EXTREME.bmp");
                break;
            case 2: TFT_draw_image_sd(EXTREME_ICON_D1_X, EXTREME_ICON_D1_Y,"EXTREME.bmp");
                break;
            case 3: TFT_draw_image_sd(EXTREME_ICON_N1_X, EXTREME_ICON_N1_Y,"EXTREME.bmp");
                break;
        }
        
        TFT_draw_image_sd(TERM_D1_ICON_X,TERM_D1_ICON_Y,"TERM.bmp");                    // disegno le icone restanti
        TFT_draw_image_sd(RAIN_D1_ICON_X,RAIN_D1_ICON_Y,"RAIN.bmp");
        TFT_draw_image_sd(WIND_ICON_X,WIND_ICON_Y,"WIND.bmp");
        
        // temperatura
        if (meteo_fc_day_1.temp_min == 0x7F)                                            // se non  ancora presente una previsione scrivo --
            TFT_write_string(METEO_TEMP_D1_X,METEO_TEMP_D1_Y,18,"--",SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR);
        else
            TFT_write_sint(METEO_TEMP_D1_X,METEO_TEMP_D1_Y,28,meteo_fc_day_1.temp_min,2, ZERO_CLEANING_ON,SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR);
                
        if (meteo_fc_day_1.temp_max == 0x7F)
            TFT_write_string(METEO_TEMP_D1_X+37,METEO_TEMP_D1_Y,18,"--",SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR);
        else 
            TFT_write_sint(METEO_TEMP_D1_X+37,METEO_TEMP_D1_Y,28,meteo_fc_day_1.temp_max,2, ZERO_CLEANING_ON,SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR);
        
        TFT_write_char(METEO_TEMP_D1_X+27,METEO_TEMP_D1_Y,'/',SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR); 
        TFT_write_char(METEO_TEMP_D1_X+65,METEO_TEMP_D1_Y,127,SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR); 
        TFT_write_char(METEO_TEMP_D1_X+71,METEO_TEMP_D1_Y,'C',SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR);     
            
        // probabilit pioggia
        if (meteo_fc_day_1.rain_pr == 0x7F)
            TFT_write_string(METEO_RAIN_PR_D1_X,METEO_RAIN_PR_D1_Y,50,"--",SAVESPACE_ON,METEO_RAIN_PR_COLOR,BACK_COLOR);
        else
            TFT_write_string(METEO_RAIN_PR_D1_X,METEO_RAIN_PR_D1_Y,50,hkw581_rain_pr[meteo_fc_day_1.rain_pr],SAVESPACE_ON,METEO_RAIN_PR_COLOR,BACK_COLOR);
        
        // vento
        if (meteo_fc_day_1.wind_dir == 0x7F)
            TFT_write_string(METEO_WIND_DIR_X,METEO_WIND_DIR_Y,102,"--",SAVESPACE_ON,METEO_WIND_COLOR,BACK_COLOR);
        else                                                                                                                                                    
            TFT_write_string(METEO_WIND_DIR_X,METEO_WIND_DIR_Y,102,hkw581_wind_dir[meteo_fc_day_1.wind_dir],SAVESPACE_ON,METEO_WIND_COLOR,BACK_COLOR);
        
        if (meteo_fc_day_1.wind_str == 0x7F)
            TFT_write_string(METEO_WIND_SPEED_X,METEO_WIND_SPEED_Y,100,"--",SAVESPACE_ON,METEO_WIND_COLOR,BACK_COLOR);
        else
            TFT_write_string(METEO_WIND_SPEED_X,METEO_WIND_SPEED_Y,100,hkw581_wind_str[meteo_fc_day_1.wind_str],SAVESPACE_ON,METEO_WIND_COLOR,BACK_COLOR);
        
        // ora aggiornamento
        if ((meteo_fc_day_1.day_fc != 0) || (meteo_fc_day_2.day_fc != 0) ||             // se ho ricevuto un qualsiasi dato meteo aggiorno l'ora di ricezione
            (meteo_fc_day_3.day_fc != 0) || (meteo_fc_day_4.day_fc != 0)){        
            TFT_draw_image_sd(M_UPD_ICON_X,M_UPD_ICON_Y,"M_UPD.bmp");
            TFT_write_PBCD(METEO_UPD_X,METEO_UPD_Y,50,meteo_update_time.day,SAVESPACE_ON,METEO_UPD_COLOR,BACK_COLOR);
            TFT_write_char(METEO_UPD_X+18,METEO_UPD_Y,'/',SAVESPACE_ON,METEO_UPD_COLOR,BACK_COLOR); 
            TFT_write_PBCD(METEO_UPD_X+28,METEO_UPD_Y,50,meteo_update_time.month,SAVESPACE_ON,METEO_UPD_COLOR,BACK_COLOR);

            TFT_write_PBCD(METEO_UPD_X+50,METEO_UPD_Y,50,meteo_update_time.hours,SAVESPACE_ON,METEO_UPD_COLOR,BACK_COLOR);
            TFT_write_char(METEO_UPD_X+70,METEO_UPD_Y,':',SAVESPACE_ON,METEO_UPD_COLOR,BACK_COLOR); 
            TFT_write_PBCD(METEO_UPD_X+76,METEO_UPD_Y,50,meteo_update_time.minutes,SAVESPACE_ON,METEO_UPD_COLOR,BACK_COLOR);
        }

        // Calcolo e disegno la fase lunare
        Draw_moon_phase();

    // 2 giorni seguenti        
    }else{
        TFT_draw_line_y(METEO_ICON_D2_Y, SENSORS_Y-4, LINE_METEO_1_X, LINE_METEO_COLOR);
        
        // giorno 2
        _day_w = rtcc_time.day_w + 1;                                                   // incremento giorno settimana (per nome giorno)

        if (_day_w == 7)
            _day_w = 0;

        switch (meteo_fc_day_2.day_fc){
            case 1: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_D_1.bmp");
                    break;
            case 2: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_D_2.bmp");
                    break;
            case 3: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_3.bmp");
                    break;
            case 4: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_4.bmp");
                    break;
            case 5: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_D_5.bmp");
                    break;
            case 6: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_6.bmp");
                    break;
            case 7: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_7.bmp");
                    break;
            case 8: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_D_8.bmp");
                    break;
            case 9: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_9.bmp");
                    break;
            case 10: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_D_10.bmp");
                    break;
            case 11: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_11.bmp");
                    break;
            case 12: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_D_12.bmp");
                    break;
            case 13: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_13.bmp");
                    break;
            case 14: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_14.bmp");
                    break;
            case 15: TFT_draw_image_sd(METEO_ICON_D2_X,METEO_ICON_D2_Y,"M_D_15.bmp");
                    break;
        }
        
        switch (meteo_fc_day_2.night_fc){
            case 1: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_N_1.bmp");
                    break;
            case 2: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_N_2.bmp");
                    break;
            case 3: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_3.bmp");
                    break;
            case 4: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_4.bmp");
                    break;
            case 5: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_N_5.bmp");
                    break;
            case 6: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_6.bmp");
                    break;
            case 7: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_7.bmp");
                    break;
            case 8: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_N_8.bmp");
                    break;
            case 9: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_9.bmp");
                    break;
            case 10: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_N_10.bmp");
                    break;
            case 11: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_11.bmp");
                    break;
            case 12: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_N_12.bmp");
                    break;
            case 13: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_13.bmp");
                    break;
            case 14: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_14.bmp");
                    break;
            case 15: TFT_draw_image_sd(METEO_ICON_N2_X,METEO_ICON_N2_Y,"M_N_15.bmp");
                    break;
        }
        
        switch (meteo_fc_day_2.extreme_w){
            case 1: TFT_draw_image_sd(EXTREME_ICON_D2_X, EXTREME_ICON_D2_Y,"EXTREME.bmp");
                    TFT_draw_image_sd(EXTREME_ICON_N2_X, EXTREME_ICON_N2_Y,"EXTREME.bmp");
                break;
            case 2: TFT_draw_image_sd(EXTREME_ICON_D2_X, EXTREME_ICON_D2_Y,"EXTREME.bmp");
                break;
            case 3: TFT_draw_image_sd(EXTREME_ICON_N2_X, EXTREME_ICON_N2_Y,"EXTREME.bmp");
                break;
        }
            
        TFT_draw_image_sd(TERM_D2_ICON_X,TERM_D2_ICON_Y,"TERM.bmp");
        TFT_draw_image_sd(RAIN_D2_ICON_X,RAIN_D2_ICON_Y,"RAIN.bmp");
        
        // temperatura
        if (meteo_fc_day_2.temp_min == 0x7F)
            TFT_write_string(METEO_TEMP_D2_X,METEO_TEMP_D2_Y,18,"--",SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR);
        else
            TFT_write_sint(METEO_TEMP_D2_X,METEO_TEMP_D2_Y,28,meteo_fc_day_2.temp_min,2, ZERO_CLEANING_ON,SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR);
                
        if (meteo_fc_day_2.temp_max == 0x7F)
            TFT_write_string(METEO_TEMP_D2_X+37,METEO_TEMP_D2_Y,28,"--",SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR);
        else 
            TFT_write_sint(METEO_TEMP_D2_X+37,METEO_TEMP_D2_Y,18,meteo_fc_day_2.temp_max,2, ZERO_CLEANING_ON,SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR);

        TFT_write_char(METEO_TEMP_D2_X+27,METEO_TEMP_D2_Y,'/',SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR); 
        TFT_write_char(METEO_TEMP_D2_X+65,METEO_TEMP_D2_Y,127,SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR); 
        TFT_write_char(METEO_TEMP_D2_X+71,METEO_TEMP_D2_Y,'C',SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR); 
        
        // probabilit pioggia
        if (meteo_fc_day_2.rain_pr == 0x7F)
            TFT_write_string(METEO_RAIN_PR_D2_X,METEO_RAIN_PR_D2_Y,50,"--",SAVESPACE_ON,METEO_RAIN_PR_COLOR,BACK_COLOR);
        else
            TFT_write_string(METEO_RAIN_PR_D2_X,METEO_RAIN_PR_D2_Y,50,hkw581_rain_pr[meteo_fc_day_2.rain_pr],SAVESPACE_ON,METEO_RAIN_PR_COLOR,BACK_COLOR);
                
        // nome giorno se almeno una previsione
        if (meteo_fc_day_2.day_fc != 0)
            TFT_write_string(METEO_DAY_D2_X,METEO_DAY_D2_Y,80,name_day[_day_w],SAVESPACE_ON,METEO_NAME_COLOR,BACK_COLOR);   
        
        // giorno 3
       _day_w = _day_w + 1;                                                             // incremento giorno settimana (per nome giorno)

        if (_day_w == 7)
            _day_w = 0;

        switch (meteo_fc_day_3.day_fc){
            case 1: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_D_1.bmp");
                    break;
            case 2: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_D_3.bmp");
                    break;
            case 3: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_3.bmp");
                    break;
            case 4: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_4.bmp");
                    break;
            case 5: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_D_5.bmp");
                    break;
            case 6: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_6.bmp");
                    break;
            case 7: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_7.bmp");
                    break;
            case 8: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_D_8.bmp");
                    break;
            case 9: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_9.bmp");
                    break;
            case 10: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_D_10.bmp");
                    break;
            case 11: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_11.bmp");
                    break;
            case 12: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_D_12.bmp");
                    break;
            case 13: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_13.bmp");
                    break;
            case 14: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_14.bmp");
                    break;
            case 15: TFT_draw_image_sd(METEO_ICON_D3_X,METEO_ICON_D3_Y,"M_D_15.bmp");
                    break;
        }
        
        switch (meteo_fc_day_3.night_fc){
            case 1: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_N_1.bmp");
                    break;
            case 2: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_N_3.bmp");
                    break;
            case 3: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_3.bmp");
                    break;
            case 4: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_4.bmp");
                    break;
            case 5: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_N_5.bmp");
                    break;
            case 6: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_6.bmp");
                    break;
            case 7: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_7.bmp");
                    break;
            case 8: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_N_8.bmp");
                    break;
            case 9: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_9.bmp");
                    break;
            case 10: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_N_10.bmp");
                    break;
            case 11: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_11.bmp");
                    break;
            case 12: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_N_12.bmp");
                    break;
            case 13: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_13.bmp");
                    break;
            case 14: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_14.bmp");
                    break;
            case 15: TFT_draw_image_sd(METEO_ICON_N3_X,METEO_ICON_N3_Y,"M_N_15.bmp");
                    break;
        }
        
        switch (meteo_fc_day_3.extreme_w){
            case 1: TFT_draw_image_sd(EXTREME_ICON_D3_X, EXTREME_ICON_D3_Y,"EXTREME.bmp");
                    TFT_draw_image_sd(EXTREME_ICON_N3_X, EXTREME_ICON_N3_Y,"EXTREME.bmp");
                break;
            case 2: TFT_draw_image_sd(EXTREME_ICON_D3_X, EXTREME_ICON_D3_Y,"EXTREME.bmp");
                break;
            case 3: TFT_draw_image_sd(EXTREME_ICON_N3_X, EXTREME_ICON_N3_Y,"EXTREME.bmp");
                break;
        }
            
        TFT_draw_image_sd(TERM_D3_ICON_X,TERM_D3_ICON_Y,"TERM.bmp");
        TFT_draw_image_sd(RAIN_D3_ICON_X,RAIN_D3_ICON_Y,"RAIN.bmp");
        
        // temperatura
        if (meteo_fc_day_3.temp_min == 0x7F)
            TFT_write_string(METEO_TEMP_D3_X,METEO_TEMP_D3_Y,18,"--",SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR);
        else
            TFT_write_sint(METEO_TEMP_D3_X,METEO_TEMP_D3_Y,28,meteo_fc_day_3.temp_min,2, ZERO_CLEANING_ON,SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR);
                
        if (meteo_fc_day_3.temp_max == 0x7F)
            TFT_write_string(METEO_TEMP_D3_X+37,METEO_TEMP_D3_Y,18,"--",SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR);
        else 
            TFT_write_sint(METEO_TEMP_D3_X+37,METEO_TEMP_D3_Y,28,meteo_fc_day_3.temp_max,2, ZERO_CLEANING_ON,SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR);

        TFT_write_char(METEO_TEMP_D3_X+27,METEO_TEMP_D3_Y,'/',SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR); 
        TFT_write_char(METEO_TEMP_D3_X+65,METEO_TEMP_D3_Y,127,SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR); 
        TFT_write_char(METEO_TEMP_D3_X+71,METEO_TEMP_D3_Y,'C',SAVESPACE_ON,METEO_TEMP_COLOR,BACK_COLOR); 
        
        // probabilit pioggia
        if (meteo_fc_day_3.rain_pr == 0x7F)
            TFT_write_string(METEO_RAIN_PR_D3_X,METEO_RAIN_PR_D3_Y,50,"--",SAVESPACE_ON,METEO_RAIN_PR_COLOR,BACK_COLOR);
        else
            TFT_write_string(METEO_RAIN_PR_D3_X,METEO_RAIN_PR_D3_Y,50,hkw581_rain_pr[meteo_fc_day_3.rain_pr],SAVESPACE_ON,METEO_RAIN_PR_COLOR,BACK_COLOR);
                
        // nome giorno se almeno una previsione                
        if (meteo_fc_day_3.day_fc != 0)
            TFT_write_string(METEO_DAY_D3_X,METEO_DAY_D3_Y,80,name_day[_day_w],SAVESPACE_ON,METEO_NAME_COLOR,BACK_COLOR);   
    }
}

/* ============================================================================================================================================= / 
 * FUNCTION:    Draw_moon_phase                                                                                                                         
 *                                                                                                                                                 
 * Overview:    Calcola la fase lunare e disegna la relativa icona          
 * Note:        Algoritmo trovato in rete http://www.voidware.com/moon_phase.htm
/* ============================================================================================================================================ */ 
void Draw_moon_phase(void){
    unsigned char _day = 0;
    unsigned char _month = 0;
    unsigned int _year = 0;
    volatile unsigned char _moon_phase = 0;
    
    // converto i valori BCD in decimali
    _day = Bcd_2_dec(rtcc_time.day);
    _month = Bcd_2_dec(rtcc_time.month);
    _year = Bcd_2_dec(rtcc_time.year); // ho tolto il  +2000 
    
    float _c,_e,_b;
    int _c_int, _e_int;
    float _jd;
    
    if (_month < 3) {
        _year--;
        _month += 12;
    }
    ++_month;
    _c = 365.25*_year;  // sarebbe 365.25*2017, ma ho tolto il 2000 e aggiungo (365.25 * 2000) direttamente al calcolo di _jd
    _c_int = (int) _c;
    _e = 30.6*_month;
    _e_int = (int) _e;
    _jd = _c_int+_e_int+_day+36460.91;     /* jd is total days elapsed */  // sarebbe  _c + _d + _day -694039.09 ma _c  calcolato con anno 17 e non 2017 quindi devo sommare anche 365.25*2000 = 730500 730500-394039.09 = +36460.91
    _jd /= 29.53;                   /* divide by the moon cycle (29.53 days) */
    _b = (int)_jd;                  /* int(jd) -> b, take integer part of jd */
    _jd -= _b;                       /* subtract integer part to leave fractional part of original jd */
    _b = _jd*8 + 0.5;               /* scale fraction from 0-8 and round by adding 0.5 */
    _moon_phase = ((unsigned int) _b) & 7;		   /* 0 and 8 are the same so turn 8 into 0 */
    
    
    // disegno relativa icona
    switch (_moon_phase){
        case 0: TFT_draw_image_sd(MOON_ICON_X, MOON_ICON_Y,"MOON_0.bmp");
            break;
        case 1: TFT_draw_image_sd(MOON_ICON_X, MOON_ICON_Y,"MOON_1.bmp");
            break;
        case 2: TFT_draw_image_sd(MOON_ICON_X, MOON_ICON_Y,"MOON_2.bmp");
            break;
        case 3: TFT_draw_image_sd(MOON_ICON_X, MOON_ICON_Y,"MOON_3.bmp");
            break;
        case 4: TFT_draw_image_sd(MOON_ICON_X, MOON_ICON_Y,"MOON_4.bmp");
            break;
        case 5: TFT_draw_image_sd(MOON_ICON_X, MOON_ICON_Y,"MOON_5.bmp");
            break;
        case 6: TFT_draw_image_sd(MOON_ICON_X, MOON_ICON_Y,"MOON_6.bmp");
            break;
        case 7: TFT_draw_image_sd(MOON_ICON_X, MOON_ICON_Y,"MOON_7.bmp");
            break;            
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Update_time
 *
 * Overview:    Legge data e ora da RTCC, salva nella variabile globale rtcc_time, scrive data e ora su display
/* ============================================================================================================================================ */
void Update_time(void){
    
//    rtcc_time = RTCC_read_all();                                                        // leggo l'orario dal RTCC     

    // scrittura ore e minuti
    TFT_set_font(font_dense_23x56);
    TFT_write_PBCD(HOUR_X,HOUR_Y,50,rtcc_time.hours,SAVESPACE_OFF,HOUR_COLOR,BACK_COLOR);
    TFT_write_PBCD(MINUTE_X,MINUTE_Y,50,rtcc_time.minutes,SAVESPACE_OFF,MINUTE_COLOR,BACK_COLOR);
    TFT_write_char(DOT_X,DOT_Y,':',SAVESPACE_ON,DOT_COLOR,BACK_COLOR);
    
    // scrittura giorno della settimana
    TFT_set_font(font_segoe_20x24);
    TFT_write_string(DAY_W_X, DAY_W_Y, 93, name_day[rtcc_time.day_w], SAVESPACE_ON, DAY_W_COLOR, BACK_COLOR);
    
    // scrittura data
    TFT_set_font(font_segoe_10x15);
    TFT_write_PBCD(DAY_X,DAY_Y,22,rtcc_time.day,SAVESPACE_OFF,DAY_COLOR,BACK_COLOR);
    TFT_write_char(SEP_1_X,SEP_1_Y,'/',SAVESPACE_ON,SEP_1_COLOR,BACK_COLOR);
    TFT_write_PBCD(MONTH_X,MONTH_Y,22,rtcc_time.month,SAVESPACE_OFF,MONTH_COLOR,BACK_COLOR);
    TFT_write_char(SEP_2_X,SEP_2_Y,'/',SAVESPACE_ON,SEP_2_COLOR,BACK_COLOR);
    TFT_write_PBCD(YEAR_X,YEAR_Y,22,rtcc_time.year,SAVESPACE_OFF,YEAR_COLOR,BACK_COLOR);
    
    TFT_set_font(font_lucida_sans_10x16);                                               // ripristino font principale
}

/* ============================================================================================================================================= /
 * FUNCTION:    Get_local_sensors_values
 *
 * Overview:    Legge i sensori, calcola i minimi e i massimi e salva i valori nelle variabili globali
/* ============================================================================================================================================ */
void Get_local_sensors_values(void){
    static unsigned int _sensors_first_read = 1;

//    lettura valori raw sensori locali
    sensors_local_raw_values.temp_hdc1080_raw =  HDC1080_read_temp_raw();               // leggo temperatura raw
    sensors_local_raw_values.humidity_raw = HDC1080_read_humidity_raw();                // leggo umindit raw

    LPS25HB_power_mode(LPS25HB_DEV_ADDR,LSP25HB_ACTIVE);                                // attivo il sensore LPS25HB
    LPS25HB_shot(LPS25HB_DEV_ADDR);

    sensors_local_raw_values.press_raw = LPS25HB_read_press_raw(LPS25HB_DEV_ADDR);      // leggo pressione e temperatura
    sensors_local_raw_values.temp_lps25_raw = LPS25HB_read_temp_raw(LPS25HB_DEV_ADDR);
    
    LPS25HB_power_mode(LPS25HB_DEV_ADDR,LSP25HB_POWER_DOWN);                            // disattivo il sensore
    
    VEML6075_power_mode(VEML6075_ACTIVE);                                               // attivo il sensore VEML6075
    VEML6075_shot();
    Delay_ms(200);                                                                      // attesa lettura e conversione
    
    sensors_local_raw_values.uv_readed_raw = VEML6075_read_uv_raw();
    
    VEML6075_power_mode(VEML6075_POWER_DOWN);     
    
    if (record_log)
        Save_sensors_log(& sensors_local_raw_values, LOCAL);                                         //salvo valori raw dei sensori
        
    
    // conversione da raw a float e offset
    local_actual.temp_hdc1080 = HDC1080_temp_raw_2_float(sensors_local_raw_values.temp_hdc1080_raw) + HDC1080_TEMP_CAL;
    local_actual.humidity = HDC1080_humidity_raw_2_float(sensors_local_raw_values.humidity_raw) + HDC1080_HUMID_CAL;  
    local_actual.press = LPS25HB_press_raw_2_float(sensors_local_raw_values.press_raw);                    
    local_actual.temp_lps25 = LPS25HB_temp_raw_2_float(sensors_local_raw_values.temp_lps25_raw) + LPS25HB_TEMP_CAL; 
    local_actual.uv_readed = VEML6075_convert_uv_raw(sensors_local_raw_values.uv_readed_raw);
    
    // converto da uW/cm^2 a mW/cm^2  
    local_actual.uv_readed.uva = local_actual.uv_readed.uva/1000;
    local_actual.uv_readed.uvb = local_actual.uv_readed.uvb/1000;   
    
    #if defined DEBUG_SENSOR_ANOMALY && defined DEBUG_LOG_SD
    if (local_actual.temp_hdc1080 > 100 || local_actual.humidity > 100 || 
        local_actual.press > 1200 || local_actual.temp_lps25 > 100 ||
        local_actual.uv_readed.uva > 50 || local_actual.uv_readed.uvb > 50 ||
        local_actual.uv_readed.uvi > 20)     
        Write_log(SENSOR_ANOMALY_LOC);
    #endif
    
    // primo salvataggio massimi minimi
    if (_sensors_first_read){
        local_max = local_actual;
        local_min = local_actual;
        _sensors_first_read = 0;
    } else {

        // check massimi
        if (local_actual.temp_hdc1080 > local_max.temp_hdc1080)
            local_max.temp_hdc1080 = local_actual.temp_hdc1080;
        if (local_actual.humidity > local_max.humidity)
            local_max.humidity = local_actual.humidity;
        if (local_actual.press > local_max.press)
            local_max.press = local_actual.press;
        if (local_actual.temp_lps25 > local_max.temp_lps25)
            local_max.temp_lps25 = local_actual.temp_lps25;
        if (local_actual.uv_readed.uva > local_max.uv_readed.uva)
            local_max.uv_readed.uva = local_actual.uv_readed.uva;
        if (local_actual.uv_readed.uvb > local_max.uv_readed.uvb)
            local_max.uv_readed.uvb = local_actual.uv_readed.uvb;
        if (local_actual.uv_readed.uvi > local_max.uv_readed.uvi)
            local_max.uv_readed.uvi = local_actual.uv_readed.uvi;

        // check minimi
        if (local_actual.temp_hdc1080 < local_min.temp_hdc1080)
            local_min.temp_hdc1080  = local_actual.temp_hdc1080;
        if (local_actual.humidity < local_min.humidity)
            local_min.humidity  = local_actual.humidity;
        if (local_actual.press < local_min.press)
            local_min.press  = local_actual.press;
        if (local_actual.temp_lps25 < local_min.temp_lps25)
            local_min.temp_lps25  = local_actual.temp_lps25;
        if (local_actual.uv_readed.uva < local_min.uv_readed.uva)
            local_min.uv_readed.uva  = local_actual.uv_readed.uva;
        if (local_actual.uv_readed.uvb < local_min.uv_readed.uvb)
            local_min.uv_readed.uvb  = local_actual.uv_readed.uvb;
        if (local_actual.uv_readed.uvi < local_min.uv_readed.uvi)
            local_min.uv_readed.uvi  = local_actual.uv_readed.uvi;   
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Get_remote_sensors_values
 *
 * Overview:    Converte i valori ricevuti dal modulo remoto e salvati in HMTRP_buffer in valori reali e li salva nella struct con variabili globali.
 *              calcola inoltre i valori minimi e massimi e li salva in variabili globali
/* ============================================================================================================================================ */
void Get_remote_sensors_values(void){
//    sensors_raw_values _raw_values;
//    veml6075_uv_raw uv_readed_raw;
    static unsigned char _sensors_first_read = 1;
    
    if (hmtrp_buffer[0] == 0xFA){                                                       // primo valore per invio dati sensori 0xFA
        // raccolgo dati ricevuti in struct raw
        sensors_remote_raw_values.temp_hdc1080_raw = ((unsigned int)hmtrp_buffer[1] << 8) | hmtrp_buffer[2];
        sensors_remote_raw_values.humidity_raw = ((unsigned int)hmtrp_buffer[3] << 8) | hmtrp_buffer[4];
        sensors_remote_raw_values.press_raw = ((unsigned long)hmtrp_buffer[5] << 24) | ((unsigned long)hmtrp_buffer[6] << 16) | ((unsigned long) hmtrp_buffer[7] << 8) |  hmtrp_buffer[8];
        sensors_remote_raw_values.temp_lps25_raw = ((unsigned int)hmtrp_buffer[9] << 8) | hmtrp_buffer[10];
        sensors_remote_raw_values.uv_readed_raw.uva = ((unsigned int)hmtrp_buffer[11] << 8) |  hmtrp_buffer[12]; 
        sensors_remote_raw_values.uv_readed_raw.uvb = ((unsigned int)hmtrp_buffer[13] << 8) |  hmtrp_buffer[14];
        sensors_remote_raw_values.uv_readed_raw.uv_comp_1 = ((unsigned int)hmtrp_buffer[15] << 8) |  hmtrp_buffer[16];
        sensors_remote_raw_values.uv_readed_raw.uv_comp_2 = ((unsigned int)hmtrp_buffer[17] << 8) |  hmtrp_buffer[18];
                
        if (record_log)
            Save_sensors_log(& sensors_remote_raw_values, REMOTE);                                    //salvo valori raw dei sensori
             
        // conversione dati ricevuti da raw a float
        remote_actual.temp_hdc1080 = HDC1080_temp_raw_2_float(sensors_remote_raw_values.temp_hdc1080_raw) + HDC1080_REM_TEMP_CAL;
        remote_actual.humidity = HDC1080_humidity_raw_2_float(sensors_remote_raw_values.humidity_raw) + HDC1080_REM_HUMID_CAL;
        remote_actual.press = LPS25HB_press_raw_2_float(sensors_remote_raw_values.press_raw);
        remote_actual.temp_lps25 = LPS25HB_temp_raw_2_float(sensors_remote_raw_values.temp_lps25_raw) + LPS25HB_REM_TEMP_CAL;
        remote_actual.uv_readed = VEML6075_convert_uv_raw(sensors_remote_raw_values.uv_readed_raw);  
        
        // converto da uW/cm^2 a mW/cm^2  
        remote_actual.uv_readed.uva = remote_actual.uv_readed.uva/1000;
        remote_actual.uv_readed.uvb = remote_actual.uv_readed.uvb/1000;  
       
        #if defined DEBUG_SENSOR_ANOMALY && defined DEBUG_LOG_SD
        if (local_actual.temp_hdc1080 > 100 || local_actual.humidity > 100 || 
            local_actual.press > 1200 || local_actual.temp_lps25 > 100 ||
            local_actual.uv_readed.uva > 50 || local_actual.uv_readed.uvb > 50 ||
            local_actual.uv_readed.uvi > 20)     
            Write_log(SENSOR_ANOMALY_REM);
        #endif
        
        
        
        // livello batteria
        remote_battery_level = hmtrp_buffer[19];
            
        // primo salvataggio massimi minimi
        if (_sensors_first_read){
            remote_max = remote_actual;
            remote_min = remote_actual;
            _sensors_first_read = 0;
        } else {
    
            // check massimi
            if (remote_actual.temp_hdc1080 > remote_max.temp_hdc1080)
                remote_max.temp_hdc1080 = remote_actual.temp_hdc1080;
            if (remote_actual.humidity > remote_max.humidity)
                remote_max.humidity = remote_actual.humidity;
            if (remote_actual.press > remote_max.press)
                remote_max.press = remote_actual.press;
            if (remote_actual.temp_lps25 > remote_max.temp_lps25)
                remote_max.temp_lps25 = remote_actual.temp_lps25;
            if (remote_actual.uv_readed.uva > remote_max.uv_readed.uva)
                remote_max.uv_readed.uva = remote_actual.uv_readed.uva;
            if (remote_actual.uv_readed.uvb > remote_max.uv_readed.uvb)
                remote_max.uv_readed.uvb = remote_actual.uv_readed.uvb;
            if (remote_actual.uv_readed.uvi > remote_max.uv_readed.uvi)
                remote_max.uv_readed.uvi = remote_actual.uv_readed.uvi;

            // check minimi
            if (remote_actual.temp_hdc1080 < remote_min.temp_hdc1080)
                remote_min.temp_hdc1080  = remote_actual.temp_hdc1080;
            if (remote_actual.humidity < remote_min.humidity)
                remote_min.humidity  = remote_actual.humidity;
            if (remote_actual.press < remote_min.press)
                remote_min.press  = remote_actual.press;
            if (remote_actual.temp_lps25 < remote_min.temp_lps25)
                remote_min.temp_lps25  = remote_actual.temp_lps25;
            if (remote_actual.uv_readed.uva < remote_min.uv_readed.uva)
                remote_min.uv_readed.uva  = remote_actual.uv_readed.uva;
            if (remote_actual.uv_readed.uvb < remote_min.uv_readed.uvb)
                remote_min.uv_readed.uvb  = remote_actual.uv_readed.uvb;
            if (remote_actual.uv_readed.uvi < remote_min.uv_readed.uvi)
                remote_min.uv_readed.uvi  = local_actual.uv_readed.uvi;   
        } 
    }
    HMTRP_reset_buffer();
}

/* ============================================================================================================================================= /
 * FUNCTION:    Write_sensor_values
 * Input:       *_value_actual: puntatore alla struct contentente i valori attuali rilevati dai sensori
 *              *_value_min: puntatore alla struct contentente i valori minimi rilevati dai sensori
 *              *_value_max: puntatore alla struct contentente i valori massimi rilevati dai sensori
 *
 * Overview:    Scrive su display i valori rilevati dai sensori 
/* ============================================================================================================================================ */
void Write_sensor_values(sensors_values *_value_actual, sensors_values *_value_min, sensors_values *_value_max){     
//    TFT_set_font(font_lucida_sans_10x16);
    float _press_temp = 0;
    
    // scrittura valori sensori colonna 1 (valori attuali)
    _press_temp = _value_actual->press;
    if (press_to_write == 1)                                                            // se  stata impostata la pressione normalizzata
        _press_temp = Normalize_press(_press_temp, _value_actual->temp_hdc1080, local_altitude); // normalizzo la pressione    
        
    TFT_write_float(SENSORS_C1_X, SENSORS_R1_Y, 48, _value_actual->temp_hdc1080, TEMP_INT_DIGITS, TEMP_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, TEMP_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C1_X, SENSORS_R2_Y, 48, _value_actual->humidity, RH_INT_DIGITS, RH_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, RH_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C1_X, SENSORS_R3_Y, 48, _press_temp, PRESS_INT_DIGITS, PRESS_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, PRES_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C1_X, SENSORS_R4_Y, 48, _value_actual->uv_readed.uva, UV_INT_DIGITS, UV_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, UVA_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C1_X, SENSORS_R5_Y, 48, _value_actual->uv_readed.uvb, UV_INT_DIGITS, UV_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, UVB_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C1_X, SENSORS_R6_Y, 48, _value_actual->uv_readed.uvi, UVI_INT_DIGITS, UVI_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, UVI_COLOR, BACK_COLOR); 
    
    // scrittura valori sensori colonna 2 (valori minimi)
    _press_temp = _value_min->press;
    if (press_to_write == 1)
        _press_temp = Normalize_press(_press_temp, _value_min->temp_hdc1080, local_altitude); 
        
    TFT_write_float(SENSORS_C2_X, SENSORS_R1_Y, 48, _value_min->temp_hdc1080, TEMP_INT_DIGITS, TEMP_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, TEMP_MIN_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C2_X, SENSORS_R2_Y, 48, _value_min->humidity, RH_INT_DIGITS, RH_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, RH_MIN_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C2_X, SENSORS_R3_Y, 48, _press_temp, PRESS_INT_DIGITS, PRESS_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, PRES_MIN_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C2_X, SENSORS_R4_Y, 48, _value_min->uv_readed.uva, UV_INT_DIGITS, UV_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, UVA_MIN_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C2_X, SENSORS_R5_Y, 48, _value_min->uv_readed.uvb, UV_INT_DIGITS, UV_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, UVB_MIN_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C2_X, SENSORS_R6_Y, 48, _value_min->uv_readed.uvi, UVI_INT_DIGITS, UVI_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, UVI_MIN_COLOR, BACK_COLOR); 
    
    // scrittura valori sensori colonna 2 (valori massimi)
    _press_temp = _value_max->press;
    if (press_to_write == 1)
        _press_temp = Normalize_press(_press_temp, _value_max->temp_hdc1080, local_altitude); 
    
    TFT_write_float(SENSORS_C3_X, SENSORS_R1_Y, 48, _value_max->temp_hdc1080, TEMP_INT_DIGITS, TEMP_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, TEMP_MAX_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C3_X, SENSORS_R2_Y, 48, _value_max->humidity, RH_INT_DIGITS, RH_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, RH_MAX_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C3_X, SENSORS_R3_Y, 48, _press_temp, PRESS_INT_DIGITS, PRESS_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, PRES_MAX_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C3_X, SENSORS_R4_Y, 48, _value_max->uv_readed.uva, UV_INT_DIGITS, UV_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, UVA_MAX_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C3_X, SENSORS_R5_Y, 48, _value_max->uv_readed.uvb, UV_INT_DIGITS, UV_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, UVB_MAX_COLOR, BACK_COLOR); 
    TFT_write_float(SENSORS_C3_X, SENSORS_R6_Y, 48, _value_max->uv_readed.uvi, UVI_INT_DIGITS, UVI_DEC_DIGITS, ZERO_CLEANING_OFF, SAVESPACE_ON, UVI_MAX_COLOR, BACK_COLOR);         
}

/* ============================================================================================================================================= /
 * FUNCTION:    Normalize_press
 * Input:       _press_absolute: pressione assoluta da convertire (float)
 *              _temp: temperatura attuale (float)
 *              _local_altitude: altitudine locale (unsigned int)
 * Output:      pressione normalizzata al livello del mare (float)             
 *
 * Overview:    Converte la pressione da assoluta a normalizzata al livello del mare
/* ============================================================================================================================================ */
float Normalize_press(float _press_absolute, float _temp, unsigned int _local_altitude){
    float _return = 0;

    _return = 1-((0.0065*_local_altitude)/(_temp+(0.0065*_local_altitude)+273.15));
    _return = pow(_return,-5.257);
    _return = _return * _press_absolute;
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    DCF77_manage_loop
 *
 * Overview:    Ciclo di gestione ricezione e analisi segnale DCF77
/* ============================================================================================================================================ */
void DCF77_manage_loop(void){
    rtcc_data _rtcc_time_temp;
    unsigned char _valid_time = 0;
    unsigned char _minute_meteo = 0;
    unsigned char _first_minute_meteo = 0;

    // RACCOLTA 3 DATI                                                              
    if (dcf77_data_ready){                                                              // DCF77 dato ricevuto (tutto il minuto)
        dcf77_data_ready = 0;
        
    _first_minute_meteo = minute_meteo + 2;
    if (_first_minute_meteo >= 60)
        _first_minute_meteo -= 60;
    _first_minute_meteo = Dec_2_bcd(_first_minute_meteo);
    
        switch (dcf77_received_counter){
            case 0: dcf77_time_0 = DCF77_decode_data();     
                    if (meteo_time != 0){                                               // se sono in ricezione meteo
                        if (dcf77_time_0.minutes == _first_minute_meteo)                // controllo che il dato ricevuto sia del primo dato meteo               
                            dcf77_received_counter++;
                    } else {
                            dcf77_received_counter++;                                   // altrimenti incremento sempre ill contatore
                    }
                    
                    break;
            case 1: dcf77_time_1 = DCF77_decode_data();     
                    dcf77_received_counter++;
                    break;
            case 2: dcf77_time_2 = DCF77_decode_data();     
                    dcf77_received_counter++;
                    break;          
        }
    }
    if (dcf77_received_counter == 3){                                                  // se ho ricevuto 3 dati
        dcf77_received_counter = 0;
        
        // LOG DATI RICEVUTI IN 3 MINUTI            
        #ifdef DEBUG_LOG_SD
        Write_log(DCF77_RECEIVED);
        #endif
        
        // controllo i 3 dati ricevuti e selezioni i valori corretti se ci sono errori
        
        _valid_time = Check_dcf77_received(&dcf77_time);
        
        if (_valid_time){                                                                // se l'orario ricevuto  valido, proveniente da ricezioni dati per sync ora o meteo
            
            _rtcc_time_temp.day = dcf77_time.day;
            _rtcc_time_temp.month = dcf77_time.month;
            _rtcc_time_temp.year = dcf77_time.year;
//            _rtcc_time_temp.day_w = dcf77_time.day_w;
            _rtcc_time_temp.hours = dcf77_time.hours;
            _rtcc_time_temp.minutes = dcf77_time.minutes;
            
            if (dcf77_time.day_w == 7)                                                             // allineo giorno 7 tra DCF77 e RTCC
                _rtcc_time_temp.day_w = 0;
            else
               _rtcc_time_temp.day_w = dcf77_time.day_w;     
            
            
            _rtcc_time_temp = Set_time_zone(_rtcc_time_temp, time_zone_utc -1);         // adatto al fuso orario (-1 perch DCF77  gi +1)
            RTCC_osc(OFF);                                                              // aggiorno RTCC
            RTCC_write_all(_rtcc_time_temp.day, _rtcc_time_temp.month, _rtcc_time_temp.year, _rtcc_time_temp.day_w, _rtcc_time_temp.hours, _rtcc_time_temp.minutes,0);
            while (dcf77_seconds_pulse == 0);                                           // attendo nuovo impulso prima di far partire oscillatore, in questo modo dcf e rtcc sono sincronizzati al secondo
            
            RTCC_osc(ON);
            
            if (page == MAIN_PAGE){                                                     // se sono in pagina MAIN aggiorno subito i valori , altrimenti si aggiornano in draw_layout al ritorno in MAIN
                rtcc_time = RTCC_read_all();                                            // leggo l'orario dal RTCC    
                Update_time();                                                          // scrivo i nuovi valori
            }
            
            if ((dcf77_time.flag_bits & 0b00001100) == DCF77_CET)                       // imposto flag orario cet cest
                cet_cest_flag = 0;
            else if ((dcf77_time.flag_bits & 0b00001100) == DCF77_CEST)
                cet_cest_flag = 1;
         
            // LOG ORA SYNC E DATI RTCC PIC
            #ifdef DEBUG_LOG_SD
            Write_log(RTCC_TIME);
            #endif
            
            Calc_meteo_time(meteo_zone,time_zone_utc, cet_cest_flag);                   // Calcolo orari di avvio DCF77 per meteo (calcolo dopo ogni ricezione orario perch cambia se cet o cest)
         
            if (first_sync == 0)                                                        // flag di prima sincronizzazione
                first_sync = 1;
            
            dcf77_success_sync_counter++;
        } else{
            dcf77_fail_sync_counter++;
        }
       
        // se sono stati ricevuti dati meteo, anche con orario non valido
        // i dati meteo vanno decodificati con l'orario d'arrivo +cet/cest, senza la time zone, quindi passa direttamente i dcf77_time_1
        if (meteo_time){   
            // calcolo i minuti da inviare al decoder che saranno sempre uguali per la zona meteo, incremento minute_meteo di 3 (il DCF77 parte  minuti prima)
            _minute_meteo = minute_meteo + 3;
            if (_minute_meteo >= 60)
                _minute_meteo -= 60;
            _minute_meteo = Dec_2_bcd(_minute_meteo);
            
//            meteo_fc_decoded = HKW581_decode_meteo_data(dcf77_time_0.meteo, dcf77_time_1.meteo, dcf77_time_2.meteo, dcf77_time_1.minutes, dcf77_time_1.hours, dcf77_time_1.day, dcf77_time_1.month, dcf77_time_1.day_w, dcf77_time_1.year);
            meteo_fc_decoded = HKW581_decode_meteo_data(dcf77_time_0.meteo, dcf77_time_1.meteo, dcf77_time_2.meteo, _minute_meteo, dcf77_time.hours, dcf77_time.day, dcf77_time.month, dcf77_time.day_w, dcf77_time.year);
         
            #ifdef DEBUG_LOG_SD
            Write_log(METEO);
            #endif     
            
            // se la decodifica  avvenuta con successo
            // salvo i risultati nelle variabili dedicate
            if ((meteo_fc_decoded.decoder_status == 1)){  
                
                if (meteo_time == 1 || meteo_time == 3 || meteo_time == 5 || meteo_time == 7){
                    meteo_fc_day_pointer->day_fc = meteo_fc_decoded.day_fc;
                    meteo_fc_day_pointer->night_fc = meteo_fc_decoded.night_fc;
                    meteo_fc_day_pointer->extreme_w = meteo_fc_decoded.extreme_w_wind_dir;
                    meteo_fc_day_pointer->rain_pr = meteo_fc_decoded.rain_pr_wind_str;
                    meteo_fc_day_pointer->anomaly = meteo_fc_decoded.anomaly;
                    meteo_fc_day_pointer->temp_max = meteo_fc_decoded.temp_fc;
                } else {
                    if (meteo_time != 8){                                               // ultimo dato meteo salvo solo wind_dir, wind_str e anomalia.
                        meteo_fc_day_pointer->day_fc = meteo_fc_decoded.day_fc;
                        meteo_fc_day_pointer->night_fc = meteo_fc_decoded.night_fc;
                        meteo_fc_day_pointer->temp_min = meteo_fc_decoded.temp_fc;
                    }
                    meteo_fc_day_pointer->wind_dir = meteo_fc_decoded.extreme_w_wind_dir;
                    meteo_fc_day_pointer->wind_str = meteo_fc_decoded.rain_pr_wind_str;
                    meteo_fc_day_pointer->anomaly = meteo_fc_decoded.anomaly;
                    
                }
                                                                                                                        
                meteo_update_time = rtcc_time;                                          // salvo l'ora attuale per aggiornamento meteo
                Save_meteo_data_eeprom(0);                                              // salvo in eeprom agli indirizzi dedicati a seconda della fascia oraria di ricezione (0 = dati ricevuti)
             
                // LOG DUMP EEPROM METEO
                #ifdef DEBUG_LOG_SD                                                     
                Write_log(METEO_EEPROM);
                #endif  
             
                if (page == MAIN_PAGE){                                                 // se sono in pagina MAIN aggiorno i valori meteo della pagina meteo visualizzata
                    //meteo_page = 0;                                                   // altrimenti si aggiornano di draw_layout al ritorno in pagina MAIN
                    Draw_meteo(meteo_page);
                }
                
                meteo_success_sync_counter++;
            } else{
                // se il meteo non  stato ricevuto correttamente cancello i vecchi valori
                // eliminato, se non  stato ricevuto correttamente mantiene il vecchio valore 
                // che deriva dalla previsione del giorno prima o dei giorni prima (la prima valida ricevuta)
          /*      if (meteo_time == 1 || meteo_time == 3 || meteo_time == 5 || meteo_time == 7){
                    meteo_fc_day_pointer->day_fc = 0x7F;
                    meteo_fc_day_pointer->night_fc = 0x7F;
                    meteo_fc_day_pointer->extreme_w = 0x7F;
                    meteo_fc_day_pointer->rain_pr = 0x7F;
                    meteo_fc_day_pointer->anomaly = 0x7F;
                    meteo_fc_day_pointer->temp_max = 0x7F;
                } else {
                    if (meteo_time != 8){                                               // ultimo dato meteo salvo solo wind_dir, wind_str e anomalia.
                        meteo_fc_day_pointer->day_fc = 0x7F;
                        meteo_fc_day_pointer->night_fc = 0x7F;
                        meteo_fc_day_pointer->temp_min = 0x7F;
                    }
                    meteo_fc_day_pointer->wind_dir = 0x7F;
                    meteo_fc_day_pointer->wind_str = 0x7F;
                    meteo_fc_day_pointer->anomaly = 0x7F;;
                }
        */        
                meteo_fail_sync_counter++;
            }
        }
         
        // se ho ricevuto un ora valida o se sono in una ricezione meteo (anche con ricezione errata) resetto i flag e fermo la ricezione
        // in caso di meteo errato l'ora non viene aggiornata
        // se invece non ho un ora valida e non sono in ricezione meteo non fermo la ricezione DCF77 e attendo un ora valida (primo avvio o resync)
        if (_valid_time || meteo_time){                                              
            meteo_time = 0;                                                             // resetto i flag
            dcf77_minute_sync = 0;                                                      // resetto il conteggio dei minuti in sincronizzazione
            
            // Debug scrittura orari avvio ricezione DCF77 per meteo aggiornati
            #ifdef DEBUG_LOG_SD_METEO_TIME
            Write_log(METEO_TIME);
            #endif 
            
//            HMTRP_mode(HMTRP_NORMAL_MODE);                                              // attivo  WIFI e HMTRP
//            hmtrp_timeout_counter = 0;
//            if (wifi_on_off == ON)
//                Wifi_connect(ssid, password);
            
            if (server_closed_for_dcf77_sync == 1)
                Wifi_open_server();
            
            DCF77_stop_receive_via_interrupt();                                         // e fermo ricezione
            dcf77_status = DCF77_DISABLED;
            
            dcf77_seconds_pulse = 1;                                                    // forzo a 0 per cancellare l'icona dell'antenna se in pagina MAIN
        }
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Check_dcf77_received
 * Input:       puntatore alla struct in cui salvare i dati corretti ricevuti tramite DCF77
 * Output       0: dati non validi, 1: dati validi (unsigned char)
 *
 * Overview:    Controlla i dati ricevuti in 3 minuti via DCF77, Torna 1 se il dato  corretto e salva il dato nella struct dichiarata
 * Note:        Controlla solamente che i dati dei 3 minuti ricevuti siano consecutivi, non controlla se sono coerenti con l'ora attuale se gi sincronizzato 
 *              Se tra i dati ricevuti viene salvato 0xAA non  stato ricevuto un dato valido
/* ============================================================================================================================================ */
unsigned char Check_dcf77_received(dcf77_decoded_data * _save_loc){
    unsigned char _temp = 0;
    unsigned char _valid_time = 0;
    unsigned char _day_current_month = 0;
    //dcf77_decoded_data _return;
    
    //valid_time = 0;
    
    /* teoria di controllo di tutti i dati tranne che dei minuti:
     * Se il dato della prima ricezione  uguale al dato della seconda ricezione o il dato della seconda  uguale al dato della terza il dato della seconda ricezione viene considerato valido
     * oppure 
     * se il dato della prima ricezione  uguale a quello della terza il dato della prima ricezione viene considerato valido
     * oppure
     * se il dato della terza ricezione  uguale dal dato della seconda ricezione +1  il dato della terza ricezione  considerato valido
     * oppure 
     * se il dato della seconda ricezione  uguale dal dato della prima ricezione +1  il dato della seconda ricezione  considerato valido
     * oppure
     * se il dato della terza ricezione  uguale dal dato della prima ricezione +1  il dato della terza ricezione  considerato valido
     * altrimenti il dato viene salvato come 0xAA a segnalare l'errore
     * 
     * teoria di controllo dei minuti:
     * se il dato della terza ricezione  uguale al dato della seconda ricezione +1 oppure il dato della terza ricezione  uguale al dato della prima ricezione +2 il dato della terza ricezione  considerato valido
     * altrimenti se il dato della seconda ricezione  uguale al dato della prima ricezione +1 il dato della seconda ricezione  incrementato di 1  considerato valido
     * 
     * In questo modo almeno 2 ricezioni su 3 devono essere corrette, altrimenti non si ha una ricezione valida
     */
    
    if (rtcc_time.month == 1 || rtcc_time.month == 3 || rtcc_time.month == 5 || rtcc_time.month == 7 || rtcc_time.month == 8 || rtcc_time.month == 10 || rtcc_time.month == 12){           // mesi con 31 giorni
        _day_current_month = 31;
    } else if (rtcc_time.month == 4 || rtcc_time.month == 6 || rtcc_time.month == 9 || rtcc_time.month == 11){              //mesi con 30 giorni
        _day_current_month = 30;
    } else {                                                                            // febbraio
        if (rtcc_time.year % 4 == 0)                                                    // anno bisestile
            _day_current_month = 29;
        else
            _day_current_month = 28;
    }
        
    if (dcf77_time_1.day == dcf77_time_2.day)
        _save_loc->day = dcf77_time_2.day;
    else if (dcf77_time_0.day == dcf77_time_2.day)
        _save_loc->day = dcf77_time_0.day;
    else if (dcf77_time_2.day == Inc_pbcd(dcf77_time_1.day, 0x01, _day_current_month))
         _save_loc->day = dcf77_time_2.day;
    else if (dcf77_time_0.day == dcf77_time_1.day)
        _save_loc->day = dcf77_time_1.day;
    else if (dcf77_time_1.day == Inc_pbcd(dcf77_time_0.day, 0x01, _day_current_month))
         _save_loc->day = dcf77_time_1.day;
    else if (dcf77_time_2.day == Inc_pbcd(dcf77_time_0.day, 0x01, _day_current_month))
         _save_loc->day = dcf77_time_2.day;
    else 
        _save_loc->day = 0xAA;  // dato non valido

    if (dcf77_time_1.month == dcf77_time_2.month)
        _save_loc->month = dcf77_time_2.month;
    else if (dcf77_time_0.month == dcf77_time_2.month)
        _save_loc->month = dcf77_time_0.month;
    else if (dcf77_time_2.month == Inc_pbcd(dcf77_time_1.month, 0x01, 0x12))
         _save_loc->month = dcf77_time_2.month;
    else if (dcf77_time_0.month == dcf77_time_1.month)
        _save_loc->month = dcf77_time_1.month;
    else if (dcf77_time_1.month == Inc_pbcd(dcf77_time_0.month, 0x01, 0x12))
         _save_loc->month = dcf77_time_1.month;
    else if (dcf77_time_2.month == Inc_pbcd(dcf77_time_0.month, 0x01, 0x12))
         _save_loc->month = dcf77_time_2.month;
    else 
        _save_loc->month = 0xAA;  // dato non valido
    
    if (dcf77_time_1.year == dcf77_time_2.year)
        _save_loc->year = dcf77_time_2.year;
    else if (dcf77_time_0.year == dcf77_time_2.year)
        _save_loc->year = dcf77_time_0.year;
    else if (dcf77_time_2.year == Inc_pbcd(dcf77_time_1.year, 0x01, 0x99))
         _save_loc->year = dcf77_time_2.year;
    else if (dcf77_time_0.year == dcf77_time_1.year)
        _save_loc->year = dcf77_time_1.year;
    else if (dcf77_time_1.year == Inc_pbcd(dcf77_time_0.year, 0x01, 0x99))
         _save_loc->year = dcf77_time_1.year;
    else if (dcf77_time_2.year == Inc_pbcd(dcf77_time_0.year, 0x01, 0x99))
         _save_loc->year = dcf77_time_2.year;
    else 
        _save_loc->year = 0xAA;  // dato non valido
    
    if (dcf77_time_1.day_w == dcf77_time_2.day_w)
        _save_loc->day_w = dcf77_time_2.day_w;
    else if (dcf77_time_0.day_w == dcf77_time_2.day_w)
        _save_loc->day_w = dcf77_time_0.day_w;
    else if (dcf77_time_2.day_w == Inc_pbcd(dcf77_time_1.day_w, 0x01, 0x07))
         _save_loc->day_w = dcf77_time_2.day_w;
    else if (dcf77_time_0.day_w == dcf77_time_1.day_w)
        _save_loc->day_w = dcf77_time_1.day_w;
    else if (dcf77_time_1.day_w == Inc_pbcd(dcf77_time_0.day_w, 0x01, 0x07))
         _save_loc->day_w = dcf77_time_1.day_w;
    else if (dcf77_time_2.day_w == Inc_pbcd(dcf77_time_0.day_w, 0x01, 0x07))
         _save_loc->day_w = dcf77_time_2.day_w;
    else 
        _save_loc->day_w = 0xAA;  // dato non valido
    
    if (dcf77_time_1.hours == dcf77_time_2.hours)
        _save_loc->hours = dcf77_time_2.hours;
    else if (dcf77_time_0.hours == dcf77_time_2.hours)
        _save_loc->hours = dcf77_time_0.hours;
    else if (dcf77_time_2.hours == Inc_pbcd(dcf77_time_1.hours, 0x01, 0x23))
         _save_loc->hours = dcf77_time_2.hours;
    else if (dcf77_time_0.hours == dcf77_time_1.hours)
        _save_loc->hours = dcf77_time_1.hours;
    else if (dcf77_time_1.hours == Inc_pbcd(dcf77_time_0.hours, 0x01, 0x23))
         _save_loc->hours = dcf77_time_1.hours;
    else if (dcf77_time_2.hours == Inc_pbcd(dcf77_time_0.hours, 0x01, 0x23))
         _save_loc->hours = dcf77_time_2.hours;
    else 
        _save_loc->hours = 0xAA;  // dato non valido
    
    if (dcf77_time_0.flag_bits == dcf77_time_1.flag_bits || dcf77_time_1.flag_bits == dcf77_time_2.flag_bits)
        _save_loc->flag_bits = dcf77_time_1.flag_bits;  
    else if (dcf77_time_0.flag_bits == dcf77_time_2.flag_bits)
        _save_loc->flag_bits = dcf77_time_2.flag_bits;  
    
    _temp = Inc_pbcd(dcf77_time_0.minutes, 0x00, 0x59);
    _temp = Inc_pbcd(_temp, 0x00, 0x59);
        
    if((dcf77_time_2.minutes == Inc_pbcd(dcf77_time_1.minutes, 0x00, 0x59)) || (_temp == dcf77_time_2.minutes))
        _save_loc->minutes = dcf77_time_2.minutes;
//    else if(dcf77_time_1.minutes == Inc_pbcd(dcf77_time_0.minutes, 0x00, 0x59))
//        _save_loc->minutes = Inc_pbcd(dcf77_time_1.minutes, 0x00, 0x59);
    else 
        _save_loc->minutes =0xAA;  // dato non valido
    

    // controllo se i dati ricevuti sono nei limiti
    if (_save_loc->day > 0x31 || _save_loc->month > 0x12 || _save_loc->year > 0x99 || _save_loc->hours > 0x23 || _save_loc->minutes > 0x59 || _save_loc->day_w > 0x07 ||
        _save_loc->day == 0 || _save_loc->month == 0 || _save_loc->year == 0){
        _valid_time = 0;
    }else{
        _valid_time = 1;
    }
        
    return _valid_time;    
}

/* ============================================================================================================================================= /
 * FUNCTION:    Check_time_2_sync
 *
 * Overview:    Controlla se l'orario corrente  impostato per l'aggiornamento dei dati via DCF77 (orario o orario + meteo) e avvia la ricezione
 *              dei dati via DCF77 se necessario.
 *              Per le fasce orarie meteo imposta la variabile meteo_time
/* ============================================================================================================================================ */
void Check_time_2_sync(void){            
    volatile unsigned char _hours = 0;
    volatile unsigned char _minutes = 0;
    
    // da BCD a decimale
    _hours = Bcd_2_dec(rtcc_time.hours);
    _minutes = Bcd_2_dec(rtcc_time.minutes);

    // aggiornamento ora alle 2:00 CET o 3:00 CEST per aggiornare direttamente sugli orari del cambio ora CET CEST
    if (_minutes == 58){
        if(_hours == 1 + cet_cest_flag){
//            HMTRP_mode(HMTRP_SLEEP_MODE);                                               // disattivo WIFI e HMTRP
//            hmtrp_timeout_counter = 0;
//            ESP8266_power(OFF, 10);
            
            // chiudo il server per evitare connessioni in ingresso che bloccherebbero la ricezione DCF77
            if (server_status == 1)
                Wifi_close_server();

            dcf77_status = DCF77_ENABLED;
            DCF77_start_receive_via_interrupt();
                                                                                        // previsioni giorno 1
            #ifdef DEBUG_LOG_SD
            Write_log(START_DCF);
            #endif   
        }
    }
    
    // aggiornamento per meteo
    else if (_minutes == minute_meteo){                                                 // se sono nel minuto dedicato all'avvio del meteo per la zona selezionata
        if (_hours == hour_meteo[0]){                                                   // controllo se sono in una delle ore dedicate 
            meteo_time = 1;                                                             // imposto il flag meteo time
            meteo_fc_day_pointer = &meteo_fc_day_1;                                     // imposto puntatore alla struct delle previsioni meteo per la fascia oraria di ricezione
        }
        else if (_hours == hour_meteo[1]){
            meteo_time = 2;
            meteo_fc_day_pointer = &meteo_fc_day_1;
        }
        else if (_hours == hour_meteo[2]){
            meteo_time = 3;
            meteo_fc_day_pointer = &meteo_fc_day_2;
        }
        else if (_hours == hour_meteo[3]){
            meteo_time = 4;
            meteo_fc_day_pointer = &meteo_fc_day_2;
        }
        else if (_hours == hour_meteo[4]){
            meteo_time = 5;
            meteo_fc_day_pointer = &meteo_fc_day_3;
        }
        else if (_hours == hour_meteo[5]){
            meteo_time = 6;
            meteo_fc_day_pointer = &meteo_fc_day_3;
        }
        else if (_hours == hour_meteo[6]){
            meteo_time = 7;
            meteo_fc_day_pointer = &meteo_fc_day_4;
        }
        else if (_hours == hour_meteo[7]){
            meteo_time = 8;
            meteo_fc_day_pointer = &meteo_fc_day_4;
        } 
        else {
            meteo_time = 0;
        }
        
        if (meteo_time != 0){                                                            // se sono in una fascia oraria
//            HMTRP_mode(HMTRP_SLEEP_MODE);                                               // disattivo WIFI e HMTRP
//            hmtrp_timeout_counter = 0;
//            ESP8266_power(OFF, 10);
        
            // chiudo il server
            if (server_status == 1)
                Wifi_close_server();
            
            dcf77_status = DCF77_ENABLED;                                               // abilita ricezione DCF77
            DCF77_start_receive_via_interrupt();
                                                                             
            #ifdef DEBUG_LOG_SD
            Write_log(START_DCF);
            #endif  
       } 
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Calc_meteo_time
 * Input:       _meteo_zone: zona meteo (unsigned char)
 *              _time_zone_utc: fuso orario (unsigned char)
 *
 * Overview:    Calcola gli orari di aggiornamento meteo e imposta le variabili globali
 * Nota:        Sono orari di avvio del DCF77, non di ricezione effettiva (avvio il DCF 2 minuti prima rispetto ai minuti dedicati alla regione selezionata)
/* ============================================================================================================================================ */
void Calc_meteo_time(unsigned char _meteo_zone, signed char _time_zone_utc, unsigned char _cet_cest_flag){
    
    _time_zone_utc -= 1; // per adattare a DCF77
    
    // imposto ora meteo fascia 1 in base alla zona meteo selezionata (orari CET)
    if(_meteo_zone <= 19)                                                               // zone da 0 a 19                                                              
        hour_meteo[0] = 23;
    else if (_meteo_zone >= 20 && _meteo_zone <= 39){                                   // zone da 20 a 39
        hour_meteo[0] = 0;
        _meteo_zone -=20;
    } else {                                                                            // zone da 40 a 59
        hour_meteo[0] = 1;
        _meteo_zone -=40;
    }
    
    if (hour_meteo[0] == 23)
        hour_meteo[1] = 2;
    else
        hour_meteo[1] = hour_meteo[0] +3;
        
    for (unsigned char _i = 1; _i < 7; _i++){
        hour_meteo[_i+1] = hour_meteo[_i] +3;
    }    

    // ricavo minuti del meteo possono essere 59,2,5,8,11,14,17,20,23,26,29,32,35,38,41,44,47,50,53,56 a seconda della zona meteo
    // _meteo_zone ora vale sempre da 0 a 19
    minute_meteo = (3 *_meteo_zone) -1;
    
    if (minute_meteo == 0xFF)                                                           // valore negativo
        minute_meteo = 59;                                                              // riporto a 59                               

    // se 59 devo partire l'ora prima, decremento le ore
    if (minute_meteo == 59) {
        
        for (unsigned char _i = 0; _i < 8; _i++){
            if (hour_meteo[_i] == 0)
                hour_meteo[_i] = 23;
            else
                hour_meteo[_i] -= 1;
        }
    }
    
    // aggiungo fuso orario
    for (unsigned char _i = 0; _i < 8; _i++){
        hour_meteo[_i] += _time_zone_utc;

        if (_time_zone_utc > 0){
            if (hour_meteo[_i] > 23)
                hour_meteo[_i] = hour_meteo[_i] - 24;
        }else if (_time_zone_utc < 0){
            if (hour_meteo[_i] > 23)
                hour_meteo[_i] = 24 - (256-hour_meteo[_i]);
        }
    }
    
    // aggiungo cet_cest_flag
    for (unsigned char _i = 0; _i < 8; _i++){
        hour_meteo[_i] +=  _cet_cest_flag;
        if (hour_meteo[_i] == 24)
            hour_meteo[_i] = 0;
    }
    
    #ifdef DEBUG_LOG_SD
    Write_log(METEO_TIME);
    #endif 
}

/* ============================================================================================================================================= /
 * FUNCTION:    Set_time_zone
 * Output:      Struct contenente i dati DCF77 adeguati alla zona UTC selezionata
 *
 * Overview:    Adatta l'orario ricevuto da DCF77 (UTC+1) al fuso orario selezionato
/* ============================================================================================================================================ */
rtcc_data Set_time_zone(rtcc_data _rtcc_time, signed char _time_zone_utc){
    rtcc_data _return;
    unsigned char _hours, _day, _day_w, _month, _year;

    // converto da bcd a decimali per comodit nei calcoli
    _hours = Bcd_2_dec(_rtcc_time.hours);
    _day = Bcd_2_dec(_rtcc_time.day);
    _day_w = _rtcc_time.day_w;
    _month = Bcd_2_dec(_rtcc_time.month);
    _year = Bcd_2_dec(_rtcc_time.year);
    
    _hours += _time_zone_utc;                                                           // aggiungo alle ore il valore del fuso orario (positivo o negativo)
    
    // controlli di overflow per fuso orario positivo
    if (_time_zone_utc > 0){
        if (_hours > 23){                                                               // se le ore sono superiori a 23
            _hours = _hours - 24;                                                       // calcolo le ore
            _day_w++;                                                                   // incremento giorno settimana
            if (_day_w == 7)
                _day_w = 0;
            _day++;                                                                     // incremento giorno
            if (_month == 1 || _month == 3 || _month == 5 || _month == 7 || _month == 8 || _month == 10 || _month == 12){           // mesi con 31 giorni
                if (_day == 32){                                                                                                    // se giorno == 32
                    _day = 1;                                                                                                       // riporto a 1
                    _month++;                                                                                                       // incremento mese
                }
            } else if (_month == 4 || _month == 6 || _month == 9 || _month == 11){      //mesi con 30 giorni
                if (_day == 31){
                    _day = 1;
                    _month++;
                }
            } else {                                                                    // febbraio
                if (_year % 4 == 0){                                                    // anno bisestile
                    if (_day == 29) {
                        _day = 1;
                        _month++;
                    }
                } else {                                                                // anno non bisestile
                    if (_day == 28) {
                        _day = 1;
                        _month++;
                    }
                }
            }
            if (_month == 13){                                                          // se mese == 13
                _month = 1;                                                             // riporto a 1 e incremento anno
                _year++;
                
                if (_year == 100)                                                       // se anno == 100
                    _year = 0;                                                          // riporto a 0
            }
        }
    } 
    
    // controlli di overflow per fuso orario negativo
    else if (_time_zone_utc < 0){                                                     

        if (_hours > 23){
            _hours = 24 - (256-_hours);
            _day_w--;
            if (_day_w == 0xFF)
                _day_w = 6;
            _day--;
            if (_day == 0){
                _month--;
                
                if (_month == 0){
                    _month = 12;
                    _year--;
                    
                    if (_month == 1 || _month == 3 || _month == 5 || _month == 7 || _month == 8 || _month == 10 || _month == 12)    // mesi con 31 giorni
                        _day = 31;
                    else if (_month == 4 || _month == 6 || _month == 9 || _month == 11)     //mesi con 30 giorni
                        _day = 30;
                    else{                                                                   // febbraio
                        if (_year % 4 == 0)                                                 // anno bisestile
                                _day = 29;
                        else 
                            _day = 28;
                    }
                    
                    if (_year == 255)
                        _year = 99;
                }
            }
        }
    }
    
    // torno i valori e dove necessario riconverto in BCD
    _return.seconds = _rtcc_time.seconds;
    _return.minutes = _rtcc_time.minutes;                                               
    _return.hours = Dec_2_bcd(_hours);
    _return.day = Dec_2_bcd(_day);
    _return.month = Dec_2_bcd(_month);
    _return.year = Dec_2_bcd(_year);
    _return.day_w = _day_w;
    
    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    Wifi_connect
 * Input:       *_ssid: puntatore ad array contentente SSID
 *              *_password: puntatore ad array contentente la password
 * 
 * Overview:    Connessione alla rete WiFi, ssid, password e porta devono essere salvati nei relativi array globali
 * Note:        Se il modulo viene disabilitato tramite funzione ESP8266_power quando viene riabilitato va rieseguita la funzione per connettersi
 *              e per ricreare il server
 *              Se la rete non  pi nelle vicinanze si riconnette in automatico quando torna visibile e il server viene ricreato in automatico
 *              dal modulo ESP8266
/* ============================================================================================================================================ */
void Wifi_connect (unsigned char *_ssid, unsigned char *_password){
    unsigned char _resp = 0;
//    ESP8266_power(ON,ESP8266_DEF_TIMEOUT);                                              // abilito modulo wifi
    ESP8266_set_mode(ESP8266_STATION_MODE, ESP8266_CURRENT_VALUE, ESP8266_DEF_TIMEOUT); // Modalita stazione

    // tentativo di connessione
    _resp = ESP8266_join_ap(_ssid,_password,ESP8266_CURRENT_VALUE, 18000);
    
    // connesso
    if (_resp == ESP8266_RESP_OK){     
        wifi_connected = 1;                                                             // flag wifi connesso
        Wifi_check_con(1);                                                              // controllo la connessione e scrivo icona segnale
        Write_log(WIFI_CON);
                 
        // imposto connessioni multiple e avvio il server
        ESP8266_multiple_con(ON, ESP8266_DEF_TIMEOUT);
        Wifi_open_server();
        
    // connessione non riuscita
    } else {
        wifi_connected = 0;
        
        #ifdef DEBUG_LOG_SD
        Write_log(WIFI_NO_CON);
        #endif
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Wifi_open_server
 * 
 * Overview:    Apre il server per le connessioni in ingresso
/* ============================================================================================================================================ */
void Wifi_open_server(void){
    server_status = ESP8266_create_server(port,ESP8266_DEF_TIMEOUT);
    
        // LOG DEBUG
        #ifdef DEBUG_LOG_SD
        Write_log(WIFI_SERVER);
        #endif
        
        // risposta server 0xFE o 0xFF porto a 0 (timeout o non gestita)
        if (server_status > 1)
            server_status = 0;
        
        server_closed_for_dcf77_sync = 0;
}

/* ============================================================================================================================================= /
 * FUNCTION:    Wifi_close_server
 * 
 * Overview:    Chiude il server per le connessioni in ingresso
/* ============================================================================================================================================ */
void Wifi_close_server(void){
    ESP8266_close_server(ESP8266_DEF_TIMEOUT);
    
    server_closed_for_dcf77_sync = 1;
    server_status = 0;
    
    // LOG DEBUG
    #ifdef DEBUG_LOG_SD
    Write_log(WIFI_SERVER);
    #endif
}


/* ============================================================================================================================================= /
 * FUNCTION:    Wifi_check_con
 * Input:       _forced: a 1 forza la scrittura dell'icona (se ritorno da altre pagine).
 * 
 * Overview:    Testa la connessione ed aggiorna l'icona sul display a seconda del livello rssi
 * Note:        Reimposta la variabile wifi_connected, se la rete sparisce e poi ricompare il modulo si riconnette da solo se era gi connesso
/* ============================================================================================================================================ */
void Wifi_check_con(unsigned char _forced){
    unsigned char _signal_str = 0;
    static unsigned char _icon_drawn = 0;
    
    if (_forced)
        _icon_drawn = 0;
    
    // test connessione AP
    if (ESP8266_check_ap_con(ESP8266_CURRENT_VALUE, ESP8266_DEF_TIMEOUT) == ESP8266_RESP_OK){       // se connesso
        wifi_connected = 1;
        ESP8266_clean_buffer_until_char('-');
        
        // ricavo forza segnale
        _signal_str = (esp8266_buffer[0] - '0') << 4;                                               
        _signal_str |= esp8266_buffer[1] - '0';
        
        // disegno icona a seconda della forza del segnale
        if (_signal_str <= 60 && _icon_drawn != 3){                                             
            TFT_draw_image_sd(WIFI_ICON_X,WIFI_ICON_Y,"WIFI_3.bmp");
            _icon_drawn = 3;
        }
        else if (_signal_str >= 61 && _signal_str <= 80 && _icon_drawn != 2) {
            TFT_draw_image_sd(WIFI_ICON_X,WIFI_ICON_Y,"WIFI_2.bmp");
            _icon_drawn = 2;
        }
        else if (_signal_str >= 81 && _icon_drawn != 1) {
            TFT_draw_image_sd(WIFI_ICON_X,WIFI_ICON_Y,"WIFI_1.bmp");
            _icon_drawn = 1;
        }    
        ESP8266_reset_buffer();
    // non connesso
    }else{                                                                              
        TFT_fill_rect(WIFI_ICON_X, WIFI_ICON_Y,19,15,BACK_COLOR);                       // cancello icona wifi
        TFT_fill_rect(TRAN_ICON_X, TRAN_ICON_Y,20,15,BACK_COLOR);                       // ed eventuale icona connessione client attiva
        _icon_drawn = 0;
        wifi_trans_icon_drawn = 0;
        wifi_connected = 0;                                                             // flag di connessione a 0   
        active_conn = 0;                                                                // annullo tutte le connessioni attive
//        server_status = 0;
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Wifi_check_incoming
 *
 * Overview:    Monitora i dati provenienti dal modulo ESP8266, gestisce le connessioni in ingresso e disegna la relativa icona.
 *              Richiama la funzione di invio dati/modifica parametri a seconda dei dati ricevuti:
 *              'A': dati real time
 *              'B': log sensori locali
 *              'D': log sensori remoti
 *              'C': Start/stop registrazione log              
/* ============================================================================================================================================ */
void Wifi_check_incoming(void){
    unsigned char _data = 0;
    unsigned char _active_connection = 0;                                               // connessione che ha richiesto i dati
    unsigned char _request = 0;
        
    _data = ESP8266_check_incoming_data();
    
    // ciclo ricezione connessione e dati    
    if (_data != ESP8266_NO_INCOMING_DATA){
    
        if ((_data & 0xF0) == ESP8266_CON_CONNECT_OFFSET){                              // connessione attiva
            active_conn++;                                                              // numero di connessioni attive
            wifi_incoming_con_counter++;
            #ifdef DEBUG_LOG_SD
            Write_log(NEW_CON);
            #endif
        } else if ((_data & 0xF0) == ESP8266_CON_CLOSE_OFFSET || (_data & 0xF0) == ESP8266_CON_FAIL_OFFSET){                    // connessione chiusa o fallita
            active_conn--;
            
            if (active_conn < 0)
                active_conn = 0;
            #ifdef DEBUG_LOG_SD
            Write_log(CLOSED_CON);
            #endif
        }
    
        if (wifi_trans_icon_drawn == 0 && active_conn != 0){                            // disegno icona connessione in ingresso se non  gi disegnata e se c' una connessione in ingresso
            TFT_draw_image_sd(TRAN_ICON_X,TRAN_ICON_Y,"TRAN.bmp");
            wifi_trans_icon_drawn = 1;
        }
        else if (wifi_trans_icon_drawn == 1 && active_conn == 0){                       // cancello icona se sono state chiuse tutte le connessioni
            TFT_fill_rect(TRAN_ICON_X, TRAN_ICON_Y,20,15,BACK_COLOR);
            wifi_trans_icon_drawn = 0;
        }        
        
        if ((_data & 0xF0) == ESP8266_CON_SEND_OFFSET){                                 // dati in arrivo da una delle 5 connessioni
            _active_connection = _data & 0x0F;
            
            #ifdef DEBUG_LOG_SD
            connection_log = _active_connection;
            Write_log(INCOMING_DATA);
            #endif  
            
            _request = esp8266_buffer[0];
            
            if (_request == 'A' || _request == 'B' || _request == 'C'){
                Delay_ms(100);              
                Wifi_send_settings(_active_connection);
            
                switch (_request){
                    case 'A':                                                           // dati RT
                        Wifi_send_rt_data(_active_connection);
                        wifi_real_time_send_counter++;
                        break;
                    case 'B':
                        Wifi_send_log_data(_active_connection, LOCAL);                  // logo sensori locali
                        wifi_log_send_counter++;
                        break;
                    case 'C':                                                           // log sensori remoti
                        Wifi_send_log_data(_active_connection, REMOTE);
                        wifi_log_send_counter++;
                        break;                                         
                }
                Wifi_close_connection(_active_connection);
            }
            
            else if (_request == 'D'){                                                  // avvia o ferma registrazione log
                if (record_log == 0) {
                    record_log = 1;
                    new_log = 1;
                    TFT_draw_image_sd(LOG_ICON_X,LOG_ICON_Y,"LOG.bmp");
                } else {
                    record_log = 0;
                    TFT_fill_rect(LOG_ICON_X,LOG_ICON_Y,13,15,BACK_COLOR);
                }
                ESP8266_send_data_over_con_l(_active_connection,"1","D",ESP8266_SEND_TIMEOUT);    // invio risposta
            }
            
            else if (_request == 'E'){                                                  // riprende la registrazione log
                record_log = 1;
                TFT_draw_image_sd(LOG_ICON_X,LOG_ICON_Y,"LOG.bmp");
                ESP8266_send_data_over_con_l(_active_connection,"1","E",ESP8266_SEND_TIMEOUT);    // invio risposta
            }
            
            else if (_request == 'F'){                                                  // invia il file LOG.TXT
                Wifi_send_debug_log(_active_connection);
                Wifi_close_connection(_active_connection);
                
            } else if (_request == 'G'){                                                // cancella tutto il contenuto del file LOG.TXT
                if (f_mount(&sd_card, "", 1) == FR_OK) {                                            // Mount SD
                    if (f_open(&file, "LOG.txt", FA_CREATE_ALWAYS | FA_READ | FA_WRITE) == FR_OK) {	// Apertura file LOG
                    f_close(&file);
                    }
                }
                ESP8266_send_data_over_con_l(_active_connection,"1","G",ESP8266_SEND_TIMEOUT);    // invio risposta
                Wifi_close_connection(_active_connection);
            }
        }
    ESP8266_reset_buffer();
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Wifi_send_settings
 * Input:       _active_connection: connessione attiva a cui inviare i dati
 * 
 * Overview:    Invia i dati di calibrazione e di settaggio alla connessione indicata  
/* ============================================================================================================================================ */
void Wifi_send_settings (unsigned char _active_connection){
    unsigned char *_string_buffer;
    unsigned char _send_buffer[5];
    
    // Invio fattore calibrazione hdc1080 temperatura locale
    _string_buffer = Float_2_string(HDC1080_TEMP_CAL,2,4,ZERO_CLEANING_ON);
    ESP8266_send_data_over_con(_active_connection,"10",_string_buffer,ESP8266_SEND_TIMEOUT);    // invio dati fino a fine stringa
    
    // Invio fattore calibrazione hdc1080 umidit locale
    _string_buffer = Float_2_string(HDC1080_HUMID_CAL,2,4,ZERO_CLEANING_ON);
    ESP8266_send_data_over_con(_active_connection,"10",_string_buffer,ESP8266_SEND_TIMEOUT);    // invio dati fino a fine stringa
    
    // Invio fattore calibrazione hdc1080 temperatura remoto
    _string_buffer = Float_2_string(HDC1080_REM_TEMP_CAL,2,4,ZERO_CLEANING_ON);
    ESP8266_send_data_over_con(_active_connection,"10",_string_buffer,ESP8266_SEND_TIMEOUT);    // invio dati fino a fine stringa
    
    // Invio fattore calibrazione hdc1080 umidit remoto  
    _string_buffer = Float_2_string(HDC1080_REM_HUMID_CAL,2,4,ZERO_CLEANING_ON);
    ESP8266_send_data_over_con(_active_connection,"10",_string_buffer,ESP8266_SEND_TIMEOUT);    // invio dati fino a fine stringa
      
    // Invio altitudine  
    _string_buffer = Float_2_string((float)local_altitude,4,0,ZERO_CLEANING_ON);
    ESP8266_send_data_over_con(_active_connection,"5",_string_buffer,ESP8266_SEND_TIMEOUT);     // invio dati fino a fine stringa
    
    // invio settaggi
    _send_buffer[0] = (unsigned char) press_to_write;
    _send_buffer[1] = (unsigned char) remote_connected;
    _send_buffer[2] = (unsigned char) remote_battery_level;
    _send_buffer[3] = (unsigned char) record_log;
    _send_buffer[4] = '\0';
    ESP8266_send_data_over_con_l(_active_connection,"4",_send_buffer,ESP8266_SEND_TIMEOUT);    // invio dati a lunghezza fissa
}
  
/* ============================================================================================================================================= /
 * FUNCTION:    Wifi_send_rt_data
 * Input:       _active_connection: connessione attiva a cui inviare i dati
 * 
 * Overview:    Invia i dati real time alla connessione indicata  
/* ============================================================================================================================================ */    
void Wifi_send_rt_data (unsigned char _active_connection){
    unsigned char _send_buffer[71];                                                     // buffer con dati da inviare
    unsigned char _index = 0;
    meteo_fc_compl *_meteo_fc_day_pointer; 
    sensors_raw_values *_sensors_values;
    
    // data e ora in buffer invio
    _send_buffer[0] = rtcc_time.day;
    _send_buffer[1] = rtcc_time.month;
    _send_buffer[2] = rtcc_time.year;
    _send_buffer[3] = rtcc_time.hours;
    _send_buffer[4] = rtcc_time.minutes;
    _send_buffer[5] = RTCC_read_seconds();
    _send_buffer[6] = rtcc_time.day_w;
      
    _index = 7;
    
    // dati meteo in buffer invio
    for (unsigned char _i=0; _i<3; _i++){
        switch (_i){
            case 0: _meteo_fc_day_pointer = &meteo_fc_day_1;
            break;
            case 1: _meteo_fc_day_pointer = &meteo_fc_day_2;
            break;
            case 2: _meteo_fc_day_pointer = &meteo_fc_day_3;
            break;
        }
        

        _send_buffer[_index] = (_meteo_fc_day_pointer->day_fc);
        _index++;
        _send_buffer[_index] = (_meteo_fc_day_pointer->night_fc);
        _index++;
        _send_buffer[_index] = (_meteo_fc_day_pointer->extreme_w);
        _index++;
        _send_buffer[_index] = (_meteo_fc_day_pointer->rain_pr);
        _index++;
        _send_buffer[_index] = (_meteo_fc_day_pointer->anomaly);
        _index++;
        _send_buffer[_index] = (_meteo_fc_day_pointer->wind_dir);
        _index++;
        _send_buffer[_index] = (_meteo_fc_day_pointer->wind_str);
        _index++;
        _send_buffer[_index] = (_meteo_fc_day_pointer->temp_min);
        _index++;
        _send_buffer[_index] = (_meteo_fc_day_pointer->temp_max);
        _index++;
    
    }
    
    // data e ora aggiornamento meteo
    _send_buffer[_index] = meteo_update_time.day;
    _index++;
    _send_buffer[_index] = meteo_update_time.month;
    _index++;
    _send_buffer[_index] = meteo_update_time.hours;
    _index++;
    _send_buffer[_index] = meteo_update_time.minutes;
    _index++;
    
    // dati raw sensori in buffer invio
    for (unsigned char _i=0; _i<2; _i++){
        
        switch (_i){
            case 0: _sensors_values = &sensors_local_raw_values;
            break;
            case 1: _sensors_values = &sensors_remote_raw_values;
            break;

        }
        
        _send_buffer[_index] = _sensors_values->temp_hdc1080_raw >> 8;
        _index++;
        _send_buffer[_index] = _sensors_values->temp_hdc1080_raw &  0xFF;
        _index++;
        _send_buffer[_index] = _sensors_values->humidity_raw >> 8;
        _index++;
        _send_buffer[_index] = _sensors_values->humidity_raw &  0xFF;
        _index++;
        _send_buffer[_index] = _sensors_values->press_raw >> 24;
        _index++;
        _send_buffer[_index] = _sensors_values->press_raw >> 16;
        _index++;
        _send_buffer[_index] = _sensors_values->press_raw >> 8;
        _index++;
        _send_buffer[_index] = _sensors_values->press_raw & 0xFF;
        _index++;
        _send_buffer[_index] = _sensors_values->uv_readed_raw.uva >> 8;
        _index++;
        _send_buffer[_index] = _sensors_values->uv_readed_raw.uva & 0xFF;
        _index++;
        _send_buffer[_index] = _sensors_values->uv_readed_raw.uvb >> 8;
        _index++;
        _send_buffer[_index] = _sensors_values->uv_readed_raw.uvb & 0xFF;
        _index++;
        _send_buffer[_index] = _sensors_values->uv_readed_raw.uv_comp_1 >> 8;
        _index++;
        _send_buffer[_index] = _sensors_values->uv_readed_raw.uv_comp_1 & 0xFF;
        _index++;
        _send_buffer[_index] = _sensors_values->uv_readed_raw.uv_comp_2 >> 8;
        _index++;
        _send_buffer[_index] = _sensors_values->uv_readed_raw.uv_comp_2 & 0xFF;
        _index++;
    }
    
/*70*/  _send_buffer[_index] = '\0';
    
    _index = 0;
    
    ESP8266_send_data_over_con_l(_active_connection,"70",_send_buffer,ESP8266_SEND_TIMEOUT);    // invio dati
}

/* ============================================================================================================================================= /
 * FUNCTION:    Wifi_send_log_data
 * Input:       _active_connection: connessione attiva a cui inviare i dati
 *              _selection: dati da inviare (LOCAL/REMOTE)
 * 
 * Overview:    Invia i dati dei log alla connessione indicata  
/* ============================================================================================================================================ */    
void Wifi_send_log_data (unsigned char _active_connection, unsigned char _selection){
    volatile unsigned int _eeprom_addr = 0;
    volatile unsigned char _eeprom_dev_address = 0;
    unsigned char _send_buffer[23];
    
    // selezione della memoria da leggere
    if (_selection == LOCAL)
        _eeprom_dev_address =  EEPROM_I2C_DEV_ADDR_B1;
    else if (_selection == REMOTE)
        _eeprom_dev_address =  EEPROM_I2C_DEV_ADDR_B2;

    // ciclo finch i dati salvato hanno lo stesso indice (della registrazione in corso o dell'ultima registrazione avviata)
    while (EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr) == eeprom_log_index){
        _eeprom_addr++;
            
        for (unsigned char _c = 0; _c < 22; _c++){
            _send_buffer[_c] =  EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
            _eeprom_addr++;
        }
        _send_buffer[22] = '\0';
            
        ESP8266_send_data_over_con_l(_active_connection,"22",_send_buffer,ESP8266_SEND_TIMEOUT);    // invio dati lunghezza fissa
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Wifi_send_debug_log
 * Input:       _active_connection: connessione attiva a cui inviare i dati
 * 
 * Overview:    Invia i dati registrati nel lod gebug 
/* ============================================================================================================================================ */ 
void Wifi_send_debug_log(unsigned char _active_connection){
    unsigned char _readed[201];
    
    if (f_mount(&sd_card, "", 1) == FR_OK) {                                            // Mount SD
        if (f_open(&file, "LOG.txt", FA_OPEN_EXISTING | FA_READ) == FR_OK) {
            do {
                Clear_buffer(_readed);
                f_read(&file,_readed,200,&bw);
                ESP8266_send_data_over_con(_active_connection,"200",_readed,ESP8266_SEND_TIMEOUT);    // invio dati lunghezza fissa o fino a fine stringa
            } while (bw == 200);
            f_close(&file);	
        } else {
            ESP8266_send_data_over_con(_active_connection,"20","File non tovato.",ESP8266_SEND_TIMEOUT);    // invio dati lunghezza fissa o fino a fine stringa
        }
    }
}

void Clear_buffer (unsigned char * _buffer){
    unsigned char _i = 0;
    while (_buffer[_i]){
        _buffer[_i] = 0;
        _i++;
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Wifi_close_connection
 * Input:       _active_connection: connessione attiva da chiudere
 * 
 * Overview:    Chiude la connessione indicata
/* ============================================================================================================================================ */    
void Wifi_close_connection(unsigned char _connection_2_close){
      ESP8266_close_connection(_connection_2_close, ESP8266_DEF_TIMEOUT);
      active_conn--;
      
      // in alcuni casi di connessioni non rilevate (connessione in ingresso mentre sta inviando i log per esempio)
      // non incrementa active_conn
      if (active_conn < 0)
        active_conn = 0;
      
      #ifdef DEBUG_LOG_SD
      Write_log(CLOSED_CON);
      #endif

      if (wifi_trans_icon_drawn == 1 && active_conn == 0){                              // cancello icona se sono state chiuse tutte le connessioni
          TFT_fill_rect(TRAN_ICON_X, TRAN_ICON_Y,20,15,BACK_COLOR);
          wifi_trans_icon_drawn = 0;
      }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Save_sensors_log
 * Input:       *_values: puntatore alla struct contenente i valori da salvare
 *              _type: sensori da salvare (LOCAL, REMOTE) (unsigned char)
 *
 * Overview:    Salva il valore dei sensori sulla EEPROM 24LC1025. I dati dei sensori locali vengono salvati nel primo banco dalla memoria (0x0000 -> 0xFFFF),
 *              i dati dei sensori remoti vengono salvati nel secondo banco (0x10000 -> 0x1FFFF).
/* ============================================================================================================================================ */
void Save_sensors_log(sensors_raw_values *_values, unsigned char _type){
    static unsigned int _page_1_address = 0;
    static unsigned int _page_2_address = 0;
    unsigned char _eeprom_dev_address = 0;
    static unsigned int *_eeprom_address;

    // se nuova registrazione azzero gli indirizzi ed incremento l'indice del log
    // i dati registrati avranno lo stesso indice per identificare la registrazione dati corrente da esportare
    if (new_log){
        _page_1_address = 0;
        _page_2_address = 0;
        
        eeprom_log_index = EEPROM_I2C_read(EEPROM_I2C_DEV_ADDR_B1, 0x00) + 1;
        EEPROM_I2C_write(EEPROM_I2C_DEV_ADDR_B1,0x00,eeprom_log_index);                 // risalvo subito indice aggionrato
        
        new_log = 0;
        
        #ifdef DEBUG_LOG_SD
        Write_log(START_RECORD_LOG);
        #endif
    }
    
    // macro di scrittura, delay ed incremento puntatore indirizzi
    #define Write_value(_value)     EEPROM_I2C_write(_eeprom_dev_address, *_eeprom_address, _value); (*_eeprom_address)++; Delay_ms(10);   
    
    // imposto l'indirizzo EEPROM (2 banchi) e il puntatore agli indirizzi a seconda dei dati da salvare
    switch (_type){
        case LOCAL: _eeprom_dev_address = EEPROM_I2C_DEV_ADDR_B1;
                    _eeprom_address = &_page_1_address;
                    break;
        case REMOTE: _eeprom_dev_address = EEPROM_I2C_DEV_ADDR_B2;
                     _eeprom_address = &_page_2_address;
                     break;
    }
    
    // salvo i dati in EEPROM
    Write_value(eeprom_log_index);                                                      // indice univoco log
    Write_value(rtcc_time.day);
    Write_value(rtcc_time.month);
    Write_value(rtcc_time.year);
    Write_value(rtcc_time.hours);
    Write_value(rtcc_time.minutes);
    Write_value(RTCC_read_seconds());
    Write_value(_values->temp_hdc1080_raw >> 8);
    Write_value(_values->temp_hdc1080_raw);
    Write_value(_values->humidity_raw >> 8);
    Write_value(_values->humidity_raw);
    Write_value(_values->press_raw >> 24);
    Write_value(_values->press_raw >> 16);
    Write_value(_values->press_raw >> 8);
    Write_value(_values->press_raw);
//    Write_value(_values->temp_lps25_raw >> 8);
//    Write_value(_values->temp_lps25_raw);
    Write_value(_values->uv_readed_raw.uva >> 8);
    Write_value(_values->uv_readed_raw.uva);
    Write_value(_values->uv_readed_raw.uvb >> 8);
    Write_value(_values->uv_readed_raw.uvb);
    Write_value(_values->uv_readed_raw.uv_comp_1 >> 8);
    Write_value(_values->uv_readed_raw.uv_comp_1);
    Write_value(_values->uv_readed_raw.uv_comp_2 >> 8);
    Write_value(_values->uv_readed_raw.uv_comp_2);
    
    if (*_eeprom_address > 0xFFF0)                                                      // ultimo salvataggio valido 
        *_eeprom_address = 0x0000;                                                      // azzero l'indirizzo eeprom per ripartire la registrazione da capo.
}

/* ============================================================================================================================================= /
 * FUNCTION:    Plot_log_chart
 * Input:       _chart_2_plot: grafico da stampare codificato: 4bit alti: 1-Locali, 2-Remoto, 3-Incrementa indice, 4-decrementa indice
 *                                                             4 bit bassi: 1-temperatura, 2-umidit, 3-pressione, 4-UVA, 5-UVB, 6-UVI
 *
 * Overview:    Disegna il grafico selezionato, abilita o disabilita lo scrolling dei dati se il log ha pi di 200 campioni.
/* ============================================================================================================================================ */
unsigned char Plot_log_chart(unsigned char _chart_2_plot){
    float _min;                                                                         // valori dei sensori minimi, massimi e dato attuale
    float _max;
    float _value;                                                                       // valore da del campione letto da eeprom
    float _value_pre;                                                                   // valore del campione precedente, per grafico
    
    // valori temporanei letture dati
    unsigned long _temp = 0;                                                            
    veml6075_uv_raw _readed_uv_raw;
    veml6075_uv _readed_uv;
    
    unsigned char *_timestamp_string;                                                   // puntatore al buffer contenente la stringa con la data e l'ora
    
    unsigned char _x = 0;                                                               // conteggio X
    
    volatile unsigned int _eeprom_addr = 0;
    volatile unsigned char _eeprom_dev_address = 0;
    unsigned int _eeprom_addr_min = 0;
    unsigned int _eeprom_addr_max = 0;
    
    float _hdc1080_temp_cal = 0.0;                                                      // valori calibrazione
    float _hdc1080_humid_cal = 0.0;
    
    unsigned char _eeprom_addr_offset_1 = 0;                                            // offset da indice a primo valore da leggere
    unsigned char _eeprom_addr_offset_2 = 0;                                            // offset da ultimo valore letto a prossimo indice
    unsigned char _eeprom_addr_offset_3 = 0;                                            // offset da ultimo valore letto a data (indietro)
    unsigned char _byte_2_read = 0;                                                     // numero di byte da leggere dalla EEPROM per singolo dato
    
    static unsigned char _log_index = 0;                                                // indice per calcolo valori da leggere se log > 200 campioni
    static unsigned char _chart_plotted = 0;                                            // grafico disegnato                  
    
    // limiti del grafico
    float _limit_min = 0;
    float _limit_max = 0;    
    
//    TFT_set_font(font_lucida_sans_10x16);
    
    // area grafico
    TFT_fill_rounded_rect(CHART_RECT_X, CHART_RECT_Y, CHART_RECT_L, CHART_RECT_H, 5, CHART_RECT_COLOR);

    // scritte statiche
    TFT_draw_line_x(LOG_SEP_LINE_X, LOG_SEP_LINE_L, LOG_SEP_LINE_Y, LOG_SEP_LINE_COLOR);  
    TFT_draw_line_x(LOG_SEP_LINE_X, LOG_SEP_LINE_L, LOG_SEP_LINE_Y+1, LOG_SEP_LINE_COLOR);
    TFT_write_string(LOG_MIN_TEXT_X,LOG_MIN_TEXT_Y,50,"MIN",SAVESPACE_ON,LOG_MIN_COLOR,BACK_COLOR);
    TFT_write_string(LOG_MAX_TEXT_X,LOG_MAX_TEXT_Y,50,"MAX",SAVESPACE_ON,LOG_MAX_COLOR,BACK_COLOR);
    TFT_fill_rect(LOG_MIN_TEXT_X+30, LOG_MIN_TEXT_Y, 80, 16, BACK_COLOR);
    TFT_fill_rect(LOG_MAX_TEXT_X+30, LOG_MAX_TEXT_Y, 80, 16, BACK_COLOR);
    
    // switch per dati successivi o precedenti
    // incrementa o decrementa l'indice e imposta il grafico disegnato precedentemente
    // se _chart_2_plot  0x30 o 0x40, altrimenti  il grafico per un nuovo valore e resetto prev e next e indice
    switch ((_chart_2_plot & 0xF0) >>4 ){   
        case 3: _log_index ++;
                _chart_2_plot = _chart_plotted;
                break;
        case 4: _log_index -=1;
                _chart_2_plot = _chart_plotted;
                break;
        default:
                log_next_allowed = 0;
                log_prev_allowed = 0;
                _log_index = 0;
                break;                
    }
    
    // switch dati sensori locali o remoti e impostazione 
    switch ((_chart_2_plot & 0xF0) >>4 ){
        case 1: _eeprom_dev_address = EEPROM_I2C_DEV_ADDR_B1;
                _hdc1080_temp_cal = HDC1080_TEMP_CAL;                                   // carico i valori di calibrazione dei sensori locali
                _hdc1080_humid_cal = HDC1080_HUMID_CAL;
                _chart_plotted = _chart_2_plot;

                break;
        case 2: _eeprom_dev_address = EEPROM_I2C_DEV_ADDR_B2;
                _hdc1080_temp_cal = HDC1080_REM_TEMP_CAL;                               // carico i valori di calibrazione dei sensori remoti
                _hdc1080_humid_cal = HDC1080_REM_HUMID_CAL;
                _chart_plotted = _chart_2_plot;
                break;
                        
    }
    
    // switch tipo di dati
    switch (_chart_2_plot & 0x0F){
                                                                                        // temperatura
        case 1: _eeprom_addr_offset_1 = 7;                                              // carico gli offset per la lettura dei dati dalla eeprom        
                _eeprom_addr_offset_2 = 14;
                _eeprom_addr_offset_3 = 8;
                _byte_2_read = 2;                                                       // numero di byte da leggere
                _limit_min = -25;
                _limit_max = 65;
                TFT_draw_line_x(CHART_X,CHART_X_END, CHART_Y + CHART_X_OFFSET_Y_TEMP, CHART_AXIS_COLOR);            // linee x y grafico e unit di misura
                TFT_draw_line_y(CHART_Y,CHART_Y_END, CHART_X + CHART_Y_OFFSET_X, CHART_AXIS_COLOR);
                TFT_write_char(LOG_MIN_TEXT_X+30,LOG_MIN_TEXT_Y,127,SAVESPACE_ON,LOG_MIN_COLOR,BACK_COLOR); 
                TFT_write_char(LOG_MIN_TEXT_X+35,LOG_MIN_TEXT_Y,'C',SAVESPACE_ON,LOG_MIN_COLOR,BACK_COLOR); 
                TFT_write_char(LOG_MAX_TEXT_X+30,LOG_MAX_TEXT_Y,127,SAVESPACE_ON,LOG_MAX_COLOR,BACK_COLOR); 
                TFT_write_char(LOG_MAX_TEXT_X+35,LOG_MAX_TEXT_Y,'C',SAVESPACE_ON,LOG_MAX_COLOR,BACK_COLOR); 
                break;
        case 2: _eeprom_addr_offset_1 = 9;                                              // RH
                _eeprom_addr_offset_2 = 12;
                _eeprom_addr_offset_3 = 10;
                _byte_2_read = 2;
                _limit_min = 0;
                _limit_max = 100;
                TFT_draw_line_x(CHART_X,CHART_X_END, CHART_Y + CHART_X_OFFSET_Y, CHART_AXIS_COLOR);
                TFT_draw_line_y(CHART_Y,CHART_Y_END, CHART_X + CHART_Y_OFFSET_X, CHART_AXIS_COLOR);
                TFT_write_char(LOG_MIN_TEXT_X+30,LOG_MIN_TEXT_Y,'%',SAVESPACE_ON,LOG_MIN_COLOR,BACK_COLOR);
                TFT_write_char(LOG_MAX_TEXT_X+30,LOG_MAX_TEXT_Y,'%',SAVESPACE_ON,LOG_MAX_COLOR,BACK_COLOR); 
                break;                
        case 3: _eeprom_addr_offset_1 = 11;                                             // pressione
                _eeprom_addr_offset_2 = 8;
                _eeprom_addr_offset_3 = 14;
                _byte_2_read = 4;
                _limit_min = 950;
                _limit_max = 1050;
                TFT_draw_line_x(CHART_X,CHART_X_END, CHART_Y + CHART_X_OFFSET_Y, CHART_AXIS_COLOR);
                TFT_draw_line_y(CHART_Y,CHART_Y_END, CHART_X + CHART_Y_OFFSET_X, CHART_AXIS_COLOR);
                TFT_write_string(LOG_MIN_TEXT_X+30,LOG_MIN_TEXT_Y,30,"hPa",SAVESPACE_ON,LOG_MIN_COLOR,BACK_COLOR);
                TFT_write_string(LOG_MAX_TEXT_X+30,LOG_MAX_TEXT_Y,30,"hPa",SAVESPACE_ON,LOG_MAX_COLOR,BACK_COLOR); 
                break;
        case 4: _eeprom_addr_offset_1 = 15;                                             // UVA
                _eeprom_addr_offset_2 = 0;
                _eeprom_addr_offset_3 = 22;
                _byte_2_read = 8;
                _limit_min = 0;
                _limit_max = 20;
                TFT_draw_line_x(CHART_X,CHART_X_END, CHART_Y + CHART_X_OFFSET_Y, CHART_AXIS_COLOR);
                TFT_draw_line_y(CHART_Y,CHART_Y_END, CHART_X + CHART_Y_OFFSET_X, CHART_AXIS_COLOR);  
                TFT_write_string(LOG_MIN_TEXT_X+33,LOG_MIN_TEXT_Y,30,"mW",SAVESPACE_ON,LOG_MIN_COLOR,BACK_COLOR);
                TFT_write_string(LOG_MAX_TEXT_X+30,LOG_MAX_TEXT_Y,30,"cmq",SAVESPACE_ON,LOG_MAX_COLOR,BACK_COLOR);
                TFT_draw_line_x(LOG_MAX_TEXT_X+30,LOG_MAX_TEXT_X+57,LOG_MAX_TEXT_Y, LOG_MAX_COLOR);  
                break; 
        case 5: _eeprom_addr_offset_1 = 15;                                             // UVB
                _eeprom_addr_offset_2 = 0;
                _eeprom_addr_offset_3 = 22;
                _byte_2_read = 8;
                _limit_min = 0;
                _limit_max = 20;
                TFT_draw_line_x(CHART_X,CHART_X_END, CHART_Y + CHART_X_OFFSET_Y, CHART_AXIS_COLOR);
                TFT_draw_line_y(CHART_Y,CHART_Y_END, CHART_X + CHART_Y_OFFSET_X, CHART_AXIS_COLOR);         
                TFT_write_string(LOG_MIN_TEXT_X+33,LOG_MIN_TEXT_Y,30,"mW",SAVESPACE_ON,LOG_MIN_COLOR,BACK_COLOR);
                TFT_write_string(LOG_MAX_TEXT_X+30,LOG_MAX_TEXT_Y,30,"cmq",SAVESPACE_ON,LOG_MAX_COLOR,BACK_COLOR);
                TFT_draw_line_x(LOG_MAX_TEXT_X+30,LOG_MAX_TEXT_X+57,LOG_MAX_TEXT_Y, LOG_MAX_COLOR); 
                break;
        case 6: _eeprom_addr_offset_1 = 15;                                             // UVI
                _eeprom_addr_offset_2 = 0;
                _eeprom_addr_offset_3 = 22;
                _byte_2_read = 8;
                _limit_min = 0;
                _limit_max = 15;
                TFT_draw_line_x(CHART_X,CHART_X_END, CHART_Y + CHART_X_OFFSET_Y, CHART_AXIS_COLOR);
                TFT_draw_line_y(CHART_Y,CHART_Y_END, CHART_X + CHART_Y_OFFSET_X, CHART_AXIS_COLOR);                
                break;                                                 
                
    }
    
    _eeprom_addr = 4600 * _log_index;                                                   // offset di 200 letture a seconda di quante ne sono gi state lette  (23 byte ogni salvataggio * 200 salvataggi = 4600 byte)
    
    while (EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr) == eeprom_log_index && _x < 200){      // leggo finch i dati fanno parte della registrazione log attuale (in corso o ultima conclusa))  o per massimo 200 campioni
        if (_x == 0){                                                                   // per primo calore scrivo la data di inizio
            _eeprom_addr++;                                                             
            _timestamp_string = Create_timestamp_string(_eeprom_dev_address,_eeprom_addr,5);
            TFT_write_string(LOG_TIME_START_X,LOG_TIME_START_Y,110,_timestamp_string,SAVESPACE_ON,LOG_TIME_START_COLOR,BACK_COLOR); 
                
            _eeprom_addr += _eeprom_addr_offset_1 -1;                                   // dati letti da lettura data per arrivare a primo dato valido
        } else {
            _eeprom_addr += _eeprom_addr_offset_1;                                      // offset primo byte dati valido se data non letta
        }
        
        // lettura dati per temperatura, umidit e pressione        
        if (_byte_2_read != 8){
            for (unsigned char _i = 0; _i < _byte_2_read; _i++){
                _temp = _temp << 8;
                _temp |= EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                _eeprom_addr ++;
            }
        // lettura dati per UVA UVB e UVI, i dati vanno letti sempre tutti per il calcolo dei valori.
        } else if (_byte_2_read == 8) {
            _readed_uv_raw.uva = EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
            _eeprom_addr++;
            _readed_uv_raw.uva = (_readed_uv_raw.uva << 8) | EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
            _eeprom_addr++;
            _readed_uv_raw.uvb = EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
            _eeprom_addr++;
            _readed_uv_raw.uvb = (_readed_uv_raw.uvb << 8) | EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
            _eeprom_addr++;
            _readed_uv_raw.uv_comp_1 = EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
            _eeprom_addr++;
            _readed_uv_raw.uv_comp_1 = (_readed_uv_raw.uv_comp_1 << 8) | EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
            _eeprom_addr++;
            _readed_uv_raw.uv_comp_2 = EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
            _eeprom_addr++;
            _readed_uv_raw.uv_comp_2 = (_readed_uv_raw.uv_comp_2 << 8) | EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
            _eeprom_addr++;
        }
        
        // calcolo il valore da esporre a seconda del tipo di dato
        switch (_chart_2_plot & 0x0F){
            case 1: _value = HDC1080_temp_raw_2_float((unsigned int)_temp) + _hdc1080_temp_cal;
                break;
            case 2: _value = HDC1080_humidity_raw_2_float((unsigned int)_temp) + _hdc1080_humid_cal; 
                break;    
            case 3: _value = LPS25HB_press_raw_2_float(_temp);
                break;
            case 4: _readed_uv = VEML6075_convert_uv_raw (_readed_uv_raw);
                    // converto da uW/cm^2 a mW/cm^2          
                    _value = _readed_uv.uva/1000;
                break;
            case 5: _readed_uv = VEML6075_convert_uv_raw (_readed_uv_raw);
                    // converto da uW/cm^2 a mW/cm^2          
                    _value = _readed_uv.uvb/1000;
                break;
            case 6: _readed_uv = VEML6075_convert_uv_raw (_readed_uv_raw);
                    _value = _readed_uv.uvi;
                break;                
        }
        
        // controllo che i valori siano nei limiti dell'area del grafico, in caso contrario imposto il valore limite
        if (_value > _limit_max)
            _value = _limit_max;
        if (_value < _limit_min)
            _value = _limit_min;
        
        // il primo valore letto lo imposto come minimo e massimo e scrivo la data        
        if (_x == 0){
            _min = _value;
            _max = _value;
            _eeprom_addr_min = _eeprom_addr;
            _eeprom_addr_max = _eeprom_addr;
        }                
        
        // check minimi e massimi
        if (_value < _min){
            _min = _value;
            _eeprom_addr_min = _eeprom_addr;
        }
        if (_value > _max){
            _max = _value; 
            _eeprom_addr_max = _eeprom_addr;
        }
        
        // adattamento valori rilevati a valori grafico da 0 a 100 (punti asse Y)
        switch (_chart_2_plot & 0x0F){
            case 1: _value += 20;                                                       // porto valori a 0 (supponendo un minimo di -20C)
                    _value *= 1.1765;                                                   // scalo da  0 a 100  (scala valori da -20 a +65 in da 0 a 100)
                    break;
//            case 2:                                                                   // non  necessario adattamento, i valori vanno gi da 0 a 100
//                    break;
            case 3: _value -= 950;                                                       
                    break;                    
            case 4: _value *= 5;                                                        // UVA e UVB massimo presupposto 20mW, *5 porto da 0 a 100
                    break;
            case 5: _value *= 5;
                    break;
            case 6: _value *= 6.667;                                                    // valori da 0 a 15
                    break;
                
        }
       
        // disegno sul grafico
        if (_x == 0)
            // punto    
            TFT_draw_pixel(_x + CHART_X + CHART_Y_OFFSET_X + 1, (100 -((signed int)_value)+ CHART_Y), CHART_LINE_COLOR);
        else
            // linea da punto precedente a nuovo punto
            TFT_draw_line_p2p(_x + CHART_X + CHART_Y_OFFSET_X, (100 -((signed int)_value_pre)+ CHART_Y), _x + CHART_X + CHART_Y_OFFSET_X + 1, (100 -((signed int)_value)+ CHART_Y), CHART_LINE_COLOR);
        
        _value_pre = _value;
        _x++;
        _eeprom_addr += _eeprom_addr_offset_2;                                          // offset da ultima lettura a prossimo indice prossimo valore
    }                                                                                   // fine ciclo lettura eeprom
    
    // se _x == 200 significa che il ciclo si  fermato per limite valori a 200 e ci sono altri valori da leggere, abilito la lettura ai prossimi valori
    if (_x == 200){
        TFT_draw_image_sd(ARROW_NEXT_X,ARROW_NEXT_Y,"NEXT.bmp");
        log_next_allowed = 1;
    // altrimenti il ciclo  finito perch sono stati letti tutti i valori con lo stesso eeprom_log_index e non ci sono altri valori
    } else if (_x < 200) {  
        TFT_fill_rect(ARROW_NEXT_X,ARROW_NEXT_Y,16,12,BACK_COLOR);
        log_next_allowed = 0;
    }
    
    // abilito o disabilito lettura dati precedenti
    if (_log_index > 0){
        TFT_draw_image_sd(ARROW_PREV_X,ARROW_PREV_Y,"PREV.bmp");
        log_prev_allowed = 1;            
    }else{
        TFT_fill_rect(ARROW_PREV_X,ARROW_PREV_Y,16,12,BACK_COLOR);
        log_prev_allowed = 0;
    }
    
    // scrivo la data di fine grafico dell'ultimo valore letto se ho letto almeno 1 valore
    if (_x != 0){
        _timestamp_string = Create_timestamp_string(_eeprom_dev_address,_eeprom_addr -22,5);
        TFT_write_string(LOG_TIME_END_X,LOG_TIME_END_Y,110,_timestamp_string,SAVESPACE_ON,WHITE,BACK_COLOR);
    
        // scrivo i valori minimi e massimi rilevati e relativa data
        TFT_write_float(LOG_MIN_VAL_X,LOG_MIN_TEXT_Y,48,_min, 5, 2, ZERO_CLEANING_ON, SAVESPACE_ON, LOG_MIN_COLOR, BACK_COLOR); 
        _timestamp_string = Create_timestamp_string(_eeprom_dev_address,_eeprom_addr_min - _eeprom_addr_offset_3,5);
        TFT_write_string(LOG_MIN_TIME_X,LOG_MIN_TEXT_Y,112,_timestamp_string,SAVESPACE_ON,LOG_MIN_COLOR,BACK_COLOR);                                                                                                                                                               
        TFT_write_float(LOG_MAX_VAL_X,LOG_MAX_TEXT_Y,48,_max, 5, 2, ZERO_CLEANING_ON, SAVESPACE_ON, LOG_MAX_COLOR, BACK_COLOR); 
        _timestamp_string = Create_timestamp_string(_eeprom_dev_address,_eeprom_addr_max - _eeprom_addr_offset_3,5);
        TFT_write_string(LOG_MAX_TIME_X,LOG_MAX_TEXT_Y,112,_timestamp_string,SAVESPACE_ON,LOG_MAX_COLOR,BACK_COLOR);
    }
    
    return _chart_plotted;
}

/* ============================================================================================================================================= /
 * FUNCTION:    Export_sensors_log
 * Input:       *_file_name[]: Stringa con nome del file
 *
 * Overview:    Esporta i dati del salvataggio log corrente su file csv nella scheda SD.
 * Nota:        Se il file  gi presente viene sovrascritto
/* ============================================================================================================================================ */
void Export_sensors_log(const unsigned char *_file_name){
    unsigned long _temp = 0;
    volatile unsigned int _eeprom_addr = 0;
    volatile unsigned char _eeprom_dev_address = 0;
    sensors_values _readed_values;
    veml6075_uv_raw _readed_uv_raw;
    unsigned char *_string_buffer;                                                      // puntatore a buffer di ritorno valori float convertiti in stringa
    unsigned char *_timestamp_string;                                                   // puntatore a buffer di ritorno contenente la stringa con data e ora 
    float _hdc1080_temp_cal = 0.0;
    float _hdc1080_humid_cal = 0.0;
//    float _lps25_temp_cal = 0.0;
    
    #ifdef DEBUG_LOG_SD
    Write_log(READ_LOG);
    #endif
    
    
    if (f_mount(&sd_card, "", 1) == FR_OK) {                                            // Mount SD
        if (f_open(&file, _file_name, FA_CREATE_ALWAYS | FA_READ | FA_WRITE) == FR_OK){ // creo sempre un nuovo file
            f_lseek(&file, file.fsize);                                                 // Lettura ultimo settore scritto

            // ciclo 2 banchi della memoria EEPORM
            for (unsigned char _g = 0; _g < 2; _g++){
                // intestazione file e variabili di offset a seconda dei dati letti
                switch (_g){
                    case 0: f_write(&file, "LOCAL;TEMP;RH%;PRESS mBar;UVA mW/cm2;UVB mW/cm2;UVI;\r\n", 54, &bw);      
                            _eeprom_dev_address = EEPROM_I2C_DEV_ADDR_B1;
                            _hdc1080_temp_cal = HDC1080_TEMP_CAL;
                            _hdc1080_humid_cal = HDC1080_HUMID_CAL;
//                            _lps25_temp_cal = LPS25HB_TEMP_CAL;
                            break;
                    case 1: f_write(&file, "REMOTE;TEMP;RH%;PRESS mBar;UVA mW/cm2;UVB mW/cm2;UVI;\r\n", 55, &bw);      
                            _eeprom_dev_address = EEPROM_I2C_DEV_ADDR_B2;
                            _hdc1080_temp_cal = HDC1080_REM_TEMP_CAL;
                            _hdc1080_humid_cal = HDC1080_REM_HUMID_CAL;
//                            _lps25_temp_cal = LPS25HB_REM_TEMP_CAL;
                            break;
                }
                
                _eeprom_addr = 0;
                
                // ciclo lettura dati salvati
                while (EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr) == eeprom_log_index){ // leggo finch i dati fanno parte della registrazione log attuale (in corso o ultima conclusa)
                    _eeprom_addr++;
                    
                    // lettura giorno , mese, anno, ora, minuti e secondi
                    _timestamp_string = Create_timestamp_string(_eeprom_dev_address,_eeprom_addr,6);
                    _eeprom_addr += 6;
                    
                    // lettura valori sensori e conversione in float
                    // temp hdc1080
                    _temp = EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _eeprom_addr++;
                    _temp = (_temp << 8) | EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _readed_values.temp_hdc1080 = HDC1080_temp_raw_2_float((unsigned int)_temp) + _hdc1080_temp_cal;
                    _eeprom_addr++;

                    // RH hdc1080
                    _temp = EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _eeprom_addr++;
                    _temp = (_temp << 8) | EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _readed_values.humidity = HDC1080_humidity_raw_2_float((unsigned int)_temp) + _hdc1080_humid_cal;
                    _eeprom_addr++;

                    // press
                    _temp = EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _eeprom_addr++;
                    _temp = (_temp << 8) | EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _eeprom_addr++;
                    _temp = (_temp << 8) | EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _eeprom_addr++;
                    _temp = (_temp << 8) | EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _readed_values.press = LPS25HB_press_raw_2_float(_temp);
                    _eeprom_addr++;

                    if (press_to_write == 1)                                            // pressione normalizzata
                        _readed_values.press = Normalize_press(_readed_values.press, _readed_values.temp_hdc1080, local_altitude);

                    // temp LPS25HB
//                    _temp = EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
//                    _eeprom_addr++;
//                    _temp = (_temp << 8) | EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
//                    _readed_values.temp_lps25 = LPS25HB_temp_raw_2_float((unsigned int)_temp) + _lps25_temp_cal;
//                    _eeprom_addr++;

                    // UVA
                    _readed_uv_raw.uva = EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _eeprom_addr++;
                    _readed_uv_raw.uva = (_readed_uv_raw.uva << 8) | EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _eeprom_addr++;

                    // UVB
                    _readed_uv_raw.uvb = EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _eeprom_addr++;
                    _readed_uv_raw.uvb = (_readed_uv_raw.uvb << 8) | EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _eeprom_addr++;

                    // UVComp1
                    _readed_uv_raw.uv_comp_1 = EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _eeprom_addr++;
                    _readed_uv_raw.uv_comp_1 = (_readed_uv_raw.uv_comp_1 << 8) | EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _eeprom_addr++;

                    // UVComp2
                    _readed_uv_raw.uv_comp_2 = EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _eeprom_addr++;
                    _readed_uv_raw.uv_comp_2 = (_readed_uv_raw.uv_comp_2 << 8) | EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
                    _eeprom_addr++;    

                    _readed_values.uv_readed = VEML6075_convert_uv_raw(_readed_uv_raw);

                    // converto da uW/cm^2 a mW/cm^2  
                    _readed_values.uv_readed.uva = _readed_values.uv_readed.uva/1000;
                    _readed_values.uv_readed.uvb = _readed_values.uv_readed.uvb/1000;  

                    // scrittura data e ora nel file
                    f_write(&file, _timestamp_string, 17, &bw);   
                    f_write(&file, ";", 1, &bw);   
                    
                    // conversione float in stringa e scrittura dei valori nel file
                    _string_buffer = Float_2_string(_readed_values.temp_hdc1080, TEMP_INT_DIGITS,TEMP_DEC_DIGITS, ZERO_CLEANING_OFF);
                    f_write(&file, _string_buffer, 6, &bw);   
                    f_write(&file, ";", 1, &bw);   

                    _string_buffer = Float_2_string(_readed_values.humidity, RH_INT_DIGITS,RH_DEC_DIGITS, ZERO_CLEANING_OFF);
                    f_write(&file, _string_buffer, 6, &bw);   
                    f_write(&file, ";", 1, &bw);  

                    _string_buffer = Float_2_string(_readed_values.press, PRESS_INT_DIGITS,PRESS_DEC_DIGITS, ZERO_CLEANING_OFF);
                    f_write(&file, _string_buffer, 6, &bw);   
                    f_write(&file, ";", 1, &bw);  

                    _string_buffer = Float_2_string(_readed_values.uv_readed.uva, UV_INT_DIGITS,UV_DEC_DIGITS, ZERO_CLEANING_OFF);
                    f_write(&file, _string_buffer, 6, &bw);   
                    f_write(&file, ";", 1, &bw);  

                    _string_buffer = Float_2_string(_readed_values.uv_readed.uvb, UV_INT_DIGITS,UV_DEC_DIGITS, ZERO_CLEANING_OFF);
                    f_write(&file, _string_buffer, 6, &bw);   
                    f_write(&file, ";", 1, &bw);

                    _string_buffer = Float_2_string(_readed_values.uv_readed.uvi, UVI_INT_DIGITS,UVI_DEC_DIGITS, ZERO_CLEANING_OFF);
                    f_write(&file, _string_buffer, 5 , &bw);   
                    f_write(&file, ";", 1, &bw);

                    f_write(&file, "\r\n", 2, &bw);   
                }
            }
            f_write(&file, "\r\n", 2, &bw);   
            f_close(&file);                                                             // chiusura file
        }
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Setup_time_manual
 *
 * Overview:    Impostazione della data e dell'ora manuali
/* ============================================================================================================================================ */
void Setup_time_manual(void){
    rtcc_data _rtcc_time;
    unsigned char *_pointer_value = 0;                                                  // puntatore a valori temporanei da modificare
    _rtcc_time = RTCC_read_all();                                                       // variabile temporanea per menu
    unsigned char _exit = 0;
    unsigned char _dcf77_user_enable = dcf77_user_enable;
    unsigned char _cet_cest_flag = cet_cest_flag;
    
    _pointer_value = &_rtcc_time.day_w;
    
    TFT_fill_rect(1,17,240,280, BACK_COLOR);
    
//    TFT_set_font(font_lucida_sans_10x16);
    TFT_write_string(SETUP_TEXT_GEN_X,SETUP_TEXT_GEN_Y,230,setup_messages[7],SAVESPACE_ON,SETUP_TITLE_COLOR,BACK_COLOR);                            // "Impostazioni data/ora"
    TFT_write_string(SETUP_TIME_DAY_W_X,SETTING_LINE_1_Y,100,setup_messages[8],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                     // "Giorno"
    TFT_write_string(SETUP_TIME_DAY_W_VALUE_X,SETTING_LINE_1_Y,80,name_day[_rtcc_time.day_w],SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
    TFT_write_string(SETUP_TIME_DATE_X,SETTING_LINE_2_Y,100,setup_messages[9],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                      // "Data"
    TFT_write_PBCD(SETUP_TIME_DAY_VALUE_X,SETTING_LINE_2_Y,SETUP_DATE_X_LEN,_rtcc_time.day,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    TFT_write_char(SETUP_DATE_SEP_1_X,SETTING_LINE_2_Y,'/',SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    TFT_write_PBCD(SETUP_TIME_MONTH_VALUE_X,SETTING_LINE_2_Y,SETUP_DATE_X_LEN,_rtcc_time.month,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    TFT_write_char(SETUP_DATE_SEP_2_X,SETTING_LINE_2_Y,'/',SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    TFT_write_PBCD(SETUP_TIME_YEAR_VALUE_X,SETTING_LINE_2_Y,SETUP_DATE_X_LEN,_rtcc_time.year,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    
    TFT_write_string(SETUP_HOUR_X,SETTING_LINE_3_Y,80,setup_messages[10],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                           // "Ora"
    TFT_write_PBCD(SETUP_HOUR_VALUE_X,SETTING_LINE_3_Y,SETUP_TIME_X_LEN,_rtcc_time.hours,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    TFT_write_char(SETUP_HOUR_DOTS_X,SETTING_LINE_3_Y,':',SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    TFT_write_PBCD(SETUP_MINUTES_VALUE_X,SETTING_LINE_3_Y,SETUP_TIME_X_LEN,_rtcc_time.minutes,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    
    TFT_write_string(SETUP_DCF77_X,SETTING_LINE_4_Y,80,setup_messages[31],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                           // "CET/CEST:"
    TFT_write_string(SETUP_DCF77_VALUE_X,SETTING_LINE_4_Y,80,cet_cest[_cet_cest_flag],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);                     // "On/off"
    
    TFT_write_string(SETUP_DCF77_X,SETTING_LINE_5_Y,80,setup_messages[30],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                           // "DCF77: "
    TFT_write_string(SETUP_DCF77_VALUE_X,SETTING_LINE_5_Y,80,on_off[_dcf77_user_enable],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);                  // "On/off"
    
    Draw_switch(SW_SETUP_INCR);
    
    XPT2046_enable();                                                                   // per settare parametri corretti SPI (SD  impostata + veloce)
    do {
        if (XPT2046_IRQ == 0) {                                                         // se  stato toccato lo schermo

            switch (Touch_screen_manage_loop(TOUCH_SETUP_TIME)){            
                case T_SETUP_HOURS:
                    _pointer_value = &_rtcc_time.hours;                                 // imposto puntatore a valore da modificare a seconda del valore selezionato
                    break;
                case T_SETUP_MINUTES:
                    _pointer_value = &_rtcc_time.minutes;
                    break;    
                case T_SETUP_DAY:
                    _pointer_value = &_rtcc_time.day;
                    break; 
                case T_SETUP_MONTH:
                    _pointer_value = &_rtcc_time.month;
                    break; 
                case T_SETUP_YEAR:
                    _pointer_value = &_rtcc_time.year;
                    break; 
                case T_SETUP_DAY_W:
                    _pointer_value = &_rtcc_time.day_w;
                    break;   
                case T_SETUP_CET_CEST:    
                    _pointer_value = &_cet_cest_flag;
                    break;   
                case T_SETUP_DCF77:
                    _pointer_value = &_dcf77_user_enable;
                    break;
                    
                case T_SW_1:                                                            // incrementi
                    Draw_switch(T_SW_PLUS);
                    if (_pointer_value == &_rtcc_time.hours)
                        *_pointer_value = Inc_pbcd(*_pointer_value, 0x00, 0x23);
                    else if (_pointer_value == &_rtcc_time.minutes)
                        *_pointer_value = Inc_pbcd(*_pointer_value, 0x00, 0x59);
                    else if (_pointer_value == &_rtcc_time.day)
                        *_pointer_value = Inc_pbcd(*_pointer_value, 0x01, 0x31);
                    else if (_pointer_value == &_rtcc_time.month)
                        *_pointer_value = Inc_pbcd(*_pointer_value, 0x01, 0x12);
                    else if (_pointer_value == &_rtcc_time.year)
                        *_pointer_value = Inc_pbcd(*_pointer_value, 0x00, 0x99);
                    else if (_pointer_value == &_rtcc_time.day_w){
                        *_pointer_value = *_pointer_value +1;
                        if (*_pointer_value == 7)
                            *_pointer_value = 0;
                    } else if (_pointer_value == &_dcf77_user_enable || _pointer_value == &_cet_cest_flag)
                        *_pointer_value = !(*_pointer_value);
                    break;
                case T_SW_2:                                                            // decrementi
                    Draw_switch(T_SW_LESS);
                    if (_pointer_value == &_rtcc_time.hours)
                        *_pointer_value = Dec_pbcd(*_pointer_value, 0x00, 0x23);
                    else if (_pointer_value == &_rtcc_time.minutes)
                        *_pointer_value = Dec_pbcd(*_pointer_value, 0x00, 0x59);
                    else if (_pointer_value == &_rtcc_time.day)
                        *_pointer_value = Dec_pbcd(*_pointer_value, 0x01, 0x31);
                    else if (_pointer_value == &_rtcc_time.month)
                        *_pointer_value = Dec_pbcd(*_pointer_value, 0x01, 0x12);
                    else if (_pointer_value == &_rtcc_time.year)
                        *_pointer_value = Dec_pbcd(*_pointer_value, 0x00, 0x99);
                    else if (_pointer_value == &_rtcc_time.day_w){
                        *_pointer_value = *_pointer_value -1;
                        if (*_pointer_value == 0xFF)
                            *_pointer_value = 6;
                    } else if (_pointer_value == &_dcf77_user_enable || _pointer_value == &_cet_cest_flag)
                        *_pointer_value = !(*_pointer_value);
                    break;

                case T_SW_3:
                    Draw_switch(T_SW_OK);
                    _exit = 1;
                    break;
                case T_SW_4:
                    Draw_switch(T_SW_EXIT);
                    _exit = 2;
                    break;    
            }
            
//            TFT_set_font(font_lucida_sans_10x16);
            
            // riscrivo tutti i valori
            TFT_write_string(SETUP_TIME_DAY_W_VALUE_X,SETTING_LINE_1_Y,80,name_day[_rtcc_time.day_w],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);     
            TFT_write_PBCD(SETUP_TIME_DAY_VALUE_X,SETTING_LINE_2_Y,SETUP_DATE_X_LEN,_rtcc_time.day,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
            TFT_write_PBCD(SETUP_TIME_MONTH_VALUE_X,SETTING_LINE_2_Y,SETUP_DATE_X_LEN,_rtcc_time.month,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
            TFT_write_PBCD(SETUP_TIME_YEAR_VALUE_X,SETTING_LINE_2_Y,SETUP_DATE_X_LEN,_rtcc_time.year,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
            TFT_write_PBCD(SETUP_HOUR_VALUE_X,SETTING_LINE_3_Y,SETUP_TIME_X_LEN,_rtcc_time.hours,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
            TFT_write_PBCD(SETUP_MINUTES_VALUE_X,SETTING_LINE_3_Y,SETUP_TIME_X_LEN,_rtcc_time.minutes,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
            TFT_write_string(SETUP_DCF77_VALUE_X,SETTING_LINE_4_Y,80,cet_cest[_cet_cest_flag],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);    
            TFT_write_string(SETUP_DCF77_VALUE_X,SETTING_LINE_5_Y,80,on_off[_dcf77_user_enable],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);    
            
            // riscrivo in rosso solo il valore selezionato o modificato
            if (_pointer_value == &_rtcc_time.day_w)
                TFT_write_string(SETUP_TIME_DAY_W_VALUE_X,SETTING_LINE_1_Y,80,name_day[_rtcc_time.day_w],SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);     
            else if (_pointer_value == &_rtcc_time.day)
                TFT_write_PBCD(SETUP_TIME_DAY_VALUE_X,SETTING_LINE_2_Y,SETUP_DATE_X_LEN,_rtcc_time.day,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
            else if (_pointer_value == &_rtcc_time.month)
                TFT_write_PBCD(SETUP_TIME_MONTH_VALUE_X,SETTING_LINE_2_Y,SETUP_DATE_X_LEN,_rtcc_time.month,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
            else if (_pointer_value == &_rtcc_time.year)
                TFT_write_PBCD(SETUP_TIME_YEAR_VALUE_X,SETTING_LINE_2_Y,SETUP_DATE_X_LEN,_rtcc_time.year,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
            else if (_pointer_value == &_rtcc_time.hours)
                TFT_write_PBCD(SETUP_HOUR_VALUE_X,SETTING_LINE_3_Y,SETUP_TIME_X_LEN,_rtcc_time.hours,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
            else if (_pointer_value == &_rtcc_time.minutes)
                TFT_write_PBCD(SETUP_MINUTES_VALUE_X,SETTING_LINE_3_Y,SETUP_TIME_X_LEN,_rtcc_time.minutes,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
            else if (_pointer_value == &_cet_cest_flag)
                TFT_write_string(SETUP_DCF77_VALUE_X,SETTING_LINE_4_Y,80,cet_cest[_cet_cest_flag],SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);                  
            else if (_pointer_value == &_dcf77_user_enable)
                TFT_write_string(SETUP_DCF77_VALUE_X,SETTING_LINE_5_Y,80,on_off[_dcf77_user_enable],SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);                  
      
//            Delay_ms(200);
        }
    }while(_exit == 0); 
    
    // se  stato premuto il tasto salva salvo i valori nel RTCC
    if (_exit == 1){
        RTCC_write_all(_rtcc_time.day, _rtcc_time.month, _rtcc_time.year, _rtcc_time.day_w, _rtcc_time.hours, _rtcc_time.minutes, 0x00);
        EEPROM_write(_dcf77_user_enable,EEPROM_DCF77_USER_EN_ADDRESS);
        dcf77_user_enable = _dcf77_user_enable;
        cet_cest_flag = _cet_cest_flag;
        
        // fermo ricezione se in corso e resetto contatore dati ricevuti
        DCF77_stop_receive_via_interrupt();
        dcf77_received_counter = 0;
        dcf77_status = DCF77_DISABLED;
        first_sync = 1;                                                                 // forzo a 1 first_sync per ricaricare i dati meteo se la data  corretta
        Calc_meteo_time(meteo_zone,time_zone_utc, cet_cest_flag);                       // Calcolo orari di avvio DCF77 per meteo
            
        // se connesso a wifi creo il server
        if (server_closed_for_dcf77_sync == 1 && server_status == 0)
            Wifi_open_server();
            
//        }
    }
    
    TFT_fill_rect(1,17,240,280, BACK_COLOR);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Setup_utc
 *
 * Overview:    Impostazione UTC ed orario
/* ============================================================================================================================================ */
void Setup_utc(void){
    unsigned char _exit = 0;  // 1 = salva, 2 = esci
    rtcc_data _rtcc_time;
    signed char _utc = 0;
    
    // leggo l'impostazione attuale
    _utc = time_zone_utc;

    // scritte iniziali
    TFT_fill_rect(1,17,240,280, BACK_COLOR);
    
//    TFT_set_font(font_lucida_sans_10x16);
    TFT_write_string(SETUP_TEXT_GEN_X,SETUP_TEXT_GEN_Y,230,setup_messages[0],SAVESPACE_ON,SETUP_TITLE_COLOR,BACK_COLOR);           // "Impostare il fuso orario UTC"
    TFT_write_sint(SET_UTC_HOUR_X,SETTING_LINE_1_Y,50,_utc,2,ZERO_CLEANING_ON,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
    
    Draw_switch(SW_SETUP_INCR);
    
    XPT2046_enable();                                                                   // per settare parametri corretti SPI (SD  impostata + veloce)
    do {
        if (XPT2046_IRQ == 0) {                                                         // se  stato toccato lo schermo

            switch (Touch_screen_manage_loop(TOUCH_SETUP_INCR)){
                case T_SW_1:
                    Draw_switch(T_SW_PLUS);
                    _utc++;
                    if (_utc == 15)                                                     // limite massimo
                        _utc = -12;
                    break;
                case T_SW_2:
                    Draw_switch(T_SW_LESS);
                    _utc--;
                    if (_utc == -13)                                                    // limite minimo
                        _utc = 14;
                    break;
                case T_SW_3:
                    Draw_switch(T_SW_OK);
                    _exit = 1;
                    break;
                case T_SW_4:
                    Draw_switch(T_SW_EXIT);
                    _exit = 2;
                    break;    
            }
//            TFT_set_font(font_lucida_sans_10x16);
            TFT_write_sint(SET_UTC_HOUR_X,SETTING_LINE_1_Y,50,_utc,2,ZERO_CLEANING_ON,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
        }
    }while(_exit == 0);                                                                 // ripeto finch non viene premuto il tasto salva o il tasto esci
    
    // se  stato premuto il tasto salva e il valore _utc  cambiato
    if (_exit == 1 && _utc != time_zone_utc) {
        _rtcc_time = RTCC_read_all();                                                   // leggo ora attuale
        _rtcc_time = Set_time_zone(_rtcc_time, (~time_zone_utc) +1);                    // riporto a UTC 0
        _rtcc_time = Set_time_zone(_rtcc_time, _utc);                                   // adatto al nuovo fuso orario e salvo i valori nel RTCC
        RTCC_write_all(_rtcc_time.day, _rtcc_time.month, _rtcc_time.year, _rtcc_time.day_w, _rtcc_time.hours, _rtcc_time.minutes, _rtcc_time.seconds);
        
        time_zone_utc = _utc;                                                           // salvo il nuovo valore in variabile e eeprom
        EEPROM_write(_utc, EEPROM_UTC_ADDRESS);
        Calc_meteo_time(meteo_zone,time_zone_utc, cet_cest_flag);                       // Calcolo orari di avvio DCF77 per meteo
    }
    
    TFT_fill_rect(1,17,240,280, BACK_COLOR);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Setup_meteo_region
 *
 * Overview:    Impostazione della regione meteo
/* ============================================================================================================================================ */
void Setup_meteo_region (void){
    unsigned char _meteo_zone = 0;
    unsigned char _exit = 0;
    
    // leggo l'impostazione attuale
    _meteo_zone = meteo_zone;

    // scritte iniziali
    TFT_fill_rect(1,17,240,280, BACK_COLOR);;
    
//    TFT_set_font(font_lucida_sans_10x16);
    TFT_write_string(SETUP_TEXT_GEN_X,SETUP_TEXT_GEN_Y,230,setup_messages[1],SAVESPACE_ON,SETUP_TITLE_COLOR,BACK_COLOR);                // "Selezionare la citta per"
    TFT_write_string(SETUP_TEXT_GEN_X,SETUP_TEXT_GEN_Y +SETTING_LINE_Y_SPACE ,230,setup_messages[2],SAVESPACE_ON,SETUP_TITLE_COLOR,BACK_COLOR);           // "le previsioni meteo"
    TFT_write_unsint(SET_METEO_REGION_X,SET_METEO_REGION_Y,25,_meteo_zone,2,ZERO_CLEANING_ON,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
    TFT_write_char(SET_METEO_REGION_X +20,SET_METEO_REGION_Y,':',SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);  
    TFT_write_string(SET_METEO_REGION_VALUE_X,SET_METEO_REGION_Y,195,city[_meteo_zone],SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);  
    
    Draw_switch(SW_SETUP_INCR);
    
    XPT2046_enable();                                                                   // per settare parametri corretti SPI (SD  impostata + veloce)
    do {
        if (XPT2046_IRQ == 0) {                                                         // se  stato toccato lo schermo

            switch (Touch_screen_manage_loop(TOUCH_SETUP_INCR)){
                case T_SW_1:
                    Draw_switch(T_SW_PLUS);
                    _meteo_zone++;
                    if (_meteo_zone == 60)                                              // limite massimo
                        _meteo_zone = 0;
                    break;
                case T_SW_2:
                    Draw_switch(T_SW_LESS);
                    _meteo_zone--;
                    if (_meteo_zone == 0xFF)                                            // limite minimo
                        _meteo_zone = 59;
                    break;
                case T_SW_3:
                    Draw_switch(T_SW_OK);
                    _exit = 1;
                    break;
                case T_SW_4:
                    Draw_switch(T_SW_EXIT);
                    _exit = 2;
                    break;    
            }
//            TFT_set_font(font_lucida_sans_10x16);
            TFT_write_unsint(SET_METEO_REGION_X,SET_METEO_REGION_Y,18,_meteo_zone,2,ZERO_CLEANING_ON,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
            TFT_write_string(SET_METEO_REGION_VALUE_X,SET_METEO_REGION_Y,195,city[_meteo_zone],SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
        }
    }while(_exit == 0);  
    
    // se  stato premuto il tasto salva e il valore _utc  cambiato
    if (_exit == 1 && _meteo_zone != meteo_zone) {
        meteo_zone = _meteo_zone;                                                       // salvo il nuovo valore in variabile e eeprom
        EEPROM_write(_meteo_zone, EEPROM_METEO_ZONE_ADDRESS);
        
        Calc_meteo_time(_meteo_zone,time_zone_utc, cet_cest_flag);                      // calcolo i nuovi orari per avvio DCF77 meteo
    }
    TFT_fill_rect(1,17,240,280, BACK_COLOR);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Setup_sensors
 *
 * Overview:    Impostazione tempo scansione sensori, pressione e altitudine. Se viene variato il tempo di scansione del modulo remoto invia le 
 *              impostazioni
/* ============================================================================================================================================ */
void Setup_sensors(void){
    unsigned int _local_sensor_refr = 0;                                                // variabili temporanee per menu
    unsigned int _remote_sensor_refr = 0;
    unsigned char _remote_sensor_refr_old = 0;
    unsigned int _press_to_write = 0;
    unsigned int _local_altitude = 0;
    unsigned int *_pointer_value = 0;                                                   // puntatore alla variabile temporanea da modificare
    unsigned char _exit = 0;

    _local_altitude = local_altitude;
    _press_to_write = press_to_write;
    _local_sensor_refr = EEPROM_read(EEPROM_REFRESH_TIME_LOCAL_ADDRESS);
    _remote_sensor_refr = EEPROM_read(EEPROM_REFRESH_TIME_REMOTE_ADDRESS);
    _remote_sensor_refr_old = _remote_sensor_refr;
    
    _pointer_value = &_local_sensor_refr;
    
    // scritte iniziali
    TFT_fill_rect(1,17,240,280, BACK_COLOR);
    
    TFT_write_string(SETUP_TEXT_GEN_X,SETUP_TEXT_GEN_Y,230,setup_messages[11],SAVESPACE_ON,SETUP_TITLE_COLOR,BACK_COLOR);                           // "Impostazioni sensori"
    TFT_write_string(SETUP_SENSOR_LOCAL_X,SETTING_LINE_1_Y,100,setup_messages[12],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                  // "Locale"
    TFT_write_string(SETUP_SENSOR_LOCAL_VALUE_X,SETTING_LINE_1_Y,100,sensor_time[_local_sensor_refr],SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
    TFT_write_string(SETUP_SENSOR_REMOTE_X,SETTING_LINE_2_Y,100,setup_messages[13],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                 // "Remoto"
    TFT_write_string(SETUP_SENSOR_REMOTE_VALUE_X,SETTING_LINE_2_Y,100,sensor_time[_remote_sensor_refr],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    TFT_write_string(SETUP_PRESS_TO_WRITE_X,SETTING_LINE_3_Y,100,setup_messages[14],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                // "Pressione"
    TFT_write_string(SETUP_PRESS_TO_WRITE_VALUE_X,SETTING_LINE_3_Y,100,press_descr[_press_to_write],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);      
    TFT_write_string(SETUP_ALTITUDE_X,SETTING_LINE_4_Y,125,setup_messages[15],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                      // "Altitudine"
    TFT_write_unsint(SETUP_ALTITUDE_VALUE_X,SETTING_LINE_4_Y,50,_local_altitude,4,ZERO_CLEANING_ON,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    
    Draw_switch(SW_SETUP_INCR);
    
    XPT2046_enable();                                                                   // per settare parametri corretti SPI (SD  impostata + veloce)
    do {
        if (XPT2046_IRQ == 0) {                                                         // se  stato toccato lo schermo

            switch (Touch_screen_manage_loop(TOUCH_SETUP_SENSORS)){
                case T_SETUP_LOCAL_SENSOR:
                    _pointer_value = &_local_sensor_refr;
                    break;
                case T_SETUP_REMOTE_SENSOR:
                    _pointer_value = &_remote_sensor_refr;
                    break;    
                case T_SETUP_PRESS_TO_WRITE:
                    _pointer_value = &_press_to_write;
                    break; 
                case T_SETUP_LOCAL_ALTITUDE:
                    _pointer_value = &_local_altitude;
                    break; 
                case T_SW_1:
                    Draw_switch(T_SW_PLUS);
                    if (_pointer_value == &_local_sensor_refr || _pointer_value == &_remote_sensor_refr){
                        *_pointer_value = *_pointer_value +1;
                        if (*_pointer_value == 9)
                            *_pointer_value = 1;
                    } else if (_pointer_value == &_press_to_write)
                        *_pointer_value = ! *_pointer_value;
                    else {
                        *_pointer_value = *_pointer_value +1;
                        if (*_pointer_value == 3000)
                            *_pointer_value = 0;
                    }
                    break;
                        
                case T_SW_2:
                    Draw_switch(T_SW_LESS);
                    if (_pointer_value == &_local_sensor_refr || _pointer_value == &_remote_sensor_refr){
                        *_pointer_value = *_pointer_value -1;
                        if (*_pointer_value == 0)
                            *_pointer_value = 8;
                    } else if (_pointer_value == &_press_to_write)
                        *_pointer_value = ! *_pointer_value;
                    else {
                        *_pointer_value = *_pointer_value -1;
                        if (*_pointer_value == 0xFFFF)
                            *_pointer_value = 3000;
                    }
                    break;
                case T_SW_3:
                    Draw_switch(T_SW_OK);
                    _exit = 1;
                    break;
                case T_SW_4:
                    Draw_switch(T_SW_EXIT);
                    _exit = 2;
                    break;    
            }

//            TFT_set_font(font_lucida_sans_10x16);
            TFT_write_string(SETUP_SENSOR_LOCAL_VALUE_X,SETTING_LINE_1_Y,100,sensor_time[_local_sensor_refr],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
            TFT_write_string(SETUP_SENSOR_REMOTE_VALUE_X,SETTING_LINE_2_Y,100,sensor_time[_remote_sensor_refr],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
            TFT_write_string(SETUP_PRESS_TO_WRITE_VALUE_X,SETTING_LINE_3_Y,100,press_descr[_press_to_write],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);      
            TFT_write_unsint(SETUP_ALTITUDE_VALUE_X,SETTING_LINE_4_Y,50,_local_altitude,4,ZERO_CLEANING_ON,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
            
            if (_pointer_value == &_local_sensor_refr)
                TFT_write_string(SETUP_SENSOR_LOCAL_VALUE_X,SETTING_LINE_1_Y,100,sensor_time[_local_sensor_refr],SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
            else if (_pointer_value == &_remote_sensor_refr)
                TFT_write_string(SETUP_SENSOR_REMOTE_VALUE_X,SETTING_LINE_2_Y,100,sensor_time[_remote_sensor_refr],SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
            else if (_pointer_value == &_press_to_write)
                TFT_write_string(SETUP_PRESS_TO_WRITE_VALUE_X,SETTING_LINE_3_Y,100,press_descr[_press_to_write],SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);      
            else if (_pointer_value == &_local_altitude)
                TFT_write_unsint(SETUP_ALTITUDE_VALUE_X,SETTING_LINE_4_Y,50,_local_altitude,4,ZERO_CLEANING_ON,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
//            Delay_ms(200);
        }
    }while(_exit == 0); 
    
    if (_exit == 1){                                                                    // se  stato premuto il tasto salva
        local_altitude = _local_altitude;                                               // salvo le variabili temporanee nelle variabili globali
        press_to_write = _press_to_write;
        
        switch (_local_sensor_refr){                                                    // carico il tempo di scansione dei sensori locali
            case 1: time_refresh_local_sensors = TEMP_SCANS_SENS_1;
                break;
            case 2: time_refresh_local_sensors = TEMP_SCANS_SENS_2;
                break;            
            case 3: time_refresh_local_sensors = TEMP_SCANS_SENS_3;
                break;            
            case 4: time_refresh_local_sensors = TEMP_SCANS_SENS_4;
                break;            
            case 5: time_refresh_local_sensors = TEMP_SCANS_SENS_5;
                break;            
            case 6: time_refresh_local_sensors = TEMP_SCANS_SENS_6;
                break;            
            case 7: time_refresh_local_sensors = TEMP_SCANS_SENS_7;
                break;            
            case 8: time_refresh_local_sensors = TEMP_SCANS_SENS_8;
                break;            
        }
        
        switch (_remote_sensor_refr){                                                   // carico il tempo di scansione dei sensori remoti
            case 1: time_refresh_remote_sensors = TEMP_SCANS_SENS_1;
                break;
            case 2: time_refresh_remote_sensors = TEMP_SCANS_SENS_2;
                break;            
            case 3: time_refresh_remote_sensors = TEMP_SCANS_SENS_3;
                break;            
            case 4: time_refresh_remote_sensors = TEMP_SCANS_SENS_4;
                break;            
            case 5: time_refresh_remote_sensors = TEMP_SCANS_SENS_5;
                break;            
            case 6: time_refresh_remote_sensors = TEMP_SCANS_SENS_6;
                break;            
            case 7: time_refresh_remote_sensors = TEMP_SCANS_SENS_7;
                break;            
            case 8: time_refresh_remote_sensors = TEMP_SCANS_SENS_8;
                break;            
        }
        
        time_refresh_remote_sensors +=2;         
        
        // azzero i contatori 
        counter_refresh_local_sensors = 0;
        hmtrp_timeout_counter = 0;
        
        // salvo i valori in eeprom
        EEPROM_write(_local_sensor_refr, EEPROM_REFRESH_TIME_LOCAL_ADDRESS);
        EEPROM_write(_remote_sensor_refr, EEPROM_REFRESH_TIME_REMOTE_ADDRESS);
        EEPROM_write_int(_local_altitude,EEPROM_ALTITUDE_ADDRESS);
        EEPROM_write(_press_to_write,EEPROM_PRESS_2_WRITE_ADDRESS);
      
        if (_remote_sensor_refr != _remote_sensor_refr_old)                             // se il tempo di scansione dei sensori remoti  stato modificato
            Set_remote_aq_time(_remote_sensor_refr);                                    // avvio la procedura per impostare il modulo remoto
    } 
    
//    TFT_clear(BACK_COLOR);
    TFT_fill_rect(1,17,240,280, BACK_COLOR);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Setup_wifi
 *
 * Overview:    Impostazione attivazione globale WiFi, selezione rete ed esposizione valori fissi (IP, server, porta)
/* ============================================================================================================================================ */
void Setup_wifi(void){
    unsigned int _x = 0;
    unsigned char _exit = 0;
    unsigned char _i = 0;
        
    TFT_fill_rect(1,17,240,280, BACK_COLOR);
    
    Draw_switch(SW_SETUP_OK_EXT);
    TFT_write_string(SET_WIFI_SCAN_SSID_X,SETTING_LINE_8_Y,150,setup_messages[23],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);               // "Cerca rete"
    
//    TFT_set_font(font_lucida_sans_10x16);
    TFT_write_string(SETUP_TEXT_GEN_X,SETUP_TEXT_GEN_Y,230,setup_messages[16],SAVESPACE_ON,SETUP_TITLE_COLOR,BACK_COLOR);                   // "Impostazioni WiFi"
    TFT_write_string(SET_WIFI_EN_X,SETTING_LINE_1_Y,180,setup_messages[17],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                 // "Connessione WiFi"
    TFT_write_string(SET_WIFI_SSID_X,SETTING_LINE_2_Y,80,setup_messages[18],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                // "SSID:"
    TFT_write_string(SET_WIFI_STAT_X,SETTING_LINE_3_Y,80,setup_messages[19],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                // "Stato:"
    TFT_write_string(SET_WIFI_IP_X,SETTING_LINE_4_Y,80,setup_messages[20],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                  // "IP:"
    TFT_write_string(SET_WIFI_SERVER_X,SETTING_LINE_5_Y,80,setup_messages[21],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);              // "Server:"
    TFT_write_string(SET_WIFI_PORT_X,SETTING_LINE_6_Y,80,setup_messages[22],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                // "Porta:"
    
    TFT_write_string(SET_WIFI_EN_VALUE_X,SETTING_LINE_1_Y,40,on_off[wifi_on_off],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);                // valore wifi on off    
    TFT_write_string(SET_WIFI_SERVER_VALUE_X,SETTING_LINE_5_Y,80,on_off[server_status],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);     // valore server on off
    TFT_write_string(SET_WIFI_PORT_VALUE_X,SETTING_LINE_6_Y,80,port,SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                        // porta 80
    
    // scrittura SSID salvato in impostazioni
    if (ssid[0] != '\0' && ssid[0] != 0xFF){                                            // solo se ho impostato una ssid  valido
        while (ssid[_i] != '\0'){                                                       // scrivo ssid salvato sul display
            _x += TFT_write_char(SET_WIFI_SSID_VALUE_X + _x ,SETTING_LINE_2_Y,ssid[_i],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);
            _x += font_space;
            _i++;
        }
        
        _x = 0;
        _i = 0;
    }

    // scrittura stato rete
    if (ESP8266_check_ap_con(ESP8266_CURRENT_VALUE, ESP8266_DEF_TIMEOUT) == ESP8266_RESP_OK){
         // stato connessione
        TFT_write_string(SET_WIFI_STAT_VALUE_X,SETTING_LINE_3_Y,100,setup_messages[26],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);             // "Connesso"
        
        // scrittura indirizzo IP assegnato dall'AP
        if (ESP8266_read_ip_sta(ESP8266_CURRENT_VALUE, ESP8266_DEF_TIMEOUT) == 1){      
            ESP8266_clean_buffer(16);                                         
            
            while ( esp8266_buffer[_i] != '"'){
                _x += TFT_write_char(SET_WIFI_IP_VALUE_X + _x ,SETTING_LINE_4_Y,esp8266_buffer[_i],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);
                _x += font_space;
                _i++;
            }
        }
    } else {
        // stato wifi non connesso
        TFT_write_string(SET_WIFI_STAT_VALUE_X,SETTING_LINE_3_Y,100,setup_messages[27],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);             // "Non connesso"    
    }
    
    XPT2046_enable();                                                                   // per settare parametri corretti SPI (SD  impostata + veloce)
    do {
        if (XPT2046_IRQ == 0) {                                                         // se  stato toccato lo schermo

            switch (Touch_screen_manage_loop(TOUCH_SETUP_WIFI)){
                case T_SETUP_WIFI_EN:
                    wifi_on_off = !wifi_on_off;
                    
                    TFT_write_string(SET_WIFI_EN_VALUE_X,SETTING_LINE_1_Y,40,on_off[wifi_on_off],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);         // On/Off
                    
                    if (wifi_on_off == ON){
                        ESP8266_power(ON,ESP8266_DEF_TIMEOUT);                                              // abilito modulo wifi
                        ESP8266_set_mode(ESP8266_STATION_MODE, ESP8266_CURRENT_VALUE, ESP8266_DEF_TIMEOUT); // Modalita stazione sempre
                    } else {
                        ESP8266_power(OFF,10);                                                              // disabilito modulo wifi
                        Wifi_check_con(0);                                                                   // cancello icona
                        server_status = 0;
                    }
                    break;
                case T_SETUP_WIFI_SCAN_SSID:
                    Wifi_select_ssid();                                                 // avvio procedura ricerca rete wifi
                    _exit = 1;                                                          // forzo uscita con salvataggio per ridisegnare i valori del menu
                    break;
                case T_SW_3:
                    Draw_switch(T_SW_OK);
                    _exit = 1;
                    break;
                case T_SW_4:
                    Draw_switch(T_SW_EXIT);
                    _exit = 2;
                    break;    
            }
        }
    } while(_exit == 0);                                                                                                            
                                                                                                                             
    // se salva
    if (_exit == 1){
        EEPROM_write(wifi_on_off,EEPROM_WIFI_EN_ADDRESS);                                       // salvo in eeprom
        if (wifi_connected == 0 && wifi_on_off == 1 && ssid[0] != '\0'){                // se non c' connessione (ssid selezionato errato)
//            TFT_set_font(font_lucida_sans_10x16);
            TFT_write_string(SETUP_TEXT_GEN_X,SETTING_LINE_10_Y,230,setup_messages[28],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);      // "Connessione in corso"
            if (wifi_on_off == ON && ssid[0] != '\0')
                Wifi_connect(ssid,password);                                          // tento connessione a rete precedente se impostata
        }
    }
    TFT_fill_rect(1,17,240,280, BACK_COLOR);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Wifi_select_ssid
 *
 * Overview:    Ricerca le reti nelle vicinanze, e richiede la password quando selezionata, testa la connessione e se riesce salva la rete come 
 *              default
/* ============================================================================================================================================ */
void Wifi_select_ssid(void){
    unsigned char _x = 0;    
    unsigned char _y = 0;    
    unsigned char _i = 0;
    unsigned char _c = 0;
    unsigned char _ssid_temp[3][32];
    unsigned char _selection = 0xFF;
    unsigned char _readed_char = 0;
    unsigned char _password_temp[32];
    unsigned char _exit = 0;
    
    unsigned char _enc = 0;
    
    wifi_connected = 0;
    
    do {
        _exit = 0;
        _selection = 0xFF;
        TFT_clear(BACK_COLOR);
        
//        TFT_set_font(font_lucida_sans_10x16);
        TFT_write_string(SETUP_TEXT_GEN_X,SETUP_TEXT_GEN_Y,230,setup_messages[24],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);      // "Ricerca reti in corso"
        ESP8266_search_ap_opt(ON,"7",ESP8266_DEF_TIMEOUT);                              // ordinamento in base alla potenza, visualizzo solo criptazione, ssi e rsi               

        if (ESP8266_search_ap(3000) == ESP8266_RESP_OK){                                // ricerca reti

            // scrittura valori reti rilevate
            // ciclo finch ci sono reti, fino a 3 reti massimo
            while (_c < 3 && esp8266_buffer[0] == '+'){                                 
                ESP8266_clean_buffer(8);
                _enc = esp8266_buffer[0] - '0';

                _x = 0;
                _y = SETTING_LINE_1_Y + (2*_c* SETTING_LINE_Y_SPACE);

                ESP8266_clean_buffer(3);

                _i = 0;

                // Salvo SSID in buffer temporaneo
                while (esp8266_buffer[_i] != '"'){                                          
                    _ssid_temp[_c][_i] = esp8266_buffer[_i];
                    _i++;
                }
                // aggiungo fine stringa
                _ssid_temp[_c][_i] = '\0';
                TFT_write_string(SET_WIFI_SSID_RES_X,_y,200,_ssid_temp[_c],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);

                ESP8266_clean_buffer_until_char(',');
                _x = 0;
                _i = 0;

                // scrivo potenza segnale
                while (_i < 3) {
                    _x += TFT_write_char(SET_WIFI_RSSI_X + _x ,_y,esp8266_buffer[_i],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
                    _x += font_space;
                    _i++;
                }
                
                TFT_write_string(SET_WIFI_RSSI_X + _x, _y ,20,"dB",SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);

                // scrivo codifica
                TFT_write_string(SET_WIFI_ENC_X,_y+SETTING_LINE_Y_SPACE,200,encryption[_enc],SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
                ESP8266_clean_buffer(4);
                _c++;
                
                // linea separazione
                TFT_draw_line_x(5,235,_y + 2* SETTING_LINE_Y_SPACE -3, SETUP_TITLE_COLOR);
            }

            // selezione di una delle 3 reti o rescan e exit
            Draw_switch(SW_RESC_ESC);
//            TFT_set_font(font_lucida_sans_10x16);
            XPT2046_enable();                                                           // per settare parametri corretti SPI (SD  impostata + veloce)
            do {
                if (XPT2046_IRQ == 0) {                                                 // se  stato toccato lo schermo

                    switch (Touch_screen_manage_loop(TOUCH_SETUP_WIFI_SSID)){
                        case T_SETUP_WIFI_SSID_1:
                            _selection = 0;
                            break;
                        case T_SETUP_WIFI_SSID_2:
                            if (_c >= 2)                                                // selezione valida solo se  stata trovata una rete
                                _selection = 1;
                            break;
                        case T_SETUP_WIFI_SSID_3:
                            if (_c >= 3)                                                // selezione valida solo se  stata trovata una rete                       
                                _selection = 2;
                            break;
                        case T_SW_3:
                            _exit = 2;                                                  // rescan
                            break;
                        case T_SW_4:
                            _exit = 1;                                                  // exit
                            break;
                    }
                }
            } while (_selection == 0xFF && _exit == 0);

            _i = 0;
            _c = 0;
            _x = 0;
            
            if (_exit == 0){                                                            // se  stata selezionata una rete
                
                TFT_clear(BACK_COLOR);
                TFT_write_string(SETUP_TEXT_GEN_X,SETUP_TEXT_GEN_Y,230,setup_messages[25],SAVESPACE_ON,SETUP_TITLE_COLOR,BACK_COLOR);      // "Inserire la password per" 
                TFT_write_string(SETUP_TEXT_GEN_X,SETTING_LINE_1_Y,230,_ssid_temp[_selection],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);       

                _y = SETTING_LINE_2_Y; 
                
                // ciclo fino a return
                do {
                    _readed_char = Handle_keyboard();                                       // leggo tastiera

                    if (_readed_char != 0 && _readed_char != 0x0D && _readed_char != 0x08 && _i < 32){  // scrivo e salvo massimo  32 caratteri

                        if (_i == 21){                                                      // contatore 22 caratteri poi 
                            _y = SETTING_LINE_3_Y;                                          // scrive su riga 2
                            _x = 0;
                        }

                        _x += TFT_write_char(SET_WIFI_PASS_X + _x ,_y,_readed_char,SAVESPACE_OFF,SETUP_VALUE_COLOR,BACK_COLOR);
                        _x += font_space;
                        _password_temp[_i] = _readed_char; 
                        _i++;

                        if (_i == 33)                                                       // mantengo contatore a 32
                            _i = 32;
    //                        _readed_char = 0x0D;                                            // forzo return ai 32 caratteri

                    // BACKSPACE cancella carattere scritto e decrementa il contatore caratteri
                    } else if (_readed_char == 0x08){                                       
                        _i--;
                        if (_i != 0xFF){
                            if (_i == 20) {
                                _y = SETTING_LINE_2_Y;
                                _x = 230;
                            }

                            _x -= font_width;
                            _x -= font_space; 
                            TFT_fill_rect(SET_WIFI_PASS_X+_x,_y,font_width+font_space,font_height, BACK_COLOR);
                        } else {
                            _i = 0;
                        }
                    }
                    Delay_ms(150);
                } while (_readed_char != 0x0D);                                             // INVIO

                _password_temp[_i] = '\0';                                                  // chiudo stringa

                TFT_fill_rect(KEYBOARD_X, KEYBOARD_Y,240,128, BACK_COLOR);                  // cancello la tastiera

                TFT_write_string(SETUP_TEXT_GEN_X,SETTING_LINE_4_Y,230,setup_messages[28],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);      // "Connessione in corso"

                #ifdef DEBUG_LOG_SD
                Write_log(WIFI_SSID_TEST);
                #endif

                // tentativo di connessione alla rete selezionata con la password inserita
                Wifi_connect(_ssid_temp[_selection], _password_temp);

                // se connessione riuscita
                if (wifi_connected){
                TFT_write_string(SETUP_TEXT_GEN_X,SETTING_LINE_4_Y,230,setup_messages[26],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);      // "Connesso"

                    _i = 0;

                    // salvo ssid temporanea in ssid globale e in eeprom
                    do {
                        ssid[_i] = _ssid_temp[_selection][_i];
                        EEPROM_write(ssid[_i],EEPROM_WIFI_SSID_ADDRESS + _i);
                        _i++;
                    } while (ssid[_i-1] != '\0');

                    _i = 0;

                    // salvo password temporanea in password globale e in eeprom
                    do {
                        password[_i] = _password_temp[_i];
                        EEPROM_write(password[_i],EEPROM_WIFI_PASSWORD_ADDRESS + _i);
                        _i++;

                    } while (password[_i-1] != '\0');                                       // fino a fine stringa compreso per salvare sempre anche in eeprom '\0'
                } else {
                    TFT_write_string(SETUP_TEXT_GEN_X,SETTING_LINE_4_Y,230,setup_messages[27],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);      // "Non Connesso"
                }
            }
        } else {
            TFT_write_string(SETUP_TEXT_GEN_X,SETTING_LINE_1_Y,180,setup_messages[29],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);          // "Nessuna rete trovata"
        }

        if (_exit == 0){                                                                // rete selezionata e password immessa
            if (wifi_connected == 0){
                Draw_switch(SW_RESC_ESC);
                XPT2046_enable(); 

                do{
                    if (XPT2046_IRQ == 0) {                                                     // se  stato toccato lo schermo

                        switch (Touch_screen_manage_loop(TOUCH_SETUP_WIFI_SSID)){
                            case T_SW_3:                                                        // rescan
                                Draw_switch(T_SW_RSC);
                                _exit = 2;
                                break;
                            case T_SW_4:
                                Draw_switch(T_SW_EXIT);
                                _exit = 1;
                                break;
                        }
                    }
                } while (_exit == 0);
            } else {
                _exit = 1;
                Delay_s(1);
            }
        } 
//        else {
//            // se durate la selezione della rete  stato premuto rescan o exit salta tutto e verifica exit all'ultimo while, se 2 ripete per fare una nuova scansione
//            // se 1 esce per exit premuto
//        }
    }while (_exit == 2);                                                                // ripeto tutta la procedura se  stato premuto il tasto rescan
}

/* ============================================================================================================================================= /
 * FUNCTION:    Setup_light
 *
 * Overview:    Impostazione della retroilluminazione e degli orari di accensione e spegnimento
/* ============================================================================================================================================ */
void Setup_light(void){
    unsigned char _exit = 0;
    unsigned char *_pointer_value = 0;                                                  // puntatore a valori temporanei da modificare
    unsigned char _ccpr4l = 0;                                                          // variabili temporanee solo per menu
    unsigned char _backill_on_h = 0;
    unsigned char _backill_on_m = 0;
    unsigned char _backill_off_h = 0;
    unsigned char _backill_off_m = 0;
        
    _ccpr4l = ccpr4l;
    _backill_on_h = backill_on_h;
    _backill_on_m = backill_on_m;
    _backill_off_h = backill_off_h;
    _backill_off_m = backill_off_m;
    
    _pointer_value = &_ccpr4l;
    
    // scritte iniziali
    TFT_fill_rect(1,17,240,280, BACK_COLOR);
    
//    TFT_set_font(font_lucida_sans_10x16);
    TFT_write_string(SETUP_TEXT_GEN_X,SETUP_TEXT_GEN_Y,230,setup_messages[3],SAVESPACE_ON,SETUP_TITLE_COLOR,BACK_COLOR);                        // "Impostazioni retroilluminazione"
    TFT_write_string(SETUP_LIGHT_LEV_X,SETTING_LINE_1_Y,100,setup_messages[4],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                  // "Livello"
    TFT_write_unsint(SETUP_LIGHT_LEV_VALUE_X,SETTING_LINE_1_Y,50,_ccpr4l,3,ZERO_CLEANING_ON,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
    TFT_write_string(SETUP_LIGHT_ON_X,SETTING_LINE_2_Y,165,setup_messages[5],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                   // "Ora accensione"
    TFT_write_PBCD(SETUP_LIGHT_HOURS_X,SETTING_LINE_2_Y,SETUP_LIGHT_X_LEN,_backill_on_h,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    TFT_write_char(SETUP_LIGHT_DOTS_X,SETTING_LINE_2_Y,':',SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    TFT_write_PBCD(SETUP_LIGHT_MINUTES_X,SETTING_LINE_2_Y,SETUP_LIGHT_X_LEN,_backill_on_m,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    TFT_write_string(SETUP_LIGHT_OFF_X,SETTING_LINE_3_Y,165,setup_messages[6],SAVESPACE_ON,SETUP_VALUE_STAT_COLOR,BACK_COLOR);                  // "Ora spegnimento"
    TFT_write_PBCD(SETUP_LIGHT_HOURS_X,SETTING_LINE_3_Y,SETUP_LIGHT_X_LEN,_backill_off_h,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    TFT_write_char(SETUP_LIGHT_DOTS_X,SETTING_LINE_3_Y,':',SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    TFT_write_PBCD(SETUP_LIGHT_MINUTES_X,SETTING_LINE_3_Y,SETUP_LIGHT_X_LEN,_backill_off_m,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
    
    Draw_switch(SW_SETUP_INCR);
    
    XPT2046_enable();                                                                   // per settare parametri corretti SPI (SD  impostata + veloce)
    do {
        if (XPT2046_IRQ == 0) {                                                         // se  stato toccato lo schermo

            switch (Touch_screen_manage_loop(TOUCH_SETUP_LIGHT)){                       // imposto il puntatore al valore da modificare a seconda di quale  stato toccato
                case T_LIGHT_LEV:
                    _pointer_value = &_ccpr4l;
                    break;
                case T_ON_HOURS:
                    _pointer_value = &_backill_on_h;
                    break;    
                case T_ON_MINUTES:
                    _pointer_value = &_backill_on_m;
                    break; 
                case T_OFF_HOURS:
                    _pointer_value = &_backill_off_h;
                    break; 
                case T_OFF_MINUTES:
                    _pointer_value = &_backill_off_m;
                    break; 
                case T_SW_1:                                                            // INCREMENTI
                    Draw_switch(T_SW_PLUS);
                    if (_pointer_value == &_ccpr4l){                                    // incremento ccpr4l temporaneo
                        *_pointer_value = *_pointer_value +10;
                        if (*_pointer_value == 250)
                            *_pointer_value = 10;
                        CCPR4L = *_pointer_value; 
                    }
                    else if (_pointer_value == &_backill_on_h || _pointer_value == &_backill_off_h)  // incremento valori ore
                        *_pointer_value = Inc_pbcd(*_pointer_value, 0x00, 0x23);
                    else if (_pointer_value == &_backill_on_m || _pointer_value == &_backill_off_m)  // incremento valori minuti
                        *_pointer_value = Inc_pbcd(*_pointer_value, 0x00, 0x59);
                    break;
                case T_SW_2:                                                            // DECREMENTI
                    Draw_switch(T_SW_LESS);
                    if (_pointer_value == &_ccpr4l){
                        *_pointer_value = *_pointer_value -10;
                        if (*_pointer_value == 0)
                            *_pointer_value = 240;
                        CCPR4L = *_pointer_value; 
                    }
                    else if (_pointer_value == &_backill_on_h || _pointer_value == &_backill_off_h)
                        *_pointer_value = Dec_pbcd(*_pointer_value, 0x00, 0x23);
                    else if (_pointer_value == &_backill_on_m || _pointer_value == &_backill_off_m)
                        *_pointer_value = Dec_pbcd(*_pointer_value, 0x00, 0x59);
                    break;
                case T_SW_3:                                                            // SALVA
                    Draw_switch(T_SW_OK);
                    _exit = 1;
                    break;
                case T_SW_4:                                                            // ESCI
                    Draw_switch(T_SW_EXIT);
                    _exit = 2;
                    break;    
            }
            
//            TFT_set_font(font_lucida_sans_10x16);
            // riscrivo tutti i valori
            TFT_write_unsint(SETUP_LIGHT_LEV_VALUE_X,SETTING_LINE_1_Y,50,_ccpr4l,3,ZERO_CLEANING_ON,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
            TFT_write_PBCD(SETUP_LIGHT_HOURS_X,SETTING_LINE_2_Y,SETUP_LIGHT_X_LEN,_backill_on_h,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
            TFT_write_PBCD(SETUP_LIGHT_MINUTES_X,SETTING_LINE_2_Y,SETUP_LIGHT_X_LEN,_backill_on_m,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
            TFT_write_PBCD(SETUP_LIGHT_HOURS_X,SETTING_LINE_3_Y,SETUP_LIGHT_X_LEN,_backill_off_h,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
            TFT_write_PBCD(SETUP_LIGHT_MINUTES_X,SETTING_LINE_3_Y,SETUP_LIGHT_X_LEN,_backill_off_m,SAVESPACE_ON,SETUP_VALUE_COLOR,BACK_COLOR);
            
            // scrivo in rosso solo il valore selezionato/modificato
            if (_pointer_value == &_ccpr4l){
                TFT_write_unsint(SETUP_LIGHT_LEV_VALUE_X,SETTING_LINE_1_Y,50,_ccpr4l,3,ZERO_CLEANING_ON,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
            } else if (_pointer_value == &_backill_on_h){
                TFT_write_PBCD(SETUP_LIGHT_HOURS_X,SETTING_LINE_2_Y,SETUP_LIGHT_X_LEN,_backill_on_h,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
            } else if (_pointer_value == &_backill_on_m){
                TFT_write_PBCD(SETUP_LIGHT_MINUTES_X,SETTING_LINE_2_Y,SETUP_LIGHT_X_LEN,_backill_on_m,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
            } else if (_pointer_value == &_backill_off_h){
                TFT_write_PBCD(SETUP_LIGHT_HOURS_X,SETTING_LINE_3_Y,SETUP_LIGHT_X_LEN,_backill_off_h,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
            } else if (_pointer_value == &_backill_off_m){
                TFT_write_PBCD(SETUP_LIGHT_MINUTES_X,SETTING_LINE_3_Y,SETUP_LIGHT_X_LEN,_backill_off_m,SAVESPACE_ON,SETUP_VALUE_SEL_COLOR,BACK_COLOR);
            }
        }
    }while(_exit == 0);                                                                 // ripeto finch non viene premuto il tasto salva o il tasto esci
    
    if (_exit == 1){                                                                    // se premuto tasto salva
        ccpr4l = _ccpr4l;                                                               // salvo i valori temporanei nelle variabili globali
        backill_on_h = _backill_on_h;
        backill_on_m = _backill_on_m;
        backill_off_h = _backill_off_h;
        backill_off_m = _backill_off_m;
        
        EEPROM_write(ccpr4l,EEPROM_CCPR4L_ADDRESS);                                     // salvo i valori in EEPROM
        EEPROM_write(backill_on_h,EEPROM_BKILL_ON_H_ADDRESS);
        EEPROM_write(backill_on_m,EEPROM_BKILL_ON_M_ADDRESS);
        EEPROM_write(backill_off_h,EEPROM_BKILL_OFF_H_ADDRESS);
        EEPROM_write(backill_off_m,EEPROM_BKILL_OFF_M_ADDRESS);
    } else                                                                              // tasto esci            
        CCPR4L = ccpr4l;                                                                // ripristino la retroilluminazione
    
    TFT_fill_rect(1,17,240,280, BACK_COLOR);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Calibrate_touch
 *
 * Overview:    Calibrazione touch screen a 2 punti. Salva in variabili globali  i valori minimi e massimi di x/y e calcola i fattori di scaling
/* ============================================================================================================================================ */
void Calibrate_touch(void){
    xpt2046_point _touched_point_raw_1;
    xpt2046_point _touched_point_raw_2;
    
    TFT_clear(BACK_COLOR);
    
//    TFT_set_font(font_lucida_sans_10x16);
    TFT_write_string(5,150,210,messages[0], SAVESPACE_ON, SETUP_VALUE_STAT_COLOR, BACK_COLOR);           // "Calibrazione touch screen, "
    TFT_write_string(5,170,235,messages[1], SAVESPACE_ON, SETUP_VALUE_STAT_COLOR, BACK_COLOR);           // "Tocca i punti rossi sullo schermo:"
    
    Delay_ms(200);
    
    TFT_fill_rect(1,1,4,4,RED);
    while (XPT2046_IRQ);
    
    _touched_point_raw_1 = XPT2046_read_point_raw();
    
    TFT_fill_rect(1,1,4,4,BACK_COLOR);
    TFT_fill_rect(X_MAX-3,Y_MAX-3,4,4,RED);
    Delay_ms(200);
    while (XPT2046_IRQ);
    
    _touched_point_raw_2 = XPT2046_read_point_raw();
    
    XPT2046_calc_scaling_factor(_touched_point_raw_1, _touched_point_raw_2);
    Delay_ms(500);
    
    TFT_clear(BACK_COLOR);
//    TFT_write_string(1,1,50,"MIN X:", SAVESPACE_ON, WHITE, BACK_COLOR);
//    TFT_write_unsint(50,1,100, _touched_point_raw_1.x, 5, ZERO_CLEANING_OFF, SAVESPACE_ON, WHITE, BACK_COLOR);
//    TFT_write_string(1,20,50,"MAX X:", SAVESPACE_ON, WHITE, BACK_COLOR);
//    TFT_write_unsint(50,20,100, _touched_point_raw_2.x, 5, ZERO_CLEANING_OFF, SAVESPACE_ON, WHITE, BACK_COLOR);
//    TFT_write_string(1,40,50,"MIN Y:", SAVESPACE_ON, WHITE, BACK_COLOR);
//    TFT_write_unsint(50,40,100, _touched_point_raw_1.y, 5, ZERO_CLEANING_OFF, SAVESPACE_ON, WHITE, BACK_COLOR);
//    TFT_write_string(1,60,50,"MAX Y:", SAVESPACE_ON, WHITE, BACK_COLOR);
//    TFT_write_unsint(50,60,100, _touched_point_raw_2.y, 5, ZERO_CLEANING_OFF, SAVESPACE_ON, WHITE, BACK_COLOR);

    TFT_write_string(5,20,230,messages[2], SAVESPACE_ON, TEST_E_HW_LABEL_COLOR, BACK_COLOR);           // "Tocca il display per uscire."
    
    // salvo i punti in EEPROM 
    EEPROM_write_int(_touched_point_raw_1.x, EEPROM_TOUCH_POINT_1_X_ADDRESS);
    EEPROM_write_int(_touched_point_raw_1.y, EEPROM_TOUCH_POINT_1_Y_ADDRESS);
    EEPROM_write_int(_touched_point_raw_2.x, EEPROM_TOUCH_POINT_2_X_ADDRESS);
    EEPROM_write_int(_touched_point_raw_2.y, EEPROM_TOUCH_POINT_2_Y_ADDRESS);
    
    while (XPT2046_IRQ);
    TFT_clear(BACK_COLOR);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Hardware_check
 *
 * Overview:    Test hardware installato
 * Note:        Il modulo remoto deve essere spento e acceso quando richiesto
/* ============================================================================================================================================ */
void Hardware_check(void){
    unsigned int counter = 0;
    unsigned char signal_foud = 0;
    
    TFT_clear(BACK_COLOR);
    
//    TFT_set_font(font_lucida_sans_10x16);
    TFT_write_string(5,1,220,messages[3], SAVESPACE_ON, TEST_E_HW_LABEL_COLOR, BACK_COLOR);       // "HARDWARE CHECK"
    
    /* TEST COMPONENTI I2C TRAMITE LETTURA DEL DEVICE ID */
    TFT_write_string(5,20,100,messages[4], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);            // "HDC1080:"
    if (HDC1080_read_device_id() == 0x1050)
        TFT_write_string(100,20,140,messages[5], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);      // "Trovato."
    else
        TFT_write_string(100,20,140,messages[6], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);      // "Non trovato."
        
    TFT_write_string(5,40,100,messages[7], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);            // "LPS25HB:"
    if (LPS25HB_read_device_id(LPS25HB_DEV_ADDR) == 0xBD)
        TFT_write_string(100,40,140,messages[5], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);      // "Trovato."
    else
        TFT_write_string(100,40,140,messages[6], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);      // "Non trovato."
    
    TFT_write_string(5,60,100,messages[8], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);            // "VEML6075:"
    if (VEML6075_read_device_id() == 0x0026)
        TFT_write_string(100,60,140,messages[5], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);
    else
        TFT_write_string(100,60,140,messages[6], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);
    
    TFT_write_string(5,80,100,messages[9], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);            // "24LC1025 B1:"
    if (EEPROM_I2C_write_e_check(EEPROM_I2C_DEV_ADDR_B1,0x00, 0xAA) == 0)
        TFT_write_string(100,80,140,messages[5], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);
    else
        TFT_write_string(100,80,140,messages[6], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);

    TFT_write_string(5,100,100,messages[10], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);          // "24LC1025 B2:"
    if (EEPROM_I2C_write_e_check(EEPROM_I2C_DEV_ADDR_B2,0x00, 0xAB) == 0)
        TFT_write_string(100,100,140,messages[5], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);
    else
        TFT_write_string(100,100,140,messages[6], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);
    
    /* TEST MODULO ESP8266 TRAMITE INVIO COMANDO AT */
    ESP8266_power(ON, ESP8266_DEF_TIMEOUT);                                                             // forzo acceso
    TFT_write_string(5,120,100,messages[11], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);          // "ESP8266:"
    if (ESP8266_at(ESP8266_DEF_TIMEOUT) == 0x01)
        TFT_write_string(100,120,140,messages[5], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);     // "Trovato."
    else
        TFT_write_string(100,120,140,messages[6], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);     // "Non trovato."
    
    if (wifi_on_off == OFF)                                                                             // se wifi  disabilitato
        ESP8266_power(OFF, 10);                                                                         // spengo il modulo
    
    /* TEST MODULO HMTRP TRAMITE IMPOSTAZIONE DELLA MODALIT DI FUNZIONAMENTO, VERIFICARE I LED SUL MODULO */
    TFT_write_string(5,140,235,messages[12], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);          // "HMTRP:"
    TFT_write_string(100,140,135,messages[13], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);        // "Tocca per iniz."
    while (XPT2046_IRQ);    
//    HMTRP_mode(HMTRP_SLEEP_MODE);
//    TFT_write_string(5,140,235,"HMTRP SLEEP:", SAVESPACE_ON, TEST_E_HARDWARE_COLOR, BACK_COLOR);
//    TFT_write_string(100,140,140,"Led Spenti.", SAVESPACE_ON, TEST_E_HARDWARE_COLOR, BACK_COLOR);
//    Delay_s(1);
//    while (XPT2046_IRQ);    
    HMTRP_mode(HMTRP_CONFIG_MODE);
    TFT_write_string(5,140,235,messages[14], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);          // "HMTRP CONF:"
    TFT_write_string(100,140,140,messages[15], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);        // "Led accesi."
    Delay_s(1);
    while (XPT2046_IRQ);    
    HMTRP_mode(HMTRP_NORMAL_MODE);
    TFT_write_string(5,140,235,messages[16], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);          // "HMTRP NORM:"
    TFT_write_string(100,140,140,messages[17], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);        // "Led Spenti."
    Delay_s(1);
    while (XPT2046_IRQ);    
    TFT_write_string(5,140,235,messages[12], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);          // "HMTRP:"
    TFT_write_string(100,140,140,messages[18], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);        // "Fine."
    
//    /* TEST INVIO E RICEZIONE MODULO REMOTO */
    TFT_write_string(5,160,235,messages[19], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);          // "REMOTO:"
    TFT_write_string(100,160,135,messages[20], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);        // "Accendere"
    
    Delay_ms(200);

    while ((hmtrp_string_received == 0) && (XPT2046_IRQ == 1));
    
    if(hmtrp_string_received){
        if(hmtrp_buffer[0] == 0xF0){
            HMTRP_reset_buffer();
            HMTRP_send_char(0xF0);
            HMTRP_send_char(0x0D);
            HMTRP_send_char(0x0A);
            TFT_write_string(100,160,135,messages[5], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR); // "Trovato."
            TFT_write_string(5,180,135,messages[4], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);   // "HDC1080:"
            TFT_write_string(5,200,135,messages[7], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);   // "LPS25HB:"
            TFT_write_string(5,220,135,messages[8], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);   // "VEML6075:"

            HMTRP_send_char(0xF1);
            HMTRP_send_char(0x0D);
            HMTRP_send_char(0x0A);
            Delay_s(2);
            if (hmtrp_string_received){
                if (hmtrp_buffer[0] == 0xF2)
                    TFT_write_string(100,180,135,messages[5], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR); // "Trovato."
                else
                   TFT_write_string(100,180,135,messages[6], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);  // "Non trovato."
                if (hmtrp_buffer[1] == 0xF2)
                    TFT_write_string(100,200,135,messages[5], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR); // "Trovato."
                else
                   TFT_write_string(100,200,135,messages[6], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);  // "Non trovato."
                if (hmtrp_buffer[2] == 0xF2)
                    TFT_write_string(100,220,135,messages[5], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR); // "Trovato."
                else
                   TFT_write_string(100,220,135,messages[6], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);  // "Non trovato."

                HMTRP_reset_buffer();
            } else {
                TFT_write_string(100,180,135,messages[21], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);    // "Nessuna risposta.", 
                TFT_write_string(100,200,135,messages[21], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);    // "Nessuna risposta."
                TFT_write_string(100,220,135,messages[21], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);    // "Nessuna risposta."
            }

        }
    } else 
       TFT_write_string(100,160,135,messages[22], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);             // "Saltato."
    
    
    /* TEST MODULO DCF77 TRAMITE ATTESA IMPULSO SECONDI */
    TFT_write_string(5,240,235,messages[23], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);                  // "DCF77:"
    
    do {
        if (DCF77_SIGNAL == 1)
            signal_foud = 1;
        Delay_ms(1);
        counter++;
    }while((counter < 5000) && (signal_foud == 0));
    
    if (signal_foud)
        TFT_write_string(100,240,140,messages[5], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);             // "Trovato." 
    else
        TFT_write_string(100,240,140,messages[6], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);             // "Non trovato."
    
    /* TEST MODULO HKW581 TRAMITE DECODIFICA DATI METEO NOTI */
    TFT_write_string(5,260,235,messages[24], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);                  // "HKW581:"                  
    TFT_write_string(100,260,140,messages[25], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);                // "Attesa BRF a 0."
    while (!HKW581_read_brf());
    TFT_write_string(100,260,140,messages[26], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);                // "Invio Dati,"
    Delay_s(1);
    meteo_fc_decoded = HKW581_decode_meteo_data(0b01111000010001, 0b11101110000011, 0b01110100011111, 0x02, 0x19, 0x02, 0x11, 0x03, 0x16);
    
    if (meteo_fc_decoded.day_fc == 0b0110)
        TFT_write_string(100,260,140,messages[27], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);            // "Decodifica ok."  
    else
        TFT_write_string(100,260,140,messages[28], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);            // "Errore."  
    
    /* TEST BUZZER TRAMITE 1 SUONI */
    TFT_write_string(5,280,235,messages[29], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);                  // "Buzzer:"
    TFT_write_string(100,280,140,messages[30], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);                // "ON"
    BUZZER = 1;
    Delay_ms(500);
    TFT_write_string(100,280,140,messages[31], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);                // "OFF"
    BUZZER = 0;
    
    /* TEST TFT TRAMITE RIEMPIMENTO ROSSO/VERDE/BLU */
    TFT_write_string(5,300,235,messages[32], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);                  //"TFT:"
    TFT_write_string(100,300,135,messages[33], SAVESPACE_ON, TEST_E_HW_TEXT_COLOR, BACK_COLOR);
    Delay_s(1);
    while (XPT2046_IRQ);    
    
    TFT_clear(RED); // Rosso
    Delay_ms(500);
    while (XPT2046_IRQ); 
    TFT_clear(GREEN); // Verde
    Delay_ms(500);
    while (XPT2046_IRQ); 
    TFT_clear(BLUE); // Blu
    while (XPT2046_IRQ); 
    
    TFT_clear(BACK_COLOR);
    TFT_write_string(5,1,220,messages[34], SAVESPACE_ON, TEST_E_HW_LABEL_COLOR, BACK_COLOR);  // "HARDWARE CHECK FINITO,"
    TFT_write_string(5,20,230,messages[2], SAVESPACE_ON, TEST_E_HW_LABEL_COLOR, BACK_COLOR);  // "Tocca il display per uscire."
    
    while (XPT2046_IRQ); 
    TFT_clear(BACK_COLOR);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Test_sensors
 *
 * Overview:    Ciclo di test per verifica dei valori rilevati dai sensori. Ad ogni ricezione dati dal modulo remoto scrive su display solo
 *              i valori letti dai sensori locali e remoti   
/* ============================================================================================================================================ */
void Test_sensors(void){   
    TFT_clear(BACK_COLOR);
    HMTRP_reset_buffer();
    HMTRP_mode(HMTRP_NORMAL_MODE);                                                      // forzo avvio
    
//    TFT_set_font(font_lucida_sans_10x16);
    TFT_write_string(5,1,220,messages[35], SAVESPACE_ON, TEST_E_HW_LABEL_COLOR, BACK_COLOR);
    
    while (XPT2046_IRQ){  
        if(hmtrp_string_received){                                                      // dati da remoto ricevuti
            
            TFT_clear(BACK_COLOR);
            TFT_write_string(1,1,150,label[1],SAVESPACE_ON,TEST_E_HW_LABEL_COLOR,BACK_COLOR);
            TFT_write_string(1,180,150,label[0],SAVESPACE_ON,TEST_E_HW_LABEL_COLOR,BACK_COLOR);
            
            Get_remote_sensors_values();                                                // converto valori remoto

//            TFT_fill_rect(1,1,220,20,BACK_COLOR);
  
            TFT_write_float(5,20,150,remote_actual.temp_hdc1080,TEMP_INT_DIGITS,TEMP_DEC_DIGITS, ZERO_CLEANING_ON, SAVESPACE_ON,TEST_E_HW_TEXT_COLOR, BACK_COLOR); 
            TFT_write_float(5,40,150,remote_actual.humidity,RH_INT_DIGITS,RH_DEC_DIGITS, ZERO_CLEANING_ON, SAVESPACE_ON,TEST_E_HW_TEXT_COLOR, BACK_COLOR); 
            TFT_write_float(5,60,150,remote_actual.press,PRESS_INT_DIGITS,PRESS_DEC_DIGITS, ZERO_CLEANING_ON, SAVESPACE_ON,TEST_E_HW_TEXT_COLOR, BACK_COLOR); 
            TFT_write_float(5,80,150,remote_actual.temp_lps25,TEMP_INT_DIGITS,TEMP_DEC_DIGITS, ZERO_CLEANING_ON, SAVESPACE_ON,TEST_E_HW_TEXT_COLOR, BACK_COLOR);
            TFT_write_float(5,100,150,remote_actual.uv_readed.uva,UV_INT_DIGITS,UV_DEC_DIGITS, ZERO_CLEANING_ON, SAVESPACE_ON,TEST_E_HW_TEXT_COLOR, BACK_COLOR); 
            TFT_write_float(5,120,150,remote_actual.uv_readed.uvb,UV_INT_DIGITS,UV_DEC_DIGITS, ZERO_CLEANING_ON, SAVESPACE_ON,TEST_E_HW_TEXT_COLOR, BACK_COLOR); 
            TFT_write_unsint(5,140,50,remote_battery_level,1, ZERO_CLEANING_ON, SAVESPACE_ON,TEST_E_HW_TEXT_COLOR, BACK_COLOR);

            Get_local_sensors_values();
            TFT_write_float(5,200,150,local_actual.temp_hdc1080,TEMP_INT_DIGITS,TEMP_DEC_DIGITS, ZERO_CLEANING_ON, SAVESPACE_ON,TEST_E_HW_TEXT_COLOR, BACK_COLOR); 
            TFT_write_float(5,220,150,local_actual.humidity,RH_INT_DIGITS,RH_DEC_DIGITS, ZERO_CLEANING_ON, SAVESPACE_ON,TEST_E_HW_TEXT_COLOR, BACK_COLOR);
            TFT_write_float(5,240,100,local_actual.press,PRESS_INT_DIGITS,PRESS_DEC_DIGITS, ZERO_CLEANING_ON, SAVESPACE_ON,TEST_E_HW_TEXT_COLOR, BACK_COLOR); 
            TFT_write_float(5,260,150,local_actual.temp_lps25,TEMP_INT_DIGITS,TEMP_DEC_DIGITS, ZERO_CLEANING_ON, SAVESPACE_ON,TEST_E_HW_TEXT_COLOR, BACK_COLOR); 
            TFT_write_float(5,280,150,local_actual.uv_readed.uva,UV_INT_DIGITS,UV_DEC_DIGITS, ZERO_CLEANING_ON, SAVESPACE_ON,TEST_E_HW_TEXT_COLOR, BACK_COLOR); 
            TFT_write_float(5,300,150,local_actual.uv_readed.uvb,UV_INT_DIGITS,UV_DEC_DIGITS, ZERO_CLEANING_ON, SAVESPACE_ON,TEST_E_HW_TEXT_COLOR, BACK_COLOR);

        }  
    }
    TFT_clear(BACK_COLOR);
}

/* ============================================================================================================================================= /
 * FUNCTION:    DCF77_check
 *
 * Overview:    Ciclo di test per verifica del segnale DCF77. Disegna su display un grafico che rappresenta il segnale DCF77, sono visualizzati
 *              4 impulsi (4 secondi) con un test del segnale ogni 20ms.
 * Nota:        Per avere un grafico pi preciso diminuire il tempo di test del segnale, ma verranno visualizzati meno impulsi (10ms = 2 impulsi, 5ms = 1 impulso)             
/* ============================================================================================================================================ */
void DCF77_check(void){
    unsigned int _x = 0;
    unsigned char _y = 0;
    unsigned int _prevoius_y;
    
    TFT_clear(BACK_COLOR);
    TFT_write_string(5,300,230,messages[2], SAVESPACE_ON, TEST_E_HW_LABEL_COLOR, BACK_COLOR);           // "Tocca il display per uscire."
    
    // area grafico
    TFT_fill_rounded_rect(CHART_RECT_X, CHART_RECT_Y, CHART_RECT_L, CHART_RECT_H, 5, CHART_RECT_COLOR);
    TFT_draw_line_x(CHART_X,CHART_X_END, CHART_Y + CHART_X_OFFSET_Y, CHART_AXIS_COLOR);
    TFT_draw_line_y(CHART_Y,CHART_Y_END, CHART_X + CHART_Y_OFFSET_X, CHART_AXIS_COLOR);
        
    DCF77_start_receive_via_interrupt();                                                // avvio ricezione DCF77
    while (dcf77_seconds_pulse == 0 && XPT2046_IRQ);                                    // attendo nuovo secondo
    
    do{                                                                                 // ripetere finch non viene toccato lo schermo
        if (DCF77_SIGNAL == DCF77_SIGNAL_ACTIVE)                                        // calcolo la y
//        if (dcf77_seconds_pulse == 1)                                                   
            _y =  CHART_Y_END - (106 - CHART_X_OFFSET_Y);
        else 
            _y = CHART_Y;
        
        TFT_draw_line_y(CHART_Y, CHART_Y_END - (106 - CHART_X_OFFSET_Y), _x + CHART_X + CHART_Y_OFFSET_X + 1, CHART_RECT_COLOR);                   // cancello valore precedente
        
        if (_x == 0)
            TFT_draw_pixel(_x + CHART_X + CHART_Y_OFFSET_X + 1, _y, RED);
        else    
            TFT_draw_line_p2p(_x + CHART_X + CHART_Y_OFFSET_X, _prevoius_y, _x + CHART_X + CHART_Y_OFFSET_X + 1, _y, RED);
        
        
        _prevoius_y = _y;
        _x++;
        
        if (_x == 200){
            _x = 0;
        }
        Delay_ms(20);
        
    }while (XPT2046_IRQ);
    
    DCF77_stop_receive_via_interrupt();    
    TFT_clear(BACK_COLOR);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Set_remote_aq_time
 * Input:       _time: tempo di refresh e invio dei sensori del modulo remoto 1~8 (unsigned char) 
 *
 * Overview:    Invia al modulo remoto il parametro per settare il tempo di refresh e invio dei sensori
 * Note:        Il modulo remoto deve essere spento e acceso quando richiesto
/* ============================================================================================================================================ */
void Set_remote_aq_time (unsigned char _time){
//    TFT_set_font(font_lucida_sans_10x16);
    TFT_fill_rect(1,17,240,320, BACK_COLOR);
    TFT_write_string(5,20,235,messages[35], SAVESPACE_ON, TEST_E_HW_LABEL_COLOR , BACK_COLOR);
    
    Delay_ms(500);
    HMTRP_reset_buffer();
    
    // attendo avvio modulo remoto o tocco display
    while ((hmtrp_string_received == 0) && (XPT2046_IRQ == 1));
        
    if(hmtrp_string_received){                                                          // modulo remoto acceso
        if(hmtrp_buffer[0] == 0xF0){                                                    // invio conferma rilevamento
            HMTRP_reset_buffer();
            HMTRP_send_char(0xF0);
            HMTRP_send_char(0x0D);
            HMTRP_send_char(0x0A);
            
            Delay_ms(5);                                                                // tempo per cancellare il buffer nel modulo remoto
                                                                                        // invio 0xF4 per settaggio tempo invio
            HMTRP_send_char(0xF4);                                                      // invio tempo
            HMTRP_send_char(_time);                                                     // e byte di chiusura
            HMTRP_send_char(0x0D);
            HMTRP_send_char(0x0A);
        }
    }    
//    TFT_clear(BACK_COLOR);
//    TFT_fill_rect(1,20,240,40, BACK_COLOR);
    TFT_clear(BACK_COLOR);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Touch_screen_manage_loop
 * Input:       _page_touch: pagina da gestire
 * Output:      Punto toccato codificato con valori noti
 *
 * Overview:    Ciclo di gestione touch screen
/* ============================================================================================================================================ */
unsigned char Touch_screen_manage_loop(unsigned char _page_touch){  
    xpt2046_point _touched_point_scaled;
    unsigned char _return = 0;
    
    _touched_point_scaled = XPT2046_read_point_scaled();                                // leggo le coordinate del punto toccato
    
    // gestione pagina main
    if (_page_touch == MAIN_PAGE){
        if (_touched_point_scaled.y > TIME_Y && _touched_point_scaled.y < TIME_Y + 56){  // se viene toccata l'ora forzo l'aggiornamento dei dati via DCF77
            if (_touched_point_scaled.x > TIME_X && _touched_point_scaled.x < TIME_X + 100)
                _return = T_HOUR;
        } else if (_touched_point_scaled.y > METEO_ICON_D1_Y && _touched_point_scaled.y <SENSORS_Y-20) // se viene toccata l'area del meteo
            _return = T_METEO;
        else if (_touched_point_scaled.y > SENSORS_Y && _touched_point_scaled.y <= SENSORS_Y +20){    
            if (_touched_point_scaled.x < 120)                                          // se viene toccata l'etichetta dei sensori locali
                _return = T_LOCAL_SENSOR;
            else   
                _return = T_REMOTE_SENSOR;// se viene toccata l'etichetta dei sensori remoti
        }
    }

    // gestione pagine menu setup
    else if (_page_touch == TOUCH_SETUP_MENU){
        if (_touched_point_scaled.y > SETUP_ICON_1_Y && _touched_point_scaled.y < SETUP_ICON_2_Y)
            _return = T_SETUP_R_1;
        else if (_touched_point_scaled.y > SETUP_ICON_2_Y && _touched_point_scaled.y < SETUP_ICON_3_Y)
            _return = T_SETUP_R_2;
        else if (_touched_point_scaled.y > SETUP_ICON_3_Y && _touched_point_scaled.y < SETUP_ICON_4_Y)
            _return = T_SETUP_R_3;
        else if (_touched_point_scaled.y > SETUP_ICON_4_Y && _touched_point_scaled.y < SETUP_ICON_5_Y)
            _return = T_SETUP_R_4;
         else if (_touched_point_scaled.y > SETUP_ICON_5_Y && _touched_point_scaled.y < SETUP_ICON_6_Y)
            _return = T_SETUP_R_5;
        else if (_touched_point_scaled.y > SETUP_ICON_6_Y && _touched_point_scaled.y < SETUP_ICON_7_Y)
            _return = T_SETUP_R_6;
        else if (_touched_point_scaled.y > SETUP_ICON_7_Y && _touched_point_scaled.y < SETUP_ICON_8_Y)
            _return = T_SETUP_R_7;
        else if (_touched_point_scaled.y > SETUP_ICON_8_Y && _touched_point_scaled.y < SETUP_ICON_9_Y)
            _return = T_SETUP_R_8;
        else if (_touched_point_scaled.y > SETUP_ICON_9_Y && _touched_point_scaled.y < SW_1_Y)
            _return = T_SETUP_R_9;
    }
    
    // gestione pagina setup luminosit
    else if (_page_touch == TOUCH_SETUP_LIGHT){
        if (_touched_point_scaled.y > SETTING_LINE_1_Y && _touched_point_scaled.y < SETTING_LINE_1_Y + SETTING_LINE_Y_SPACE)
            _return = T_LIGHT_LEV;
        else if (_touched_point_scaled.y > SETTING_LINE_2_Y && _touched_point_scaled.y < SETTING_LINE_2_Y + SETTING_LINE_Y_SPACE){
            if (_touched_point_scaled.x > SETUP_LIGHT_HOURS_X  && _touched_point_scaled.x < SETUP_LIGHT_HOURS_X + SETUP_LIGHT_X_LEN)
                _return = T_ON_HOURS;
            else if (_touched_point_scaled.x > SETUP_LIGHT_MINUTES_X  && _touched_point_scaled.x < SETUP_LIGHT_MINUTES_X + SETUP_LIGHT_X_LEN)
                _return = T_ON_MINUTES;
        } else if (_touched_point_scaled.y > SETTING_LINE_3_Y && _touched_point_scaled.y < SETTING_LINE_3_Y + SETTING_LINE_Y_SPACE){
            if (_touched_point_scaled.x > SETUP_LIGHT_HOURS_X  && _touched_point_scaled.x < SETUP_LIGHT_HOURS_X + SETUP_LIGHT_X_LEN)
                _return = T_OFF_HOURS;
            else if (_touched_point_scaled.x > SETUP_LIGHT_MINUTES_X  && _touched_point_scaled.x < SETUP_LIGHT_MINUTES_X + SETUP_LIGHT_X_LEN)
                _return = T_OFF_MINUTES;
        }
    }
    
    // gestione pagina setup ora manuale
    else if (_page_touch == TOUCH_SETUP_TIME){
        if (_touched_point_scaled.y > SETTING_LINE_1_Y && _touched_point_scaled.y < SETTING_LINE_1_Y + SETTING_LINE_Y_SPACE)
            _return = T_SETUP_DAY_W;
        else if (_touched_point_scaled.y > SETTING_LINE_2_Y && _touched_point_scaled.y < SETTING_LINE_2_Y + SETTING_LINE_Y_SPACE){
            if (_touched_point_scaled.x > SETUP_TIME_DAY_VALUE_X && _touched_point_scaled.x < SETUP_TIME_DAY_VALUE_X + SETUP_DATE_X_LEN)
                _return = T_SETUP_DAY;
            else if (_touched_point_scaled.x > SETUP_TIME_MONTH_VALUE_X && _touched_point_scaled.x < SETUP_TIME_MONTH_VALUE_X + SETUP_DATE_X_LEN)
                _return = T_SETUP_MONTH;
            else if (_touched_point_scaled.x > SETUP_TIME_YEAR_VALUE_X && _touched_point_scaled.x < SETUP_TIME_YEAR_VALUE_X + SETUP_DATE_X_LEN)
                _return = T_SETUP_YEAR;
        } else if (_touched_point_scaled.y > SETTING_LINE_3_Y && _touched_point_scaled.y < SETTING_LINE_3_Y + SETTING_LINE_Y_SPACE){
            if (_touched_point_scaled.x > SETUP_HOUR_VALUE_X && _touched_point_scaled.x < SETUP_HOUR_VALUE_X + SETUP_TIME_X_LEN)
                _return = T_SETUP_HOURS;
            else if (_touched_point_scaled.x > SETUP_MINUTES_VALUE_X && _touched_point_scaled.x < SETUP_MINUTES_VALUE_X + SETUP_TIME_X_LEN)
                _return = T_SETUP_MINUTES;
        } else if (_touched_point_scaled.y > SETTING_LINE_4_Y && _touched_point_scaled.y < SETTING_LINE_4_Y + SETTING_LINE_Y_SPACE){
            _return = T_SETUP_CET_CEST;
        } else if (_touched_point_scaled.y > SETTING_LINE_5_Y && _touched_point_scaled.y < SETTING_LINE_5_Y + SETTING_LINE_Y_SPACE){
            _return = T_SETUP_DCF77;
        }
    }
    
    // gestione pagina setup sensori
    else if (_page_touch == TOUCH_SETUP_SENSORS){
        if (_touched_point_scaled.y > SETTING_LINE_1_Y && _touched_point_scaled.y < SETTING_LINE_1_Y + SETTING_LINE_Y_SPACE)
            _return = T_SETUP_LOCAL_SENSOR;
        else if (_touched_point_scaled.y > SETTING_LINE_2_Y && _touched_point_scaled.y < SETTING_LINE_2_Y + SETTING_LINE_Y_SPACE)
            _return = T_SETUP_REMOTE_SENSOR;
        else if (_touched_point_scaled.y > SETTING_LINE_3_Y && _touched_point_scaled.y < SETTING_LINE_3_Y + SETTING_LINE_Y_SPACE)
            _return = T_SETUP_PRESS_TO_WRITE;
        else if (_touched_point_scaled.y > SETTING_LINE_4_Y && _touched_point_scaled.y < SETTING_LINE_4_Y + SETTING_LINE_Y_SPACE)
            _return = T_SETUP_LOCAL_ALTITUDE;
    }
    
    // gestione pagina setup wifi
    else if (_page_touch == TOUCH_SETUP_WIFI){
        if (_touched_point_scaled.y > SETTING_LINE_1_Y && _touched_point_scaled.y < SETTING_LINE_1_Y + SETTING_LINE_Y_SPACE)
            _return = T_SETUP_WIFI_EN;
        else if (_touched_point_scaled.y > SETTING_LINE_8_Y && _touched_point_scaled.y < SETTING_LINE_8_Y + SETTING_LINE_Y_SPACE)
            _return = T_SETUP_WIFI_SCAN_SSID;           
    }
    
    // gestione pagina selezione ssid
    else if (_page_touch == TOUCH_SETUP_WIFI_SSID){
        if (_touched_point_scaled.y > SETTING_LINE_1_Y && _touched_point_scaled.y < SETTING_LINE_1_Y + 2*SETTING_LINE_Y_SPACE)
            _return = T_SETUP_WIFI_SSID_1;
        else if (_touched_point_scaled.y > SETTING_LINE_2_Y && _touched_point_scaled.y < SETTING_LINE_3_Y + 2*SETTING_LINE_Y_SPACE)
            _return = T_SETUP_WIFI_SSID_2;
        else if (_touched_point_scaled.y > SETTING_LINE_3_Y && _touched_point_scaled.y < SETTING_LINE_5_Y + 2*SETTING_LINE_Y_SPACE)
            _return = T_SETUP_WIFI_SSID_3;    
    }
    
    // gestione pulsanti pagina log
    else if (_page_touch == TOUCH_LOG){
        if (_touched_point_scaled.y > ARROW_NEXT_Y  && _touched_point_scaled.y < ARROW_NEXT_Y + 12){
            if (_touched_point_scaled.x > ARROW_PREV_X-20  && _touched_point_scaled.x < ARROW_PREV_X + 30)
                _return = T_ARROW_PREV;
            else if (_touched_point_scaled.x > ARROW_NEXT_X-20  && _touched_point_scaled.x < ARROW_NEXT_X + 30)
                _return = T_ARROW_NEXT;
        }else if (_touched_point_scaled.y > SW_5_Y && _touched_point_scaled.y < SW_9_Y ){
            if (_touched_point_scaled.x > SW_5_X  && _touched_point_scaled.x < SW_5_X+40)
                _return = T_SW_5;
            else if (_touched_point_scaled.x > SW_6_X  && _touched_point_scaled.x < SW_6_X+40)
                _return = T_SW_6;
            else if (_touched_point_scaled.x > SW_7_X  && _touched_point_scaled.x < SW_7_X+40)
                _return = T_SW_7;
            else if (_touched_point_scaled.x > SW_8_X  && _touched_point_scaled.x < SW_8_X+40)
                _return = T_SW_8;
        } else if (_touched_point_scaled.y > SW_9_Y && _touched_point_scaled.y < SW_13_Y ){
            if (_touched_point_scaled.x > SW_9_X  && _touched_point_scaled.x < SW_9_X+40)
                _return = T_SW_9;
            else if (_touched_point_scaled.x > SW_10_X  && _touched_point_scaled.x < SW_10_X+40)
                _return = T_SW_10;
            else if (_touched_point_scaled.x > SW_11_X  && _touched_point_scaled.x < SW_11_X+40)
                _return = T_SW_11;
            else if (_touched_point_scaled.x > SW_12_X  && _touched_point_scaled.x < SW_12_X+40)
                _return = T_SW_12;
        } else if (_touched_point_scaled.y > SW_13_Y && _touched_point_scaled.y < SW_1_Y ){
            if (_touched_point_scaled.x > SW_13_X  && _touched_point_scaled.x < SW_13_X+40)
                _return = T_SW_13;
            else if (_touched_point_scaled.x > SW_14_X  && _touched_point_scaled.x < SW_14_X+40)
                _return = T_SW_14;
            else if (_touched_point_scaled.x > SW_15_X  && _touched_point_scaled.x < SW_15_X+40)
                _return = T_SW_15;
            else if (_touched_point_scaled.x > SW_16_X  && _touched_point_scaled.x < SW_16_X+40)
                _return = T_SW_16;
        } 
    }
  
//    if (_page_touch == MAIN_PAGE || _page_touch == TOUCH_SETUP_MENU){ 
        if (_touched_point_scaled.y > SW_1_Y){                                                   // gestione pulsanti
            if (_touched_point_scaled.x > SW_1_X  && _touched_point_scaled.x < SW_1_X+40)
                _return = T_SW_1;
            else if (_touched_point_scaled.x > SW_2_X  && _touched_point_scaled.x < SW_2_X+40)
                _return = T_SW_2;
            else if (_touched_point_scaled.x > SW_3_X  && _touched_point_scaled.x < SW_3_X+40)
                _return = T_SW_3;
            else if (_touched_point_scaled.x > SW_4_X  && _touched_point_scaled.x < SW_4_X+40)
                _return = T_SW_4;
        }
        Delay_ms(100);
//    }

    return _return;
}

/* ============================================================================================================================================= /
 * FUNCTION:    Handle_keyboard
 * Output:      Carattere letto dalla tastiera, o 0x00 se non  stato premuto nulla
 * 
 * Overview:    Disegna la tastiera e restituisce il valore del tasto premuto
/* ============================================================================================================================================ */
unsigned char Handle_keyboard(void){
    xpt2046_point _touched_point_scaled;
    unsigned char _c = 0;
    unsigned char _r = 0;
    static unsigned char _keyboard = 0;
    unsigned char _key_pressed = 0;
    
//    static unsigned char _keyboard_0_drawed = 0;
//    static unsigned char _keyboard_1_drawed = 0;
//    static unsigned char _keyboard_2_drawed = 0;
    
    static unsigned char _keyboard_drawed = 0;
    
    // array tastiera
    const unsigned char key[4][10] = {{'q','w','e','r','t','y','u','i','o','p'},
                                      {'a','s','d','f','g','h','j','k','l',0x08},       // 0x08 = BACKSPACE     
                                      {0x0F,'z','x','c','v','b','n','m',0x11,0x11},     // 0x0F = SHIFT IN, 0x11 = NUMERI E SIMBOLI
                                      {' ',' ',' ',' ',' ',' ',' ',' ',0x0D,0x0D}};     // 0x0D = ENTER

    const unsigned char key_cap[4][10] = {{'Q','W','E','R','T','Y','U','I','O','P'},
                                      {'A','S','D','F','G','H','J','K','L',0x08},       // 0x08 = BACKSPACE     
                                      {0x0E,'Z','X','C','V','B','N','M',0x11,0x11},     // 0x0E = SHIFT OUT, 0x11 = NUMERI E SIMBOLI
                                      {' ',' ',' ',' ',' ',' ',' ',' ',0x0D,0x0D}};     // 0x0D = ENTER

    const unsigned char key_num[4][10] = {{'1','2','3','4','5','6','7','8','9','0'},
                                      {'-','/',':',';','(',')','$','&','@',0x08},       // 0x08 = BACKSPACE     
                                      {0x00,'.',',','?','!',0x27,'%','_',0x12,0x12},    // 0x0F = SHIFT, 0x12 = LETTERE
                                      {' ',' ',' ',' ',' ',' ',' ',' ',0x0D,0x0D}};     // 0x0D = ENTER 
    
    switch (_keyboard){                                                                 // pannello della tastiera da disegnare
        case 0: if (_keyboard_drawed != 1){
                    TFT_draw_image_sd(KEYBOARD_X,KEYBOARD_Y,"K_0.bmp");
                    _keyboard_drawed = 1;                                               // flag tastiera disegnata per disegnare solo 1 volta 
                }
            break; 
        case 1: if (_keyboard_drawed != 2){
                    TFT_draw_image_sd(KEYBOARD_X,KEYBOARD_Y,"K_1.bmp");
                    _keyboard_drawed = 2; 
                }
            break; 
        case 2: if (_keyboard_drawed != 3){
                    TFT_draw_image_sd(KEYBOARD_X,KEYBOARD_Y,"K_2.bmp");
                    _keyboard_drawed = 3; 
                }
            break; 
    }

    XPT2046_enable();   
    
    if (XPT2046_IRQ == 0){                                                              // se il display  stato toccato
        _touched_point_scaled = XPT2046_read_point_scaled();                            // leggo il punto

        // cerco la riga
        if (_touched_point_scaled.y > KEYBOARD_Y && _touched_point_scaled.y < KEYBOARD_R_1)
            _r = 0;
        else if (_touched_point_scaled.y > KEYBOARD_R_1 && _touched_point_scaled.y < KEYBOARD_R_2)
            _r = 1;
        else if (_touched_point_scaled.y > KEYBOARD_R_2 && _touched_point_scaled.y < KEYBOARD_R_3)
            _r = 2;
        else if (_touched_point_scaled.y > KEYBOARD_R_3 && _touched_point_scaled.y < KEYBOARD_R_4)
            _r = 3;  
        else
            _r = 0xFF; // riga fuori dai limiti
        
        // se la riga  nei limiti cerco la colonna
        if (_r != 0xFF){
            if (_touched_point_scaled.x > KEYBOARD_X && _touched_point_scaled.x < KEYBOARD_C_1)
                _c = 0;
            else if (_touched_point_scaled.x > KEYBOARD_C_1 && _touched_point_scaled.x < KEYBOARD_C_2)
                _c = 1;
            else if (_touched_point_scaled.x > KEYBOARD_C_2 && _touched_point_scaled.x < KEYBOARD_C_3)
                _c = 2;
            else if (_touched_point_scaled.x > KEYBOARD_C_3 && _touched_point_scaled.x < KEYBOARD_C_4)
                _c = 3;
            else if (_touched_point_scaled.x > KEYBOARD_C_4 && _touched_point_scaled.x < KEYBOARD_C_5)
                _c = 4;
            else if (_touched_point_scaled.x > KEYBOARD_C_5 && _touched_point_scaled.x < KEYBOARD_C_6)
                _c = 5;
            else if (_touched_point_scaled.x > KEYBOARD_C_6 && _touched_point_scaled.x < KEYBOARD_C_7)
                _c = 6;
            else if (_touched_point_scaled.x > KEYBOARD_C_7 && _touched_point_scaled.x < KEYBOARD_C_8)
                _c = 7;
            else if (_touched_point_scaled.x > KEYBOARD_C_8 && _touched_point_scaled.x < KEYBOARD_C_9)
                _c = 8;
            else if (_touched_point_scaled.x > KEYBOARD_C_9 && _touched_point_scaled.x < KEYBOARD_C_10)
                _c = 9;  
        

            // e leggo il carattere dall'array alla riga e colonna premuti
            switch (_keyboard){
                case 0: _key_pressed = key[_r][_c];
                    break;
                case 1: _key_pressed = key_cap[_r][_c];
                    break;
                case 2: _key_pressed = key_num[_r][_c];
                    break;  
            }
        
            // caratteri speciali per cambio pannello tastiera
            if (_key_pressed == 0x0F){                                                  // maiuscole
                _keyboard = 1;
                _key_pressed = 0;
            }else if (_key_pressed == 0x11){                                            // numeri e simboli
                _keyboard = 2;
                _key_pressed = 0;
            }else if (_key_pressed == 0x0E || _key_pressed == 0x12){                    // normale
                _keyboard = 0;
                _key_pressed = 0;
            } else if (_key_pressed == 0x0D){                                           // invio significa che poi cancello la tastiera
                _keyboard_drawed = 0;
                _keyboard = 0;
            }
        }
        // se la riga non  nei limiti torna 0
        
    }
    return _key_pressed;
}

/* ============================================================================================================================================= /
 * FUNCTION:    Drawing
 *
 * Overview:    Disegno a mano libera
/* ============================================================================================================================================ */
void Drawing(void){
    xpt2046_point _touched_point_scaled;
    unsigned char _exit = 0;

    TFT_clear(WHITE);
    unsigned int color_selected = 0;
    
    XPT2046_enable();
    
    TFT_fill_rect(1,301,20,20,BLACK); // Nero
    TFT_fill_rect(21,301,20,20,BLUE); // Blu
    TFT_fill_rect(41,301,20,20,RED); // Rosso
    TFT_fill_rect(61,301,20,20,GREEN); // Verde
    TFT_fill_rect(81,301,20,20,YELLOW);// Giallo
    TFT_fill_rect(101,301,20,20,BROWN);// Marrone
    TFT_fill_rect(121,301,20,20,PURPLE);// Viola
    TFT_fill_rect(141,301,20,20,GRAY);// Grigio
    TFT_fill_rect(161,301,20,20,CYAN);// Ciano
    TFT_fill_rect(181,301,20,20,ORANGE);// Arancione
    TFT_draw_rect(201,301,20,20,BLACK);// Clear Display
    TFT_draw_line_p2p(222,301,239,319, RED);
    TFT_draw_line_p2p(239,301,222,319, RED);
    
//    TFT_write_char(20,1,'F',SAVESPACE_OFF,BACK_COLOR,WHITE);
//    TFT_write_char(40,1,'M',SAVESPACE_OFF,BACK_COLOR,WHITE);
//    TFT_write_char(60,1,'L',SAVESPACE_OFF,BACK_COLOR,WHITE);

    
    do{
        if (XPT2046_IRQ == 0) {
            _touched_point_scaled = XPT2046_read_point_scaled();
            
            if (_touched_point_scaled.y > 300){
                if (_touched_point_scaled.x < 20)
                    color_selected = BLACK;
                else if (_touched_point_scaled.x > 21 && _touched_point_scaled.x < 40)
                    color_selected = BLUE;
                else if (_touched_point_scaled.x > 41 && _touched_point_scaled.x < 60)
                    color_selected = RED;
                else if (_touched_point_scaled.x > 61 && _touched_point_scaled.x < 80)
                    color_selected = GREEN;
                else if (_touched_point_scaled.x > 81 && _touched_point_scaled.x < 100)
                    color_selected = YELLOW;
                else if (_touched_point_scaled.x > 101 && _touched_point_scaled.x < 120)
                    color_selected = 0x7800;  
                else if (_touched_point_scaled.x > 121 && _touched_point_scaled.x < 140)
                    color_selected = 0x780F;
                else if (_touched_point_scaled.x > 141 && _touched_point_scaled.x < 160)
                    color_selected = 0xC618;  
                else if (_touched_point_scaled.x > 161 && _touched_point_scaled.x < 180)
                    color_selected = 0x07FF;
                else if (_touched_point_scaled.x > 181 && _touched_point_scaled.x < 200)
                    color_selected = 0xFD20;
                else if (_touched_point_scaled.x > 201 && _touched_point_scaled.x < 220)
                    TFT_fill_rect(1,1,240,300,WHITE);
                else if (_touched_point_scaled.x > 221)
                    _exit = 1;   
            }
            
//            if (_touched_point_scaled.y < 20){
//                 if (_touched_point_scaled.x > 20 && _touched_point_scaled.x < 40)
//                     width = 1;
//                 else if (_touched_point_scaled.x > 40 && _touched_point_scaled.x < 60)
//                     width = 2;
//                 else if (_touched_point_scaled.x > 60 && _touched_point_scaled.x < 80)
//                     width = 3;
//            }
            
            if (_touched_point_scaled.y < 300){
                TFT_draw_pixel(_touched_point_scaled.x,_touched_point_scaled.y,color_selected);
                TFT_draw_pixel(_touched_point_scaled.x+1,_touched_point_scaled.y,color_selected);
                TFT_draw_pixel(_touched_point_scaled.x,_touched_point_scaled.y+1,color_selected);
                TFT_draw_pixel(_touched_point_scaled.x+1,_touched_point_scaled.y+1,color_selected);
            }
            
        }
    } while (_exit == 0);
    
    TFT_clear(BACK_COLOR);
}       

/* ============================================================================================================================================= /
 * FUNCTION:    Save_eeprom_default_variables
 *
 * Overview:    Salva nella EEPROM interna i valori di partenza
 * Note:        Eseguita solo durante primo avvio dopo programmazione 
/* ============================================================================================================================================ */
void Save_eeprom_default_variables(void){
    EEPROM_write_int(230, EEPROM_TOUCH_POINT_1_X_ADDRESS);
    EEPROM_write_int(225, EEPROM_TOUCH_POINT_1_Y_ADDRESS);
    EEPROM_write_int(3800, EEPROM_TOUCH_POINT_2_X_ADDRESS);
    EEPROM_write_int(3740, EEPROM_TOUCH_POINT_2_Y_ADDRESS);
    EEPROM_write(200,EEPROM_CCPR4L_ADDRESS);
    EEPROM_write(0x07,EEPROM_BKILL_ON_H_ADDRESS);
    EEPROM_write(0x00,EEPROM_BKILL_ON_M_ADDRESS);
    EEPROM_write(0x22,EEPROM_BKILL_OFF_H_ADDRESS);
    EEPROM_write(0x30,EEPROM_BKILL_OFF_M_ADDRESS);
    EEPROM_write_int(192,EEPROM_ALTITUDE_ADDRESS);
    EEPROM_write(1,EEPROM_UTC_ADDRESS);
    EEPROM_write(40,EEPROM_METEO_ZONE_ADDRESS);
    EEPROM_write(1,EEPROM_PRESS_2_WRITE_ADDRESS);
    EEPROM_write(2,EEPROM_REFRESH_TIME_LOCAL_ADDRESS);
    EEPROM_write(2,EEPROM_REFRESH_TIME_REMOTE_ADDRESS);
    EEPROM_write(1,EEPROM_DCF77_USER_EN_ADDRESS);
    EEPROM_write(0,EEPROM_WIFI_EN_ADDRESS);
    
    // dati meteo a 0x7F
    for (unsigned char _i = 0; _i < 64; _i++){
        EEPROM_write(0x7F, EEPROM_FC_D1_ADDRESS + _i);
    }
    
    EEPROM_write(0x70,EEPROM_START_CHECK);                                              // byte di controllo primo avvio dopo programmazione avvenuto
} 

/* ============================================================================================================================================= /
 * FUNCTION:    Load_eeprom_variables
 *
 * Overview:    Legge i dati dall EEPROM interna e li salva nelle relative variabili globali
/* ============================================================================================================================================ */
void Load_eeprom_variables(void){
    xpt2046_point _touched_point_raw_1;
    xpt2046_point _touched_point_raw_2;
    unsigned char _i = 0;
    
    _touched_point_raw_1.x = EEPROM_read_int(EEPROM_TOUCH_POINT_1_X_ADDRESS);
    _touched_point_raw_1.y = EEPROM_read_int(EEPROM_TOUCH_POINT_1_Y_ADDRESS);    
    _touched_point_raw_2.x = EEPROM_read_int(EEPROM_TOUCH_POINT_2_X_ADDRESS);
    _touched_point_raw_2.y = EEPROM_read_int(EEPROM_TOUCH_POINT_2_Y_ADDRESS);
    
    // calcolo il fattore di scaling per il touch screen
    XPT2046_calc_scaling_factor(_touched_point_raw_1, _touched_point_raw_2);
    
    // imposto la retroilluminazione
    ccpr4l = EEPROM_read(EEPROM_CCPR4L_ADDRESS);
    CCPR4L = ccpr4l;
    
    // orari di accensione e spegnimento
    backill_on_h = EEPROM_read(EEPROM_BKILL_ON_H_ADDRESS);
    backill_on_m = EEPROM_read(EEPROM_BKILL_ON_M_ADDRESS);
    backill_off_h = EEPROM_read(EEPROM_BKILL_OFF_H_ADDRESS);
    backill_off_m = EEPROM_read(EEPROM_BKILL_OFF_M_ADDRESS); 
    
    local_altitude = EEPROM_read_int(EEPROM_ALTITUDE_ADDRESS); 
    time_zone_utc = EEPROM_read(EEPROM_UTC_ADDRESS);
    meteo_zone = EEPROM_read(EEPROM_METEO_ZONE_ADDRESS);
    press_to_write = EEPROM_read(EEPROM_PRESS_2_WRITE_ADDRESS);
    
    dcf77_user_enable = EEPROM_read(EEPROM_DCF77_USER_EN_ADDRESS);
    
    wifi_on_off = EEPROM_read(EEPROM_WIFI_EN_ADDRESS); 
    
    // ricavo i tempi di scansione sei sensori locali
    unsigned char _temp = 0;
    _temp = EEPROM_read(EEPROM_REFRESH_TIME_LOCAL_ADDRESS);
    
    switch (_temp){
        case 1: time_refresh_local_sensors = TEMP_SCANS_SENS_1;
            break;
        case 2: time_refresh_local_sensors = TEMP_SCANS_SENS_2;
            break;            
        case 3: time_refresh_local_sensors = TEMP_SCANS_SENS_3;
            break;            
        case 4: time_refresh_local_sensors = TEMP_SCANS_SENS_4;
            break;            
        case 5: time_refresh_local_sensors = TEMP_SCANS_SENS_5;
            break;            
        case 6: time_refresh_local_sensors = TEMP_SCANS_SENS_6;
            break;            
        case 7: time_refresh_local_sensors = TEMP_SCANS_SENS_7;
            break;            
        case 8: time_refresh_local_sensors = TEMP_SCANS_SENS_8;
            break;            
    }
    
    // ricavo i tempi di scansione dei sensori remoti
    _temp = EEPROM_read(EEPROM_REFRESH_TIME_REMOTE_ADDRESS);
    
    switch (_temp){
        case 1: time_refresh_remote_sensors = TEMP_SCANS_SENS_1;
            break;
        case 2: time_refresh_remote_sensors = TEMP_SCANS_SENS_2;
            break;            
        case 3: time_refresh_remote_sensors = TEMP_SCANS_SENS_3;
            break;            
        case 4: time_refresh_remote_sensors = TEMP_SCANS_SENS_4;
            break;            
        case 5: time_refresh_remote_sensors = TEMP_SCANS_SENS_5;
            break;            
        case 6: time_refresh_remote_sensors = TEMP_SCANS_SENS_6;
            break;            
        case 7: time_refresh_remote_sensors = TEMP_SCANS_SENS_7;
            break;            
        case 8: time_refresh_remote_sensors = TEMP_SCANS_SENS_8;
            break;            
    }
    
    time_refresh_remote_sensors +=2;                                                    // aggiungo 2 secondi di margine
       
    _i = 0;
    
    // se c' un ssid salvato leggo l'ssid da EEPROM
    if (EEPROM_read(EEPROM_WIFI_SSID_ADDRESS) != 0xFF) {
        do{
            ssid[_i] = EEPROM_read(EEPROM_WIFI_SSID_ADDRESS + _i);
            _i++;
        } while (ssid[_i-1] != '\0');
    }
    
    _i = 0;
    
    // se c' una password salvata leggo la password da EEPROM
    if (EEPROM_read(EEPROM_WIFI_PASSWORD_ADDRESS) != 0xFF) {
        do {
            password[_i] = EEPROM_read(EEPROM_WIFI_PASSWORD_ADDRESS + _i);
            _i++;
        } while (password[_i-1] != '\0' && ssid[_i] != 0xFF);
    }
    
    // carico l'ultimo indice del log sensori
    eeprom_log_index = EEPROM_I2C_read(EEPROM_I2C_DEV_ADDR_B1,0x00);
}   

/* ============================================================================================================================================= /
 * FUNCTION:    Save_meteo_data_eeprom
 * Input:       _update_all: 1 = sposta indietro i dati precedentemente salvati, 0 salva il dato a seconda del puntatore attuale (globale)
 *              
 * Overview:    Salva i dati meteo decodificati in EEPROM
/* ============================================================================================================================================ */
void Save_meteo_data_eeprom(unsigned char _update_all){
    
    unsigned char _eeprom_addr;
    
    // imposto il primo indirizzo EEPROM a seconda del puntatore alla previsione meteo attuale
    if (_update_all != 1){
        if (meteo_fc_day_pointer == &meteo_fc_day_1){
            _eeprom_addr = EEPROM_FC_D1_ADDRESS;
        }
        else if (meteo_fc_day_pointer == &meteo_fc_day_2){
            _eeprom_addr = EEPROM_FC_D2_ADDRESS;
        }
        else if (meteo_fc_day_pointer == &meteo_fc_day_3){
            _eeprom_addr = EEPROM_FC_D3_ADDRESS;
        }
        else if (meteo_fc_day_pointer == &meteo_fc_day_4){
            _eeprom_addr = EEPROM_FC_D4_ADDRESS;
        }

        EEPROM_write(rtcc_time.day,_eeprom_addr);
        EEPROM_write(rtcc_time.month,_eeprom_addr +1);
        EEPROM_write(rtcc_time.hours,_eeprom_addr +2);
        EEPROM_write(rtcc_time.minutes,_eeprom_addr +3);
        EEPROM_write(meteo_fc_day_pointer->day_fc,_eeprom_addr +4);
        EEPROM_write(meteo_fc_day_pointer->night_fc,_eeprom_addr +5);
        EEPROM_write(meteo_fc_day_pointer->extreme_w,_eeprom_addr +6);
        EEPROM_write(meteo_fc_day_pointer->rain_pr,_eeprom_addr +7);
        EEPROM_write(meteo_fc_day_pointer->anomaly,_eeprom_addr +8);
        EEPROM_write(meteo_fc_day_pointer->wind_dir,_eeprom_addr +9);
        EEPROM_write(meteo_fc_day_pointer->wind_str,_eeprom_addr +10);
        EEPROM_write(meteo_fc_day_pointer->temp_min,_eeprom_addr +11);
        EEPROM_write(meteo_fc_day_pointer->temp_max,_eeprom_addr +12);
    } else {
        // _meteo_time == 1 sposto indietro di 1 giorno i dati salvati
        for (unsigned char _i = 0; _i<48; _i++){
             EEPROM_write(EEPROM_read(EEPROM_FC_D2_ADDRESS +_i),EEPROM_FC_D1_ADDRESS + _i);
        }
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Load_meteo_data_eeprom
 *              
 * Overview:    Legge i dati meteo salvati in EEPROM e li salva nelle rispettive variabili globali
/* ============================================================================================================================================ */
void Load_meteo_data_eeprom(void){
    
    meteo_fc_compl *_meteo_fc_day_pointer;
    unsigned char _eeprom_addr;
    
    _eeprom_addr = EEPROM_FC_D1_ADDRESS;
    
    // imposto il puntatore alla previsione meteo
    for(unsigned char _i = 0; _i < 4; _i++){
        switch (_i){
            case 0: _meteo_fc_day_pointer = &meteo_fc_day_1;
                break;
            case 1: _meteo_fc_day_pointer = &meteo_fc_day_2;
                break;
            case 2: _meteo_fc_day_pointer = &meteo_fc_day_3;
                break;
            case 3: _meteo_fc_day_pointer = &meteo_fc_day_4;
                break;
        }
        
        // leggo i valori dalla memoria EEPROM e li salvo nella relativa struct di variabili
        _meteo_fc_day_pointer->day_fc = EEPROM_read(_eeprom_addr+4);
        _meteo_fc_day_pointer->night_fc = EEPROM_read(_eeprom_addr+5);
        _meteo_fc_day_pointer->extreme_w = EEPROM_read(_eeprom_addr+6);
        _meteo_fc_day_pointer->rain_pr = EEPROM_read(_eeprom_addr+7);
        _meteo_fc_day_pointer->anomaly = EEPROM_read(_eeprom_addr+8);
        _meteo_fc_day_pointer->wind_dir = EEPROM_read(_eeprom_addr+9);
        _meteo_fc_day_pointer->wind_str = EEPROM_read(_eeprom_addr+10);
        _meteo_fc_day_pointer->temp_min = EEPROM_read(_eeprom_addr+11);
        _meteo_fc_day_pointer->temp_max = EEPROM_read(_eeprom_addr+12);
        
        _eeprom_addr += 0x10;
    }   
}

/* ============================================================================================================================================= /
 * FUNCTION:    Init_meteo_data
 *              
 * Overview:    Imposta a 0x7F le variabili meteo per scrivere -- su display finch non  stato ricevuto un valore valido
/* ============================================================================================================================================ */
void Init_meteo_data(void){
meteo_fc_compl *_meteo_fc_day_pointer;
    
    for(unsigned char _i = 0; _i < 4; _i++){
        switch (_i){
            case 0: _meteo_fc_day_pointer = &meteo_fc_day_1;
            break;
            case 1: _meteo_fc_day_pointer = &meteo_fc_day_2;
            break;
            case 2: _meteo_fc_day_pointer = &meteo_fc_day_3;
            break;
            case 3: _meteo_fc_day_pointer = &meteo_fc_day_4;
            break;
        }
        
        _meteo_fc_day_pointer->day_fc = 0x7F;
        _meteo_fc_day_pointer->night_fc = 0x7F;
        _meteo_fc_day_pointer->extreme_w = 0x7F;
        _meteo_fc_day_pointer->rain_pr = 0x7F;
        _meteo_fc_day_pointer->anomaly = 0x7F;
        _meteo_fc_day_pointer->wind_dir = 0x7F;
        _meteo_fc_day_pointer->wind_str = 0x7F;
        _meteo_fc_day_pointer->temp_min = 0x7F;
        _meteo_fc_day_pointer->temp_max = 0x7F;
    }
}

/* ============================================================================================================================================= /
 * FUNCTION:    Inc_pbcd
 * Input:       _value: Valore da incrementare (unsigned char BCD)
 *              _min: valore minimo (unsigned char BCD)
 *              _max: limite massimo (unsigned char BCD)
 * Output:      Valore incrementato (unsigned char BCD)
 *
 * Overview:    Incrementa di 1 il byte codificato in BCD
/* ============================================================================================================================================ */
unsigned char Inc_pbcd(unsigned char _value, unsigned char _min, unsigned char _max) {
    _value++;                                                                           // incremento il valore
    if ((_value & 0b00001111) == 10)                                                    // se risultato degli ultimi 4 bit  10 sommo 6 al risultato
        _value = _value + 6;
    
    if (_value > _max)
        _value = _min;
    return(_value);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Dec_pbcd
 * Input:       _value: valore attuale da decrementare
 *              _min: valore minimo consentito del risultato
 *              _max: valore massimo consentito del risultato
 * Output:      Ritorna il valore decrementato di 1 unit
 *
 * Overview:    Decrementa di 1 unit il singolo byte (unsigned char) in packed BCD (4bit alti: decine, 4 bit bassi: unit (max 9) mantenendo il risultato
 *              tra il valore minimo e il valore massimo consentito
/* ============================================================================================================================================ */
unsigned char Dec_pbcd(unsigned char _value, unsigned char _min, unsigned char _max) {
    _value--;                                                                           // decremento il valore
    if ((_value & 0b00001111) == 15) 
        _value = _value - 6;                                                            // se il risultato degli ultimi 4 bit  15 sottraggo 6 al risultato
    if (_value < _min || _value > _max) _value = _max;                                  // se il risultato  minore del valore minimo consentito o maggiore
                                                                                        // del valore massimo (overflow) riporto il risultato al valore massimo
    return(_value);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Bcd_2_dec
 * Input:       _byte: byte da convertire (unsigned char BCD)
 * Output:      Valore convertito (unsigned char)
 *
 * Overview:    Converte un byte BCD in valore decimale
/* ============================================================================================================================================ */
unsigned char Bcd_2_dec(unsigned char _byte){
    return ((_byte >>4)*10) + (_byte & 0x0F);
}

/* ============================================================================================================================================= /
 * FUNCTION:    Dec_2_bcd
 * Input:       _byte: byte da convertire (unsigned char)
 * Output:      Valore convertito (unsigned char BCD)
 *
 * Overview:    Converte un byte decimale in valore BCD 
/* ============================================================================================================================================ */
unsigned char Dec_2_bcd(unsigned char _value){
    unsigned char _h = 0;
        
    while (_value > 9){
        _h++;
        _value -= 10;
    }
    
    return (_h << 4) | _value;
}

/* ============================================================================================================================================= /
 * FUNCTION:    Create_timestamp_string
 * Input:       _eeprom_dev_address: Indirizzo dispositivo memoria EEPROM (banco 1 o banco 2)
 *              _eeprom_addr: primo indirizzo eeprom contenente la data
 *              _data_2_read: numero di dati da leggere (5: giorno, mese, anno, ore, minuti; 6:  giorno, mese, anno, ore, minuti, secondi)
 * Output       puntatore all'array contentente la stringa             
 * 
 * Overview:    Legge i dati in memoria EEPROM I2C e crea la stringa con data e orario partendo dall'indirizzo indicato, usata per scrivere la data
 *              dei valori minimi e massimi in esposizione log su display
/* ============================================================================================================================================ */
unsigned char * Create_timestamp_string (unsigned char _eeprom_dev_address, volatile unsigned int _eeprom_addr, unsigned char _data_2_read){
    static unsigned char _time_stamp_buffer[18];                                               // buffer con data e ora
    unsigned char _time_stamp_buffer_index = 0;
    unsigned char _readed = 0;
    unsigned long _temp = 0;
    
    _time_stamp_buffer[2] = '/';
    _time_stamp_buffer[5] = '/';
    _time_stamp_buffer[8] = ' ';
    _time_stamp_buffer[11] = ':';
    
    
    for (unsigned char _i = 0; _i < _data_2_read; _i++){                                // leggo dati
        _readed = EEPROM_I2C_read(_eeprom_dev_address,_eeprom_addr);
        _temp = (_readed >> 4) & 0x0F;
        _time_stamp_buffer[_time_stamp_buffer_index] = _temp + '0';
        _time_stamp_buffer_index++;
        _time_stamp_buffer[_time_stamp_buffer_index] = (_readed & 0x0F) + '0';
        _time_stamp_buffer_index +=2;
        _eeprom_addr++;
    }
    
    switch (_data_2_read){
        case 5: _time_stamp_buffer[14] = '\0';
            break;
        case 6: _time_stamp_buffer[14] = ':'; 
                _time_stamp_buffer[17] = '\0';   
    }
    
    return _time_stamp_buffer;                                                          // torno indirizzo al primo valore dell'array
}

/* ============================================================================================================================================= /
 * FUNCTION:    Float_2_string
 * Input:       _float: valore da convertire (float)
 *              _digits_int: numero massimo di cifre intere (unsigned char)
 *              _digits_part: numero massimo di cifre decimali (unsigned char)
 *              _zero_cleaning: on/off per eliminare gli zeri iniziali se presenti (unsigned char)
 * Output:      puntatore all'array contentente la stringa    
 *              
 * Overview:    Converte in stringa un valore float e restituisce un puntatore all'array contenente la stringa
/* ============================================================================================================================================ */
unsigned char * Float_2_string (float _float, unsigned char _digits_int, unsigned char _digits_part, unsigned char _zero_cleaning) {
    signed int _int = 0;
    float _part = 0;
    unsigned char _converted[5] = {0};
    unsigned char _string_part[10] = {0};
    static unsigned char _string_compl[18] = {0};                                       // stringa finale segno + max 5 interi + , + max 10 decimali + carattere di fine stringa
    unsigned char _string_index = 0;
    
    unsigned int _denom = 0;
    unsigned char _first_non_zero = 0;
    
    if (_digits_int > 5)                                                                // valore massimo di interi
        _digits_int = 5;
    
    if (_digits_part > 10)                                                              // valore massimo di decimali = 10
        _digits_part = 10;
        
    if (_float < 0){                                                                    // valore negativo
        _float *= -1;                                                                   // porto positivo
       _string_compl[0] = '-';                                                          // e scrivo segno meno nella stringa finale
       _string_index++;
    }
    
    _int = (int) _float;                                                                // estraggo la parte intera
    _part = _float - (float)_int;                                                       // lascio solo la parte decimale
    
    // converto decimali in stringa
    for (unsigned char _i = 0; _i < _digits_part +1; _i++ ){                            // per ogni decimale richiesto +1 per calcolo approssimazione
        _part = _part *10;                                                              // moltiplico per 10
        _string_part[_i] = (unsigned char)_part;                                        // estraggo la parte intera
        _part = _part - (float) _string_part[_i];                                       // dalla parte decimale sottraggo la nuova intera
    }    
    
    // arrotondamento decimali
    if (_digits_part == 0) {                                                            // 0 cifre decimali
        if (_string_part[0] > 4){                                                       // se la prima decimale  maggiore di 4 aggiungo 1 agli interi
            if (_int >= 0)                                                              // se positivo
                _int++;
            else                                                                        // altrimenti sottraggo 1
                _int--;    
        }
    } else {                                                                            // se ho pi cifre decimali
        if (_string_part[_digits_part] > 4){                                            // se l'ultima decimale (aggiunta solo per l'arrotondamento)  maggiore di 4
                _string_part[_digits_part - 1]++;                                       // aggiungo uno alla decimale precedente
        }
        
        for (unsigned char _i = 0; _i < _digits_part -1; _i++ ){                        // controlla dalla penultima decimale alla prima
            if (_string_part[_digits_part - 1 - _i] > 9){                               // se maggiore di 9
                _string_part[_digits_part - 1 - _i] = 0;                                // la porto a 0
                _string_part[_digits_part - 2 - _i]++;                                  // e aggiungo uno alla precedente
            }
        }

        if (_string_part[0] > 9){                                                       // se la prima decimale  maggiore di 9
            _string_part[0] = 0;                                                        // la azzero e incremento o decremento gli interi
            if (_int >= 0)
                _int++;
            else
                _int--;
        }
    
    }
    
     // conversione intero in stringa
    for (unsigned char _i=0; _i<4; _i++) {                                              // converto unsigned int
        if (_i==0) {_denom = 10000;}                                                    
        if (_i==1) {_denom = 1000;}                                                     
        if (_i==2) {_denom = 100;}                                                      
        if (_i==3) {_denom = 10;}                                                       

        _converted[_i] = _int/_denom;                                                   // in array da 5 caratteri
        if (_converted[_i]) {                                                           // se risultato != 0
            _int = _int - (_denom *_converted[_i]);                                    // sottraggo il valore gi  scritto
        }
    }
    _converted[4]= _int;                                                                // riporto il resto nell'array

                                                                                        // in array _converted ho il dato completo di 0 iniziali
                                                                                        // sposto in stringa finale solo le cifre massime definite da _digits_int e da _zero_cleaning 
    for (unsigned char _c = 0; _c < _digits_int-1 ; _c++) {                             // ciclo per numero di cifre -1 (l'ultima la scrivo sempre)
        if (_converted[_c + (5 -_digits_int)] != 0) {
            _string_compl[_string_index] = _converted[_c + (5 -_digits_int)] + 0b00110000;
            _first_non_zero = 1;
        } else {
            if (_zero_cleaning == ZERO_CLEANING_ON && _first_non_zero == 0)
                _string_index--;
            else
                _string_compl[_string_index] = '0';
        }
        _string_index++;
    }
    _string_compl[_string_index] =  _converted[4] + 0b00110000;
    
    _string_index++;
    
    if (_digits_part > 0){
        _string_compl[_string_index] = ',';                                                 // salvo carattere , nella stringa
        _string_index++;

        for (unsigned char _i = 0; _i < _digits_part; _i++){
            _string_compl[_string_index] = _string_part[_i] |= 0b00110000;                  // sposto i valori dall'array _string_part a _string_compl convertendolo in carattere ASCII
            _string_index++;
        }
    }
        
    _string_compl[_string_index] = '\0';                                                // aggiungo carattere di fine stringa
    
    return _string_compl;
} 

